/* File: SidebarWindow.java
 *
 * Contains class SidebarWindow
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.SystemColor;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;

import edu.cmu.hcii.calo.controller.NotificationCenterController;
import edu.cmu.hcii.calo.controller.SchedulePaneController;
import edu.cmu.hcii.calo.controller.SidebarController;
import edu.cmu.hcii.calo.controller.TaskPaneController;
import edu.cmu.hcii.calo.support.SidebarPropertyNames;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.CALOScrollPane;
import edu.cmu.hcii.calo.view.FlatButton;
import edu.cmu.hcii.calo.view.IconToggleButton;
import edu.cmu.hcii.calo.view.iconwell.IconWellView;
import edu.cmu.hcii.calo.view.notification.NotificationCenterView;
import edu.cmu.hcii.calo.view.panes.CALOPane;
import edu.cmu.hcii.calo.view.relevantstuff.RelevantStuffPaneView;
import edu.cmu.hcii.calo.view.schedule.SchedulePaneView;
import edu.cmu.hcii.calo.view.task.TaskPaneView;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The sidebar window itself. This window hooks up all the controllers to the views, lays out the
 * panes, handles the transition between sidebar and minibar mode, handles the collapsing and
 * expanding of panes, and quite a lot more.
 * 
 * @author Brian Ellis
 * 
 */
public class SidebarWindow extends WinAppBar {

	/**
	 * Listens for clicks on one of the buttons in the minibar that show and hide panes, and shows
	 * or hides the pane appopriately.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	final class PaneButtonListener implements ActionListener {

		/**
		 * The pane this listener will show when invoked.
		 */
		private CALOPane pane;

		/**
		 * Constructs a PaneButtonListener that shows the given pane.
		 */
		public PaneButtonListener(CALOPane pane) {
			this.pane = pane;
		}

		/**
		 * Invoked when the button is clicked. Shows or hides the appropriate pane.
		 * 
		 * @param e the action event
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {

			if (SidebarWindow.this.taskPaneButton != e.getSource()) {
				SidebarWindow.this.taskPaneButton.setSelected(false);
			}
			if (SidebarWindow.this.schedulePaneButton != e.getSource()) {
				SidebarWindow.this.schedulePaneButton.setSelected(false);
			}
			if (SidebarWindow.this.relevantStuffPaneButton != e.getSource()) {
				SidebarWindow.this.relevantStuffPaneButton.setSelected(false);
			}

			if (SidebarWindow.this.containerFrame != null
					&& SidebarWindow.this.containerFrame.isVisible()) {
				// It's visible, so close it.
				SidebarWindow.this.containerFrame.setVisible(false);
				if (SidebarWindow.this.currentlyVisiblePane == this.pane) {
					// We clicked on the same button we used to open it, so
					// don't open anything
					// else.
					SidebarWindow.this.currentlyVisiblePane = null;
					return;
				}
			}
			SidebarWindow.this.currentlyVisiblePane = this.pane;

			SidebarWindow.this.containerFrame = new JDialog();
			SidebarWindow.this.containerFrame.setLayout(new RelativeLayout());
			SidebarWindow.this.containerFrame.setUndecorated(true);
			SidebarWindow.this.containerFrame.add(this.pane, new RelativeConstraints());
			Point p = ((Component)e.getSource()).getLocation();
			SwingUtilities.convertPointToScreen(p, SidebarWindow.this.getContentPane());
			SidebarWindow.this.containerFrame.setLocation(SidebarWindow.this.getX()
					- SidebarWindow.MINIBAR_PANE_WIDTH, p.y);
			// Set up the initial pane sizes:
			int paneHeight = Toolkit.getDefaultToolkit().getScreenSize().height - p.y - 30;

			this.pane.setPreferredSize(new Dimension(SidebarWindow.MINIBAR_PANE_WIDTH, paneHeight));

			SidebarWindow.this.containerFrame.pack();
			SidebarWindow.this.containerFrame.setVisible(true);

		}
	}

	/**
	 * Listens for property changes indicating that a pane has collapsed or expanded, and updates
	 * the sidebar accordingly.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	final class PaneExpansionListener implements PropertyChangeListener {

		/**
		 * Invoked when a pane is expanded or collapsed. Updates the sidebar.
		 * 
		 * @param e the property change event
		 * 
		 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
		 */
		public void propertyChange(PropertyChangeEvent e) {

			if (e.getPropertyName().equals(SidebarPropertyNames.PANE_EXPANDED)) {
				assert e.getOldValue() instanceof Boolean && e.getNewValue() instanceof Boolean : "got a pane expanded event whose values weren't booleans.";
				SidebarWindow.this.update();
			}
		}
	}

	/**
	 * Listens for property changes indicating that a pane's size has changed, and updates the
	 * sidebar accordingly.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	final class PaneResizeListener implements PropertyChangeListener {

		/**
		 * The height of a collapsed pane.
		 */
		private static final int COLLAPSED_PANE_HEIGHT = 26;

		/**
		 * The minimum size we should let the last pane reach before we stop letting the user shrink
		 * it.
		 */
		private static final int MINIMUM_LAST_PANE_HEIGHT = 55;

		/**
		 * The pane that should have a fluid size when other panes are resized. This shouldn't be
		 * hardcoded, but it is.
		 */
		private final CALOPane LAST_PANE = SidebarWindow.this.schedulePaneContainerPane;

		/**
		 * The pane we're listening to.
		 */
		private CALOPane pane;

		/**
		 * Constructs a PaneResizeListener for the given pane.
		 * 
		 * @param pane the pane to listen to
		 */
		public PaneResizeListener(CALOPane pane) {
			this.pane = pane;
		}

		/**
		 * Invoked when a pane resizes. Updates the sidebar.
		 * 
		 * @param evt the property change event
		 */
		public void propertyChange(PropertyChangeEvent evt) {
			Dimension d = (Dimension)evt.getNewValue();
			if (d.height < 26) {
				// Dragging it here would cause the pane to subduct under the
				// previous pane's title region, so we'll stop it:
				this.pane.setPreferredSize(new Dimension(	d.width,
															PaneResizeListener.COLLAPSED_PANE_HEIGHT));
			} else {
				int heightChange = d.height - ((Dimension)evt.getOldValue()).height;
				if (this.LAST_PANE.getHeight() - heightChange > PaneResizeListener.MINIMUM_LAST_PANE_HEIGHT) {
					// We're all clear:
					this.pane.setPreferredSize(d);
				} else {
					// Dragging it here would crush the last pane (the relevant
					// stuff pane in our case), so we'll stop it:
					int degreeOfTooLargeness =
						PaneResizeListener.MINIMUM_LAST_PANE_HEIGHT
								- (this.LAST_PANE.getHeight() - heightChange);
					this.pane.setPreferredSize(new Dimension(d.width, d.height
							- degreeOfTooLargeness));
				}
			}
			// Revalidate the pane, which actually serves to revalidate all the
			// other panes too since the pane is not a validate root.
			this.pane.revalidate();
		}
	}

	/** The default width of the full sidebar */
	private static final int FULLBAR_DEFAULT_WIDTH = 200;

	/**
	 * The width of the minibar (cannot be changed by the user).
	 */
	private static final int MINIBAR_WIDTH = 45;

	/**
	 * The width of panes that are shown from the minibar.
	 */
	private static final int MINIBAR_PANE_WIDTH = 300;

	/** Prevent serialization collisions */
	private static final long serialVersionUID = 1L;

	/**
	 * @name Buttons
	 * 
	 * The buttons that show various panes in the minibar.
	 */

	// @{
	protected IconToggleButton taskPaneButton;
	protected IconToggleButton schedulePaneButton;
	protected IconToggleButton relevantStuffPaneButton;
	// @}

	/**
	 * @name Panes
	 * 
	 * The various panes we show in the sidebar.
	 */

	// @{
	/** The notification center visualization. */
	private NotificationCenterView notificationCenterView;

	/** The task pane visualization. */
	private TaskPaneView taskPaneView;

	/** The schedule pane visualization. */
	private SchedulePaneView schedulePaneView;

	/** The relevant stuff pane visualization. */
	protected CALOComponent relevantStuffPaneView;

	/** The icon well visualization. */
	private IconWellView iconWellView;

	// @}

	/**
	 * @name Container Panes
	 * 
	 * Most panes need a title bar, a resize widget, and other such decorations. These container
	 * panes provide those.
	 */

	// @{
	/** The scrollable pane containing the task pane. */
	private CALOPane taskPaneContainerPane;

	/** The scrollable pane containing the schedule pane. */
	private CALOPane schedulePaneContainerPane;

	/** The pane containing the notification center. */
	private CALOPane notificationCenterContainerPane;

	/** The scrollable pane containing the relevant stuff pane. */
	protected CALOPane relevantStuffPaneContainerPane;

	// @}

	/**
	 * @name Other Controls
	 * 
	 * Miscellaneous controls that appear in the sidebar.
	 */

	// @{
	private JDialog containerFrame;
	private CALOPane currentlyVisiblePane;
	private FlatButton sidebarMiniaturizeButton;
	// @}

	/**
	 * @name Controllers
	 * 
	 * The controllers that control the various panes, as well as one for the sidebar itself.
	 */

	// @{
	/** Controls the sidebar display. */
	@SuppressWarnings("unused")
	// This is used in the thread, but not locally
	private SidebarController sidebarController;

	/** Controls the notification center display. */
	@SuppressWarnings("unused")
	// This is used in the thread, but not locally
	private NotificationCenterController notificationCenterController;

	/** Controls the task pane display. */
	@SuppressWarnings("unused")
	// This is used in the thread, but not locally
	private TaskPaneController taskPaneController;

	/** Controls the schedule pane display. */
	@SuppressWarnings("unused")
	// This is used in the thread, but not locally
	private SchedulePaneController schedulePaneController;
	// @}

	/** Whether or not the sidebar is in minibar mode. */
	private boolean isMiniaturized;

	/** The map of components to their preferred sizes when not collapsed. */
	private Map<Component, Dimension> oldPreferredSizes;

	/** The current width of the full sidebar. */
	private int fullSidebarWidth;

	/**
	 * Constructs a SidebarWindow in either full or minibar mode.
	 * 
	 * @param isMiniaturized if \c true, the sidebar appears in minibar mode
	 */
	public SidebarWindow(boolean isMiniaturized) {
		this.oldPreferredSizes = new HashMap<Component, Dimension>();
		this.fullSidebarWidth = SidebarWindow.FULLBAR_DEFAULT_WIDTH;

		this.isMiniaturized = isMiniaturized;

		this.setTitle("CALO Sidebar");

		SwingUtilities.invokeLater(new Runnable() {

			@SuppressWarnings("synthetic-access")
			public void run() {

				try {
					setSize(SidebarWindow.FULLBAR_DEFAULT_WIDTH,
							SidebarWindow.FULLBAR_DEFAULT_WIDTH);
				} catch (UnsatisfiedLinkError e) {
					// do nothing
				}
				// Create panes:
				SidebarWindow.this.notificationCenterView = new NotificationCenterView(false);
				SidebarWindow.this.taskPaneView = new TaskPaneView();
				SidebarWindow.this.schedulePaneView = new SchedulePaneView();
				SidebarWindow.this.relevantStuffPaneView = new RelevantStuffPaneView();
				SidebarWindow.this.iconWellView = new IconWellView();

				// Create controllers:
				// We MUST create the sidebar controller first, or bad things
				// happen for mysterious reasons.
				SidebarWindow.this.sidebarController = new SidebarController(SidebarWindow.this);
				SidebarWindow.this.notificationCenterController =
					new NotificationCenterController(SidebarWindow.this.notificationCenterView);
				SidebarWindow.this.taskPaneController =
					new TaskPaneController(SidebarWindow.this.taskPaneView);
				SidebarWindow.this.schedulePaneController =
					new SchedulePaneController(SidebarWindow.this.schedulePaneView);

				// Create container panes:
				SidebarWindow.this.notificationCenterContainerPane =
					new CALOPane(	"Notification Center",
									SidebarWindow.this.notificationCenterView,
									false,
									false,
									false);
				SidebarWindow.this.taskPaneContainerPane =
					new CALOPane("Tasks", SidebarWindow.this.taskPaneView, true, true, true);
				SidebarWindow.this.schedulePaneContainerPane =
					new CALOPane("Schedule", SidebarWindow.this.schedulePaneView, true, true, false);
				CALOScrollPane scrollPane =
					new CALOScrollPane(SidebarWindow.this.relevantStuffPaneView);
				scrollPane.setBackground(SystemColor.text);
				SidebarWindow.this.relevantStuffPaneContainerPane =
					new CALOPane("CALO Suggestions", scrollPane, true, true, true);

				// Create buttons:
				SidebarWindow.this.sidebarMiniaturizeButton =
					new FlatButton(new ImageIcon("Resources/miniaturize.png"));

				SidebarWindow.this.taskPaneButton =
					new IconToggleButton(new ImageIcon("Resources/mini_TaskPane.png"));

				SidebarWindow.this.schedulePaneButton =
					new IconToggleButton(new ImageIcon("Resources/mini_Calendar.png"));

				SidebarWindow.this.relevantStuffPaneButton =
					new IconToggleButton(new ImageIcon("Resources/mini_CALO_Suggestion.png"));

				// Call setup:
				SidebarWindow.this.setup();
			}
		});

	}

	/**
	 * Closes the sidebar window.
	 */
	public void closeWindow() {
		System.exit(0);
	}

	/**
	 * Fix up the visual look of this sidebar.
	 */
	public void update() {
		this.taskPaneContainerPane.update();
		this.schedulePaneContainerPane.update();
		this.relevantStuffPaneContainerPane.update();
	}

	/**
	 * Returns whether the sidebar is in minibar mode.
	 * 
	 * @return \c true if the sidebar is miniaturized, \c false otherwise
	 */
	private boolean isMiniaturized() {
		return this.isMiniaturized;
	}

	/**
	 * Performs the layout of the full sidebar.
	 */
	private void layoutFullSidebar() {

		this.setResizable(true);
		this.getContentPane().setBackground(SystemColor.controlShadow);

		try {
			this.setSidebarSize(this.fullSidebarWidth, this.getSize().height);
		} catch (UnsatisfiedLinkError e) {
			// do nothing
		}
		this.setSize(this.fullSidebarWidth, this.getSize().height);

		this.notificationCenterView.setMini(false);

		// Recreate container panes:
		SidebarWindow.this.notificationCenterContainerPane =
			new CALOPane(	"Notification Center",
							SidebarWindow.this.notificationCenterView,
							false,
							false,
							false);

		PaneExpansionListener expansionListener = new PaneExpansionListener();

		this.taskPaneContainerPane.setResizable(true);
		this.taskPaneContainerPane.setCollapsible(true);

		this.schedulePaneContainerPane.setCollapsible(true);

		this.relevantStuffPaneContainerPane.setCollapsible(true);

		// Add our property change listeners, first for expansion, then for
		// resize:
		this.taskPaneContainerPane.addPropertyChangeListener(expansionListener);
		this.schedulePaneContainerPane.addPropertyChangeListener(expansionListener);
		this.relevantStuffPaneContainerPane.addPropertyChangeListener(expansionListener);

		this.taskPaneContainerPane.addPropertyChangeListener(	"DesiredPaneSize",
																new PaneResizeListener(this.taskPaneContainerPane));
		this.schedulePaneContainerPane.addPropertyChangeListener(	"DesiredPaneSize",
																	new PaneResizeListener(this.schedulePaneContainerPane));
		this.relevantStuffPaneContainerPane.addPropertyChangeListener(	"DesiredPaneSize",
																		new PaneResizeListener(this.relevantStuffPaneContainerPane));

		// Construct constraints for each pane:
		BindingMaker bm = new BindingMaker(0, 0, 0, 0, 0, 0);
		RelativeConstraints notificationCenterConstraints =
			new RelativeConstraints(bm.leftEdge(),
									bm.rightEdge(),
									bm.directlyBelow(this.sidebarMiniaturizeButton));
		RelativeConstraints taskPaneConstraints =
			new RelativeConstraints(bm.leftEdge(),
									bm.rightEdge(),
									bm.below(this.notificationCenterContainerPane));
		RelativeConstraints schedulePaneConstraints =
			new RelativeConstraints(bm.leftEdge(),
									bm.rightEdge(),
									bm.above(this.iconWellView),
									bm.below(this.relevantStuffPaneContainerPane));
		RelativeConstraints relevantStuffPaneConstraints =
			new RelativeConstraints(bm.leftEdge(),
									bm.rightEdge(),
									bm.below(this.taskPaneContainerPane));
		RelativeConstraints iconWellConstraints =
			new RelativeConstraints(bm.leftEdge(), bm.rightEdge(), bm.bottomEdge());

		// Add each pane:
		this.add(this.notificationCenterContainerPane, notificationCenterConstraints);
		this.add(this.taskPaneContainerPane, taskPaneConstraints);
		this.add(this.schedulePaneContainerPane, schedulePaneConstraints);
		this.add(this.relevantStuffPaneContainerPane, relevantStuffPaneConstraints);
		this.add(this.iconWellView, iconWellConstraints);

		int availableHeight =
			Toolkit.getDefaultToolkit().getScreenSize().height - this.getContentPane().getY()
					- this.relevantStuffPaneContainerPane.getPreferredSize().height
					- this.iconWellView.getPreferredSize().height;

		// Set up the initial pane sizes:
		if (!this.taskPaneContainerPane.isPreferredSizeSet()) {
			int taskPaneHeight = (int)(2.0f / 3.0f * availableHeight);
			int schedulePaneHeight = (int)(1.0f / 3.0f * availableHeight);

			// System.out.println(taskPaneHeight + ", " + schedulePaneHeight);

			this.taskPaneContainerPane.setPreferredSize(new Dimension(	this.taskPaneContainerPane.getPreferredSize().width,
																		taskPaneHeight));

			this.schedulePaneContainerPane.setPreferredSize(new Dimension(	this.schedulePaneContainerPane.getPreferredSize().width,
																			schedulePaneHeight));
		}

		// Remove borders from the panes:
		this.taskPaneContainerPane.setBorder(null);
		this.schedulePaneContainerPane.setBorder(null);
		this.relevantStuffPaneContainerPane.setBorder(null);

	}

	/**
	 * Performs the layout of the minibar.
	 */
	private void layoutMinibar() {
		this.getContentPane().setBackground(SystemColor.control);

		this.fullSidebarWidth = this.getWidth();

		this.setResizable(false);

		try {
			this.setSidebarSize(SidebarWindow.MINIBAR_WIDTH, this.getSize().height);
		} catch (UnsatisfiedLinkError e) {
			// do nothing
		}
		this.setSize(SidebarWindow.MINIBAR_WIDTH, this.getSize().height);

		this.notificationCenterView.setMini(true);
		this.notificationCenterView.setBorder(new MatteBorder(	0,
																0,
																1,
																0,
																SystemColor.controlDkShadow));

		this.taskPaneContainerPane.setResizable(false);
		this.taskPaneContainerPane.setCollapsible(false);

		this.schedulePaneContainerPane.setCollapsible(false);

		this.relevantStuffPaneContainerPane.setCollapsible(false);

		// Add actions to our pane buttons:
		this.taskPaneButton.addActionListener(new PaneButtonListener(this.taskPaneContainerPane));
		this.schedulePaneButton.addActionListener(new PaneButtonListener(this.schedulePaneContainerPane));
		this.relevantStuffPaneButton.addActionListener(new PaneButtonListener(this.relevantStuffPaneContainerPane));

		// Construct constraints for each pane button:
		BindingMaker bm = new BindingMaker(2, 2, 2, 2, 0, 2);

		RelativeConstraints notificationCenterConstraints =
			new RelativeConstraints(bm.directLeftEdge(),
									bm.directRightEdge(),
									bm.directlyBelow(this.sidebarMiniaturizeButton));
		RelativeConstraints taskPaneButtonConstraints =
			new RelativeConstraints(bm.horizontallyCenterAlign(null),
									bm.below(this.notificationCenterView));
		RelativeConstraints schedulePaneButtonConstraints =
			new RelativeConstraints(bm.horizontallyCenterAlign(null),
									bm.below(this.relevantStuffPaneButton));
		RelativeConstraints relevantStuffPaneButtonConstraints =
			new RelativeConstraints(bm.horizontallyCenterAlign(null), bm.below(this.taskPaneButton));
		RelativeConstraints iconWellConstraints =
			new RelativeConstraints(bm.directLeftEdge(),
									bm.directRightEdge(),
									bm.directBottomEdge());

		// Add each pane:
		this.add(this.notificationCenterView, notificationCenterConstraints);
		this.add(this.taskPaneButton, taskPaneButtonConstraints);
		this.add(this.schedulePaneButton, schedulePaneButtonConstraints);
		this.add(this.relevantStuffPaneButton, relevantStuffPaneButtonConstraints);
		this.add(this.iconWellView, iconWellConstraints);

		// Add borders to the panes for when they're in their own windows:
		this.taskPaneContainerPane.setBorder(new LineBorder(SystemColor.controlDkShadow));
		this.schedulePaneContainerPane.setBorder(new LineBorder(SystemColor.controlDkShadow));
		this.relevantStuffPaneContainerPane.setBorder(new LineBorder(SystemColor.controlDkShadow));

	}

	/**
	 * Sets the mode of the sidebar to either full or minibar mode.
	 * 
	 * @param isMiniaturized \c true if the sidebar should go into minibar mode, \c false otherwise
	 */
	private void setMiniaturized(boolean isMiniaturized) {
		this.isMiniaturized = isMiniaturized;

		this.remove(this.notificationCenterContainerPane);
		this.remove(this.taskPaneContainerPane);
		this.remove(this.schedulePaneContainerPane);
		this.remove(this.relevantStuffPaneContainerPane);
		this.remove(this.iconWellView);
		this.remove(this.taskPaneButton);
		this.remove(this.schedulePaneButton);
		this.remove(this.relevantStuffPaneButton);

		if (!this.isMiniaturized) {
			this.notificationCenterView.closeNotificationGroupOutsideSidebar();
		}

		if (this.currentlyVisiblePane != null) {
			this.currentlyVisiblePane.getTopLevelAncestor().setVisible(false);
			this.currentlyVisiblePane = null;
		}

		ActionListener[] actionListeners = this.taskPaneButton.getActionListeners();
		for (ActionListener actionListener : actionListeners) {
			this.taskPaneButton.removeActionListener(actionListener);
		}

		actionListeners = this.schedulePaneButton.getActionListeners();
		for (ActionListener actionListener : actionListeners) {
			this.schedulePaneButton.removeActionListener(actionListener);
		}

		actionListeners = this.relevantStuffPaneButton.getActionListeners();
		for (ActionListener actionListener : actionListeners) {
			this.relevantStuffPaneButton.removeActionListener(actionListener);
		}

		PropertyChangeListener[] propertyChangeListeners =
			this.taskPaneContainerPane.getPropertyChangeListeners();
		for (PropertyChangeListener listener : propertyChangeListeners) {
			this.taskPaneContainerPane.removePropertyChangeListener(listener);
			this.schedulePaneContainerPane.removePropertyChangeListener(listener);
			this.relevantStuffPaneContainerPane.removePropertyChangeListener(listener);
		}

		propertyChangeListeners =
			this.taskPaneContainerPane.getPropertyChangeListeners("DesiredPaneSize");
		for (PropertyChangeListener listener : propertyChangeListeners) {
			this.taskPaneContainerPane.removePropertyChangeListener("DesiredPaneSize", listener);
		}
		propertyChangeListeners =
			this.schedulePaneContainerPane.getPropertyChangeListeners("DesiredPaneSize");
		for (PropertyChangeListener listener : propertyChangeListeners) {
			this.schedulePaneContainerPane.removePropertyChangeListener("DesiredPaneSize", listener);
		}
		propertyChangeListeners =
			this.relevantStuffPaneContainerPane.getPropertyChangeListeners("DesiredPaneSize");
		for (PropertyChangeListener listener : propertyChangeListeners) {
			this.relevantStuffPaneContainerPane.removePropertyChangeListener(	"DesiredPaneSize",
																				listener);
		}

		if (this.isMiniaturized()) {
			this.oldPreferredSizes.put(	this.taskPaneContainerPane,
										this.taskPaneContainerPane.getPreferredSize());
			this.oldPreferredSizes.put(	this.schedulePaneContainerPane,
										this.schedulePaneContainerPane.getPreferredSize());
			this.oldPreferredSizes.put(	this.relevantStuffPaneContainerPane,
										this.relevantStuffPaneContainerPane.getPreferredSize());

			layoutMinibar();
		} else {
			this.taskPaneContainerPane.setPreferredSize(this.oldPreferredSizes.get(this.taskPaneContainerPane));
			this.schedulePaneContainerPane.setPreferredSize(this.oldPreferredSizes.get(this.schedulePaneContainerPane));
			this.relevantStuffPaneContainerPane.setPreferredSize(this.oldPreferredSizes.get(this.relevantStuffPaneContainerPane));

			layoutFullSidebar();
		}

	}

	/**
	 * Sets up the visual appearance of this window.
	 */
	private void setup() {

		// Sleep for 50 ms to give Windows a chance to catch up with the window
		// handle before we sidebarize it. Yes, this is a dirty dirty hack, but
		// it's a race condition in the JVM -- what else are we supposed to do?
		try {
			Thread.sleep(50);
		} catch (InterruptedException e) {
			// This catch block intentionally left blank
		}

		this.setLayout(new RelativeLayout());

		this.sidebarMiniaturizeButton.setHorizontalAlignment(SwingConstants.RIGHT);
		this.sidebarMiniaturizeButton.setBackground(SystemColor.control);
		this.sidebarMiniaturizeButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				SidebarWindow.this.setMiniaturized(!SidebarWindow.this.isMiniaturized());
				ImageIcon newIcon =
					(SidebarWindow.this.isMiniaturized()
							? new ImageIcon("Resources/deminiaturize.png")
							: new ImageIcon("Resources/miniaturize.png"));
				SidebarWindow.this.sidebarMiniaturizeButton.setHorizontalAlignment(SidebarWindow.this.isMiniaturized()
						? SwingConstants.LEFT
						: SwingConstants.RIGHT);
				SidebarWindow.this.sidebarMiniaturizeButton.setIcon(newIcon);
				firePropertyChange("Miniaturized", isMiniaturized(), !isMiniaturized());
			}

		});

		this.taskPaneButton.setPreferredSize(new Dimension(	SidebarWindow.MINIBAR_WIDTH - 6,
															this.taskPaneButton.getPreferredSize().height));
		this.schedulePaneButton.setPreferredSize(new Dimension(	SidebarWindow.MINIBAR_WIDTH - 6,
																this.schedulePaneButton.getPreferredSize().height));
		this.relevantStuffPaneButton.setPreferredSize(new Dimension(SidebarWindow.MINIBAR_WIDTH - 6,
																	this.schedulePaneButton.getPreferredSize().height));

		this.taskPaneButton.setUnselectedToolTipText("Click to show the Tasks pane.");
		this.schedulePaneButton.setUnselectedToolTipText("Click to show the Schedule pane.");
		this.relevantStuffPaneButton.setUnselectedToolTipText("Click to show the CALO Suggestions pane.");

		this.taskPaneButton.setSelectedToolTipText("Click to hide the Tasks pane.");
		this.schedulePaneButton.setSelectedToolTipText("Click to hide the Schedule pane.");
		this.relevantStuffPaneButton.setSelectedToolTipText("Click to hide the CALO Suggestions pane.");

		BindingMaker bm = new BindingMaker(0, 0, 0, 0, 0, 0);

		this.add(this.sidebarMiniaturizeButton, new RelativeConstraints(bm.directTopEdge(),
																		bm.directLeftEdge(),
																		bm.directRightEdge()));

		layoutFullSidebar();

		this.iconWellView.setOpaque(true);
		this.iconWellView.setBackground(SystemColor.control);

		try {
			this.setAsBar(this.getTitle(), WinAppBar.RIGHT);
		} catch (UnsatisfiedLinkError e) {
			// do nothing
		}
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent en) {
				try {
					SidebarWindow.this.unSetBar();
				} catch (UnsatisfiedLinkError e) {
					// do nothing
				}
			}
		});
		this.update();
		this.validate();
	}
}
