/* File: WinAppBar.java
 *
 * Contains class WinAppBar
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo;

import javax.swing.JFrame;

/**
 * A native class that allows a Java frame to be a Windows sidebar. A frame wishing to become a
 * sidebar must call setAsBar() after becoming visible (it is best to wait briefly before calling
 * setAsBar() so that Windows can catch up) and pass one of the edge constants to determine where
 * the sidebar will be anchored. When the application is about to exit, the frame should call
 * unSetBar() to relinquish its status as a sidebar. When the width of the window changes
 * programmatically, the window should call setSidebarSize() with its new width and height to keep
 * Windows informed about the change.
 * 
 * @warning If your application exits for any reason (even an unhandled RuntimeException) and you
 *          fail to call unSetBar(), the space claimed by the sidebar will be unavailable to all
 *          windows (even maximized ones) until you restart the computer or call setAppBar() or
 *          unSetBar() in a subsequent run of the application. So make sure that, however badly you
 *          crash, you always find a way to call unSetBar() before you die.
 * 
 * @author Brian Ellis
 * 
 */
public class WinAppBar extends JFrame {

	/**
	 * Pass to setAsBar to indicate that the sidebar should appear on the left edge of the screen.
	 */
	public static final int LEFT = 1;

	/**
	 * Pass to setAsBar to indicate that the sidebar should appear on the top edge of the screen.
	 */
	public static final int TOP = 2;

	/**
	 * Pass to setAsBar to indicate that the sidebar should appear on the right edge of the screen.
	 */
	public static final int RIGHT = 3;

	/**
	 * Pass to setAsBar to indicate that the sidebar should appear on the bottom edge of the screen.
	 */
	public static final int BOTTOM = 4;

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 1L;

	/**
	 * Attempts to load the WinAppBar library, printing a non-fatal error message if it cannot do
	 * so.
	 */
	static {
		try {
			System.loadLibrary("WinAppBar");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Can't load WinAppBar library.  Not being a sidebar.");
		}
	}

	/**
	 * Sets this window as a sidebar and places it on the specified edge of the screen.
	 * 
	 * @warning the WinAppBar library uses the window title to find a window handle for the window.
	 *          Therefore, not only must the \p title argument match the title of the window, but it
	 *          must be unique across ALL open windows in every application. It may be worthwhile to
	 *          temporarily set the window title to a UUID or other guaranteed-unique identifier
	 *          (the application name and date stamp would suffice in a pinch), then call setAppBar,
	 *          then set it back when you're done.
	 * @param title the current title of this window
	 * @param position the desired position of the sidebar
	 */
	public native void setAsBar(String title, int position);

	/**
	 * Informs the WinAppBar library that the size of the sidebar has changed. Call this whenever
	 * you set the window size programmatically from within Java.
	 * 
	 * @param width the new width of the sidebar
	 * @param height the new height of the sidebar
	 */
	public native void setSidebarSize(int width, int height);

	/**
	 * Causes this window to relinquish its status as a Windows sidebar. You must call unSetBar()
	 * before your application exits, or the space claimed by the sidebar will not be available to
	 * any other windows (even maximized ones) until you restart the computer or call setAsBar() or
	 * unSetBar() in a future run of the application.
	 */
	public native void unSetBar();
}
