/*
 * File: NotificationCenterController.java
 * 
 * Contains class NotificationCenterController
 * 
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.controller;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import java.util.Observable;

import javax.swing.SwingUtilities;

import edu.cmu.hcii.calo.engine.WizardOfOzEngine;
import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.MessageTarget;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage.WizardOfOzMessageType;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.view.notification.NotificationCenterView;
import edu.cmu.hcii.calo.view.notification.NotificationGroupView;
import edu.cmu.hcii.calo.view.notification.support.NotificationGroupState;
import edu.cmu.hcii.calo.view.notification.support.NotificationState;
import edu.cmu.hcii.calo.view.notification.support.NotificationViewPropertyNames;

/**
 * The controller that attaches the notification manager to its corresponding views.
 * 
 * @author Will Haines
 */
public class NotificationCenterController extends MessageTarget implements PropertyChangeListener {

	/** The view to control. */
	private NotificationCenterView view;

	/** Tells the controller to ignore model messages to assure atomic updates. */
	private boolean ignoreModelMessages;

	/** The last group shown outside of the mini bar. */
	private NotificationGroupView oldOutsideViewReference;

	/**
	 * Constructs a NotificationCenterController that sends updates to (and listens for property
	 * changes from) the given NotificationCenterView.
	 * 
	 * @param view the view to send updates to
	 */
	public NotificationCenterController(NotificationCenterView view) {
		this.view = view;
		this.ignoreModelMessages = false;
		view.addPropertyChangeListener(this);

		NotificationManager.getManager().addObserver(this);
		WizardOfOzEngine.getEngine().addObserver(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.messagePassing.MessageTarget#onMessageReceived(java.util.Observable,
	 *      edu.cmu.hcii.calo.messagePassing.Message)
	 */
	@Override
	public void onMessageReceived(Observable sender, Message msg) {
		assert (msg instanceof NotificationMessage || msg instanceof WizardOfOzMessage) : "notification center controller got a message of an unhandleable type";

		// Set controller ignore state as desired by the wizard engine
		if (msg.getMessageType() == WizardOfOzMessageType.NOTIFICATION_CONTROLLER_IGNORE_CHANGE) {
			this.ignoreModelMessages = (Boolean)msg.getUserInfo().get(WizardOfOzMessage.NEW_IGNORE);
		}

		// Ensure atomic animation updates
		if (this.ignoreModelMessages) {
			return;
		}

		if (SwingUtilities.isEventDispatchThread()) {
			this.view.update();
			this.view.repaint();
		} else {
			SwingUtilities.invokeLater(new Runnable() {

				@SuppressWarnings("synthetic-access")
				public void run() {
					NotificationCenterController.this.view.update();
					NotificationCenterController.this.view.repaint();
				}

			});
		}
	}

	/**
	 * This method gets called when a bound property is changed.
	 * 
	 * @param e A PropertyChangeEvent object describing the event source and the property that has
	 *            changed.
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent e) {

		// Handle notification property changes
		if (e.getPropertyName().equals(NotificationViewPropertyNames.NOTIFICATION_STATE)) {
			assert e.getOldValue() instanceof NotificationState
					&& e.getNewValue() instanceof NotificationState : "got a NotificationState property change event whose "
					+ "values weren't NotificationStates.";

			@SuppressWarnings("unused")
			NotificationState oldState = (NotificationState)e.getOldValue();
			NotificationState newState = (NotificationState)e.getNewValue();
			Notification n = newState.getNotification();

			switch (newState.getProperty()) {
			case NOTIFICATION_DELETED:
				assert newState.getValue() instanceof Boolean : "new deleted status was not a boolean";

				if ((Boolean)newState.getValue()) {
					NotificationManager.getManager().removeNotification(n);
				}
				break;
			}
		}

		// Handle notification group property changes
		else if (e.getPropertyName().equals(NotificationViewPropertyNames.NOTIFICATION_GROUP_STATE)) {
			assert e.getOldValue() instanceof NotificationGroupState
					&& e.getNewValue() instanceof NotificationGroupState : "got a NotificationGroupState property change event whose "
					+ "values weren't NotificationGroupStates.";

			@SuppressWarnings("unused")
			NotificationGroupState oldState = (NotificationGroupState)e.getOldValue();
			NotificationGroupState newState = (NotificationGroupState)e.getNewValue();
			List<Notification> nl = newState.getNotifications();
			switch (newState.getProperty()) {
			case NOTIFICATION_GROUP_DELETED:
				assert newState.getValue() instanceof Boolean : "new deleted status was not a boolean";

				// Close the window!
				this.view.closeNotificationGroupOutsideSidebar();
				this.oldOutsideViewReference = null;

				// Make sure the removal is atomic
				this.ignoreModelMessages = true;

				if ((Boolean)newState.getValue()) {

					for (Notification n : nl) {
						NotificationManager.getManager().removeNotification(n);
					}
				}

				this.ignoreModelMessages = false;

				this.view.update();
				this.view.repaint();

				break;
			default:
				break;
			}
		}

		// Handle expands
		else if (e.getPropertyName().equals(NotificationViewPropertyNames.NOTIFICATION_EXPANDED)) {
			this.view.update();
			this.view.repaint();
		}

		// Handle placing a notification outside the sidebar
		else if (e.getPropertyName().equals(NotificationViewPropertyNames.NOTIFICATION_OUTSIDE_MOVE)) {

			NotificationGroupView newOutsideView = (NotificationGroupView)e.getNewValue();
			// You clicked twice on the same group, close instead open
			if (newOutsideView.equals(this.oldOutsideViewReference)) {
				this.view.hideNotificationGroupOutsideSidebar();
				this.oldOutsideViewReference = null;
			} else {
				this.view.showNotificationGroupOutsideSidebar(newOutsideView);
				this.oldOutsideViewReference = newOutsideView;
			}
		}

		// Handle a change to the outside notification
		else if (e.getPropertyName().equals(NotificationViewPropertyNames.NOTIFICATION_OUTSIDE_CHANGE)) {
			this.oldOutsideViewReference = (NotificationGroupView)e.getNewValue();
		}
	}
}
