/* File: TaskPaneController.java
 *
 * Contains class TaskPaneController
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.controller;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Set;

import javax.swing.SwingUtilities;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.MessageTarget;
import edu.cmu.hcii.calo.messagePassing.TaskMessage;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage;
import edu.cmu.hcii.calo.messagePassing.TaskMessage.TaskMessageType;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.ResourceList;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.task.TaskPaneView;
import edu.cmu.hcii.calo.view.task.support.TaskOrdering;
import edu.cmu.hcii.calo.view.task.support.TaskState;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;

/**
 * The controller that attaches the task manager to the task pane view in the sidebar.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskPaneController extends MessageTarget implements PropertyChangeListener {

	/**
	 * The task pane.
	 */
	private TaskPaneView view;

	/** Tells the controller to ignore model messages to assure atomic updates. */
	private boolean ignoreModelMessages = false;

	/**
	 * Constructs a TaskPaneController that sends updates to (and listens for property changes from)
	 * the given TaskPaneView.
	 * 
	 * @param view the view to send updates to
	 */
	public TaskPaneController(TaskPaneView view) {
		this.view = view;
		view.addPropertyChangeListener(this);

		TaskManager.getManager().addObserver(this);
	}

	/**
	 * @see edu.cmu.hcii.calo.messagePassing.MessageTarget#onMessageReceived(java.util.Observable,
	 *      edu.cmu.hcii.calo.messagePassing.Message)
	 */
	@Override
	public void onMessageReceived(Observable sender, final Message msg) {
		assert (msg instanceof TaskMessage || msg instanceof TasklikeObjectMessage);
		assert msg.getSender() == TaskManager.getManager();

		if (!this.isIgnoringModelMessages()) {
			if (SwingUtilities.isEventDispatchThread()) {
				performUpdate(msg);
			} else {
				SwingUtilities.invokeLater(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {
						performUpdate(msg);
					}

				});
			}
		}
	}

	/**
	 * This method gets called when a bound property is changed.
	 * 
	 * @param e A PropertyChangeEvent object describing the event source and the property that has
	 *            changed.
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName().equals(TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME)) {
			this.handleTaskStateChange(e);
		} else if (e.getPropertyName().equals(TaskViewPropertyNames.TASK_ORDER_PROPERTY_NAME)) {
			this.handleTaskReorder(e);
		} else if (e.getPropertyName().equals(TaskViewPropertyNames.EXPANDED_PROPERTY_NAME)
				|| e.getPropertyName().equals(TaskViewPropertyNames.SUBPANE_COLLAPSED_PROPERTY_NAME)
				|| e.getPropertyName().equals(TaskViewPropertyNames.SUBPANE_SHOWING_ALL_PROPERTY_NAME)) {
			this.view.update();
		}
	}

	/**
	 * A helper method that twiddles task priorities to put tasks in the correct order.
	 * 
	 * @param e the PropertyChangeEvent that we're handling, whose name must be
	 *            TaskViewPropertyNames.TASK_ORDER_PROPERTY_NAME
	 * @todo This should call into the Engine to ask what to do rather than always using the
	 *       algorithm defined here.
	 * @bug This algorithm doesn't always work correctly when a bunch of tasks all have the same
	 *      priority.
	 */
	@SuppressWarnings("unchecked")
	private void handleTaskReorder(PropertyChangeEvent e) {
		this.setIgnoreModelMessages(true);
		TaskOrdering taskOrdering = (TaskOrdering)e.getNewValue();

		Task newHighPriorityTask = taskOrdering.getPreviousTask();
		Task newLowPriorityTask = taskOrdering.getNextTask();

		double firstPriority =
			taskOrdering.hasPreviousTask() ? newHighPriorityTask.getPriority() : 0;
		double secondPriority = taskOrdering.hasNextTask() ? newLowPriorityTask.getPriority() : 0;

		Task reorderedTask = taskOrdering.getReorderedTask();
		if (!taskOrdering.hasPreviousTask()) {
			reorderedTask.setPriority(Math.nextAfter(	Math.min(firstPriority, secondPriority),
														Double.NEGATIVE_INFINITY));
		} else if (!taskOrdering.hasNextTask()) {
			reorderedTask.setPriority(Math.nextAfter(	Math.max(firstPriority, secondPriority),
														Double.POSITIVE_INFINITY));
		} else {
			if (firstPriority == secondPriority) {
				newHighPriorityTask.setPriority(Math.nextAfter(	newHighPriorityTask.getPriority(),
																Double.POSITIVE_INFINITY));
				newLowPriorityTask.setPriority(Math.nextAfter(	newLowPriorityTask.getPriority(),
																Double.NEGATIVE_INFINITY));
			}
			reorderedTask.setPriority((firstPriority + secondPriority) / 2);

		}

		this.setIgnoreModelMessages(false);

		this.view.update();
	}

	/**
	 * A helper method that handles changes to that task views that should result in changes to the
	 * Task model.
	 * 
	 * @param e the PropertyChangeEvent that we're handling, whose name must be
	 *            TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME
	 */
	private void handleTaskStateChange(PropertyChangeEvent e) {
		assert e.getOldValue() instanceof TaskState && e.getNewValue() instanceof TaskState : "got a TaskState property change event whose values weren't TaskStates.";
		@SuppressWarnings("unused")
		TaskState oldState = (TaskState)e.getOldValue();
		TaskState newState = (TaskState)e.getNewValue();
		Task t = newState.getTask();

		switch (newState.getProperty()) {
		case NAME:
			assert newState.getValue() instanceof String : "new name was not a string";
			t.setName((String)newState.getValue());
			break;
		case DUE_DATE:
			assert newState.getValue() == null || newState.getValue() instanceof Calendar : "new due date was not a Calendar instance";
			t.setDueDate((Calendar)newState.getValue());
			break;
		case COMPLETE:
			assert newState.getValue() instanceof Boolean : "new complete status was not a boolean";
			t.setComplete((Boolean)newState.getValue());
			break;
		case DATE_COMPLETED:
			assert newState.getValue() instanceof Calendar : "new date completed was not a Calendar instance";
			this.setIgnoreModelMessages(true);
			t.setDateCompleted((Calendar)newState.getValue());
			this.setIgnoreModelMessages(false);
			break;
		case ON_HOLD:
			assert newState.getValue() instanceof PendingState : "new on-hold status was not a PendingState";
			t.setPendingState((PendingState)newState.getValue());
			break;
		case STARRED:
			assert newState.getValue() instanceof Boolean : "new starred status was not a boolean";
			t.setStarred((Boolean)newState.getValue());
			break;
		case TAGS:
			assert newState.getValue() instanceof Set : "new tags were not a Set containing Strings";
			t.clearTags();

			@SuppressWarnings("unchecked")
			Iterator<String> iter = ((Set<String>)newState.getValue()).iterator();
			while (iter.hasNext()) {
				String tag = iter.next();
				t.addTag(tag);
			}
			break;
		case RESOURCES:
			assert newState.getValue() instanceof ResourceList;

			t.setResources((ResourceList)newState.getValue());
			break;

		case DELETED:
			assert newState.getValue() instanceof Boolean : "new deleted status was not a boolean";
			if ((Boolean)newState.getValue()) {
				TaskManager.getManager().removeTasklikeObject(t);
			}
			break;
		}
	}

	/**
	 * Accessor method for ignoreModelMessages
	 * 
	 * @return the value of ignoreModelMessages
	 */
	private boolean isIgnoringModelMessages() {
		return this.ignoreModelMessages;
	}

	/**
	 * Determines whether the task pane or the individual task views need to be updated based on the
	 * message that has been received from the TaskManager, and invokes the appropriate operations.
	 * 
	 * @param msg the message received from the manager
	 */
	private void performUpdate(Message msg) {
		assert SwingUtilities.isEventDispatchThread() : "we are not on the event dispatch thread!";
		if (msg.getMessageType() == TaskMessageType.TASK_DUE_DATE_CHANGED
				|| msg.getMessageType() == TaskMessageType.TASK_STARRED_CHANGED
				|| msg.getMessageType() == TaskMessageType.TASK_RESOURCES_CHANGED
				|| msg.getMessageType() == TaskMessageType.TASK_PROJECT_CHANGED
				|| msg.getMessageType() == TaskMessageType.TASK_NAME_CHANGED
				|| msg.getMessageType() == TaskMessageType.TASK_TAGS_CHANGED) {
			this.view.updateTasks();
		} else {
			this.view.update();
		}
	}

	/**
	 * Setter method for ignoringUpdates.
	 * 
	 * @param ignoreModelMessages the ignoringUpdates to set
	 */
	private void setIgnoreModelMessages(boolean ignoreModelMessages) {
		this.ignoreModelMessages = ignoreModelMessages;
	}
}
