/*
 * File: AutomatedEngine.java
 * 
 * Contains class AutomatedEngine
 * 
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.engine;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.SwingUtilities;

import edu.cmu.hcii.calo.manager.HistoryManager;
import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.manager.RelevantInfoManager;
import edu.cmu.hcii.calo.manager.ScheduleManager;
import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.messagePassing.HistoryMessage;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.MessageTarget;
import edu.cmu.hcii.calo.messagePassing.MessageType;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage;
import edu.cmu.hcii.calo.messagePassing.TaskMessage;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage.NotificationMessageType;
import edu.cmu.hcii.calo.messagePassing.TaskMessage.TaskMessageType;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage.TasklikeObjectMessageType;
import edu.cmu.hcii.calo.model.HistoryItem;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Person;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.Notification.NotificationType;

/**
 * An AI simulator for the final prototype. Receives changes from the GUI and generates "AI"
 * decisions that drive the various managers.
 * 
 * @author Will Haines
 */
public class AutomatedEngine extends MessageTarget implements Engine {

	/** The singleton instance of this engine. */
	private static AutomatedEngine uniqueInstance = null;

	/**
	 * Returns the singleton instance of this engine.
	 * 
	 * @return the singleton instance
	 */
	public static AutomatedEngine getEngine() {
		if (AutomatedEngine.uniqueInstance == null) {
			AutomatedEngine.uniqueInstance = new AutomatedEngine();
		}
		return AutomatedEngine.uniqueInstance;
	}

	/** The timer to defer scripted actions. */
	private Timer timer;

	/** A mapping to keep track of notifications uniquely. */
	private Map<String, Notification> notificationIDs;

	/** A mapping to keep track of tasks uniquely. */
	Map<String, Task> taskIDs;

	/**
	 * Constructs the AutomatedEngine instance. Private because this is a singleton.
	 */
	private AutomatedEngine() {
		super();

		// Make sure it can receive messages from all component managers
		NotificationManager.getManager().addObserver(this);
		RelevantInfoManager.getManager().addObserver(this);
		ScheduleManager.getManager().addObserver(this);
		TaskManager.getManager().addObserver(this);
		HistoryManager.getManager().addObserver(this);

		// Set up the timer to delay executions
		this.timer = new Timer();
		this.notificationIDs = new HashMap<String, Notification>();
		this.taskIDs = new HashMap<String, Task>();
	}

	/**
	 * Adds a notification to the notification manager.
	 * 
	 * @param id the identifier to assign this notification
	 * @param type this notification's type
	 * @param name this notification's name
	 * @param text this notification's text
	 * @param person this notificaiton's associated person
	 * @param priority this notification's priority
	 * @param time the time at which to add this notification
	 */
	public void addNotification(String id,
			NotificationType type,
			String name,
			String text,
			Person person,
			double priority,
			long time) {
		final Notification n = new Notification(type, name, text, person);
		n.setPriority(priority);
		this.notificationIDs.put(id, n);

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					public void run() {
						NotificationManager.getManager().addNotification(n);
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for adding notification " + id);
		}
	}

	/**
	 * Adds a task to the task manager.
	 * 
	 * @param id this task's unique identifier
	 * @param taskName this task's name
	 * @param isOnHold whether this task is on hold
	 * @param isComplete whether this task is complete
	 * @param dueDate the date when this task is due
	 * @param priority this tasks priority
	 * @param time the time at which to add this task
	 */
	public void addTask(String id,
			String taskName,
			boolean isOnHold,
			boolean isComplete,
			Calendar dueDate,
			double priority,
			long time) {
		final Task t = new Task(taskName, true);
		double newPriority = priority;

		// If the priority was unspecified, set its priority based on due date
		if (priority == Double.NEGATIVE_INFINITY) {
			newPriority = t.getDateAdded().getTimeInMillis();

			if (dueDate != null) {
				newPriority = (Long.MAX_VALUE - t.getDueDate().getTimeInMillis());
			}
		} else {
			t.setPriority(newPriority);
		}

		if (dueDate != null) {
			t.setDueDate(dueDate);
		}
		t.setComplete(isComplete);
		t.setPendingState(new PendingState(isOnHold));

		this.taskIDs.put(id, t);

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					public void run() {
						TaskManager.getManager().addTasklikeObject(t);
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for adding task " + id);
		}
	}

	/**
	 * Set the priority of a task based on its due date, with things that are nearly due at the
	 * highest priority. If something doesn't have a due date, its priority is lower than all due
	 * date tasks, but based on add time.
	 * 
	 * @param task the task whose priority will be updated
	 */
	private void automatedSetTaskPriority(Task task) {
		double priority = task.getDateAdded().getTimeInMillis();

		if (task.getDueDate() != null) {
			priority = (Long.MAX_VALUE - task.getDueDate().getTimeInMillis());
		}

		task.setPriority(priority);
	}

	/**
	 * Determines and sets the priority of a notification.
	 * 
	 * @param id the identifier to assign this notification
	 * @param noteName this notification's name
	 * @param text this notification's text
	 * @param person this notificaiton's associated person
	 * @param priority this notification's priority
	 * @param time the time at which to change this notification
	 */
	public void changeNotification(String id,
			String noteName,
			String text,
			Person person,
			double priority,
			long time) {
		final String toChange = id;
		final String newNoteName = noteName;
		final String newText = text;
		final Person newPerson = person;
		final double newPriority = priority;

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {
						Notification n = AutomatedEngine.this.notificationIDs.get(toChange);

						// Ignore non-existant notifications
						if (n == null) {
							return;
						}

						if (!newNoteName.equals("Unknown")) {
							n.setName(newNoteName);
						}
						if (!newText.equals("Unknown")) {
							n.setText(newText);
						}
						if (!newPerson.getName().equals("Unknown")) {
							n.setPerson(newPerson);
						}
						n.setPriority(newPriority);
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for changing notification " + id);
		}
	}

	/**
	 * Changes a task in the task manager.
	 * 
	 * @param id this task's unique identifier
	 * @param taskName this task's name
	 * @param isOnHold whether this task is on hold
	 * @param isComplete whether this task is complete
	 * @param dueDate the date when this task is due
	 * @param priority this tasks priority
	 * @param time the time at which to add this task
	 */
	public void changeTask(String id,
			String taskName,
			boolean isOnHold,
			boolean isComplete,
			Calendar dueDate,
			double priority,
			long time) {
		final String toChange = id;
		final String newTaskName = taskName;
		final boolean newOnHold = isOnHold;
		final boolean newComplete = isComplete;
		final Calendar newDueDate = dueDate;
		final double desiredPriority = priority;

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					public void run() {
						Task t = AutomatedEngine.this.taskIDs.get(toChange);

						// Ignore non-existant tasks
						if (t == null) {
							return;
						}

						double newPriority = desiredPriority;

						// If the priority was unspecified, set its priority based on the old
						// priority
						if (desiredPriority == Double.NEGATIVE_INFINITY) {
							newPriority = t.getPriority();
						}

						t.setName(newTaskName);
						t.setPendingState(new PendingState(newOnHold));
						t.setComplete(newComplete);
						double oldPriority = t.getPriority();

						// Only change things that can cause animations
						// if absolutely necessary
						if (!t.getDueDate().equals(newDueDate)) {
							t.setDueDate(newDueDate);
						}
						if (oldPriority != newPriority) {
							t.setPriority(newPriority);
						}
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for changing task " + id);
		}
	}

	/**
	 * Deletes a notification.
	 * 
	 * @param id the identifier of the notification to delete
	 * @param time the time at which to delete this notification
	 */
	public void deleteNotification(String id, long time) {
		final String toDelete = id;

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {
						Notification n = AutomatedEngine.this.notificationIDs.get(toDelete);

						// Ignore non-existant notifications
						if (n == null) {
							return;
						}

						NotificationManager.getManager().removeNotification(n);
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for deleting notification " + id);
		}
	}

	/**
	 * Deletes a task.
	 * 
	 * @param id the identifier of the task to delete
	 * @param time the time at which to delete this task
	 */
	public void deleteTask(String id, long time) {
		final String toDelete = id;

		TimerTask scheduledTask = new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {
						Task t = AutomatedEngine.this.taskIDs.get(toDelete);

						// Ignore non-existant tasks
						if (t == null) {
							return;
						}

						TaskManager.getManager().removeTasklikeObject(t);
					}
				});
			}
		};

		try {
			this.timer.schedule(scheduledTask, time);
		} catch (IllegalArgumentException e) {
			System.err.println("Warning: Negative time offset for deleting task " + id);
		}
	}

	/**
	 * Handles messages from various managers that are going to let it know that some changes went
	 * down.
	 * 
	 * @param sender The manager that sent this message
	 * @param msg The message to handle
	 */
	@Override
	public void onMessageReceived(Observable sender, Message msg) {

		MessageType curType = msg.getMessageType();

		/**
		 * Check the message for its type and respond appropriately.
		 */
		if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED) {
			Task task = (Task)((TasklikeObjectMessage)msg).getTasklikeObject();
			if (!task.isCaloAdded()) {
				this.automatedSetTaskPriority(task);
			}
		} else if (curType == TaskMessageType.TASK_DUE_DATE_CHANGED) {
			this.automatedSetTaskPriority(((TaskMessage)msg).getTask());
		} else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED) {
			Task task = (Task)((TasklikeObjectMessage)msg).getTasklikeObject();
			String toRemove = null;
			for (String key : this.taskIDs.keySet()) {
				if (this.taskIDs.get(key).equals(task)) {
					toRemove = key;
				}
			}
			this.taskIDs.remove(toRemove);
		} else if (curType == NotificationMessageType.NOTIFICATION_REMOVED) {
			Notification notification = ((NotificationMessage)msg).getNotification();
			String toRemove = null;
			for (String key : this.notificationIDs.keySet()) {
				if (this.notificationIDs.get(key).equals(notification)) {
					toRemove = key;
				}
			}
			this.notificationIDs.remove(toRemove);
		}

		// Store the message into the history manager
		this.setHistoryManager(msg);
	}

	/**
	 * Resets this engine back to the way it was when it was first created.
	 */
	public void reset() {
		AutomatedEngine.uniqueInstance = null;

		NotificationManager.getManager().deleteObserver(this);
		RelevantInfoManager.getManager().deleteObserver(this);
		ScheduleManager.getManager().deleteObserver(this);
		TaskManager.getManager().deleteObserver(this);
		HistoryManager.getManager().deleteObserver(this);
	}

	/**
	 * Stores a message in the history manager
	 * 
	 * @param message the message to make a history item out of
	 */
	private void setHistoryManager(Message message) {

		/**
		 * @todo This prevents infinite looping due to generating History Messages. We may want to
		 *       filter on other types later.
		 */
		if (!(message instanceof HistoryMessage)) {
			HistoryManager.getManager().addHistoryItem(new HistoryItem(message, true));
		}
	}
}
