/* File: WizardOfOzEngine.java
 *
 * Contains class WizardOfOzEngine
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.engine;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.HistoryManager;
import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.manager.RelevantInfoManager;
import edu.cmu.hcii.calo.manager.ScheduleManager;
import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.MessageTarget;
import edu.cmu.hcii.calo.messagePassing.MessageType;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage;
import edu.cmu.hcii.calo.messagePassing.TaskMessage;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage.NotificationMessageType;
import edu.cmu.hcii.calo.messagePassing.TaskMessage.TaskMessageType;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage.TasklikeObjectMessageType;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage.WizardOfOzMessageType;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Person;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TasklikeObject;

/**
 * An AI simulator for purposes of Wizard Of Oz testing. This half actually performs the tasks as
 * forwarded by the WizardOfOzFrontEnd.
 * 
 * @author Will Haines
 */
public class WizardOfOzEngine extends MessageTarget implements Engine, Serializable {

	/** Prevent serialization collisons. */
	private static final long serialVersionUID = -2476562601562287691L;

	/** The singleton instance of the engine. */
	private static WizardOfOzEngine uniqueInstance = null;

	/**
	 * Returns the singleton instance of this engine.
	 * 
	 * @return the singleton instance
	 */
	public static WizardOfOzEngine getEngine() {
		if (WizardOfOzEngine.uniqueInstance == null) {
			WizardOfOzEngine.uniqueInstance = new WizardOfOzEngine();
		}
		return WizardOfOzEngine.uniqueInstance;
	}

	/**
	 * Runs a the Wizard of Oz engine. This is a server; run it first.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		WizardOfOzEngine.getEngine().startInputReadingLoop();
	}

	/**
	 * The stream of serialized object that this engine reads to allow the front end to communicate
	 * to it.
	 */
	private ObjectInputStream input;

	/**
	 * The stream of serialized objects that this engine writes to to allow the front end to
	 * communicate to it.
	 */
	private ObjectOutputStream output;

	/**
	 * The socket this engine will listen on.
	 */
	private ServerSocket serverSocket = null;

	/**
	 * Tracks if the engine should still be running.
	 */
	private boolean done;

	/**
	 * Constructs the WizardOfOzEngine instance. Private because this is a singleton.
	 */
	private WizardOfOzEngine() {
		super();
	}

	/**
	 * Builds the server socket and gets a client. Also attaches listeners when everything is
	 * kosher.
	 */
	private void buildSocket() {
		Socket clientSocket = null;

		// Listen on specified port
		try {
			this.serverSocket = new ServerSocket(48192);
		} catch (IOException e) {
			System.err.println("Couldn't set up server socket to listen on port: 48192");
			System.exit(-1);
			return;
		}

		// Accept client connection
		try {
			clientSocket = this.serverSocket.accept();
		} catch (IOException e) {
			System.err.println("Engine: Accept failed: 48192");
			System.exit(-1);
			return;
		}

		if (clientSocket == null) {
			System.err.println("Engine: Could not create socket.");
			System.exit(-1);
			return;
		}

		// Create an object input stream to read from
		try {
			this.output = new ObjectOutputStream(clientSocket.getOutputStream());
			this.output.flush();
			this.input = new ObjectInputStream(clientSocket.getInputStream());
		} catch (IOException e) {
			System.err.println("Engine: Could not create ObjectInputStream.");
			System.exit(-1);
		}

		// Make sure it can receive messages from all component managers
		NotificationManager.getManager().addObserver(this);
		RelevantInfoManager.getManager().addObserver(this);
		ScheduleManager.getManager().addObserver(this);
		TaskManager.getManager().addObserver(this);
		HistoryManager.getManager().addObserver(this);
		this.done = false;
	}

	/**
	 * Adds a notification to the notification manager.
	 * 
	 * @param notification the notification to add
	 */
	private void addNotification(Notification notification) {
		NotificationManager.getManager().addNotification(notification);
	}

	/**
	 * Removes a notification from the notification manager.
	 * 
	 * @param notification the notification to remove
	 */
	private void deleteNotification(Notification notification) {
		NotificationManager.getManager().removeNotification(notification);
	}

	/**
	 * Adds a tasklike object to the task manager.
	 * 
	 * @param tasklikeObject the tasklike object to add
	 */
	private void addTasklikeObject(TasklikeObject tasklikeObject) {
		TaskManager.getManager().addTasklikeObject(tasklikeObject);
	}

	/**
	 * Removes a tasklike object from the task manager.
	 * 
	 * @param task the tasklike object to remove
	 */
	private void deleteTasklikeObject(TasklikeObject task) {
		TaskManager.getManager().removeTasklikeObject(task);
	}

	/**
	 * A getter for an iterator over the notification manager's notifications.
	 * 
	 * @return an iterator over all notification in the notification manager
	 */
	private Iterator<Notification> getNotificationIterator() {
		return NotificationManager.getManager().notificationIterator();
	}

	/**
	 * A getter for an iterator over the notification manager's notifications.
	 * 
	 * @return an iterator over all notification in the notification manager
	 */
	private Iterator<TasklikeObject> getTaskIterator() {
		return TaskManager.getManager().taskIterator();
	}

	/**
	 * Takes input from off of the input stream and determines what to do with the messages
	 * received. It ignores things that it can't translate.
	 * 
	 * @param o the object sent across the socket
	 * @throws IOException
	 */
	private synchronized void onInputReceived(Object o) throws IOException {

		// Ignore things that are not messages
		if (!(o instanceof Message)) {
			return;
		}

		Message msg = (Message)o;
		MessageType curType = msg.getMessageType();
		Map<String, Object> userInfo = new HashMap<String, Object>();

		// Parse the message

		// Close the engine successful if asked by the FrontEnd
		if (curType == WizardOfOzMessageType.CLOSE) {
			try {
				this.serverSocket.close();
			} catch (IOException e) {
				System.err.println("Server socket failed to close.");
			}
			this.done = true;
		}
		// Set the notification controller ignore status
		else if (curType == WizardOfOzMessageType.NOTIFICATION_CONTROLLER_IGNORE_CHANGE) {
			this.setChanged();
			this.notifyObservers(msg);
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASKLIKE_OBJECT_ADD_STATUS,
															userInfo));
		}
		// Add a task
		else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED) {
			this.addTasklikeObject(((TasklikeObjectMessage)msg).getTasklikeObject());
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASKLIKE_OBJECT_ADD_STATUS,
															userInfo));
		}
		// Give back a list of tasks
		else if (curType == WizardOfOzMessageType.TASKLIKE_OBJECT_ITERATOR_REQUEST) {
			Iterator<TasklikeObject> iterator = this.getTaskIterator();
			ArrayList<TasklikeObject> serializableList = new ArrayList<TasklikeObject>();
			while (iterator.hasNext()) {
				serializableList.add(iterator.next());
			}
			userInfo.put(WizardOfOzMessage.TASK_ITERATOR, serializableList);
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASKLIKE_OBJECT_ITERATOR_REQUEST,
															userInfo));
		}
		// Change a task priority
		else if (curType == TaskMessageType.TASK_PRIORITY_CHANGED) {
			this.setTaskPriority(	((TaskMessage)msg).getTask(),
									(Double)msg.getUserInfo().get(TaskMessage.NEW_PRIORITY));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASK_PRIORITY_CHANGED_STATUS,
															userInfo));
		}
		// Change the task's completed status
		else if (curType == TaskMessageType.TASK_COMPLETE_CHANGED) {
			this.setTaskCompleted(	((TaskMessage)msg).getTask(),
									(Boolean)msg.getUserInfo().get(TaskMessage.NEW_COMPLETE));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASK_COMPLETED_CHANGED_STATUS,
															userInfo));
		}
		// Change the task's pending state
		else if (curType == TaskMessageType.TASK_PENDING_STATE_CHANGED) {
			this.setTaskPending(((TaskMessage)msg).getTask(),
								(PendingState)msg.getUserInfo().get(TaskMessage.NEW_PENDING_STATE));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASK_PENDING_CHANGED_STATUS,
															userInfo));
		}
		// Delete a task
		else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED) {
			this.deleteTasklikeObject(((TasklikeObjectMessage)msg).getTasklikeObject());
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.TASK_DELETED_STATUS,
															userInfo));
		}
		// Add a notification
		else if (curType == NotificationMessageType.NOTIFICATION_ADDED) {
			this.addNotification(((NotificationMessage)msg).getNotification());
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_ADD_STATUS,
															userInfo));
		}
		// Remove a notification
		else if (curType == NotificationMessageType.NOTIFICATION_REMOVED) {
			this.deleteNotification(((NotificationMessage)msg).getNotification());
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_DELETED_STATUS,
															userInfo));
		}
		// Give back a list of the notifications
		else if (curType == WizardOfOzMessageType.NOTIFICATION_ITERATOR_REQUEST) {
			Iterator<Notification> iterator = this.getNotificationIterator();
			ArrayList<Notification> serializableList = new ArrayList<Notification>();
			while (iterator.hasNext()) {
				serializableList.add(iterator.next());
			}
			userInfo.put(WizardOfOzMessage.NOTIFICATION_ITERATOR, serializableList);
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_ITERATOR_REQUEST,
															userInfo));
		}
		// Change a notification name
		else if (curType == NotificationMessageType.NOTIFICATION_NAME_CHANGED) {
			this.setNotificationName(	((NotificationMessage)msg).getNotification(),
										(String)msg.getUserInfo().get(NotificationMessage.NEW_NAME));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_NAME_CHANGED_STATUS,
															userInfo));
		}
		// Change a notification text
		else if (curType == NotificationMessageType.NOTIFICATION_TEXT_CHANGED) {
			this.setNotificationText(	((NotificationMessage)msg).getNotification(),
										(String)msg.getUserInfo().get(NotificationMessage.NEW_TEXT));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_TEXT_CHANGED_STATUS,
															userInfo));
		}
		// Change a notification person
		else if (curType == NotificationMessageType.NOTIFICATION_PERSON_CHANGED) {
			this.setNotificationPerson(	((NotificationMessage)msg).getNotification(),
										(Person)msg.getUserInfo().get(NotificationMessage.NEW_PERSON));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_PERSON_CHANGED_STATUS,
															userInfo));
		}
		// Change a notification priority
		else if (curType == NotificationMessageType.NOTIFICATION_PRIORITY_CHANGED) {
			this.setNotificationPriority(	((NotificationMessage)msg).getNotification(),
											(Double)msg.getUserInfo().get(NotificationMessage.NEW_PRIORITY));
			userInfo.put(WizardOfOzMessage.SUCCESS_STATUS, true);
			this.output.writeObject(new WizardOfOzMessage(	this,
															WizardOfOzMessageType.NOTIFICATION_PRIORITY_CHANGED_STATUS,
															userInfo));
		}

		// Things will break if we don't flush and reset.
		// @see
		// http://java.sun.com/developer/technicalArticles/Programming/serialization/
		this.output.flush();
		this.output.reset();
	}

	/**
	 * Handles messages from various managers that are going to let it know that some changes went
	 * down.
	 * 
	 * @param sender The manager that sent this message
	 * @param msg The message to handle
	 */
	@Override
	public synchronized void onMessageReceived(Observable sender, Message msg) {
		MessageType curType = msg.getMessageType();

		/*
		 * Check the message for its type and respond appropriately. Forwards to the front end for
		 * display.
		 */
		try {

			// Tasklike Object message forwarding
			if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED) {
				assert msg instanceof TasklikeObjectMessage : "tried to add a tasklike object without sending a TasklikeObjectMessage";
				this.output.writeObject(msg);
				this.output.flush();

			} else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED) {
				assert msg instanceof TasklikeObjectMessage : "tried to remove a tasklike object without sending a TasklikeObjectMessage";
				this.output.writeObject(msg);
				this.output.flush();

				// Task message forwarding
			} else if (curType == TaskMessageType.TASK_PRIORITY_CHANGED) {
				assert msg instanceof TaskMessage : "tried to reprioritize a task without sending a TaskMessage";
				this.output.writeObject(msg);
				this.output.flush();

			} else if (curType == TaskMessageType.TASK_COMPLETE_CHANGED) {
				assert msg instanceof TaskMessage : "tried to change a task complete without sending a TaskMessage";
				this.output.writeObject(msg);
				this.output.flush();

			} else if (curType == TaskMessageType.TASK_PENDING_STATE_CHANGED) {
				assert msg instanceof TaskMessage : "tried to change a task pending without sending a TaskMessage";
				this.output.writeObject(msg);
				this.output.flush();

			} else if (curType == TaskMessageType.TASK_NAME_CHANGED) {
				assert msg instanceof TaskMessage : "tried to change a task name without sending a TaskMessage";
				this.output.writeObject(msg);
				this.output.flush();

				// Notification message forwarding
			} else if (curType == NotificationMessageType.NOTIFICATION_REMOVED) {
				assert msg instanceof NotificationMessage : "tried to delete a notification without sending a NotificationMessage";
				this.output.writeObject(msg);
				this.output.flush();
			}

		} catch (IOException e) {
			System.err.println("Engine: Failed to forward message " + curType);
		}
	}

	/**
	 * Reads in a serialized engine. Just checks against the serialVersionUID since this is a
	 * singleton.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != WizardOfOzEngine.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Resets this engine back to the way it was when it was first created.
	 */
	public void reset() {
		NotificationManager.getManager().deleteObserver(this);
		RelevantInfoManager.getManager().deleteObserver(this);
		ScheduleManager.getManager().deleteObserver(this);
		TaskManager.getManager().deleteObserver(this);
		HistoryManager.getManager().deleteObserver(this);

		try {
			this.serverSocket.close();
		} catch (IOException e) {
			System.err.println("Server socket failed to close.");
		}

		WizardOfOzEngine.uniqueInstance = null;

		this.done = true;
	}

	/**
	 * Determines and sets the name of a notification.
	 * 
	 * @param notification the notification whose name will be updated
	 * @param newName the new name
	 */
	private void setNotificationName(Notification notification, String newName) {
		Notification managedNotification =
			NotificationManager.getManager().getNotification(notification);
		managedNotification.setName(newName);
	}

	/**
	 * Determines and sets the person for a notification.
	 * 
	 * @param notification the notification whose person will be updated
	 * @param newPerson the new person
	 */
	private void setNotificationPerson(Notification notification, Person newPerson) {
		Notification managedNotification =
			NotificationManager.getManager().getNotification(notification);
		managedNotification.setPerson(newPerson);
	}

	/**
	 * Determines and sets the priority of a notification.
	 * 
	 * @param notification the notification whose priority will be updated
	 * @param newPriority the new priority
	 */
	private void setNotificationPriority(Notification notification, double newPriority) {
		Notification managedNotification =
			NotificationManager.getManager().getNotification(notification);
		managedNotification.setPriority(newPriority);
	}

	/**
	 * Determines and sets the text of a notification.
	 * 
	 * @param notification the notification whose text will be updated
	 * @param newText the new text
	 * @param newText the new text with which to update
	 */
	private void setNotificationText(Notification notification, String newText) {
		Notification managedNotification =
			NotificationManager.getManager().getNotification(notification);
		managedNotification.setText(newText);
	}

	/**
	 * Determines and sets the completed status of a task.
	 * 
	 * @param task the task whose completion will be updated
	 * @param newCompleted the completed state to update to
	 */
	private void setTaskCompleted(Task task, boolean newCompleted) {
		Task managedTask = (Task)TaskManager.getManager().getTasklikeObject(task);
		managedTask.setComplete(newCompleted);
	}

	/**
	 * Determines and sets the pending state of a task.
	 * 
	 * @param task the task whose pending state will be updated
	 * @param newPending the pending state to update to
	 */
	private void setTaskPending(Task task, PendingState newPending) {
		Task managedTask = (Task)TaskManager.getManager().getTasklikeObject(task);
		managedTask.setPendingState(newPending);
	}

	/**
	 * Determines and sets the priority of a task.
	 * 
	 * @param task the task whose priority will be updated
	 * @param newPriority the priority to update to
	 */
	private void setTaskPriority(Task task, double newPriority) {
		Task managedTask = (Task)TaskManager.getManager().getTasklikeObject(task);
		managedTask.setPriority(newPriority);
	}

	/**
	 * Loops to read from the input stream until done is set to true.
	 */
	public void startInputReadingLoop() {

		// Do socket and listener setup
		buildSocket();

		// Read input from the input stream
		while (!this.done) {
			try {
				this.onInputReceived(this.input.readObject());
			} catch (IOException e) {
				System.err.println("Engine: Error reading from stream.");
				e.printStackTrace();
				WizardOfOzEngine.getEngine().reset();
				WizardOfOzEngine.getEngine().startInputReadingLoop();
			} catch (ClassNotFoundException e) {
				System.err.println("Engine: Could not find class for object read from stream.");
			}
		}
	}

	/**
	 * Writes this engine to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(WizardOfOzEngine.serialVersionUID);
	}
}
