/* File: HistoryManager.java
 *
 * Contains class HistoryManager
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import edu.cmu.hcii.calo.messagePassing.HistoryMessage;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.model.HistoryItem;

/**
 * Manages the addition of items to a persistant history as displayed in the learning log. The
 * history manager just manages messages since all the things that we want to track are the results
 * of specific messages. The engine will interact with this manager directly to add the messages and
 * the controller will access it directly because there is nothing to model past the list since we
 * only support adding things.
 * 
 * @author Will Haines
 */
public class HistoryManager extends AbstractManager implements Iterable<HistoryItem> {

	/** Prevent serialization collisions */
	private static final long serialVersionUID = 458809417869277326L;

	/** The singleton instance of the notification manager. */
	private static HistoryManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static HistoryManager getManager() {
		if (HistoryManager.uniqueInstance == null) {
			HistoryManager.uniqueInstance = new HistoryManager();
		}
		return HistoryManager.uniqueInstance;
	}

	/** The notifications, sorted by priority. */
	private List<HistoryItem> history;

	/**
	 * Constructs the HistoryManager instance. Private because this is a singleton.
	 */
	private HistoryManager() {
		super();
		this.history = new ArrayList<HistoryItem>();
	}

	/**
	 * Adds a history item to be managed by the manager.
	 * 
	 * @param h the history item to be added
	 */
	public boolean addHistoryItem(HistoryItem h) {
		h.setManaged(true);
		h.setId(this.getNextId());
		boolean success = this.history.add(h);
		if (success) {
			Message msg =
				new HistoryMessage(this, HistoryMessage.HistoryMessageType.HISTORY_ITEM_ADDED, h);

			this.setChanged();
			this.notifyObservers(msg);
		}
		return success;
	}

	/**
	 * Counts the number of history items currently being managed.
	 * 
	 * @return the number of history items being managed
	 * @see java.util.Set#size()
	 */
	public int countHistoryItems() {
		return this.history.size();
	}

	/**
	 * Returns whether or not the manager is managing the given history item.
	 * 
	 * @param h the history item to ask about
	 * @return true if the history item is present, false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean hasHistoryItem(HistoryItem h) {
		return this.history.contains(h);
	}

	/**
	 * Returns an iterator, sorted by add time, over the history items managed by the manager.
	 * 
	 * @return an iterator to the managed history items
	 * @see java.util.Set#iterator()
	 */
	public Iterator<HistoryItem> iterator() {
		return this.history.iterator();
	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 */
	public void reset() {
		HistoryManager.uniqueInstance = null;
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != HistoryManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}

	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(HistoryManager.serialVersionUID);
	}
}
