/* File: NotificationManager.java
 *
 * Contains class NotificationManager
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.sorting.KeyFunctor;
import edu.cmu.hcii.calo.model.sorting.MutableSortedSet;
import edu.cmu.hcii.calo.util.Utility;

/**
 * Manages the addition, removal, and modifications of notifications as visualized in the
 * notification center.
 * 
 * @author Will Haines
 */
public class NotificationManager extends AbstractManager {

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -1008874479493769120L;

	/** The singleton instance of the notification manager. */
	private static NotificationManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static NotificationManager getManager() {
		if (NotificationManager.uniqueInstance == null) {
			NotificationManager.uniqueInstance = new NotificationManager();
		}
		return NotificationManager.uniqueInstance;
	}

	/** The notifications, sorted by priority. */
	private MutableSortedSet<Notification> notifications;

	/**
	 * Constructs the NotificationManager instance. Private because this is a singleton.
	 */
	private NotificationManager() {
		super();
		this.notifications = new MutableSortedSet<Notification>(new KeyFunctor<Notification>() {

			@Override
			public double getKey(Notification item) {
				return item.getPriority();
			}

		});
	}

	/**
	 * Adds a notification to be managed by the manager.
	 * 
	 * @param n the notification to be added
	 */
	public void addNotification(Notification n) {
		assert !this.notifications.contains(n) : "tried to add a notification that is already being managed.";
		n.setManaged(true);
		n.setId(this.getNextId());
		this.notifications.add(n);
		Message msg =
			new NotificationMessage(this,
									NotificationMessage.NotificationMessageType.NOTIFICATION_ADDED,
									n);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Counts the number of notifications currently being managed.
	 * 
	 * @return the number of notifications being managed
	 * @see java.util.Set#size()
	 */
	public int countNotifications() {
		return this.notifications.size();
	}

	/**
	 * Gets the given notification in this manager if it exists. Otherwise, return null.
	 * 
	 * @param n the notification to get
	 * 
	 * @return the notification if it is found, null otherwise
	 */
	public Notification getNotification(Notification n) {
		return this.notifications.get(n);
	}

	/**
	 * Returns an array containing all the notifications in the manager. Use this instead of
	 * notificationIterator() to avoid concurrent modification due to threading issues, if that's
	 * your thing.
	 * 
	 * @return the notifications currently in the manager
	 */
	public Notification[] getNotifications() {
		Object[] notificationList = this.notifications.toArray();
		Notification[] ret = new Notification[notificationList.length];
		for (int i = 0; i < notificationList.length; i++ ) {
			ret[i] = (Notification)notificationList[i];
		}
		return ret;
	}

	/**
	 * Returns whether or not the manager is managing the given notification.
	 * 
	 * @param n the notification to ask about
	 * @return true if the notification is present, false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean hasNotification(Notification n) {
		return this.notifications.contains(n);
	}

	/**
	 * Called by a notification when its priority is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param n the notification that has been altered
	 * @param oldPriority the former priority of this task
	 * @param newPriority the new priority of this task
	 */
	public void noteNotificationPriorityChanged(Notification n,
			double oldPriority,
			double newPriority) {
		assert this.notifications.contains(n) : "tried to change a notification that is not "
				+ "currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(NotificationMessage.OLD_PRIORITY, oldPriority);
		userInfo.put(NotificationMessage.NEW_PRIORITY, newPriority);
		Message msg =
			new NotificationMessage(this,
									NotificationMessage.NotificationMessageType.NOTIFICATION_PRIORITY_CHANGED,
									n,
									userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a notification when its text is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param n the notification that has been altered
	 * @param oldText the former text of this notification
	 * @param newText the new text of this notification
	 */
	public void noteNotificationTextChanged(Notification n, String oldText, String newText) {
		assert this.notifications.contains(n) : "tried to change a notification that is not "
				+ "currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(NotificationMessage.OLD_TEXT, oldText);
		userInfo.put(NotificationMessage.NEW_TEXT, newText);
		Message msg =
			new NotificationMessage(this,
									NotificationMessage.NotificationMessageType.NOTIFICATION_TEXT_CHANGED,
									n,
									userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Returns an iterator, sorted by priority, over the notifications managed by the manager.
	 * 
	 * @return an iterator to the managed notifications
	 * @see java.util.Set#iterator()
	 */
	public Iterator<Notification> notificationIterator() {
		return Utility.unchecked_cast(this.notifications.iterator());
	}

	/**
	 * Removes the given notification from being managed by this manager.
	 * 
	 * @param n the notification to remove
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public void removeNotification(Notification n) {
		assert this.notifications.contains(n) : "tried to remove a notification that is not currently being managed.";
		this.notifications.remove(n);
		n.setManaged(false);
		Message msg =
			new NotificationMessage(this,
									NotificationMessage.NotificationMessageType.NOTIFICATION_REMOVED,
									n);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 */
	public void reset() {
		NotificationManager.uniqueInstance = null;
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != NotificationManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(NotificationManager.serialVersionUID);
	}
}
