/* File: PackManager.java
 *
 * Contains class PackManager
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.PackMessage;
import edu.cmu.hcii.calo.model.Pack;
import edu.cmu.hcii.calo.model.ResourceList;

/**
 * Manages packs, detailed lists of resources relevant to a particular Task, ScheduleItem, or other
 * entity.
 * 
 * @author Brian Ellis
 * 
 */
public class PackManager extends AbstractManager {

	/** Prevent serialization collisions */
	private static final long serialVersionUID = 7775469491476373563L;

	/** The singleton instance of this manager. */
	private static PackManager uniqueInstance = null;

	/**
	 * Gets the singleton manager instance.
	 * 
	 * @return the PackManager
	 */
	public static PackManager getManager() {
		if (PackManager.uniqueInstance == null) {
			PackManager.uniqueInstance = new PackManager();
		}
		return PackManager.uniqueInstance;
	}

	/**
	 * The set of packs managed by this manager.
	 */
	private Set<Pack> packs;

	/**
	 * Constructor. Private because this is a singleton.
	 * 
	 */
	private PackManager() {
		this.packs = new HashSet<Pack>();
	}

	/**
	 * Adds the given pack to this manager.
	 * 
	 * @param pack the pack to add
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public void addPack(Pack pack) {
		assert !this.packs.contains(pack) : "tried to add a pack that is already being managed.";
		pack.setManaged(true);
		pack.setId(this.getNextId());
		this.packs.add(pack);
		Message msg = new PackMessage(this, PackMessage.PackMessageType.PACK_ADDED, pack);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Returns the number of packs maanged by this manager.
	 * 
	 * @return the number of packs
	 * @see java.util.Set#size()
	 */
	public int countPacks() {
		return this.packs.size();
	}

	/**
	 * Returns \c true if the given pack is managed by this manager, \c false otherwise.
	 * 
	 * @param pack the pack to ask about
	 * @return whether this manager has the pack
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean hasPack(Pack pack) {
		return this.packs.contains(pack);
	}

	/**
	 * Called by a Pack when it is changed so that the manager can notify all interested parties of
	 * the change.
	 * 
	 * @param pack the Pack that has been altered
	 * @param list
	 * @param oldResources
	 */
	public void notePackChanged(Pack pack, ResourceList oldResources, ResourceList list) {
		assert this.packs.contains(pack) : "tried to change a Pack that is not currently being managed.";
		Message msg =
			new PackMessage(this, PackMessage.PackMessageType.PACK_RESOURCES_CHANGED, pack);
		msg.getUserInfo().put(PackMessage.OLD_RESOURCES, oldResources);
		msg.getUserInfo().put(PackMessage.NEW_RESOURCES, list);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Removes a given pack from the manager.
	 * 
	 * @param pack the pack to remove
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public void removePack(Pack pack) {
		assert this.packs.contains(pack) : "tried to remove a pack that is not currently being managed.";
		pack.setManaged(false);
		this.packs.remove(pack);

		Message msg = new PackMessage(this, PackMessage.PackMessageType.PACK_REMOVED, pack);

		this.setChanged();
		this.notifyObservers(msg);

	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 */
	public void reset() {
		PackManager.uniqueInstance = null;
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != PackManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(PackManager.serialVersionUID);
	}
}
