/* File: RelevantInfoManager.java
 *
 * Contains class RelevantInfoManager
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.RelevantActionMessage;
import edu.cmu.hcii.calo.messagePassing.RelevantResourceMessage;
import edu.cmu.hcii.calo.messagePassing.RelevantActionMessage.RelevantActionMessageType;
import edu.cmu.hcii.calo.messagePassing.RelevantResourceMessage.RelevantResourceMessageType;
import edu.cmu.hcii.calo.model.RelevantAction;
import edu.cmu.hcii.calo.model.ResourceList;

/**
 * Manages relevant resources and actions to the currently focused application or CALO sidebar view.
 * These will be changed through the engine.
 * 
 * @author Brian Ellis
 */
public class RelevantInfoManager extends AbstractManager {

	/** Prevents serialization collisions */
	private static final long serialVersionUID = 9096459218853621393L;

	/** The singleton instance of the notification manager. */
	private static RelevantInfoManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static RelevantInfoManager getManager() {
		if (RelevantInfoManager.uniqueInstance == null) {
			RelevantInfoManager.uniqueInstance = new RelevantInfoManager();
		}
		return RelevantInfoManager.uniqueInstance;
	}

	/** The list of resources that will be shown in the Relevant Info pane. */
	private ResourceList resources;

	/** The list of actions that will be shown in the Relevant Info pane. */
	private List<RelevantAction> actions;

	/**
	 * Constructs the TaskManager instance. Private because this is a singleton.
	 */
	private RelevantInfoManager() {
		super();
		this.resources = new ResourceList();
		this.actions = new ArrayList<RelevantAction>();
	}

	/**
	 * Accessor method for actions
	 * 
	 * @return the value of actions
	 */
	public List<RelevantAction> getActions() {
		return this.actions;
	}

	/**
	 * Accessor method for resources
	 * 
	 * @return the value of resources
	 */
	public ResourceList getResources() {
		return this.resources;
	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 */
	public void reset() {
		RelevantInfoManager.uniqueInstance = null;
	}

	/**
	 * Setter method for actions.
	 * 
	 * @param actions the actions to set
	 */
	public void setActions(List<RelevantAction> actions) {
		if (this.actions == actions) {
			return;
		}
		this.actions = actions;
		Message message =
			new RelevantActionMessage(this, RelevantActionMessageType.ACTIONS_CHANGED, actions);
		this.setChanged();
		this.notifyObservers(message);
	}

	/**
	 * Setter method for resources.
	 * 
	 * @param resources the resources to set
	 */
	public void setResources(ResourceList resources) {
		if (this.resources == resources) {
			return;
		}
		this.resources = resources;
		Message message =
			new RelevantResourceMessage(this,
										RelevantResourceMessageType.RESOURCES_CHANGED,
										resources);
		this.setChanged();
		this.notifyObservers(message);
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != RelevantInfoManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(RelevantInfoManager.serialVersionUID);
	}

}
