/* File: ScheduleManager.java
 *
 * Contains class ScheduleManager
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.ScheduleMessage;
import edu.cmu.hcii.calo.model.Project;
import edu.cmu.hcii.calo.model.ScheduleItem;
import edu.cmu.hcii.calo.model.sorting.KeyFunctor;
import edu.cmu.hcii.calo.model.sorting.MutableSortedSet;

/**
 * Manages the addition, removal, and modification of items on the schedule.
 * 
 * @author Brian Ellis
 */
public class ScheduleManager extends AbstractManager {

	/** Prevents serialization collisions */
	private static final long serialVersionUID = 8963616139767956524L;

	/** The singleton instance of the notification manager. */
	private static ScheduleManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static ScheduleManager getManager() {
		if (ScheduleManager.uniqueInstance == null) {
			ScheduleManager.uniqueInstance = new ScheduleManager();
		}
		return ScheduleManager.uniqueInstance;
	}

	/** The items managed by this schedule manager. */
	private MutableSortedSet<ScheduleItem> scheduledItems;

	/**
	 * Constructs a new ScheduleManager. Private because this is a singleton.
	 */
	private ScheduleManager() {
		this.scheduledItems = new MutableSortedSet<ScheduleItem>(new KeyFunctor<ScheduleItem>() {

			@Override
			public double getKey(ScheduleItem item) {
				return item.getStartTime().getTimeInMillis();
			}

		});
	}

	/**
	 * Adds a ScheduleItem to the schedule.
	 * 
	 * @param o the item to add
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public void addItem(ScheduleItem o) {
		assert !this.scheduledItems.contains(o) : "tried to add a scheduled item that is already being managed.";
		o.setManaged(true);
		o.setId(this.getNextId());
		this.scheduledItems.add(o);
		Message msg =
			new ScheduleMessage(this, ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_ADDED, o);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Returns the number of items currently in the schedule.
	 * 
	 * @return the number of items in the schedule
	 * @see java.util.Set#size()
	 */
	public int countItems() {
		return this.scheduledItems.size();
	}

	/**
	 * Retrieves the ScheduleItem with the earliest start time.
	 * 
	 * @return the earliest item in the schedule
	 * @see java.util.SortedSet#first()
	 */
	public ScheduleItem earliestItem() {
		return this.scheduledItems.first();
	}

	/**
	 * Checks whether the given ScheduleItem is currently being managed by this manager.
	 * 
	 * @param o the ScheduleItem to check for
	 * @return \c true if the item exists, \c false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean hasScheduleItem(ScheduleItem o) {
		return this.scheduledItems.contains(o);
	}

	/**
	 * Retrieves the ScheduleItem with the latest start time.
	 * 
	 * @return the latest item in the schedule
	 * @see java.util.SortedSet#last()
	 */
	public ScheduleItem latestItem() {
		return this.scheduledItems.last();
	}

	/**
	 * Called by a ScheduleItem when its state has generically changed so that the manager can
	 * notify all interested parties of the change.
	 * 
	 * @param item the ScheduleItem that has been altered
	 */
	public void noteItemChanged(ScheduleItem item) {
		assert this.scheduledItems.contains(item) : "tried to change a scheduled item that is not currently being managed.";
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_CHANGED,
								item);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a ScheduleItem when its end time is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param item the ScheduleItem that has been altered
	 * @param oldEndTime the former end time of this ScheduleItem
	 * @param newEndTime the new end time of this ScheduleItem
	 */
	public void noteItemEndTimeChanged(ScheduleItem item, Calendar oldEndTime, Calendar newEndTime) {
		assert this.scheduledItems.contains(item) : "tried to change a ScheduleItem that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(ScheduleMessage.OLD_END_TIME, oldEndTime);
		userInfo.put(ScheduleMessage.NEW_END_TIME, newEndTime);
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_END_TIME_CHANGED,
								item,
								userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a ScheduleItem when its reminder status is changed so that the manager can notify
	 * all interested parties of the change.
	 * 
	 * @param item the ScheduleItem that has been altered
	 * @param oldHasReminder the former reminder status of this ScheduleItem
	 * @param newHasReminder the new reminder status of this ScheduleItem
	 */
	public void noteItemHasReminderChanged(ScheduleItem item,
			boolean oldHasReminder,
			boolean newHasReminder) {
		assert this.scheduledItems.contains(item) : "tried to change a ScheduleItem that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(ScheduleMessage.OLD_HAS_REMINDER, oldHasReminder);
		userInfo.put(ScheduleMessage.NEW_HAS_REMINDER, newHasReminder);
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_HAS_REMINDER_CHANGED,
								item,
								userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a ScheduledItem when its due date is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param item the item that has been altered
	 * @param oldName the former name of this item
	 * @param name the new name of this item
	 */
	public void noteItemNameChanged(ScheduleItem item, String oldName, String name) {
		assert this.scheduledItems.contains(item) : "tried to change a scheduled item that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(ScheduleMessage.OLD_NAME, oldName);
		userInfo.put(ScheduleMessage.NEW_NAME, name);
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_NAME_CHANGED,
								item,
								userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a ScheduleItem when its project is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param item the ScheduleItem that has been altered
	 * @param oldProject the former project of this ScheduleItem
	 * @param newProject the new project of this ScheduleItem
	 */
	public void noteItemProjectChanged(ScheduleItem item, Project oldProject, Project newProject) {
		assert this.scheduledItems.contains(item) : "tried to change a ScheduleItem that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(ScheduleMessage.OLD_PROJECT, oldProject);
		userInfo.put(ScheduleMessage.NEW_PROJECT, newProject);
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_PROJECT_CHANGED,
								item,
								userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	// /**
	// * Called by a ScheduleItem when its tags are changed so that the manager
	// * can notify all interested parties of the change.
	// *
	// * @param item
	// * the ScheduleItem that has been altered
	// * @param oldTags
	// * the former tags of this ScheduleItem
	// * @param newTags
	// * the new tags of this ScheduleItem
	// */
	// public void noteItemTagsChanged(ScheduleItem item, HashSet<String>
	// oldTags,
	// HashSet<String> newTag s) {
	// assert this.scheduledItems.contains(item) : "tried to change a
	// ScheduleItem that is not currently being managed.";
	// Map<String, Object> userInfo = new HashMap<String, Object>();
	// userInfo.put(ScheduleMessage.OLD_TAGS, oldTags);
	// userInfo.put(ScheduleMessage.NEW_TAGS, newTags);
	// Message msg = new ScheduleMessage(this,
	// ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_TAGS_CHANGED,
	// item, userInfo);
	// this.setChanged();
	// this.notifyObservers(msg);
	// }

	/**
	 * Called by a ScheduleItem when its start time is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param item the ScheduleItem that has been altered
	 * @param oldStartTime the former start time of this ScheduleItem
	 * @param newStartTime the new start time of this ScheduleItem
	 */
	public void noteItemStartTimeChanged(ScheduleItem item,
			Calendar oldStartTime,
			Calendar newStartTime) {
		assert this.scheduledItems.contains(item) : "tried to change a ScheduleItem that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(ScheduleMessage.OLD_START_TIME, oldStartTime);
		userInfo.put(ScheduleMessage.NEW_START_TIME, newStartTime);
		Message msg =
			new ScheduleMessage(this,
								ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_START_TIME_CHANGED,
								item,
								userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Removes a ScheduleItem from the schedule.
	 * 
	 * @param o the ScheduleItem to remove
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public void removeItem(ScheduleItem o) {
		assert this.scheduledItems.contains(o) : "tried to remove a scheduled item that is not currently being managed.";
		this.scheduledItems.remove(o);
		o.setManaged(false);

		Message msg =
			new ScheduleMessage(this, ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_REMOVED, o);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 */
	public void reset() {
		ScheduleManager.uniqueInstance = null;
	}

	/**
	 * Retrieves an iterator over all the ScheduleItems in the schedule. The iterator will return
	 * schedule items in order from earliest to latest start times.
	 * 
	 * @return the schedule iterator
	 * @see java.util.Set#iterator()
	 */
	@SuppressWarnings("unchecked")
	public Iterator<ScheduleItem> scheduleIterator() {
		return this.scheduledItems.iterator();
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != ScheduleManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(ScheduleManager.serialVersionUID);
	}

	// /**
	// * Called by a ScheduleItem when its resources are changed so that the
	// * manager can notify all interested parties of the change.
	// *
	// * @param item
	// * the ScheduleItem that has been altered
	// * @param oldResources
	// * the former resources of this ScheduleItem
	// * @param newResources
	// * the new resources of this ScheduleItem
	// */
	// public void noteItemResourcesChanged(ScheduleItem item,
	// ResourceList oldResources, ResourceList newResources) {
	// assert this.scheduledItems.contains(item) : "tried to change a
	// ScheduleItem that is not currently being managed.";
	// Map<String, Object> userInfo = new HashMap<String, Object>();
	// userInfo.put(ScheduleMessage.OLD_RESOURCES, oldResources);
	// userInfo.put(ScheduleMessage.NEW_RESOURCES, newResources);
	// Message msg = new ScheduleMessage(
	// this,
	// ScheduleMessage.ScheduleMessageType.SCHEDULE_ITEM_RESOURCES_CHANGED,
	// item, userInfo);
	// this.setChanged();
	// this.notifyObservers(msg);
	// }

}
