/* File: TaskManager.java
 *
 * Contains class TaskManager
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.manager;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.TaskGroupMessage;
import edu.cmu.hcii.calo.messagePassing.TaskMessage;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Project;
import edu.cmu.hcii.calo.model.ResourceList;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TaskGroup;
import edu.cmu.hcii.calo.model.TasklikeObject;
import edu.cmu.hcii.calo.model.sorting.KeyFunctor;
import edu.cmu.hcii.calo.model.sorting.MutableSortedSet;

/**
 * Manages the addition, removal, and modification of Task instances.
 * 
 * @author Brian Ellis
 */
public class TaskManager extends AbstractManager {

	/** Prevents serialization collisons */
	private static final long serialVersionUID = -5072485063150524741L;

	/** The singleton instance of the task manager. */
	private static TaskManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static TaskManager getManager() {
		if (TaskManager.uniqueInstance == null) {
			TaskManager.uniqueInstance = new TaskManager();
		}
		return TaskManager.uniqueInstance;
	}

	/** The actual task list, sorted by priority. */
	private MutableSortedSet<TasklikeObject> tasks;

	/**
	 * Constructs the TaskManager instance. Private because this is a singleton.
	 */
	@SuppressWarnings("unchecked")
	private TaskManager() {
		super();
		this.tasks = new MutableSortedSet<TasklikeObject>(new KeyFunctor() {

			@Override
			public double getKey(Object o) {
				TasklikeObject e = (TasklikeObject)o;
				return e.getPriority();
			}

		});
	}

	/**
	 * Adds a task to be managed by the task manager.
	 * 
	 * @param t the task to add
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public void addTasklikeObject(TasklikeObject t) {
		assert !this.tasks.contains(t) : "tried to add a task that is already being managed.";
		if (t instanceof TaskGroup) {
			for (Task subtask : ((TaskGroup)t).getTasks()) {
				assert !this.tasks.contains(subtask) : "tried to add a task group containing a task that is already being managed.";
			}
		}
		t.setManaged(true);
		t.setId(this.getNextId());
		this.tasks.add(t);

		Message msg =
			new TasklikeObjectMessage(	this,
										TasklikeObjectMessage.TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED,
										t);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Counts the number of tasks currently being managed.
	 * 
	 * @return the number of tasks being managed
	 * @see java.util.Set#size()
	 */
	public int countTasks() {
		return this.tasks.size();
	}

	/**
	 * Gets the given tasklike object in this manager if it exists. Otherwise, return null.
	 * 
	 * @param t the tasklike object to get
	 * 
	 * @return the tasklike object if it is found, null otherwise
	 */
	public TasklikeObject getTasklikeObject(TasklikeObject t) {
		return this.tasks.get(t);
	}

	/**
	 * Returns an array containing all the tasks in the manager. Use this instead of taskIterator()
	 * to avoid concurrent modification due to threading issues, if that's your thing.
	 * 
	 * @return the tasks currently in the manager
	 */
	public TasklikeObject[] getTasks() {
		Object[] objects = this.tasks.toArray();
		TasklikeObject[] ret = new TasklikeObject[objects.length];
		for (int i = 0; i < objects.length; i++ ) {
			ret[i] = (TasklikeObject)objects[i];
		}
		return ret;
	}

	/**
	 * Returns whether or not the manager is managing the given task.
	 * 
	 * @param t the task to ask about
	 * @return true if the task is present, false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean hasTask(TasklikeObject t) {
		return this.tasks.contains(t);
	}

	/**
	 * Called by a Task when its completeness is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task whose completeness changed
	 * @param wasComplete was the task complete before?
	 * @param isComplete is it now?
	 */
	public void noteCompletenessChanged(Task t, boolean wasComplete, boolean isComplete) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_COMPLETE, wasComplete);
		userInfo.put(TaskMessage.NEW_COMPLETE, isComplete);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_COMPLETE_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its starredness (or "whether or not it's starred", for those among us
	 * with a healthy respect for English) is changed so that the manager can notify all interested
	 * parties of the change.
	 * 
	 * @param t the task whose starredness changed
	 * @param wasStarred was the task starred before?
	 * @param isStarred is it now?
	 */
	public void noteStarrednessChanged(Task t, boolean wasStarred, boolean isStarred) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_STARRED, wasStarred);
		userInfo.put(TaskMessage.NEW_STARRED, isStarred);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_STARRED_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its due date is changed so that the manager can notify all interested
	 * parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldDueDate the former due date of this task
	 * @param newDueDate the new due date of this task
	 */
	public void noteTaskDueDateChanged(Task t, Calendar oldDueDate, Calendar newDueDate) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_DUE_DATE, oldDueDate);
		userInfo.put(TaskMessage.NEW_DUE_DATE, newDueDate);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_DUE_DATE_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its associated subtasks are changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldSubtasks the old subtasks of the task
	 * @param newSubtasks the new subtasks of the task
	 */
	public void noteTaskGroupTasksChanged(TaskGroup t,
			List<Task> oldSubtasks,
			List<Task> newSubtasks) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskGroupMessage.OLD_SUBTASKS, oldSubtasks);
		userInfo.put(TaskGroupMessage.NEW_SUBTASKS, newSubtasks);
		Message msg =
			new TaskGroupMessage(	this,
									TaskGroupMessage.TaskGroupMessageType.TASK_GROUP_SUBTASKS_CHANGED,
									t,
									userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its name is changed so that the manager can notify all interested
	 * parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldName the former name of this task
	 * @param newName the new name of this task
	 */
	public void noteTaskNameChanged(Task t, String oldName, String newName) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_NAME, oldName);
		userInfo.put(TaskMessage.NEW_NAME, newName);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_NAME_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its pending state is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldPendingState the old pending state of the task
	 * @param newPendingState the new pending state of the task
	 */
	public void noteTaskPendingStateChanged(Task t,
			PendingState oldPendingState,
			PendingState newPendingState) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_PENDING_STATE, oldPendingState);
		userInfo.put(TaskMessage.NEW_PENDING_STATE, newPendingState);
		Message msg =
			new TaskMessage(this,
							TaskMessage.TaskMessageType.TASK_PENDING_STATE_CHANGED,
							t,
							userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its priority is changed so that the manager can notify all interested
	 * parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldPriority the former priority of this task
	 * @param newPriority the new priority of this task
	 */
	public void noteTaskPriorityChanged(Task t, double oldPriority, double newPriority) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";

		// Re-sort the item:
		this.tasks.resort(t);

		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_PRIORITY, oldPriority);
		userInfo.put(TaskMessage.NEW_PRIORITY, newPriority);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_PRIORITY_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its associated project is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldProject the old project of the task
	 * @param newProject the new project of the task
	 */
	public void noteTaskProjectChanged(Task t, Project oldProject, Project newProject) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_PROJECT, oldProject);
		userInfo.put(TaskMessage.NEW_PROJECT, newProject);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_PROJECT_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its associated resources are changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldResources the old resources associated with the task
	 * @param newResources the new resources associated with the task
	 */
	public void noteTaskResourcesChanged(Task t,
			ResourceList oldResources,
			ResourceList newResources) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_RESOURCES, oldResources);
		userInfo.put(TaskMessage.NEW_RESOURCES, newResources);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_RESOURCES_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Called by a Task when its associated tags are changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldTags the old tags of the task
	 * @param newTags the new tags of the task
	 */
	public void noteTaskTagsChanged(Task t, Set<String> oldTags, HashSet<String> newTags) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_TAGS, oldTags);
		userInfo.put(TaskMessage.NEW_TAGS, newTags);
		Message msg =
			new TaskMessage(this, TaskMessage.TaskMessageType.TASK_TAGS_CHANGED, t, userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Removes the given task from being managed by this manager.
	 * 
	 * @param t the task to remove
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public void removeTasklikeObject(TasklikeObject t) {
		assert this.tasks.contains(t) : "tried to remove a task that is not currently being managed.";
		this.tasks.remove(t);
		t.setManaged(false);
		Message msg =
			new TasklikeObjectMessage(	this,
										TasklikeObjectMessage.TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED,
										t);

		this.setChanged();
		this.notifyObservers(msg);
	}

	/**
	 * Resets this manager back to the way it was when it was first created.
	 * 
	 * @todo Remove me after testing.
	 */
	public void reset() {
		TaskManager.uniqueInstance = null;
	}

	/**
	 * Returns an iterator, sorted by priority, over the tasks managed by the manager.
	 * 
	 * @return an iterator to the managed tasks
	 * @see java.util.Set#iterator()
	 */
	@SuppressWarnings("unchecked")
	public Iterator<TasklikeObject> taskIterator() {
		return this.tasks.iterator();
	}

	/**
	 * Returns a string representation of this TaskManager. Useful only for debugging.
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String ret = "TaskManager:" + this.countTasks() + " tasks:";
		Iterator<TasklikeObject> iter = taskIterator();
		while (iter.hasNext()) {
			TasklikeObject element = iter.next();
			ret += "{" + element.toString() + "}, ";
		}
		return ret;
	}

	/**
	 * Reads in a serialized manager. Just checks against the serialVersionUID.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != TaskManager.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Writes this manager to stream. Since I don't want to deal with everything below this manager
	 * being serialized and it is a singleton, I just write the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(TaskManager.serialVersionUID);
	}

	/**
	 * Called by a Task when its date completed is changed so that the manager can notify all
	 * interested parties of the change.
	 * 
	 * @param t the task that has been altered
	 * @param oldDateCompleted the old date completed of the task
	 * @param newDateCompleted the new date completed of the task
	 */
	public void noteDateCompletedChanged(Task t,
			Calendar oldDateCompleted,
			Calendar newDateCompleted) {
		assert this.tasks.contains(t) : "tried to change a task that is not currently being managed.";
		Map<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(TaskMessage.OLD_DATE_COMPLETED, oldDateCompleted);
		userInfo.put(TaskMessage.NEW_DATE_COMPLETED, newDateCompleted);
		Message msg =
			new TaskMessage(this,
							TaskMessage.TaskMessageType.TASK_DATE_COMPLETED_CHANGED,
							t,
							userInfo);
		this.setChanged();
		this.notifyObservers(msg);
	}
}
