/* File: HistoryMessage.java
 *
 * Contains class HistoryMessage
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.HistoryManager;
import edu.cmu.hcii.calo.model.HistoryItem;

/**
 * A type of Message that pertains to the modification and management of History instances.
 * 
 * @author Will Haines
 */
public class HistoryMessage extends Message {

	/**
	 * The possible types of a HistoryMessage.
	 * 
	 * @author Will Haines
	 * 
	 */
	public enum HistoryMessageType implements MessageType {
		/** The History has just been added to the manager. */
		HISTORY_ITEM_ADDED,
	}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -746178295541244767L;

	/** The History to which this message pertains. */
	private HistoryItem historyItem;

	/**
	 * Constructs a History Message for the given manager, message type, and history item.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param h the History to which the message pertains
	 */
	public HistoryMessage(HistoryManager manager, HistoryMessageType type, HistoryItem h) {
		this(manager, type);
		this.historyItem = h;
	}

	/**
	 * Constructs a HistoryMessage for the given manager, message type, history item, and user info
	 * map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param h the History to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public HistoryMessage(HistoryManager manager,
			HistoryMessageType type,
			HistoryItem h,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.historyItem = h;
	}

	/**
	 * Private constructor. Use HistoryMessage#HistoryMessage(Observable, MessageType, History)
	 * instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private HistoryMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use HistoryMessage#HistoryMessage(Observable, MessageType, History, Map)
	 * instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private HistoryMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for a history item
	 * 
	 * @return the value of the history item
	 */
	public HistoryItem getHistoryItem() {
		return this.historyItem;
	}
}
