/* File: NotificationMessage.java
 *
 * Contains class NotificationMessage
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.messagePassing;

import java.io.Serializable;
import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.model.Notification;

/**
 * A type of Message that pertains to the modification and management of notification instances.
 * 
 * @author Will Haines
 */
public class NotificationMessage extends Message implements Serializable {

	/**
	 * The possible types of a NotificationMessage.
	 */
	public enum NotificationMessageType implements MessageType {
		/** The notification has just been added to the manager. */
		NOTIFICATION_ADDED,
		/** The notification has just been removed from the manager. */
		NOTIFICATION_REMOVED,
		/** The notification's priority has been changed. */
		NOTIFICATION_PRIORITY_CHANGED,
		/** The notification's text has been changed. */
		NOTIFICATION_TEXT_CHANGED,
		/** The notification's name has been changed. */
		NOTIFICATION_NAME_CHANGED,
		/** The notification's associated person has been changed. */
		NOTIFICATION_PERSON_CHANGED
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	/** The old priority. */
	public static final String OLD_PRIORITY = "OldPriority";

	/** The new priority. */
	public static final String NEW_PRIORITY = "NewPriority";

	/** The old text. */
	public static final String OLD_TEXT = "OldText";

	/** The new text. */
	public static final String NEW_TEXT = "NewText";

	/** The old person. */
	public static final String OLD_PERSON = "OldPerson";

	/** The new person. */
	public static final String NEW_PERSON = "NewPerson";

	/** The old name. */
	public static final String OLD_NAME = "OldName";

	/** The new name. */
	public static final String NEW_NAME = "NewName";

	// @}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = 1708206550814911259L;

	/**
	 * The notification to which this message pertains.
	 */
	private Notification notification;

	/**
	 * Constructs a Notification Message for the given manager, message type, and notification.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param n the notification to which the message pertains
	 */
	public NotificationMessage(NotificationManager manager,
			NotificationMessageType type,
			Notification n) {
		this(manager, type);
		this.notification = n;
	}

	/**
	 * Constructs a NotificationMessage for the given manager, message type, notification, and user
	 * info map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param n the notification to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public NotificationMessage(NotificationManager manager,
			NotificationMessageType type,
			Notification n,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.notification = n;
	}

	/**
	 * Private constructor. Use NotificationMessage#NotificationMessage(Observable, MessageType,
	 * Notification) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private NotificationMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use NotificationMessage#NotificationMessage(Observable, MessageType,
	 * Notification, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private NotificationMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for notification
	 * 
	 * @return the value of notification
	 */
	public Notification getNotification() {
		return this.notification;
	}
}
