/* File: PackMessage.java
 *
 * Contains class PackMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.PackManager;
import edu.cmu.hcii.calo.model.Pack;

/**
 * A type of Message that pertains to the modification and management of Packs in the PackManager.
 * 
 * @author Brian Ellis
 * 
 */
public class PackMessage extends Message {

	/**
	 * The possible types of a PackMessage.
	 */
	public enum PackMessageType implements MessageType {
		/**
		 * The Pack has just been added to the manager.
		 */
		PACK_ADDED,
		/**
		 * The Pack's resources have changed.
		 */
		PACK_RESOURCES_CHANGED,
		/**
		 * The Pack has just been removed from the manager.
		 */
		PACK_REMOVED
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	public static final String OLD_RESOURCES = "OldResources";

	public static final String NEW_RESOURCES = "NewResources";
	// @}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = 7840310614703757328L;

	/**
	 * The pack to which this message pertains.
	 */
	private Pack pack;

	/**
	 * Constructs a RelevantActionMessage for the given manager, message type, and action list.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param pack the pack to which the message pertains
	 */
	public PackMessage(PackManager manager, PackMessageType type, Pack pack) {
		this(manager, type);
		this.pack = pack;
	}

	/**
	 * Constructs a RelevantActionMessage for the given manager, message type, resource list, and
	 * user info map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param pack the pack to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public PackMessage(PackManager manager,
			PackMessageType type,
			Pack pack,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.pack = pack;
	}

	/**
	 * Private constructor. Use RelevantActionMessage#RelevantActionMessage(Observable, MessageType,
	 * List) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private PackMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use RelevantActionMessage#RelevantActionMessage(Observable, MessageType,
	 * List, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private PackMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for pack
	 * 
	 * @return the value of pack
	 */
	public Pack getPack() {
		return this.pack;
	}

}
