/* File: RelevantActionMessage.java
 *
 * Contains class RelevantActionMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.List;
import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.RelevantInfoManager;
import edu.cmu.hcii.calo.model.RelevantAction;

/**
 * A type of Message that pertains to the modification and management of relevant actions in the
 * RelevantInfoManager.
 * 
 * @author Brian Ellis
 */
public class RelevantActionMessage extends Message {

	/**
	 * The possible types of a RelevantActionMessage.
	 * 
	 * Currently there is only one of these, because the relevant action list will typically change
	 * wholesale rather than selective elements being added or deleted.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum RelevantActionMessageType implements MessageType {
		/** The relevant actions have generically changed. */
		ACTIONS_CHANGED,
	}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = 4916357308108903899L;

	/**
	 * The actions to which this message pertains.
	 */
	private List<RelevantAction> actions;

	/**
	 * Constructs a RelevantActionMessage for the given manager, message type, and action list.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param actions the action list to which the message pertains
	 */
	public RelevantActionMessage(RelevantInfoManager manager,
			RelevantActionMessageType type,
			List<RelevantAction> actions) {
		this(manager, type);
		this.actions = actions;
	}

	/**
	 * Constructs a RelevantActionMessage for the given manager, message type, resource list, and
	 * user info map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param actions the action list to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public RelevantActionMessage(RelevantInfoManager manager,
			RelevantActionMessageType type,
			List<RelevantAction> actions,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.actions = actions;
	}

	/**
	 * Private constructor. Use RelevantActionMessage#RelevantActionMessage(Observable, MessageType,
	 * List) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private RelevantActionMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use RelevantActionMessage#RelevantActionMessage(Observable, MessageType,
	 * List, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private RelevantActionMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for actions
	 * 
	 * @return the value of actions
	 */
	public List<RelevantAction> getActions() {
		return this.actions;
	}
}
