/* File: RelevantResourceMessage.java
 *
 * Contains class RelevantResourceMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.RelevantInfoManager;
import edu.cmu.hcii.calo.model.ResourceList;

/**
 * A type of Message that pertains to the modification and management of relevant resources in the
 * RelevantInfoManager.
 * 
 * @author Brian Ellis
 */
public class RelevantResourceMessage extends Message {

	/**
	 * The possible types of a RelevantResourceMessage.
	 * 
	 * Currently there is only one of these, because the relevant resource list will typically
	 * change wholesale rather than selective elements being added or deleted.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum RelevantResourceMessageType implements MessageType {
		/** The relevant resources have generically changed. */
		RESOURCES_CHANGED,
	}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -6953136420412472710L;

	/**
	 * The resources to which this message pertains.
	 */
	private ResourceList resources;

	/**
	 * Constructs a RelevantResourceMessage for the given manager, message type, and resource list.
	 * 
	 * @param manager the manager sending the message
	 * @param r the resource list to which the message pertains
	 * @param type the type of the message
	 */
	public RelevantResourceMessage(RelevantInfoManager manager,
			RelevantResourceMessageType type,
			ResourceList r) {
		this(manager, type);
		this.resources = r;
	}

	/**
	 * Constructs a RelevantResourceMessage for the given manager, message type, resource list, and
	 * user info map.
	 * 
	 * @param manager the manager sending the message
	 * @param r the resource list to which the message pertains
	 * @param type the type of the message
	 * @param userInfo the user info map for this message
	 */
	public RelevantResourceMessage(RelevantInfoManager manager,
			RelevantResourceMessageType type,
			ResourceList r,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.resources = r;
	}

	/**
	 * Private constructor. Use RelevantResourceMessage#RelevantResourceMessage(Observable,
	 * MessageType, ResourceList) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private RelevantResourceMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use RelevantResourceMessage#RelevantResourceMessage(Observable,
	 * MessageType, ResourceList, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private RelevantResourceMessage(Observable sender,
			MessageType type,
			Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for resources
	 * 
	 * @return the value of resources
	 */
	public ResourceList getResources() {
		return this.resources;
	}
}
