/* File: ScheduleMessage.java
 *
 * Contains class ScheduleMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.ScheduleManager;
import edu.cmu.hcii.calo.model.ScheduleItem;

/**
 * A type of Message that pertains to the modification and management of ScheduleItem instances.
 * 
 * @author Brian Ellis
 */
public class ScheduleMessage extends Message {

	/**
	 * The possible types of a ScheduleMessage.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum ScheduleMessageType implements MessageType {
		/** The scheduleItem has been changed in some generic way. */
		SCHEDULE_ITEM_CHANGED,
		/** The scheduleItem's name has been changed. */
		SCHEDULE_ITEM_NAME_CHANGED,
		/** The scheduleItem's associated project has been changed. */
		SCHEDULE_ITEM_PROJECT_CHANGED,
		/** The scheduleItem's start time has been changed. */
		SCHEDULE_ITEM_START_TIME_CHANGED,
		/** The scheduleItem's end time has been changed. */
		SCHEDULE_ITEM_END_TIME_CHANGED,
		/** Whether or not the scheduleItem has a reminder has been changed. */
		SCHEDULE_ITEM_HAS_REMINDER_CHANGED,
		/** The scheduleItem has been added to the manager. */
		SCHEDULE_ITEM_ADDED,
		/** The scheduleItem has been removed from the manager. */
		SCHEDULE_ITEM_REMOVED
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	/** The old due date. */
	public static final String OLD_START_TIME = "OldStartTime";

	/** The new due date. */
	public static final String NEW_START_TIME = "NewStartTime";

	/** The old name. */
	public static final String OLD_NAME = "OldName";

	/** The new name. */
	public static final String NEW_NAME = "NewName";

	/** The old priority. */
	public static final String OLD_END_TIME = "OldEndTime";

	/** The new priority. */
	public static final String NEW_END_TIME = "NewEndTime";

	/** The old pending state. */
	public static final String OLD_HAS_REMINDER = "OldHasReminder";

	/** The new pending state. */
	public static final String NEW_HAS_REMINDER = "NewHasReminder";

	/** The old resources. */
	public static final String OLD_RESOURCES = "OldResources";

	/** The new resources. */
	public static final String NEW_RESOURCES = "NewResources";

	/** The old tags. */
	public static final String OLD_TAGS = "OldTags";

	/** The new tags. */
	public static final String NEW_TAGS = "NewTags";

	/** The old project. */
	public static final String OLD_PROJECT = "OldProject";

	/** The new project. */
	public static final String NEW_PROJECT = "NewProject";

	// @}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -3987279722886374248L;

	/**
	 * The ScheduleItem to which this message pertains.
	 */
	private ScheduleItem scheduleItem;

	/**
	 * Constructs a TaskMessage for the given manager, message type, and scheduleItem.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param item the scheduleItem to which the message pertains
	 */
	public ScheduleMessage(ScheduleManager manager, ScheduleMessageType type, ScheduleItem item) {
		this(manager, type);
		this.scheduleItem = item;
	}

	/**
	 * Constructs a TaskMessage for the given manager, message type, scheduleItem, and user info
	 * map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param item the scheduleItem to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public ScheduleMessage(ScheduleManager manager,
			ScheduleMessageType type,
			ScheduleItem item,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.scheduleItem = item;
	}

	/**
	 * Private constructor. Use ScheduleMessage#ScheduleMessage(Observable, MessageType,
	 * ScheduleItem) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private ScheduleMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use ScheduleMessage#ScheduleMessage(Observable, MessageType,
	 * ScheduleItem, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private ScheduleMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	// @}

	/**
	 * Accessor method for scheduleItem
	 * 
	 * @return the value of scheduleItem
	 */
	public ScheduleItem getScheduleItem() {
		return this.scheduleItem;
	}

}
