/* File: TaskGroupMessage.java
 *
 * Contains class TaskGroupMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.AbstractManager;
import edu.cmu.hcii.calo.model.TaskGroup;

/**
 * A type of Message that pertains to the modification and management of Prioritizable instances.
 * 
 * @author Brian Ellis
 */
public class TaskGroupMessage extends Message {

	/**
	 * The possible types of a PrioritizedObjectMessage
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum TaskGroupMessageType implements MessageType {
		/** The task group's subtasks have been changed. */
		TASK_GROUP_SUBTASKS_CHANGED,
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	/** The old subtasks. */
	public static final String OLD_SUBTASKS = "OldSubtasks";

	/** The new subtasks. */
	public static final String NEW_SUBTASKS = "NewSubtasks";

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -7455516326133953320L;

	// @}

	/**
	 * The Prioritizable to which this message pertains.
	 */
	private TaskGroup taskGroup;

	/**
	 * Constructs a TaskGroupMessage for the given manager, message type, and taskGroup.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param t the TaskGroup to which the message pertains
	 */
	public TaskGroupMessage(AbstractManager manager, TaskGroupMessageType type, TaskGroup t) {
		this(manager, type);
		this.taskGroup = t;
	}

	/**
	 * Constructs a PrioritizedObjectMessage for the given manager, message type, taskGroup, and
	 * user info map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param t the Prioritizable to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public TaskGroupMessage(AbstractManager manager,
			TaskGroupMessageType type,
			TaskGroup t,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.taskGroup = t;
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */

	/**
	 * Private constructor. Use TaskGroupMessage#TaskGroupMessage(Observable, MessageType,
	 * TaskGroup) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private TaskGroupMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use TaskGroupMessage#TaskGroupMessage(Observable, MessageType,
	 * TaskGroup, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private TaskGroupMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for taskGroup
	 * 
	 * @return the value of taskGroup
	 */
	public TaskGroup getTaskGroup() {
		return this.taskGroup;
	}

}
