/* File: TaskMessage.java
 *
 * Contains class TaskMessage
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.Map;
import java.util.Observable;

import edu.cmu.hcii.calo.manager.AbstractManager;
import edu.cmu.hcii.calo.model.Task;

/**
 * A type of Message that pertains to the modification and management of Task instances.
 * 
 * @author Brian Ellis
 */
public class TaskMessage extends Message {

	/**
	 * The possible types of a TaskMessage
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum TaskMessageType implements MessageType {
		/** The task has been changed in some generic way. */
		TASK_CHANGED,
		/** The task's name has been changed. */
		TASK_NAME_CHANGED,
		/** The task's priority has been changed. */
		TASK_PRIORITY_CHANGED,
		/** The task's resource list has been changed. */
		TASK_RESOURCES_CHANGED,
		/** The task's pending state has been changed. */
		TASK_PENDING_STATE_CHANGED,
		/** The task's due date has been changed. */
		TASK_DUE_DATE_CHANGED,
		/** The task's tags have been changed. */
		TASK_TAGS_CHANGED,
		/** The task's associated project has been changed. */
		TASK_PROJECT_CHANGED,
		/** The task's completion value has changed. */
		TASK_COMPLETE_CHANGED,
		/** The task's starred value has changed. */
		TASK_STARRED_CHANGED,
		/** The task's date completed has changed. */
		TASK_DATE_COMPLETED_CHANGED,
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	/** The old due date. */
	public static final String OLD_DUE_DATE = "OldDueDate";

	/** The new due date. */
	public static final String NEW_DUE_DATE = "NewDueDate";

	/** The old name. */
	public static final String OLD_NAME = "OldName";

	/** The new name. */
	public static final String NEW_NAME = "NewName";

	/** The old priority. */
	public static final String OLD_PRIORITY = "OldPriority";

	/** The new priority. */
	public static final String NEW_PRIORITY = "NewPriority";

	/** The old pending state. */
	public static final String OLD_PENDING_STATE = "OldPendingState";

	/** The new pending state. */
	public static final String NEW_PENDING_STATE = "NewPendingState";

	/** The old resources. */
	public static final String OLD_RESOURCES = "OldResources";

	/** The new resources. */
	public static final String NEW_RESOURCES = "NewResources";

	/** The old tags. */
	public static final String OLD_TAGS = "OldTags";

	/** The new tags. */
	public static final String NEW_TAGS = "NewTags";

	/** The old project. */
	public static final String OLD_PROJECT = "OldProject";

	/** The new project. */
	public static final String NEW_PROJECT = "NewProject";

	/** The old task complete status. */
	public static final String OLD_COMPLETE = "OldComplete";

	/** The new task complete status. */
	public static final String NEW_COMPLETE = "NewComplete";

	/** The old starredness status. */
	public static final String OLD_STARRED = "OldStarred";

	/** The new starredness status. */
	public static final String NEW_STARRED = "NewStarred";

	public static final String OLD_DATE_COMPLETED = "OldDateCompleted";

	public static final String NEW_DATE_COMPLETED = "NewDateCompleted";

	// @}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = 2199548101869340621L;

	/**
	 * The task to which this message pertains.
	 */
	private Task task;

	/**
	 * Constructs a TaskMessage for the given manager, message type, and task.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param t the task to which the message pertains
	 */
	public TaskMessage(AbstractManager manager, TaskMessageType type, Task t) {
		this(manager, type);
		this.task = t;
	}

	/**
	 * Constructs a TaskMessage for the given manager, message type, task, and user info map.
	 * 
	 * @param manager the manager sending the message
	 * @param type the type of the message
	 * @param t the task to which the message pertains
	 * @param userInfo the user info map for this message
	 */
	public TaskMessage(AbstractManager manager,
			TaskMessageType type,
			Task t,
			Map<String, Object> userInfo) {
		this(manager, type, userInfo);
		this.task = t;
	}

	/**
	 * Private constructor. Use TaskMessage#TaskMessage(Observable, MessageType, Task) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 */
	private TaskMessage(Observable sender, MessageType type) {
		super(sender, type);
	}

	/**
	 * Private constructor. Use TaskMessage#TaskMessage(Observable, MessageType, Task, Map) instead.
	 * 
	 * @param sender the object that sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	private TaskMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}

	/**
	 * Accessor method for task
	 * 
	 * @return the value of task
	 */
	public Task getTask() {
		return this.task;
	}

}
