/* File: WizardOfOzMessage.java
 *
 * Contains class WizardOfOzMessage
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.messagePassing;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;

/**
 * Allows the Wizard of Oz engine and front end to communicate about things that are specific to
 * their operation.
 * 
 * @author Will Haines
 */
public class WizardOfOzMessage extends Message {

	/**
	 * The possible types of a WizardOfOzEngineMessage.
	 */
	public enum WizardOfOzMessageType implements MessageType {
		/** The front end wants the engine to close. */
		CLOSE,
		
		/** The front end wants a controller to ignore updates for atomicity. */
		NOTIFICATION_CONTROLLER_IGNORE_CHANGE,
		/** The engine wants to update the status of its notification add. */
		NOTIFICATION_ADD_STATUS,
		/** The engine wants to update the status of its notification delete. */
		NOTIFICATION_DELETED_STATUS,
		/** The engine wants to update the status of its notification name change. */
		NOTIFICATION_NAME_CHANGED_STATUS,
		/** The engine wants to update the status of its notification text change. */
		NOTIFICATION_TEXT_CHANGED_STATUS,
		/** The engine wants to update the status of its notification person change. */
		NOTIFICATION_PERSON_CHANGED_STATUS,
		/**
		 * The engine wants to update the status of its notification priority change.
		 */
		NOTIFICATION_PRIORITY_CHANGED_STATUS,
		/** The front end wants an iterator over the current notifications. */
		NOTIFICATION_ITERATOR_REQUEST,
		
		/** The engine wants to update the status of its task priority change. */
		TASK_PRIORITY_CHANGED_STATUS,
		/** The front end wants an iterator over the current tasklike objects. */
		TASKLIKE_OBJECT_ITERATOR_REQUEST,
		/** The engine wants to update the status of its tasklike object add. */
		TASKLIKE_OBJECT_ADD_STATUS,
		/** The engine wants to update the status of its task completed change. */
		TASK_COMPLETED_CHANGED_STATUS,
		/** The engine wants to update the status of its task pending change. */
		TASK_PENDING_CHANGED_STATUS,
		/** The engine wants to update the status of its task delete. */
		TASK_DELETED_STATUS,
	}

	/**
	 * @name User info map keys
	 * 
	 * These strings are the defined keys for use in the user info dictionary.
	 */
	// @{
	/** The success status of some engine state change. */
	public static final String SUCCESS_STATUS = "SuccessStatus";

	/** The notification iterator. */
	public static final String NOTIFICATION_ITERATOR = "NotificationIterator";

	/** The task iterator. */
	public static final String TASK_ITERATOR = "TaskIterator";
	
	/** The old ignore updates status. */
	public static final String OLD_IGNORE = "OldIgnore";
	
	/** The new ignore updates status. */
	public static final String NEW_IGNORE = "NewIgnore";

	// @}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -4298492140113423285L;
	
	/**
	 * Constructs a new WizardOfOzMessage.
	 * 
	 * @param sender the Observable that has sent this message
	 * @param type the type of this message
	 */
	public WizardOfOzMessage(Observable sender, MessageType type) {
		this(sender, type, new HashMap<String, Object>());
	}

	/**
	 * Constructs a new WizardOfOzMessage.
	 * 
	 * @param sender the Observable that has sent this message
	 * @param type the type of this message
	 * @param userInfo the map of message-specific data associated with this message
	 */
	public WizardOfOzMessage(Observable sender, MessageType type, Map<String, Object> userInfo) {
		super(sender, type, userInfo);
	}
}
