/* File: ManagedObject.java
 *
 * Contains class ManagedObject
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.io.Serializable;
import java.util.Calendar;

/**
 * Represents an object that is designed to be managed by one of CALO's manager classes.
 * 
 * @author Brian Ellis
 * 
 */
public abstract class ManagedObject implements Serializable {

	/**
	 * Whether or not this object is currently managed by a manager. (When objects are created, they
	 * are not managed, and changes to the object will not be propagated to any controllers.)
	 */
	private boolean isManaged = false;

	/** Was this item added by the AI engine? */
	private boolean caloAdded;

	/** The date on which this item was added. */
	private Calendar dateAdded;

	/** The unique id of this item for lookup in its manager. */
	private int id = -1;

	/**
	 * Tests if an object is equal to this managed object. If the object is unmanaged, it is just a
	 * reference equals. If this object is managed, then we compare unique management ids to
	 * determine equality. Don't assume something else; you will be wrong.
	 * 
	 * @param obj the object to test equality with
	 * @return true if these objects are equal, false otherwise
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this.isManaged()) {
			if (this == obj) {
				return true;
			}
			if (getClass() != obj.getClass()) {
				return false;
			}
			final ManagedObject other = (ManagedObject)obj;
			if (this.id != other.id) {
				return false;
			}
			return true;
		}
		return super.equals(obj);
	}

	/**
	 * Accessor method for whether or not this item was added by the AI engine
	 * 
	 * @return true if added by CALO
	 */
	public boolean isCaloAdded() {
		return this.caloAdded;
	}

	/**
	 * Accessor method for dateAdded.
	 * 
	 * @return the value of dateAdded
	 */
	public Calendar getDateAdded() {
		return this.dateAdded;
	}

	/**
	 * Returns a hash code for this managed object. If the object is unmanaged, it is a reference
	 * hash; otherwise, it is based on the unique management id. Don't assume otherwise; you will be
	 * wrong.
	 * 
	 * @return a hash code for this object
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		if (this.isManaged()) {
			final int PRIME = 31;
			int result = PRIME * this.id;
			return result;
		}
		return super.hashCode();
	}

	/**
	 * Accessor method for isManaged
	 * 
	 * @return the value of isManaged
	 */
	public boolean isManaged() {
		return this.isManaged;
	}

	/**
	 * Sets the ID of this managed object. This should be called by the manager managing this
	 * object, and ensures that its equals() and hashCode() methods will continue to work if it is
	 * serialized and deserialized somewhere else.
	 * 
	 * @param id the new ID of this object
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Setter method for isManaged.
	 * 
	 * @param isManaged the isManaged to set
	 */
	public void setManaged(boolean isManaged) {
		this.isManaged = isManaged;
	}

	/**
	 * Accessor method for id.
	 * 
	 * @return the value of id
	 */
	protected int getId() {
		return this.id;
	}

	/**
	 * Setter method for caloAdded.
	 * 
	 * @param caloAdded the caloAdded to set
	 */
	protected void setCaloAdded(boolean caloAdded) {
		this.caloAdded = caloAdded;
	}

	/**
	 * Setter method for dateAdded.
	 * 
	 * @param dateAdded the dateAdded to set
	 */
	protected void setDateAdded(Calendar dateAdded) {
		this.dateAdded = dateAdded;
	}

}
