/* File: Notification.java
 *
 * Contains class Notification
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.model;

import java.util.Calendar;

import edu.cmu.hcii.calo.manager.NotificationManager;

/**
 * Represents a notification as generated by the CALO engine and visualized in the notification
 * center.
 * 
 * @author Will Haines
 */
public class Notification extends PrioritizedObject {

	/**
	 * Possible notification types
	 */
	public enum NotificationType {
		CALO_ACTION,
		EMAIL,
		REMINDER
	}

	/**
	 * To prevent serialization collisons.
	 */
	private static final long serialVersionUID = -7704300813511007785L;

	/** The notification's type for purposes of coalescing */
	private NotificationType type;

	/** This notification's name */
	private String name;

	/** The text associated with the notification */
	private String text;

	/** The person associated with this notification if it is an email */
	private Person person;

	/** The notification's priority */
	private double priority;

	/**
	 * Construct a notification
	 * 
	 * @param type this notification's type for coalescing
	 * @param name this notification's name
	 * @param text the text to display for this notification
	 * @param person the person associated with this notification
	 */
	public Notification(NotificationType type, String name, String text, Person person) {
		this.type = type;
		this.name = name;
		this.text = text;
		this.person = person;
		this.setCaloAdded(true); // All notifications are added by CALO.
		this.setDateAdded(Calendar.getInstance());

		// This value is a placeholder. The engine should update it upon
		// adding the notification.
		this.priority = -1.0;
	}

	/**
	 * Construct a notification with an unknown person.
	 * 
	 * @param type this notification's type for coalescing
	 * @param name this notification's name
	 * @param text the text to display for this notification
	 */
	public Notification(NotificationType type, String name, String text) {
		this.type = type;
		this.name = name;
		this.text = text;
		this.person = new Person();
		this.setCaloAdded(true); // All notifications are added by CALO.
		this.setDateAdded(Calendar.getInstance());

		// This value is a placeholder. The engine should update it upon
		// adding the notification.
		this.priority = -1.0;
	}

	/**
	 * Accessor method for notification name.
	 * 
	 * @return this notification's name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Getter method for notification name.
	 * 
	 * @param name this notification's new name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Accessor method for priority
	 * 
	 * @return the value of priority
	 */
	@Override
	public double getPriority() {
		return this.priority;
	}

	/**
	 * Accessor method for notification text.
	 * 
	 * @return this notification's associated text
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * Accessor method for this notification's type
	 * 
	 * @return the value of this notification's type
	 */
	public NotificationType getType() {
		return this.type;
	}

	/**
	 * Setter method for priority.
	 * 
	 * @param priority the priority to set
	 */
	public void setPriority(double priority) {
		double oldPriority = this.priority;
		this.priority = priority;
		if (this.isManaged()) {
			NotificationManager.getManager().noteNotificationPriorityChanged(	this,
																				oldPriority,
																				this.priority);
		}
	}

	/**
	 * Setter method for associated text.
	 * 
	 * @param text the text to set
	 */
	public void setText(String text) {
		String oldText = this.text;
		this.text = text;
		if (this.isManaged()) {
			NotificationManager.getManager().noteNotificationTextChanged(this, oldText, this.text);
		}
	}

	/**
	 * Returns a string representation of this notification. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return "Notification: ID = " + this.getId() + " " + this.getName() + " "
				+ this.getPriority();
	}

	/**
	 * A getter for the person associated with this notification.
	 * 
	 * @return the person
	 */
	public Person getPerson() {
		return this.person;
	}

	/**
	 * A setter for the person associated with this notification.
	 * 
	 * @param person the person to set
	 */
	public void setPerson(Person person) {
		this.person = person;
	}
}
