/* File: PendingState.java
 *
 * Contains class PendingState
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.io.Serializable;

/**
 * Represents the state of a task as pending or non-pending
 * 
 * @author Brian Ellis
 */
public class PendingState implements Serializable {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -8421898332536226325L;

	/** Whether this task is currently pending on something. */
	private boolean isPending;

	/**
	 * The textual note the user entered as the reason this task is pending, if any such note
	 * exists. (If not, this will be the empty string.)
	 */
	private String pendingNote;

	/**
	 * The name of the person the user entered as the reason this task is pending, if any such name
	 * exists. (If not, this will be the empty string.)
	 */
	private String person;

	/**
	 * Creates a default pending state; this means that isPending is false and pendingNote is null;
	 */
	public PendingState() {
		this.isPending = false;
		this.pendingNote = "";
		this.person = "";
	}
	
	/**
	 * Creates a pending state with the given pending status.
	 * 
	 * @param isPending the pending status to set
	 */
	public PendingState(boolean isPending) {
		this.isPending = isPending;
		this.pendingNote = "";
		this.person = "";		
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * 
	 * @param obj the object this one may or may not be equal to
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final PendingState other = (PendingState)obj;
		if (this.isPending != other.isPending) {
			return false;
		}
		if (this.pendingNote == null) {
			if (other.pendingNote != null) {
				return false;
			}
		} else if (!this.pendingNote.equals(other.pendingNote)) {
			return false;
		}
		if (this.person == null) {
			if (other.person != null) {
				return false;
			}
		} else if (!this.person.equals(other.person)) {
			return false;
		}
		return true;
	}

	/**
	 * Accessor method for pendingNote
	 * 
	 * @return the value of pendingNote
	 */
	public String getPendingNote() {
		assert this.pendingNote != null : "pending note was null";
		return this.pendingNote;
	}

	/**
	 * Accessor method for person
	 * 
	 * @return the value of person
	 */
	public String getPerson() {
		assert this.person != null : "person was null";
		return this.person;
	}

	/**
	 * Returns a hash code value for the object. This method is supported for the benefit of
	 * hashtables such as those provided by <code>java.util.Hashtable</code>.
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + (this.isPending ? 1231 : 1237);
		result = PRIME * result + ((this.pendingNote == null) ? 0 : this.pendingNote.hashCode());
		return result;
	}

	/**
	 * Accessor method for whether or not the associated task is pending.
	 * 
	 * @return true if the associated task is pending
	 */
	public boolean isPending() {
		return this.isPending;
	}

	/**
	 * Setter method whether or not the associated task is pending.
	 * 
	 * @param isPending the pending value to set
	 */
	public void setPending(boolean isPending) {
		this.isPending = isPending;
	}

	/**
	 * Setter method for pendingNote.
	 * 
	 * @param pendingNote the pendingNote to set
	 */
	public void setPendingNote(String pendingNote) {
		if (pendingNote == null) {
			throw new IllegalArgumentException("Pending notes cannot be null.");
		}
		this.pendingNote = pendingNote;
	}

	/**
	 * Returns a string representation of this PendingState. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return this.isPending + " (" + this.pendingNote + ")";
	}

	/**
	 * Sets the name of the person associated with this pending state.
	 * 
	 * @param person the name of the person
	 */
	public void setPerson(String person) {
		if (person == null) {
			throw new IllegalArgumentException("People cannot be null.");
		}
		this.person = person;
	}
}
