/* File: Person.java
 *
 * Contains class Person
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Represents a person known to CALO.
 * 
 * This class is not used in the current implementation.
 * 
 * @author Brian Ellis
 * 
 */
public class Person implements Serializable {

	/** Prevents serialziation collisions. */
	private static final long serialVersionUID = 2297518203901454182L;

	/** The name of this person (as much of it as is known). */
	private String name;

	/** The email address associated with this person. */
	private String emailAddress;

	/** The set of projects associated with this person. */
	private Set<Project> projects;

	/**
	 * A map containing any other information about the person the user wishes to add.
	 */
	private Map<String, String> attributes;

	/**
	 * Create a new unknown person.
	 */
	public Person() {
		this.name = "Unknown";
	}

	/**
	 * Create a new person.
	 */
	public Person(String name) {
		this.name = name;
	}

	/**
	 * Associates this person with the given project.
	 * 
	 * @param p the project to add
	 * @return true if this person was not already associated with the project (but is now), false
	 *         otherwise
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public boolean addProject(Project p) {
		return this.projects.add(p);
	}

	/**
	 * Removes all attributes associated with this person.
	 * 
	 * @see java.util.Map#clear()
	 */
	public void clearAttributes() {
		this.attributes.clear();
	}

	/**
	 * Returns true if the attributes for this person includes a certain attribute, false otherwise.
	 * 
	 * @param key the attribute to find
	 * @return true if the attributes for this person contain this attribute
	 * @see java.util.Map#containsKey(java.lang.Object)
	 */
	public boolean containsAttribute(String key) {
		return this.attributes.containsKey(key);
	}

	/**
	 * Returns the number of attributes associated with this person.
	 * 
	 * @return the number of attributes associated with this person
	 * @see java.util.Map#size()
	 */
	public int countAttributes() {
		return this.attributes.size();
	}

	/**
	 * Returns the number of projects with which this person is associated.
	 * 
	 * @return
	 * @see java.util.Set#size()
	 */
	public int countProjects() {
		return this.projects.size();
	}

	/**
	 * Returns the value of this person's given attribute, or null if it is not defined.
	 * 
	 * @param key The attribute to get the value of
	 * @return the value of that attribute
	 * @see java.util.Map#get(java.lang.Object)
	 */
	public String getAttribute(String key) {
		return this.attributes.get(key);
	}

	/**
	 * Accessor method for emailAddress
	 * 
	 * @return the value of emailAddress
	 */
	public String getEmailAddress() {
		return this.emailAddress;
	}

	/**
	 * Accessor method for name
	 * 
	 * @return the value of name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Returns true if this person is associated with the given project, false otherwise.
	 * 
	 * @param p the project to ask about
	 * @return true if this person is associated with the given project, false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean isAssociatedWithProject(Project p) {
		return this.projects.contains(p);
	}

	/**
	 * Returns an iterator over the projects with which this person is associated.
	 * 
	 * @return an iterator over the projects with which this person is associated
	 * @see java.util.Set#iterator()
	 */
	public Iterator<Project> projectIterator() {
		return this.projects.iterator();
	}

	/**
	 * Sets the value of the given attribute, adding it if it did not already exist.
	 * 
	 * @param key the attribute to add or set
	 * @param value the new value of this attribute
	 * @return the previous value associated with specified attribute, or null if the attribute did
	 *         not previously exist
	 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
	 */
	public String putAttribute(String key, String value) {
		return this.attributes.put(key, value);
	}

	/**
	 * Disassociates this person from all projects with which he or she is currently associated.
	 * 
	 * @see java.util.Set#clear()
	 */
	public void removeAllProjects() {
		this.projects.clear();
	}

	/**
	 * Removes the given attribute from this person's attributes if it existed.
	 * 
	 * @param key the attribute to remove
	 * @return the value of the attribute before it was removed, or null if the attribute did not
	 *         previously exist
	 * @see java.util.Map#remove(java.lang.Object)
	 */
	public String removeAttribute(String key) {
		return this.attributes.remove(key);
	}

	/**
	 * Disassociates this person from the given project.
	 * 
	 * @param p the project to remove
	 * @return true if the project was previously associated with this person (and is no longer),
	 *         false otherwise
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public boolean removeProject(Project p) {
		return this.projects.remove(p);
	}

	/**
	 * Setter method for emailAddress.
	 * 
	 * @param emailAddress the emailAddress to set
	 */
	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	/**
	 * Setter method for name.
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
}
