/* File: RelevantAction.java
 *
 * Contains class RelevantAction
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.awt.Image;

/**
 * Represents a CALO action, displayed in the Relevant Info pane. This class is
 * intended to be anonymously subclassed so that the performAction() method can
 * be implemented on a case-by-case basis.
 * 
 * @author Brian Ellis
 * 
 */
public abstract class RelevantAction {

	/**
	 * The title of this action, shown in the interface.
	 */
	private String title;

	/**
	 * The icon for this action.
	 */
	private Image icon;

	/**
	 * Constructs a new RelevantAction.
	 * 
	 * @param title
	 *            the title of this action
	 * @param icon
	 *            the icon of this action
	 * @throws IllegalArgumentException
	 *             if title or icon are \c null
	 */
	public RelevantAction(String title, Image icon) {
		if (title == null) {
			throw new IllegalArgumentException(
					"Cannot create an action with no title.");
		}
		if (icon == null) {
			throw new IllegalArgumentException(
					"Cannot create an action with no icon.");
		}
		this.title = title;
		this.icon = icon;
	}

	/**
	 * Accessor method for icon
	 * 
	 * @return the value of icon
	 */
	public Image getIcon() {
		assert this.icon != null : "this action's icon is null";
		return this.icon;
	}

	/**
	 * Accessor method for title
	 * 
	 * @return the value of title
	 */
	public String getTitle() {
		assert this.title != null : "this action's title is null";
		return this.title;
	}

	/**
	 * Performs the action associated with this RelevantAction. This is an
	 * abstract method because each RelevantAction has a unique associated
	 * action that it performs.
	 */
	public abstract void performAction();

}
