/* File: Resource.java
 *
 * Contains class Resource
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

/**
 * Represents a resource (such as a file) that can be opened or printed.
 * 
 * @todo This doesn't yet reconstitute the File reference when deserialized.
 * 
 * @author Brian Ellis
 * 
 */
public class Resource implements Comparable<Object>, Serializable {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 903129020567497504L;

	/** The file system object represented by this resource. */
	transient private File associatedFile;

	/** The name of the file to set as a resource. */
	private String fileName;

	/**
	 * The relevance value of this resource to whatever it is being displayed with.
	 */
	private double relevance;

	/**
	 * Constructs a Resource from a particular file.
	 * 
	 * @param associatedFile the file associated with the resource
	 */
	public Resource(File associatedFile) {
		if (associatedFile == null) {
			throw new IllegalArgumentException("Cannot construct a resource with no associated file.");
		}
		this.associatedFile = associatedFile;
		this.fileName = associatedFile.getName();
	}

	/**
	 * Comparator method to sort resources in order from highest to lowest relevance.
	 * 
	 * @param otherObj the object to compare this one to
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object otherObj) {
		if (otherObj instanceof Resource) {
			Resource otherResource = ((Resource)otherObj);
			if (this.relevance > otherResource.relevance) {
				return -1;
			} else if (this.relevance == otherResource.relevance) {
				return 0;
			} else {
				return 1;
			}
		}
		return 0;
	}

	/**
	 * Accessor method for associatedFile
	 * 
	 * @return the value of associatedFile
	 */
	public File getAssociatedFile() {
		assert this.associatedFile != null : "this resource's associated file was null";
		return this.associatedFile;
	}

	/**
	 * A getter for fileName.
	 * 
	 * @return the fileName
	 */
	public String getFileName() {
		return this.fileName;
	}

	/**
	 * Accessor method for relevance
	 * 
	 * @return the value of relevance
	 */
	public double getRelevance() {
		return this.relevance;
	}

	/**
	 * Allows serialization to deal with files correctly.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		this.associatedFile = new File(this.fileName);
	}

	/**
	 * Setter method for relevance.
	 * 
	 * @param relevance the relevance to set
	 */
	public void setRelevance(double relevance) {
		this.relevance = relevance;
	}

	/**
	 * Returns a string representation of this Resource. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return this.fileName;
	}
}
