/* File: ResourceList.java
 *
 * Contains class ResourceList
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.io.Serializable;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Aggregates resources for use by tasks and the relevant info view.
 * 
 * @note Although ResourceLists are managed by the RelevantInfoManager, the ResourceList class does
 *       not subclass ManagedObject because there it is not possible to send an event about a
 *       ResourceList that is not being managed.
 * 
 * @author Brian Ellis
 * 
 */
public class ResourceList implements Serializable {

	/**
	 * A comparator that compares (and sorts) resources within a resource list.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class ResourceListComparator implements Comparator<Resource>, Serializable {

		/** Prevent serialization collisions */
		private static final long serialVersionUID = -2996889433605558422L;

		/**
		 * Compares two Resources.
		 * 
		 * @param o1 the first object
		 * @param o2 the second object
		 */
		public int compare(Resource o1, Resource o2) {
			if (o1.equals(o2)) {
				return 0;
			} else if (o1.getRelevance() > o2.getRelevance()) {
				return 1;
			} else if (o2.getRelevance() > o1.getRelevance()) {
				return -1;
			} else {
				return o1.hashCode() - o2.hashCode();
			}
		}
	}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 8112257147108845110L;

	/** The resources in the list. */
	private SortedSet<Resource> resources;

	/**
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	public ResourceList() {
		this.resources = new TreeSet<Resource>(new ResourceListComparator());
	}

	/**
	 * Adds a resource to the list if it is not already there.
	 * 
	 * @param o the resource to add
	 * @return true if the resource was not already in the list, false otherwise
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public boolean add(Resource o) {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.add(o);
	}

	/**
	 * Returns the resource at the given index.
	 * 
	 * @param index the index to look up
	 * @return the Resource at \p index
	 */
	public Resource at(int index) {
		int i = 0;
		Iterator<Resource> iter = this.resources.iterator();
		while (i < index) {
			++i;
			iter.next();
		}

		Resource next = iter.next();
		return next;
	}

	/**
	 * Removes all resources from the list.
	 * 
	 * @see java.util.Set#clear()
	 */
	public void clear() {
		assert this.resources != null : "this resource list's resources were set to null";
		this.resources.clear();
	}

	/**
	 * Determines whether the resource list contains the given resource.
	 * 
	 * @param o the resource to look for
	 * @return true if the resource is in the list, false otherwise
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean contains(Resource o) {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.contains(o);
	}

	/**
	 * Returns the first (most relevant) resource in the list.
	 * 
	 * @return the first resource
	 * @see java.util.SortedSet#first()
	 */
	public Resource first() {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.first();
	}

	/**
	 * Returns an iterator over the resources in the list, sorted by relevance.
	 * 
	 * @return an iterator over the resources in the list
	 * @see java.util.Set#iterator()
	 */
	public Iterator<Resource> iterator() {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.iterator();
	}

	/**
	 * Returns the last (least relevant) resource in the list.
	 * 
	 * @return the last resource in the list
	 * @see java.util.SortedSet#last()
	 */
	public Resource last() {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.last();
	}

	/**
	 * Removes the given resource from the list if it exists there.
	 * 
	 * @param o the resource to remove
	 * @return true if the resource was in the list and has been removed, false if it was not there
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public boolean remove(Resource o) {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.remove(o);
	}

	/**
	 * Returns the number of resources in the list.
	 * 
	 * @return the number of resources
	 * @see java.util.Set#size()
	 */
	public int size() {
		assert this.resources != null : "this resource list's resources were set to null";
		return this.resources.size();
	}

	/**
	 * Returns a string representation of this ResourceList. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return this.resources.toString();
	}
}
