/* File: ScheduleItem.java
 *
 * Contains class ScheduleItem
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.model;

import java.util.Calendar;

import edu.cmu.hcii.calo.manager.ScheduleManager;

/**
 * Represents a single item in a schedule, to be visualized by the SchedulePane.
 * 
 * @author Will Haines
 */
public class ScheduleItem extends ManagedObject {

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -4948371014705883547L;

	/** The name of this schedule item */
	private String name;

	/** Whether or not this schedule item has a reminder. */
	private boolean hasReminder;

	// /** The set of tags associated with this schedule item. */
	// private HashSet<String> tags;

	// /**
	// * The list of Resources associated with this schedule item (may be
	// unused).
	// */
	// private ResourceList resources;

	/** The project with which this schedule item is associated, if any. */
	private Project project;

	/** The time when this item starts. */
	private Calendar startTime;

	/** The time when this item ends. */
	private Calendar endTime;

	/**
	 * Constructs a ScheduleItem.
	 * 
	 * @param caloAdded whether CALO added this item
	 * @param startTime when this item starts
	 * @param endTime when this item ends
	 */
	public ScheduleItem(boolean caloAdded, Calendar startTime, Calendar endTime) {
		if (startTime == null) {
			throw new IllegalArgumentException("Cannot create a schedule item with a null start time.");
		}
		if (endTime == null) {
			throw new IllegalArgumentException("Cannot create a schedule item with a null end time.");
		}
		if (endTime.before(startTime)) {
			this.startTime = endTime;
			this.endTime = startTime;
		} else {
			this.startTime = startTime;
			this.endTime = endTime;
		}
		this.setCaloAdded(caloAdded);
		this.setDateAdded(Calendar.getInstance());
		// this.tags = new HashSet<String>();
	}

	/**
	 * Returns the number of (possibly fractional) minutes between the start and end of the
	 * scheduled item.
	 * 
	 * @return the number of minutes between the start and end
	 */
	public float getDurationMinutes() {
		return (float)(this.endTime.getTimeInMillis() - this.startTime.getTimeInMillis()) / 1000 / 60;
	}

	/**
	 * Acessor method for endTime
	 * 
	 * @return the value of endTime
	 */
	public Calendar getEndTime() {
		return this.endTime;
	}

	/**
	 * Accessor method for name
	 * 
	 * @return the value of name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Acessor method for startTime
	 * 
	 * @return the value of startTime
	 */
	public Calendar getStartTime() {
		return this.startTime;
	}

	/**
	 * Sets the end time of this ScheduleItem. If the new end time is before this ScheduleItem's
	 * start time, they will be swapped.
	 * 
	 * @param newEndTime the end time to set
	 */
	public void setEndTime(Calendar newEndTime) {
		Calendar oldStartTime = this.startTime;
		Calendar oldEndTime = this.endTime;

		if (newEndTime.before(oldStartTime)) {
			this.startTime = newEndTime;
			this.endTime = oldStartTime;

			if (this.isManaged()) {
				ScheduleManager.getManager().noteItemStartTimeChanged(	this,
																		oldStartTime,
																		this.startTime);
				ScheduleManager.getManager().noteItemEndTimeChanged(this, oldEndTime, this.endTime);
			}
		} else {
			this.endTime = newEndTime;

			if (this.isManaged()) {
				ScheduleManager.getManager().noteItemEndTimeChanged(this, oldEndTime, this.endTime);
			}
		}
	}

	/**
	 * Sets whether this ScheduleItem has an associated reminder.
	 * 
	 * @param hasReminder \c true if this item has a reminder, \c false otherwise
	 */
	public void setHasReminder(boolean hasReminder) {
		boolean oldHasReminder = this.hasReminder;
		this.hasReminder = hasReminder;
		if (this.isManaged()) {
			ScheduleManager.getManager().noteItemHasReminderChanged(this,
																	oldHasReminder,
																	this.hasReminder);
		}
	}

	/**
	 * Setter method for name.
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		String oldName = this.name;
		this.name = name;
		if (this.isManaged()) {
			ScheduleManager.getManager().noteItemNameChanged(this, oldName, this.name);
		}
	}

	/**
	 * Sets the project associated with this ScheduleItem, if any.
	 * 
	 * @param project the project to set
	 */
	public void setProject(Project project) {
		Project oldProject = this.project;
		this.project = project;
		if (this.isManaged()) {
			ScheduleManager.getManager().noteItemProjectChanged(this, oldProject, this.project);
		}
	}

	/**
	 * Sets the start time of this ScheduleItem. If the new start time is after this ScheduleItem's
	 * end time, they will be swapped.
	 * 
	 * @param newStartTime the start time to set
	 */
	public void setStartTime(Calendar newStartTime) {
		Calendar oldStartTime = this.startTime;
		Calendar oldEndTime = this.endTime;

		if (newStartTime.after(oldEndTime)) {
			this.startTime = oldEndTime;
			this.endTime = newStartTime;

			if (this.isManaged()) {
				ScheduleManager.getManager().noteItemEndTimeChanged(this, oldEndTime, this.endTime);
				ScheduleManager.getManager().noteItemStartTimeChanged(	this,
																		oldStartTime,
																		this.startTime);
			}
		} else {
			this.startTime = newStartTime;

			if (this.isManaged()) {
				ScheduleManager.getManager().noteItemStartTimeChanged(	this,
																		oldStartTime,
																		this.startTime);
			}
		}
	}
}
