/* File: Task.java
 *
 * Contains class Task
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.util.Utility;

/**
 * An abstract superclass for tasks and subtasks.
 * 
 * @author Brian Ellis
 * 
 */
public class Task extends TasklikeObject implements Cloneable {

	/**
	 * Pseudointelligently formats a given date as a due date string based on its proximity to the
	 * current date and time. Dates that are closer to the present are displayed in a form that
	 * conveys more precision than those further away, and the length of the textual representation
	 * is kept to a minimum wherever possible.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private static final class SmartDueDateFormatter {

		/**
		 * @name Time constants
		 * 
		 * These constants express the given quantity of time in milliseconds.
		 */

		// @{
		private static final long ONE_MINUTE = 1000 * 60;
		private static final long ONE_HOUR = SmartDueDateFormatter.ONE_MINUTE * 60;
		private static final long ONE_DAY = SmartDueDateFormatter.ONE_HOUR * 24;
		private static final long ONE_WEEK = SmartDueDateFormatter.ONE_DAY * 7;
		// On average! :)
		private static final long ONE_MONTH = (long)(SmartDueDateFormatter.ONE_DAY * 30.416667);
		private static final long ONE_YEAR = SmartDueDateFormatter.ONE_MONTH * 12;

		// @}

		/**
		 * Formats the given date as a String.
		 * 
		 * @param date the date to format
		 */
		public static String formatDate(Calendar date) {

			Calendar now = Calendar.getInstance();

			long diff = date.getTimeInMillis() - now.getTimeInMillis();

			// First, check for the unlikely possibility that it's due RIGHT
			// NOW:
			if (Math.abs(diff) < SmartDueDateFormatter.ONE_MINUTE) {
				return "Due immediately";
			} else if (date.before(now)) { // overdue!
				diff = now.getTimeInMillis() - date.getTimeInMillis();

				String diffString;
				if (diff < SmartDueDateFormatter.ONE_HOUR) {
					Long minutes = new Long(diff / SmartDueDateFormatter.ONE_MINUTE);
					diffString = minutes.toString();
					diffString += (minutes == 1 ? " minute" : " minutes");
				} else if (diff < SmartDueDateFormatter.ONE_DAY) {
					Long hours = new Long(diff / SmartDueDateFormatter.ONE_HOUR);
					diffString = hours.toString();
					diffString += (hours == 1 ? " hour" : " hours");
				} else if (diff < SmartDueDateFormatter.ONE_WEEK) {
					Long days = new Long(diff / SmartDueDateFormatter.ONE_DAY);
					diffString = days.toString();
					diffString += (days == 1 ? " day" : " days");
				} else if (diff < SmartDueDateFormatter.ONE_MONTH) {
					Long weeks = new Long(diff / SmartDueDateFormatter.ONE_WEEK);
					diffString = weeks.toString();
					diffString += (weeks == 1 ? " week" : " weeks");
				} else if (diff < SmartDueDateFormatter.ONE_YEAR) {
					Long months = new Long(diff / SmartDueDateFormatter.ONE_MONTH);
					diffString = months.toString();
					diffString += (months == 1 ? " month" : " months");
				} else {
					Long years = new Long(diff / SmartDueDateFormatter.ONE_YEAR);
					diffString = years.toString();
					diffString += (years == 1 ? " year" : " years");
				}
				return diffString + " overdue";
			} else { // due in the future
				String diffString;
				if (diff < SmartDueDateFormatter.ONE_HOUR) {
					// Due in less than an hour
					// => "Due in 42 minutes"
					long minutes = diff / SmartDueDateFormatter.ONE_MINUTE;
					diffString = "Due in " + new Long(minutes).toString();
					diffString += (minutes == 1 ? " minute" : " minutes");
				} else if (diff < SmartDueDateFormatter.ONE_DAY
						&& date.get(Calendar.DAY_OF_MONTH) == now.get(Calendar.DAY_OF_MONTH)) {
					// Due before midnight tonight
					// => "Due today at 5:00"
					diffString =
						"Due today at " + new SimpleDateFormat("h:mm").format(date.getTime());
				} else if (diff < 2 * SmartDueDateFormatter.ONE_DAY
						&& date.get(Calendar.DAY_OF_MONTH) == now.get(Calendar.DAY_OF_MONTH) + 1) {
					// Due after midnight tonight but before midnight tomorrow
					// night
					// => "Due tomorrow at 5:00"
					diffString =
						"Due tomorrow at " + new SimpleDateFormat("h:mm").format(date.getTime());
				} else if (diff < SmartDueDateFormatter.ONE_WEEK) {
					// Due after tomorrow, but in less than a week
					// => "Due this Thursday"
					diffString = "Due this " + new SimpleDateFormat("EEEE").format(date.getTime());
				} else if (diff < SmartDueDateFormatter.ONE_YEAR) {
					// Due in more than a week, but less than a year
					// => "Due Aug 2"
					diffString = "Due " + new SimpleDateFormat("MMM d").format(date.getTime());
				} else {
					// Due in more than a year
					// => "Due Aug 2, 2009"
					diffString =
						"Due " + new SimpleDateFormat("MMM d, yyyy").format(date.getTime());
				}
				return diffString;
			}
		}
	}

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = 894751782165268552L;

	/** The name or title of the task. */
	private String name;

	/** True if the task is marked as completed, false otherwise. */
	private boolean isComplete;

	/**
	 * The date and time the task was most recently marked as complete, or \c null if it has not
	 * been.
	 */
	private Calendar dateCompleted;

	/** True if the task is starred, false otherwise. */
	private boolean isStarred;

	/** The set of tags associated with this task. */
	private HashSet<String> tags;

	/**
	 * The date on which this task is due, or \c null if the task has no due date.
	 */
	private Calendar dueDate;

	/** The project with which this task is associated, if any. */
	private Project project;

	/**
	 * The priority value for this task. This value influences the task's sort order.
	 */
	private double priority;

	/**
	 * The pending state of the task. If the task is not marked as pending, the PendingState
	 * object's isPending() method will return false. Otherwise, it will return true and more
	 * information about the pending state of the task can be found by querying this object.
	 */
	private PendingState pendingState;

	/** The list of Resources associated with this task. */
	private ResourceList resources;

	/**
	 * Constructs an Task with a name
	 * 
	 * @param name the name of this task
	 * @param caloAdded whether or not this task was AI added
	 */
	public Task(String name, boolean caloAdded) {
		if (name == null) {
			throw new IllegalArgumentException("Cannot add a task with a null name.");
		}
		this.name = name;
		this.setCaloAdded(caloAdded);
		this.setDateAdded(Calendar.getInstance());
		this.tags = new HashSet<String>();
		this.isComplete = false;
		this.isStarred = false;

		// Sets a default priority--pending is false and note is null
		this.pendingState = new PendingState();

		/**
		 * @todo This should call into the engine to determine its priority!
		 * 
		 * @see edu.cmu.hcii.calo.model.Task.getPriority()
		 */
		this.priority = 1000000;

		this.resources = new ResourceList();

		// // This is for debugging only!
		// this.resources.add(new Resource(new File("C:/Documents and
		// Settings/Administrator/Desktop/randomTagline.js")));
		// this.resources.add(new Resource(new File("C:/Documents and
		// Settings/Administrator/Desktop/points.txt")));
		// this.dueDate = Calendar.getInstance();

	}

	/**
	 * Adds the given tag to this task.
	 * 
	 * @param o the tag to add
	 * @return true if the tag was not already associated with the task (and is now), false
	 *         otherwise.s
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public boolean addTag(String o) {
		assert this.tags != null : "this task's tags were set to null";
		// This creates a SHALLOW copy:
		Set<String> oldTags = Utility.unchecked_cast(this.tags.clone());
		boolean success = this.tags.add(o);
		if (success) {
			if (this.isManaged()) {
				TaskManager.getManager().noteTaskTagsChanged(this, oldTags, this.tags);
			}
		}
		return success;
	}

	/**
	 * Removes all tags from this task.
	 * 
	 * @see java.util.Set#clear()
	 */
	public void clearTags() {
		assert this.tags != null : "this task's tags were set to null";
		// This creates a SHALLOW copy:
		Set<String> oldTags = Utility.unchecked_cast(this.tags.clone());
		this.tags.clear();
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskTagsChanged(this, oldTags, this.tags);
		}
	}

	/**
	 * Counts the number of tags associated with this task.
	 * 
	 * @return the number of tags associated with the task
	 * @see java.util.Set#size()
	 */
	public int countTags() {
		assert this.tags != null : "this task's tags were set to null";
		return this.tags.size();
	}

	/**
	 * Accessor method for dueDate. Tasks can have no set due date (this is in fact the default), in
	 * which case this method returns \c null.
	 * 
	 * @return the value of dueDate
	 */
	public Calendar getDueDate() {
		return this.dueDate;
	}

	/**
	 * Returns the due date of this Task as an intelligently formatted string.
	 * 
	 * @return the string representation of the due date
	 */
	public String getDueDateString() {
		if (this.dueDate == null) {
			return "";
		}
		return SmartDueDateFormatter.formatDate(this.getDueDate());
	}

	/**
	 * Accessor method for name
	 * 
	 * @return the value of name
	 */
	public String getName() {
		assert this.name != null : "this task's name was null";
		return this.name;
	}

	/**
	 * Accessor method for pendingState
	 * 
	 * @return the value of pendingState
	 */
	public PendingState getPendingState() {
		assert this.pendingState != null : "the pending state of the task was null.";
		return this.pendingState;
	}

	/**
	 * Accessor method for priority
	 * 
	 * @return the value of priority
	 */
	@Override
	public double getPriority() {
		return this.priority;
	}

	/**
	 * Accessor method for project
	 * 
	 * @return the value of project
	 */
	public Project getProject() {
		return this.project;
	}

	/**
	 * Accessor method for resources
	 * 
	 * @return the value of resources
	 */
	public ResourceList getResources() {
		assert this.resources != null : "this task's resources were set to null";
		return this.resources;
	}

	/**
	 * Accessor method for tags
	 * 
	 * @return the value of tags
	 */
	public Set<String> getTags() {
		assert this.tags != null : "this task's tags were set to null";
		return this.tags;
	}

	/**
	 * Accessor method for isComplete
	 * 
	 * @return the value of isComplete
	 */
	public boolean isComplete() {
		return this.isComplete;
	}

	/**
	 * Accessor method for isStarred
	 * 
	 * @return the value of isStarred
	 */
	public boolean isStarred() {
		return this.isStarred;
	}

	/**
	 * Removes the given tag from this task.
	 * 
	 * @param o the tag to remove
	 * @return true if the tag was previously associated with the task (and is no longer), false
	 *         otherwise
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public boolean removeTag(Object o) {
		assert this.tags != null : "this task's tags were set to null";
		// This creates a SHALLOW copy:
		Set<String> oldTags = Utility.unchecked_cast(this.tags.clone());
		boolean success = this.tags.remove(o);
		if (success) {
			if (this.isManaged()) {
				TaskManager.getManager().noteTaskTagsChanged(this, oldTags, this.tags);
			}
		}
		return success;
	}

	/**
	 * Setter method for isComplete.
	 * 
	 * @param isComplete the isComplete to set
	 */
	public void setComplete(boolean isComplete) {
		boolean wasComplete = this.isComplete;
		this.isComplete = isComplete;
		if (this.isManaged()) {
			if (this.isComplete != wasComplete) {
				TaskManager.getManager().noteCompletenessChanged(this, wasComplete, this.isComplete);
			}
		}
	}

	/**
	 * Setter method for dueDate. Pass to \c null to indicate that this task has no set due date.
	 * 
	 * @param dueDate the dueDate to set, or \c null for no due date
	 */
	public void setDueDate(Calendar dueDate) {
		Calendar oldDueDate = this.dueDate;
		this.dueDate = dueDate;
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskDueDateChanged(this, oldDueDate, this.dueDate);
		}
	}

	/**
	 * Setter method for name.
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		if (name == null) {
			throw new IllegalArgumentException("Task names cannot be null.");
		}
		String oldName = this.name;
		this.name = name;
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskNameChanged(this, oldName, this.name);
		}
	}

	/**
	 * Setter method for pendingState.
	 * 
	 * @param pendingState the pendingState to set
	 */
	public void setPendingState(PendingState pendingState) {
		if (pendingState == null) {
			throw new IllegalArgumentException("Pending states cannot be null.");
		}
		PendingState oldPendingState = this.pendingState;
		this.pendingState = pendingState;
		if (this.isManaged() && !this.pendingState.equals(oldPendingState)) {
			TaskManager.getManager().noteTaskPendingStateChanged(	this,
																	oldPendingState,
																	this.pendingState);
		}
	}

	/**
	 * Setter method for priority.
	 * 
	 * @param priority the priority to set
	 */
	public void setPriority(double priority) {
		double oldPriority = this.priority;
		this.priority = priority;
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskPriorityChanged(this, oldPriority, this.priority);
		}
	}

	/**
	 * Setter method for project.
	 * 
	 * @param project the project to set
	 */
	public void setProject(Project project) {
		Project oldProject = this.project;
		this.project = project;
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskProjectChanged(this, oldProject, this.project);
		}
	}

	/**
	 * Setter method for resources.
	 * 
	 * @param resources the resources to set
	 */
	public void setResources(ResourceList resources) {
		if (resources == null) {
			throw new IllegalArgumentException("Resource lists cannot be null.");
		}
		ResourceList oldResources = this.resources;
		this.resources = resources;
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskResourcesChanged(this, oldResources, this.resources);
		}
	}

	//
	// @SuppressWarnings("unchecked")
	// @Override
	// public Object clone() throws CloneNotSupportedException {
	// Task t = new Task(this.name, this.getCaloAdded());
	// t.isComplete = this.isComplete;
	// t.isStarred = this.isStarred;
	// t.dueDate = (Calendar)this.dueDate.clone();
	// t.pendingState = (PendingState)this.pendingState.clone();
	// t.priority = this.priority;
	// t.project = this.project;
	// t.resources = (ResourceList)this.resources.clone();
	// t.tags = (HashSet<String>)this.tags.clone();
	// return t;
	// }

	/**
	 * Setter method for isStarred.
	 * 
	 * @param isStarred the isStarred to set
	 */
	public void setStarred(boolean isStarred) {
		boolean wasStarred = this.isStarred;
		this.isStarred = isStarred;
		if (this.isManaged()) {
			if (this.isStarred != wasStarred) {
				TaskManager.getManager().noteStarrednessChanged(this, wasStarred, this.isStarred);
			}
		}
	}

	/**
	 * Returns a (partial) string representation of this task. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return this.getName() + " (priority " + this.getPriority() + ") (pending "
				+ this.getPendingState().isPending() + ") (completed " + this.isComplete + ")";
	}

	/**
	 * Returns the date and time this Task was most recently marked as completed, or \c null if it
	 * has never been marked as completed.
	 * 
	 * @note This method may return a non-null value even if the task is not currently marked as
	 *       complete, so be sure to check isComplete() before assuming a non-null response means
	 *       the task is complete.
	 * @return the date and time this task was completed
	 */
	public Calendar getDateCompleted() {
		return this.dateCompleted;
	}

	/**
	 * Sets the completed date of this task to the given value.
	 * 
	 * @param dateCompleted the date and time this task was completed
	 */
	public void setDateCompleted(Calendar dateCompleted) {
		Calendar oldDateCompleted = this.dateCompleted;
		this.dateCompleted = dateCompleted;
		if (this.isManaged()) {
			if (this.dateCompleted != oldDateCompleted) {
				TaskManager.getManager().noteDateCompletedChanged(	this,
																	oldDateCompleted,
																	this.dateCompleted);
			}
		}
	}

}
