/* File: TaskGroup.java
 *
 * Contains class TaskGroup
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model;

import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.util.Utility;

/**
 * A group of tasks. The group has a priority based on its component subtasks as
 * computed by the engine.
 * 
 * @author Brian Ellis
 */
public class TaskGroup extends TasklikeObject {

	/** To prevent serialization collisions. */
	private static final long serialVersionUID = -1214130792328705063L;

	/**
	 * The list of tasks of this group, if any. If the task has no tasks, this
	 * list should be empty but non-null.
	 */
	private LinkedList<Task> tasks;

	/**
	 * Constructs a new TaskGroup.
	 */
	public TaskGroup() {
		this.tasks = new LinkedList<Task>();
	}

	/**
	 * Appends a Task onto the end of this task's tasks.
	 * 
	 * @param task
	 *            the Task to add
	 * @return true if the task was not already there, false otherwise
	 * @see java.util.List#add(java.lang.Object)
	 */
	public boolean appendTask(Task task) {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		boolean success = this.tasks.add(task);
		if (success) {
			if (this.isManaged()) {
				TaskManager.getManager().noteTaskGroupTasksChanged(this,
						oldTasks, this.tasks);
			}
		}
		return success;
	}

	/**
	 * Counts the number of tasks of this task.
	 * 
	 * @return the number of tasks of this task, with 0 meaning the task has no
	 *         tasks
	 * @see java.util.List#size()
	 */
	public int countTasks() {
		assert this.tasks != null : "this task group's tasks were set to null";
		return this.tasks.size();
	}

	/**
	 * Gets the date added of this TaskGroup. The "date added" of a TaskGroup is
	 * defined to be the date on which the oldest task in the group was added.
	 * 
	 * @see edu.cmu.hcii.calo.model.ManagedObject#getDateAdded()
	 */
	@Override
	public Calendar getDateAdded() {
		Calendar minDate = Calendar.getInstance();
		for (Task thisTask : this.tasks) {
			if (thisTask.getDateAdded().before(minDate)) {
				minDate = thisTask.getDateAdded();
			}
		}
		return minDate;
	}

	/**
	 * Returns the priority of this group, as computed by CALO.
	 * 
	 * @todo This should call into the engine to determine its priority!
	 * 
	 * @see edu.cmu.hcii.calo.model.Prioritizable#getPriority()
	 */
	@Override
	public double getPriority() {
		return -1;
	}

	/**
	 * Gets the Task at the given index.
	 * 
	 * @param index
	 *            the index to get
	 * @return the Task at that index
	 * @throws IndexOutOfBoundsException
	 *             if the index is invalid
	 * @see java.util.List#get(int)
	 */
	public Task getTaskAt(int index) {
		assert this.tasks != null : "this task group's tasks were set to null";
		return this.tasks.get(index);
	}

	/**
	 * Accessor method for tasks
	 * 
	 * @return the value of tasks
	 */
	public List<Task> getTasks() {
		assert this.tasks != null : "this task group's tasks were set to null";
		return this.tasks;
	}

	/**
	 * Returns the index of the given Task, or -1 if it's not in the list.
	 * 
	 * @param task
	 *            the Task to look for
	 * @return the index, or -1 if the Task was not found
	 * @see java.util.List#indexOf(java.lang.Object)
	 */
	public int indexOfTask(Task task) {
		assert this.tasks != null : "this task group's tasks were set to null";
		return this.tasks.indexOf(task);
	}

	/**
	 * Inserts a Task of this task at a given index.
	 * 
	 * @param index
	 *            the index to insert the Task at
	 * @param task
	 *            the Task to insert
	 * @see java.util.List#add(int, java.lang.Object)
	 */
	public void insertTask(int index, Task task) {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		this.tasks.add(index, task);
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskGroupTasksChanged(this, oldTasks,
					this.tasks);
		}
	}

	/**
	 * Removes all tasks from this task.
	 * 
	 * @see java.util.List#clear()
	 */
	public void removeAllTasks() {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		this.tasks.clear();
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskGroupTasksChanged(this, oldTasks,
					this.tasks);
		}
	}

	/**
	 * Removes the given Task from this task's Task list, if it was present.
	 * 
	 * @param task
	 *            the Task to remove
	 * @return true if this Task existed (and was removed), false otherwise
	 * @see java.util.List#remove(java.lang.Object)
	 */
	public boolean removeTask(Task task) {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		boolean success = this.tasks.remove(task);
		if (success) {
			if (this.isManaged()) {
				TaskManager.getManager().noteTaskGroupTasksChanged(this,
						oldTasks, this.tasks);
			}
		}
		return success;
	}

	/**
	 * Removes the Task of this task at the given index.
	 * 
	 * @param index
	 *            the index of the Task to remove
	 * @return the removed Task
	 * @throws IndexOutOfBoundsException
	 *             if the index is invalid
	 * @see java.util.List#remove(int)
	 */
	public Task removeTaskAt(int index) {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		Task removed = this.tasks.remove(index);
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskGroupTasksChanged(this, oldTasks,
					this.tasks);
		}
		return removed;
	}

	/**
	 * Replaces the Task of this task at the given index with the new Task.
	 * 
	 * @param index
	 *            the index of the Task to remove
	 * @param newTask
	 *            the Task to replace it with
	 * @return the removed Task
	 * @throws IndexOutOfBoundsException
	 *             if the index is invalid
	 * @see java.util.List#set(int, java.lang.Object)
	 */
	public Task replaceTaskAt(int index, Task newTask) {
		assert this.tasks != null : "this task group's tasks were set to null";
		// This creates a SHALLOW copy of the list:
		List<Task> oldTasks = Utility.unchecked_cast(this.tasks.clone());
		Task replaced = this.tasks.set(index, newTask);
		if (this.isManaged()) {
			TaskManager.getManager().noteTaskGroupTasksChanged(this, oldTasks,
					this.tasks);
		}
		return replaced;
	}

	/**
	 * Gets an iterator over this task's tasks.
	 * 
	 * @return the iterator
	 * @see java.util.List#iterator()
	 */
	public Iterator<Task> TaskIterator() {
		assert this.tasks != null : "this task group's tasks were set to null";
		return this.tasks.iterator();
	}

}
