/* File: MutableSortedSet.java
 *
 * Contains class MutableSortedSet
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.model.sorting;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import edu.cmu.hcii.calo.util.Utility;

/**
 * Implements a sorted set that allows sorting on any arbitrary key that can be derived from the
 * data of the objects in the set.
 * 
 * Rather than taking a Comparator or objects implementing Comparable, the mutable sorted set takes
 * a KeyFunctor object that is used to generate sort keys from the data on the fly.
 * 
 * The existing TreeSet class only sorts objects when they are initially added, and any subsequent
 * mutation of any data members used in the Comparator used to sort the objects will put the Set in
 * an undefined state. (This is what we call a Bad Thing.) MutableSortedSet overcomes this by
 * computing the sort key when the object is added to the set, and then storing the key apart from
 * the object. Thus, the objects in the set will remain sorted based on their state at the time they
 * were added to the set, regardless of any subsequent mutations. Additionally, an item can be
 * re-sorted at any time by calling MutableSortedSet.resort(E), which will remove the object, then
 * re-add it with a re-computed key.
 * 
 * As far as efficiency, MutableSortedSet is actually somewhat faster than TreeSet for some
 * operations, including contains(Object), and has the same time complexity as TreeSet for all other
 * operations.
 * 
 * @author Brian Ellis
 * 
 */
public class MutableSortedSet<E> implements SortedSet<E> {

	/**
	 * Represents an individual node in the sorted set.
	 * 
	 * @author Brian Ellis
	 */
	private class SetNode<T> {

		/**
		 * The cached key value for this node.
		 */
		private double nodeKey;

		/**
		 * The data for this node.
		 */
		private T nodeData;

		/**
		 * Constructs a node with the given key and data.
		 * 
		 * @param key the key
		 * @param data the data
		 */
		public SetNode(double key, T data) {
			this.nodeKey = key;
			this.nodeData = data;
		}

		/**
		 * Returns the data for this node.
		 * 
		 * @return the data
		 */
		public T getData() {
			return this.nodeData;
		}

		/**
		 * Returns the cached key for this node.
		 * 
		 * @return the key
		 */
		public double getKey() {
			return this.nodeKey;
		}

		/**
		 * Sets the key for this node to the new key.
		 * 
		 * @param newKey the new key
		 */
		public void setKey(double newKey) {
			this.nodeKey = newKey;
		}

		/**
		 * Returns a string representation of this node. Useful for debugging.
		 */
		@Override
		public String toString() {
			return "Node: " + this.getKey() + " = " + this.getData();
		}
	}

	/**
	 * The sorted treeset containing the set nodes.
	 */
	TreeSet<SetNode<E>> nodes;

	/**
	 * The functor we use to generate a sort key from each object in the set.
	 */
	KeyFunctor<E> functor;

	/**
	 * The mapping between the items being managed in the list and the nodes containing them.
	 */
	private HashMap<E, SetNode<E>> mappings;

	/**
	 * Constructs a MutableSortedSet with the given KeyFunctor.
	 * 
	 * @param func the functor that will be used to generate keys when objects are added
	 */
	@SuppressWarnings("unchecked")
	public MutableSortedSet(KeyFunctor<E> func) {
		this.nodes = new TreeSet<SetNode<E>>(this.internalComparator());
		this.mappings = new HashMap<E, SetNode<E>>();
		this.functor = func;
	}

	/**
	 * @see java.util.Set#add(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public synchronized boolean add(Object o) {
		if (this.mappings.containsKey(o)) {
			return false;
		}
		E e = (E)o;
		SetNode<E> newNode = new SetNode<E>(this.functor.getKey(e), e);
		this.mappings.put(e, newNode);
		return this.nodes.add(newNode);
	}

	/**
	 * @see java.util.Set#addAll(java.util.Collection)
	 */
	public synchronized boolean addAll(Collection<? extends E> c) {

		boolean success = true;
		for (Object object : c) {
			success &= this.add(object);
		}
		return success;
	}

	/**
	 * @see java.util.Set#clear()
	 */
	public synchronized void clear() {
		this.nodes.clear();
	}

	/**
	 * Returns the comparator this set will use to compare nodes in the set.
	 * 
	 * @see SortedSet#comparator()
	 */

	public Comparator<E> comparator() {
		return new Comparator<E>() {

			public int compare(E o1, E o2) {
				return internalComparator().compare(new SetNode<E>(	MutableSortedSet.this.functor.getKey(o1),
																	o2),
													new SetNode<E>(	MutableSortedSet.this.functor.getKey(o2),
																	o2));
			}
		};
	}

	/**
	 * @see java.util.Set#contains(java.lang.Object)
	 */
	public boolean contains(Object o) {
		return (this.mappings.containsKey(o));
	}

	/**
	 * @see java.util.Set#containsAll(java.util.Collection)
	 */
	public boolean containsAll(Collection<?> c) {
		for (Object object : c) {
			if (!(this.contains(object))) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @see java.util.SortedSet#first()
	 */
	public E first() {
		return this.nodes.first().getData();
	}

	/**
	 * Gets the given item in this set if it exists. Otherwise, return null.
	 * 
	 * @param item the item to get
	 * 
	 * @return the item if it is found, null otherwise
	 */
	public E get(E item) {
		return Utility.unchecked_cast(this.mappings.get(item).getData());
	}

	/**
	 * @see java.util.SortedSet#headSet(java.lang.Object)
	 */
	public SortedSet<E> headSet(Object toElement) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.util.Set#isEmpty()
	 */
	public boolean isEmpty() {
		return this.nodes.isEmpty();
	}

	/**
	 * @see java.util.Set#iterator()
	 */
	public Iterator<E> iterator() {
		return new Iterator<E>() {

			private Iterator<SetNode<E>> privIter = MutableSortedSet.this.nodes.iterator();

			public boolean hasNext() {
				return this.privIter.hasNext();
			}

			@SuppressWarnings("unchecked")
			public E next() {
				SetNode nextNode = this.privIter.next();
				return (E)nextNode.getData();
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}

		};
	}

	/**
	 * @see java.util.SortedSet#last()
	 */
	public E last() {
		return this.nodes.last().getData();
	}

	/**
	 * @see java.util.Set#remove(java.lang.Object)
	 */
	public synchronized boolean remove(Object o) {
		SetNode<E> setNode = this.mappings.get(o);
		assert setNode != null : "could not find mapping for this object";
		this.mappings.remove(o);
		boolean success = this.nodes.remove(setNode);
		return success;
	}

	/**
	 * @see java.util.Set#removeAll(java.util.Collection)
	 */
	public synchronized boolean removeAll(Collection<?> c) {
		boolean success = true;
		for (Object object : c) {
			success &= this.remove(object);
		}
		return success;
	}

	/**
	 * Re-sorts the given item into the set, updating its sort key based on its current state.
	 * 
	 * @param item the item to re-sort
	 */
	public synchronized void resort(E item) {
		SetNode<E> node = this.mappings.get(item);
		assert node != null;
		this.mappings.remove(item);
		boolean success = this.nodes.remove(node);
		assert success;
		success = this.add(item);
		assert success;
	}

	/**
	 * @see java.util.Set#retainAll(java.util.Collection)
	 */
	public synchronized boolean retainAll(Collection<?> c) {

		boolean success = true;
		for (SetNode<E> node : this.nodes) {
			if (!(c.contains(node.getData()))) {
				success &= this.nodes.remove(node);
			}
		}
		return success;
	}

	/**
	 * @see java.util.Set#size()
	 */
	public int size() {
		return this.nodes.size();
	}

	/**
	 * @see java.util.SortedSet#subSet(java.lang.Object, java.lang.Object)
	 */
	public SortedSet<E> subSet(E fromElement, E toElement) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.util.SortedSet#tailSet(java.lang.Object)
	 */
	public SortedSet<E> tailSet(E fromElement) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.util.Set#toArray()
	 */
	public synchronized Object[] toArray() {
		Object[] array = new Object[this.nodes.size()];
		int i = 0;
		for (SetNode<E> element : this.nodes) {
			array[i] = element.getData();
			++i;
		}
		return array;
	}

	/**
	 * @see java.util.Set#toArray(T[])
	 */
	@SuppressWarnings("unchecked")
	public Object[] toArray(Object[] a) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Returns a comparator over set nodes.
	 * 
	 * @return a comparator over set nodes.
	 */
	Comparator<SetNode<E>> internalComparator() {
		return new Comparator<SetNode<E>>() {

			public int compare(SetNode<E> o1, SetNode<E> o2) {
				E e1 = o1.getData();
				double k1 = o1.getKey();
				E e2 = o2.getData();
				double k2 = o2.getKey();

				if (e1.equals(e2)) {
					return 0;
				} else if (k1 > k2) {
					return -1;
				} else if (k1 == k2) {
					return e1.hashCode() - e2.hashCode();
				} else {
					return 1;
				}
			}

		};
	}

}
