/* File: IPAddressFinder.java
 *
 * Contains class IPAddressFinder
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.thinkaloud;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A simple standalone application class that tries to determine the best IP address to use to
 * connect the Wizard of Oz front-end to the sidebar running on another machine.
 * 
 * @author Brian Ellis
 * 
 */
public class IPAddressFinder extends JFrame {

	/**
	 * Prevents serialization collisions.
	 */
	private static final long serialVersionUID = -8860770222463693065L;

	/**
	 * Main method. Run me to launch the IP Address Finder.
	 * 
	 * @param args command-line args, none of which do anything special
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		new IPAddressFinder().setVisible(true);
	}

	/**
	 * Makes a new IPAddressFinder.
	 */
	public IPAddressFinder() {
		this.setTitle("IP Address Finder");
		this.setLayout(new RelativeLayout());

		JLabel explanation =
			new JLabel("<html><p>Some possible IP addresses for this computer are listed below.  "
					+ "It is a good idea to try these <strong>in the order listed</strong>, "
					+ "since care has been taken to show more likely candidates first.</p></html>");

		List<String> ipAddrs = this.getIpAddresses();

		JList ips = new JList(ipAddrs.toArray());
		ips.setFont(ips.getFont().deriveFont(18.0f));
		this.add(explanation, new RelativeConstraints(	new Binding(Edge.TOP,
																	10,
																	Direction.BELOW,
																	Edge.TOP,
																	null),
														new Binding(Edge.LEFT,
																	20,
																	Direction.RIGHT,
																	Edge.LEFT,
																	null),
														new Binding(Edge.RIGHT,
																	20,
																	Direction.LEFT,
																	Edge.RIGHT,
																	null)));
		this.add(new JScrollPane(ips), new RelativeConstraints(	new Binding(Edge.TOP,
																			8,
																			Direction.BELOW,
																			Edge.BOTTOM,
																			explanation),
																new Binding(Edge.BOTTOM,
																			20,
																			Direction.ABOVE,
																			Edge.BOTTOM,
																			null),
																new Binding(Edge.LEFT,
																			20,
																			Direction.RIGHT,
																			Edge.LEFT,
																			null),
																new Binding(Edge.RIGHT,
																			20,
																			Direction.LEFT,
																			Edge.RIGHT,
																			null)));

		this.setSize(512, 384);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * Gets a list of all potential IP addresses for this machine, based on everything from
	 * interrogating the routing tables to loading whatismyip.com and screen-scraping the HTML to
	 * get its opinion of our external IP.
	 * 
	 * Also displays a nice little dialog while we hunt for addresses.
	 * 
	 * @return the list of IPs
	 */
	private List<String> getIpAddresses() {
		JFrame holdOnDialog = new JFrame("Please Hold On");
		JLabel label =
			new JLabel("Please hold on while I look for IP addresses for this computer...");
		label.setBorder(new EmptyBorder(20, 20, 20, 20));
		holdOnDialog.add(label);
		holdOnDialog.pack();
		holdOnDialog.setVisible(true);

		List<String> ret = new ArrayList<String>();

		try {

			InetAddress addr;

			// Add the localhost address first. This is likely to be the local
			// LAN IP, which is
			// perfect for our use. If it's loopback, though, we shouldn't print
			// it.
			addr = InetAddress.getLocalHost();
			if (!addr.isLoopbackAddress()) {
				ret.add(addr.getHostAddress());
			}

			// Now we loop through all the network interfaces looking for other
			// IPs to try, also
			// excluding the loopback address. We also exclude any IPv6
			// addresses.
			Enumeration<NetworkInterface> ints = NetworkInterface.getNetworkInterfaces();
			while (ints.hasMoreElements()) {
				NetworkInterface thisInterface = ints.nextElement();
				Enumeration<InetAddress> addys = thisInterface.getInetAddresses();
				while (addys.hasMoreElements()) {
					InetAddress addy = addys.nextElement();
					if (addy.equals(addr)) {
						continue;
					}
					if (addy.getAddress().length == 4 && !addy.isLoopbackAddress()) {
						String hostAddress = addy.getHostAddress();
						ret.add(hostAddress);
					}
				}
			}

			// Lastly, we're going to go out to the Net and look for our
			// Internet IP address. This
			// will be the IP address used by all the machines on our local
			// subnet if we're behind a
			// router, and therefore it may not work, but in some cases it may
			// be more useful (if
			// we're on a different subnet, for instance).

			URLConnection conn = new URL("http://www.whatismyip.com/").openConnection();
			InputStream stream = conn.getInputStream();
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(stream));
			String line = bufferedReader.readLine();
			while (line != null) {
				Pattern p = Pattern.compile(".*displaycopy\\('(\\d+\\.\\d+\\.\\d+\\.\\d+)'\\).*");
				Matcher m = p.matcher(line);
				if (m.matches()) {
					String ip = m.group(1);
					ret.add(ip);
				}
				line = bufferedReader.readLine();
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			holdOnDialog.setVisible(false);
		}

		return ret;

	}

}
