/* File: WOZFrontEndFrame.java
 *
 * Contains class WOZFrontEndFrame
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.thinkaloud;

import java.awt.Component;
import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.text.JTextComponent;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;
import edu.cmu.relativelayout.RelativePosition;

/**
 * The frame for the Wizard front-end. The main function that creates this is in WizardOfOzFrontEnd.
 * 
 * @author Brian Ellis
 * 
 */
public class WOZFrontEndFrame extends JFrame {

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = 8225789452284206486L;

	/**
	 * Sets the opacity of all subcomponents when the opacity of the given component changes.
	 * 
	 * @param comp the component whose opacity is changing
	 * @param enable whether the component should be opaque
	 */
	public static void setOpaque(JComponent comp, boolean enable) {
		comp.setOpaque(enable);
		Component[] comps = comp.getComponents();
		for (Component cc : comps) {
			if (cc instanceof JComponent && !(cc instanceof JTextComponent)
					&& !(cc instanceof JList)) {
				WOZFrontEndFrame.setOpaque((JComponent)cc, enable);
			}
		}
	}

	/**
	 * The console text area.
	 */
	private JTextArea console;

	/**
	 * The task panel.
	 */
	private WOZFrontEndTaskPanel taskPanel;

	/**
	 * The tab panel containing the Tasks and Notifications tabs.
	 */
	private JTabbedPane tabs;

	/**
	 * The front-end to which this frame reports.
	 */
	private WizardOfOzFrontEnd frontEnd;

	/**
	 * The notification panel.
	 */
	private WOZFrontEndNotificationPanel notificationPanel;

	/**
	 * Constructs a WOZFrontEndFrame attached to the given front-end.
	 * 
	 * @param frontEnd the front-end this is attached to
	 * @throws HeadlessException if there's no display to show this on
	 */
	public WOZFrontEndFrame(WizardOfOzFrontEnd frontEnd) throws HeadlessException {
		super("CALO Wizard Front-End");
		this.frontEnd = frontEnd;
		this.setup();
	}

	/**
	 * Constructs a WOZFrontEndFrame in a very badly specified state. Don't call this. That's why
	 * it's private.
	 * 
	 * @param arg0 who knows?
	 */
	@SuppressWarnings("unused")
	private WOZFrontEndFrame(GraphicsConfiguration arg0) {
		super(arg0);
		this.setup();
	}

	/**
	 * Constructs a WOZFrontEndFrame in a very badly specified state. Don't call this. That's why
	 * it's private.
	 * 
	 * @param arg0 beats us
	 * @throws HeadlessException
	 */
	@SuppressWarnings("unused")
	private WOZFrontEndFrame(String arg0) throws HeadlessException {
		super(arg0);
		this.setup();
	}

	/**
	 * Constructs a WOZFrontEndFrame in a very badly specified state. Don't call this. That's why
	 * it's private.
	 * 
	 * @param arg0 something
	 * @param arg1 something else
	 */
	@SuppressWarnings("unused")
	private WOZFrontEndFrame(String arg0, GraphicsConfiguration arg1) {
		super(arg0, arg1);
		this.setup();
	}

	/**
	 * Gets the console text area.
	 * 
	 * @return the console
	 */
	public JTextArea getConsole() {
		return this.console;
	}

	/**
	 * Gets the notification list.
	 * 
	 * @return the notification list
	 */
	public JList getNotificationList() {
		return this.notificationPanel.getNotificationList();
	}

	/**
	 * Gets the task list.
	 * 
	 * @return the task list
	 */
	public JList getTaskList() {
		return this.taskPanel.getTaskList();
	}

	/**
	 * Sets which notification is selected in the notification list.
	 * 
	 * @param note the notification to be selected
	 */
	public void setSelectedNotification(Notification note) {
		this.notificationPanel.setSelectedNotification(note);
	}

	/**
	 * Sets which task is selected in the notification list.
	 * 
	 * @param task the task to be selected
	 */
	public void setSelectedTask(Task task) {
		this.taskPanel.setSelectedTask(task);
	}

	/**
	 * Sets up the visual appearance of the frame.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		this.taskPanel = new WOZFrontEndTaskPanel(this.frontEnd);
		WOZFrontEndFrame.setOpaque(this.taskPanel, false);
		this.notificationPanel = new WOZFrontEndNotificationPanel(this.frontEnd);
		WOZFrontEndFrame.setOpaque(this.notificationPanel, false);
		this.tabs = new JTabbedPane();
		this.tabs.insertTab("Tasks", null, this.taskPanel, "Tasks are in here.", 0);
		this.tabs.insertTab("Notifications",
							null,
							this.notificationPanel,
							"Notifications are in here.",
							1);

		this.console = new JTextArea(10, 50);
		this.console.setEditable(false);

		Binding TOP_EDGE = new Binding(Edge.TOP, 0, Direction.BELOW, Edge.TOP, null);
		Binding LEFT_EDGE = new Binding(Edge.LEFT, 0, Direction.RIGHT, Edge.LEFT, null);
		Binding RIGHT_EDGE = new Binding(Edge.RIGHT, 0, Direction.LEFT, Edge.RIGHT, null);
		Binding BOTTOM_EDGE = new Binding(Edge.BOTTOM, 0, Direction.ABOVE, Edge.BOTTOM, null);
		RelativePosition ABOVE = new RelativePosition(Edge.BOTTOM, 0, Direction.ABOVE, Edge.TOP);

		// RelativeLayout.setDebug(true);

		JScrollPane scrollPane = new JScrollPane(this.console);
		this.add(this.tabs, new RelativeConstraints(TOP_EDGE,
													LEFT_EDGE,
													RIGHT_EDGE,
													new Binding(ABOVE, scrollPane)));
		this.add(scrollPane, new RelativeConstraints(BOTTOM_EDGE, LEFT_EDGE, RIGHT_EDGE));

	}
}
