/* File: WOZFrontEndNotificationPanel.java
 *
 * Contains class WOZFrontEndNotificationPanel
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.thinkaloud;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.Person;
import edu.cmu.hcii.calo.model.Notification.NotificationType;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.notification.NotificationView;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The notification panel inside the WOZFrontEndFrame.
 * 
 * @author Brian Ellis
 * 
 */
public class WOZFrontEndNotificationPanel extends CALOComponent {

	/**
	 * An action listener subclass that tells the front-end to remotely change the selected
	 * notification when the change button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public final class ChangeButtonActionListener implements ActionListener {

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndNotificationPanel.this.frontEnd.changeSelectedNotification(	WOZFrontEndNotificationPanel.this.nameChangeField.getText(),
																					WOZFrontEndNotificationPanel.this.textChangeField.getText(),
																					new Person(WOZFrontEndNotificationPanel.this.personChangeField.getText()),
																					new Double(WOZFrontEndNotificationPanel.this.priorityChangeField.getText()).doubleValue());
		}

	}

	/**
	 * An action listener subclass that tells the front-end to remotely delete the selected
	 * notification when the delete button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public final class DeleteButtonActionListener implements ActionListener {

		/**
		 * Called when the delete button is clicked.
		 * 
		 * @param e the action event
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndNotificationPanel.this.frontEnd.deleteSelectedNotification();
		}
	}

	/**
	 * An action listener subclass that tells the front-end to remotely add a notification when the
	 * add button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class AddButtonActionListener implements ActionListener {

		/**
		 * @name Controls
		 * 
		 * Controls we need to query when the add button is pressed.
		 */

		// @{
		private JTextField newPriorityField;
		private JTextField newNameField;
		private JTextField newTextField;
		private JTextField newPersonField;
		private JComboBox newTypePopup;

		// @}

		/**
		 * Constructs an AddButtonActionListener with the given controls. These controls will be
		 * queried for their state when the add button is pressed so that the notification that's
		 * added has the right information associated with it.
		 * 
		 * @param newPriorityField the priority field
		 * @param newNameField the name field
		 * @param newTextField the text field
		 * @param newPersonField the person field
		 * @param newTypePopup the type popup
		 */
		public AddButtonActionListener(JTextField newPriorityField,
				JTextField newNameField,
				JTextField newTextField,
				JTextField newPersonField,
				JComboBox newTypePopup) {
			this.newPriorityField = newPriorityField;
			this.newNameField = newNameField;
			this.newTextField = newTextField;
			this.newPersonField = newPersonField;
			this.newTypePopup = newTypePopup;
		}

		/**
		 * Called when the user clicks the Add button.
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndNotificationPanel.this.frontEnd.addNewNotification(	this.newNameField.getText(),
																			this.newTextField.getText(),
																			new Person(this.newPersonField.getText()),
																			new Double(this.newPriorityField.getText()).doubleValue(),
																			(NotificationType)this.newTypePopup.getSelectedItem());
		}
	}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 5789542579865178675L;

	/**
	 * The width of the labels on the text fields.
	 */
	private static final int LABEL_WIDTH = 75;

	/**
	 * The width of the text fields.
	 */
	private static final int TEXT_FIELD_WIDTH = 34;

	/**
	 * The width of the buttons.
	 */
	private static final int BUTTON_WIDTH = 122;

	/**
	 * @name Controls
	 * 
	 * Swing controls that exist in the panel.
	 */

	// @{
	private JTextField nameChangeField;
	private JTextField textChangeField;
	private JTextField personChangeField;

	private JTextField priorityChangeField;
	private JPanel newNotificationPanel;

	private JPanel changeNotificationPanel;

	private JList notificationList;
	private JPanel notificationViewPanel;
	private JButton deleteButton;

	private JButton addButton;
	private JButton changeButton;

	// @}

	/**
	 * The front end this panel reports to.
	 */
	private WizardOfOzFrontEnd frontEnd;

	/**
	 * A binding maker for layout purposes.
	 */
	private BindingMaker bm;

	/**
	 * Construct a WOZFrontEndNotificationPanel attached to the current front-end.
	 * 
	 * @param frontEnd the front-end to attach to
	 * 
	 */
	public WOZFrontEndNotificationPanel(WizardOfOzFrontEnd frontEnd) {
		this.frontEnd = frontEnd;
		this.bm = new BindingMaker(4, 4, 4, 4, 4, 4);
		this.setup();
	}

	/**
	 * Gets the notification list.
	 * 
	 * @return the notification list
	 */
	public JList getNotificationList() {
		return this.notificationList;
	}

	/**
	 * Sets the selected notification to that given.
	 * 
	 * @param note the new selected notification
	 */
	public void setSelectedNotification(Notification note) {
		WOZFrontEndNotificationPanel.this.changeNotificationPanel.setEnabled(note != null);
		WOZFrontEndNotificationPanel.this.notificationViewPanel.removeAll();

		if (note != null) {

			NotificationView notificationView = new NotificationView(note);
			notificationView.setEnabled(false);
			WOZFrontEndNotificationPanel.this.notificationViewPanel.add(notificationView,
																		new RelativeConstraints(this.bm.directBottomEdge(),
																								this.bm.directLeftEdge(),
																								this.bm.directRightEdge()));

			this.populateChangeNotificationPanel(note);

			this.validate();
			this.repaint();

		}

	}

	/**
	 * Gets a pair of RelativeConstraints objects for a label and a text field that will lay them
	 * out in the appropriate spots.
	 * 
	 * @param label the label for this pair
	 * @param field the field for this pair
	 * @param previousLineField the field from the previous line, or \c null if this is the first
	 *            line
	 * @param stretch \c true if the field should stretch all the way to the edge of the window, \c
	 *            false otherwise
	 * @return a pair of RelativeConstraints, the first for the label, the second for the field
	 */
	private RelativeConstraints[] constraintsForLabelPair(Component label,
			Component field,
			Component previousLineField,
			boolean stretch) {
		RelativeConstraints labelConstraints = new RelativeConstraints(this.bm.leftEdge());
		if (field == null) {
			labelConstraints.addBinding(this.bm.below(previousLineField));
		} else {
			labelConstraints.addBinding(this.bm.verticallyCenterAlign(field));
		}

		RelativeConstraints fieldConstraints = new RelativeConstraints(this.bm.rightOf(label));
		if (previousLineField == null) {
			fieldConstraints.addBinding(this.bm.topEdge());
		} else {
			fieldConstraints.addBinding(this.bm.below(previousLineField));
		}
		if (stretch) {
			fieldConstraints.addBinding(new Binding(Edge.RIGHT,
													20,
													Direction.LEFT,
													Edge.RIGHT,
													null));
		}

		return new RelativeConstraints[] { labelConstraints, fieldConstraints };
	}

	/**
	 * Creates and returns the "Change Notification" panel.
	 * 
	 * @return the "Change Notification" panel
	 */
	private JPanel createChangeNotificationPanel() {
		@SuppressWarnings("hiding")
		JPanel changeNotificationPanel = new JPanel(new RelativeLayout());

		JLabel nameLabel = new JLabel("Name:");
		JLabel textLabel = new JLabel("Text:");
		JLabel personLabel = new JLabel("Person:");
		JLabel priorityLabel = new JLabel("Priority:");
		this.nameChangeField =
			new JTextField("New Notification", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		this.textChangeField =
			new JTextField("New Text", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		this.personChangeField =
			new JTextField("New Person", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		this.priorityChangeField =
			new JTextField("100", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);

		this.resizeToFit(nameLabel);
		this.resizeToFit(textLabel);
		this.resizeToFit(personLabel);
		this.resizeToFit(priorityLabel);

		this.changeButton = new JButton("Change Notification");
		this.changeButton.addActionListener(new ChangeButtonActionListener());

		this.deleteButton = new JButton("Delete Notification");
		this.deleteButton.setPreferredSize(new Dimension(	WOZFrontEndNotificationPanel.BUTTON_WIDTH,
															this.deleteButton.getPreferredSize().height));
		this.deleteButton.addActionListener(new DeleteButtonActionListener());

		RelativeConstraints[] nameConstraints =
			this.constraintsForLabelPair(nameLabel, this.nameChangeField, null, false);
		RelativeConstraints[] textConstraints =
			this.constraintsForLabelPair(	textLabel,
											this.textChangeField,
											this.nameChangeField,
											false);
		RelativeConstraints[] personConstraints =
			this.constraintsForLabelPair(	nameLabel,
											this.personChangeField,
											this.textChangeField,
											false);
		RelativeConstraints[] priorityConstraints =
			this.constraintsForLabelPair(	priorityLabel,
											this.priorityChangeField,
											this.personChangeField,
											false);

		changeNotificationPanel.add(nameLabel, nameConstraints[0]);
		changeNotificationPanel.add(this.nameChangeField, nameConstraints[1]);

		changeNotificationPanel.add(textLabel, textConstraints[0]);
		changeNotificationPanel.add(this.textChangeField, textConstraints[1]);

		changeNotificationPanel.add(personLabel, personConstraints[0]);
		changeNotificationPanel.add(this.personChangeField, personConstraints[1]);

		changeNotificationPanel.add(priorityLabel, priorityConstraints[0]);
		changeNotificationPanel.add(this.priorityChangeField, priorityConstraints[1]);

		changeNotificationPanel.add(this.deleteButton,
									new RelativeConstraints(new Binding(Edge.LEFT,
																		20,
																		Direction.RIGHT,
																		Edge.LEFT,
																		null),
															this.bm.below(this.priorityChangeField)));
		changeNotificationPanel.add(this.changeButton,
									new RelativeConstraints(new Binding(Edge.RIGHT,
																		20,
																		Direction.LEFT,
																		Edge.RIGHT,
																		null),
															this.bm.below(this.priorityChangeField)));

		changeNotificationPanel.setBorder(BorderFactory.createTitledBorder("Change Notification"));

		return changeNotificationPanel;
	}

	/**
	 * Creates and returns the "New Notification" panel.
	 * 
	 * @return the "New Notification" panel
	 */
	@SuppressWarnings("hiding")
	private JPanel createNewNotificationPanel() {
		JPanel newNotificationPanel = new JPanel(new RelativeLayout());

		JLabel nameLabel = new JLabel("Name:");
		JLabel textLabel = new JLabel("Text:");
		JLabel personLabel = new JLabel("Person:");
		JLabel priorityLabel = new JLabel("Priority:");
		JLabel typeLabel = new JLabel("Type:");
		final JTextField newNameField =
			new JTextField("Buy a Scanner", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		final JTextField newTextField =
			new JTextField(	"You should buy a scanner.",
							WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		final JTextField newPersonField =
			new JTextField("Harry Potter", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		final JTextField newPriorityField =
			new JTextField("100", WOZFrontEndNotificationPanel.TEXT_FIELD_WIDTH);
		final JComboBox newTypePopup =
			new JComboBox(new Object[] { Notification.NotificationType.EMAIL,
					Notification.NotificationType.REMINDER,
					Notification.NotificationType.CALO_ACTION });

		this.addButton = new JButton("Add Notification");
		this.addButton.setPreferredSize(new Dimension(	WOZFrontEndNotificationPanel.BUTTON_WIDTH,
														this.addButton.getPreferredSize().height));
		this.addButton.addActionListener(new AddButtonActionListener(	newPriorityField,
																		newNameField,
																		newTextField,
																		newPersonField,
																		newTypePopup));
		this.resizeToFit(nameLabel);
		this.resizeToFit(textLabel);
		this.resizeToFit(personLabel);
		this.resizeToFit(priorityLabel);
		this.resizeToFit(typeLabel);
		nameLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		priorityLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		RelativeConstraints[] nameConstraints =
			this.constraintsForLabelPair(nameLabel, newNameField, null, false);
		RelativeConstraints[] textConstraints =
			this.constraintsForLabelPair(textLabel, newTextField, newNameField, false);
		RelativeConstraints[] personConstraints =
			this.constraintsForLabelPair(personLabel, newPersonField, newTextField, false);
		RelativeConstraints[] priorityConstraints =
			this.constraintsForLabelPair(priorityLabel, newPriorityField, newPersonField, false);
		RelativeConstraints[] typeConstraints =
			this.constraintsForLabelPair(typeLabel, newTypePopup, newPriorityField, false);

		newNotificationPanel.setBorder(BorderFactory.createTitledBorder("Add Notification"));

		newNotificationPanel.add(nameLabel, nameConstraints[0]);
		newNotificationPanel.add(newNameField, nameConstraints[1]);
		newNotificationPanel.add(textLabel, textConstraints[0]);
		newNotificationPanel.add(newTextField, textConstraints[1]);
		newNotificationPanel.add(personLabel, personConstraints[0]);
		newNotificationPanel.add(newPersonField, personConstraints[1]);
		newNotificationPanel.add(priorityLabel, priorityConstraints[0]);
		newNotificationPanel.add(newPriorityField, priorityConstraints[1]);
		newNotificationPanel.add(typeLabel, typeConstraints[0]);
		newNotificationPanel.add(newTypePopup, typeConstraints[1]);
		newNotificationPanel.add(	this.addButton,
									new RelativeConstraints(new Binding(Edge.RIGHT,
																		20,
																		Direction.LEFT,
																		Edge.RIGHT,
																		null),
															this.bm.below(newTypePopup)));

		return newNotificationPanel;
	}

	/**
	 * Populates the "Change Notification" panel with data from the given notification.
	 * 
	 * @param note the notification to use when populating the panel
	 */
	private void populateChangeNotificationPanel(Notification note) {
		this.nameChangeField.setText(note.getName());
		this.textChangeField.setText(note.getText());
		this.personChangeField.setText(note.getPerson().getName());
		this.priorityChangeField.setText(new Double(note.getPriority()).toString());
	}

	/**
	 * Resizes the given label to have a width of LABEL_WIDTH.
	 * 
	 * @param nameLabel the label to resize
	 */
	private void resizeToFit(JLabel nameLabel) {
		nameLabel.setPreferredSize(new Dimension(	WOZFrontEndNotificationPanel.LABEL_WIDTH,
													nameLabel.getPreferredSize().height));
		nameLabel.setHorizontalAlignment(SwingConstants.RIGHT);
	}

	/**
	 * Sets up the visual appearance of this panel.
	 */
	@SuppressWarnings("synthetic-access")
	private void setup() {
		this.setLayout(new RelativeLayout());

		this.newNotificationPanel = this.createNewNotificationPanel();
		this.changeNotificationPanel = this.createChangeNotificationPanel();
		this.notificationList = new JList();

		this.notificationViewPanel = new JPanel(new RelativeLayout());

		this.add(	this.newNotificationPanel,
					new RelativeConstraints(this.bm.topEdge(),
											this.bm.leftEdge(),
											new Binding(Edge.RIGHT,
														0,
														Direction.LEFT,
														Edge.RIGHT,
														this.changeNotificationPanel)));
		this.add(	this.changeNotificationPanel,
					new RelativeConstraints(this.bm.below(this.newNotificationPanel),
											this.bm.leftEdge()));
		this.add(	new JScrollPane(this.notificationList),
					new RelativeConstraints(this.bm.topEdge(),
											this.bm.rightOf(this.changeNotificationPanel),
											this.bm.rightEdge(),
											this.bm.above(this.notificationViewPanel)));
		this.add(	this.notificationViewPanel,
					new RelativeConstraints(this.bm.bottomEdge(),
											this.bm.rightEdge(),
											this.bm.rightOf(this.changeNotificationPanel)));
	}
}
