/* File: WOZFrontEndTaskPanel.java
 *
 * Contains class WOZFrontEndTaskPanel
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.thinkaloud;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import com.toedter.calendar.JDateChooser;

import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.TimeSpinner;
import edu.cmu.hcii.calo.view.task.TaskView;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The task panel inside the WOZFrontEndFrame.
 * 
 * @author Brian Ellis
 * 
 */
public class WOZFrontEndTaskPanel extends CALOComponent {

	/**
	 * An action listener subclass that tells the front-end to remotely change the selected task
	 * when the change button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public final class ChangeButtonActionListener implements ActionListener {

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndTaskPanel.this.frontEnd.changeSelectedTask(	WOZFrontEndTaskPanel.this.nameChangeField.getText(),
																	new Double(WOZFrontEndTaskPanel.this.priorityChangeField.getText()),
																	WOZFrontEndTaskPanel.this.onHoldChangeBox.isSelected(),
																	WOZFrontEndTaskPanel.this.completeChangeBox.isSelected(),
																	WOZFrontEndTaskPanel.this.dueDateChangeBox.isSelected(),
																	WOZFrontEndTaskPanel.getDate(	WOZFrontEndTaskPanel.this.dateChangeChooser,
																									WOZFrontEndTaskPanel.this.timeChangeSpinner));
		}

	}

	/**
	 * An action listener subclass that tells the front-end to remotely delete the selected task
	 * when the delete button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public final class DeleteButtonActionListener implements ActionListener {

		/**
		 * Called when the delete button is clicked.
		 * 
		 * @param e the action event
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndTaskPanel.this.frontEnd.deleteSelectedTask();
		}
	}

	/**
	 * An action listener subclass that tells the front-end to remotely add a task when the add
	 * button is clicked.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class AddButtonActionListener implements ActionListener {

		/**
		 * @name Controls
		 * 
		 * Controls we need to query when the add button is pressed.
		 */

		// @{
		private final JDateChooser newDateChooser;
		private final JTextField newPriorityField;
		private final JCheckBox newCompleteBox;
		private final TimeSpinner newTimeSpinner;
		private final JTextField newNameField;
		private final JCheckBox newDueDateBox;
		private final JCheckBox newOnHoldBox;

		// @}

		/**
		 * Constructs an AddButtonActionListener with the given controls. These controls will be
		 * queried for their state when the add button is pressed so that the task that's added has
		 * the right information associated with it.
		 * 
		 * @param newDateChooser the date chooser
		 * @param newPriorityField the priority field
		 * @param newCompleteBox the complete box
		 * @param newTimeSpinner the time spinner
		 * @param newNameField the name field
		 * @param newDueDateBox the due date box
		 * @param newOnHoldBox the on hold box
		 */
		private AddButtonActionListener(JDateChooser newDateChooser,
				JTextField newPriorityField,
				JCheckBox newCompleteBox,
				TimeSpinner newTimeSpinner,
				JTextField newNameField,
				JCheckBox newDueDateBox,
				JCheckBox newOnHoldBox) {
			this.newDateChooser = newDateChooser;
			this.newPriorityField = newPriorityField;
			this.newCompleteBox = newCompleteBox;
			this.newTimeSpinner = newTimeSpinner;
			this.newNameField = newNameField;
			this.newDueDateBox = newDueDateBox;
			this.newOnHoldBox = newOnHoldBox;
		}

		/**
		 * Called when the user clicks the Add button.
		 */
		@SuppressWarnings("synthetic-access")
		public void actionPerformed(ActionEvent e) {
			WOZFrontEndTaskPanel.this.frontEnd.addNewTask(	this.newNameField.getText(),
															new Double(this.newPriorityField.getText()).doubleValue(),
															this.newOnHoldBox.isSelected(),
															this.newCompleteBox.isSelected(),
															this.newDueDateBox.isSelected(),
															WOZFrontEndTaskPanel.getDate(	this.newDateChooser,
																							this.newTimeSpinner));
		}
	}

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = -2631720313349085236L;

	/**
	 * The width of the labels.
	 */
	private static final int LABEL_WIDTH = 75;

	/**
	 * The width of the text fields.
	 */
	private static final int TEXT_FIELD_WIDTH = 34;

	/**
	 * The width of the buttons.
	 */
	private static final int BUTTON_WIDTH = 92;

	/**
	 * @name Controls
	 * 
	 * Swing controls that exist in the panel.
	 */

	// @{
	private JPanel newTaskPanel;

	private JPanel changeTaskPanel;

	private JList taskList;
	private JPanel taskViewPanel;
	private JTextField nameChangeField;
	private JTextField priorityChangeField;
	private JCheckBox onHoldChangeBox;
	private JCheckBox completeChangeBox;
	private JCheckBox dueDateChangeBox;
	private JDateChooser dateChangeChooser;
	private TimeSpinner timeChangeSpinner;
	private JButton deleteButton;
	private JButton addButton;
	private JButton changeButton;

	// @}

	/**
	 * The front end this panel reports to.
	 */
	private WizardOfOzFrontEnd frontEnd;

	/**
	 * A binding maker for layout purposes.
	 */
	private BindingMaker bm;

	/**
	 * Constructs a WOZFrontEndTaskPanel attached to the given front-end.
	 * 
	 * @param frontEnd the front-end to attach to
	 * 
	 */
	public WOZFrontEndTaskPanel(WizardOfOzFrontEnd frontEnd) {
		this.frontEnd = frontEnd;
		this.bm = new BindingMaker(4, 4, 4, 4, 4, 4);
		this.setup();
	}

	/**
	 * Gets the task list.
	 * 
	 * @return the task list
	 */
	public JList getTaskList() {
		return this.taskList;
	}

	/**
	 * Sets the selected task to that given.
	 * 
	 * @param task the new selected task
	 */
	public void setSelectedTask(Task task) {
		WOZFrontEndTaskPanel.this.changeTaskPanel.setEnabled(task != null);
		WOZFrontEndTaskPanel.this.taskViewPanel.removeAll();

		if (task != null) {

			TaskView taskView = new TaskView(task);
			taskView.setEnabled(false);
			taskView.setExpanded(true);
			taskView.addPropertyChangeListener(	TaskViewPropertyNames.EXPANDED_PROPERTY_NAME,
												new PropertyChangeListener() {

													public void propertyChange(PropertyChangeEvent evt) {
														WOZFrontEndTaskPanel.this.validate();
													}

												});
			WOZFrontEndTaskPanel.this.taskViewPanel.add(taskView,
														new RelativeConstraints(this.bm.directBottomEdge(),
																				this.bm.directLeftEdge(),
																				this.bm.directRightEdge()));

			this.populateChangeTaskPanel(task);

			this.validate();
			this.repaint();

		}

	}

	/**
	 * Constructs and returns a date using the information in the given JDateChooser and TimeSpinner
	 * instances.
	 * 
	 * @param chooser the date chooser
	 * @param spinner the time spinner
	 * 
	 * @return the date and time represented by a combination of the date chooser and time spinner
	 */
	private static Calendar getDate(JDateChooser chooser, TimeSpinner spinner) {
		Date d = chooser.getDate();
		Date od = (Date)spinner.getValue();
		if (d == null || od == null) {
			return null;
		}
		Calendar c = new GregorianCalendar();
		c.setTime(d);
		Calendar oc = new GregorianCalendar();
		oc.setTime(od);
		c.set(Calendar.HOUR_OF_DAY, oc.get(Calendar.HOUR_OF_DAY));
		c.set(Calendar.MINUTE, oc.get(Calendar.MINUTE));
		c.set(Calendar.SECOND, oc.get(Calendar.SECOND));
		return c;
	}

	/**
	 * Gets a pair of RelativeConstraints objects for a label and a text field that will lay them
	 * out in the appropriate spots.
	 * 
	 * @param label the label for this pair
	 * @param field the field for this pair
	 * @param previousLineField the field from the previous line, or \c null if this is the first
	 *            line
	 * @param stretch \c true if the field should stretch all the way to the edge of the window, \c
	 *            false otherwise
	 * @return a pair of RelativeConstraints, the first for the label, the second for the field
	 */
	private RelativeConstraints[] constraintsForLabelPair(Component label,
			Component field,
			Component previousLineField,
			boolean stretch) {
		RelativeConstraints labelConstraints = new RelativeConstraints();
		if (field == null) {
			labelConstraints.addBinding(this.bm.below(previousLineField));
		} else {
			labelConstraints.addBinding(this.bm.verticallyCenterAlign(field));
		}

		RelativeConstraints fieldConstraints = new RelativeConstraints(this.bm.rightOf(label));
		if (previousLineField == null) {
			fieldConstraints.addBinding(this.bm.topEdge());
		} else {
			fieldConstraints.addBinding(this.bm.below(previousLineField));
		}
		if (stretch) {
			fieldConstraints.addBinding(new Binding(Edge.RIGHT,
													20,
													Direction.LEFT,
													Edge.RIGHT,
													null));
		}

		return new RelativeConstraints[] { labelConstraints, fieldConstraints };
	}

	/**
<<<<<<< .mine
	 * Creates and returns the "Change Task" panel.
	 * 
	 * @return the "Change Task" panel
=======
	 * 
	 * @param label
	 * @param field
	 * @param button
	 * @param previousLineField
	 * @param stretch
	 * @return
>>>>>>> .r244
	 */
	private JPanel createChangeTaskPanel() {
		@SuppressWarnings("hiding")
		JPanel changeTaskPanel = new JPanel(new RelativeLayout());

		JLabel nameLabel = new JLabel("Name:");
		JLabel priorityLabel = new JLabel("Priority:");
		this.nameChangeField = new JTextField("New Task", WOZFrontEndTaskPanel.TEXT_FIELD_WIDTH);
		this.priorityChangeField = new JTextField("100", WOZFrontEndTaskPanel.TEXT_FIELD_WIDTH);
		JLabel onHoldLabel = new JLabel("On Hold:");
		JLabel completeLabel = new JLabel("Complete:");
		JLabel dueDateLabel = new JLabel("Due Date:");

		this.resizeToFit(nameLabel);
		this.resizeToFit(priorityLabel);
		this.resizeToFit(onHoldLabel);
		this.resizeToFit(completeLabel);
		this.resizeToFit(dueDateLabel);

		this.onHoldChangeBox = new JCheckBox("On Hold");
		this.completeChangeBox = new JCheckBox("Complete");
		this.dueDateChangeBox = new JCheckBox("Has Due Date:");
		this.dateChangeChooser = new JDateChooser();
		this.timeChangeSpinner = new TimeSpinner();

		this.changeButton = new JButton("Change Task");
		this.changeButton.addActionListener(new ChangeButtonActionListener());

		this.deleteButton = new JButton("Delete Task");
		this.deleteButton.setPreferredSize(new Dimension(	WOZFrontEndTaskPanel.BUTTON_WIDTH,
															this.deleteButton.getPreferredSize().height));
		this.deleteButton.addActionListener(new DeleteButtonActionListener());

		JPanel dueDatePanel = new JPanel(new FlowLayout());
		dueDatePanel.add(this.dueDateChangeBox);
		dueDatePanel.add(this.dateChangeChooser);
		dueDatePanel.add(this.timeChangeSpinner);

		RelativeConstraints[] nameConstraints =
			this.constraintsForLabelPair(nameLabel, this.nameChangeField, null, false);
		RelativeConstraints[] priorityConstraints =
			this.constraintsForLabelPair(	priorityLabel,
											this.priorityChangeField,
											this.nameChangeField,
											false);
		RelativeConstraints[] onHoldConstraints =
			this.constraintsForLabelPair(	onHoldLabel,
											this.onHoldChangeBox,
											this.priorityChangeField,
											false);
		RelativeConstraints[] completeConstraints =
			this.constraintsForLabelPair(	completeLabel,
											this.completeChangeBox,
											this.onHoldChangeBox,
											false);
		RelativeConstraints[] dueDateConstraints =
			this.constraintsForLabelPair(dueDateLabel, dueDatePanel, this.completeChangeBox, false);

		changeTaskPanel.add(nameLabel, nameConstraints[0]);
		changeTaskPanel.add(this.nameChangeField, nameConstraints[1]);

		changeTaskPanel.add(priorityLabel, priorityConstraints[0]);
		changeTaskPanel.add(this.priorityChangeField, priorityConstraints[1]);

		changeTaskPanel.add(onHoldLabel, onHoldConstraints[0]);
		changeTaskPanel.add(this.onHoldChangeBox, onHoldConstraints[1]);

		changeTaskPanel.add(completeLabel, completeConstraints[0]);
		changeTaskPanel.add(this.completeChangeBox, completeConstraints[1]);

		changeTaskPanel.add(dueDateLabel, dueDateConstraints[0]);
		changeTaskPanel.add(dueDatePanel, dueDateConstraints[1]);

		changeTaskPanel.add(this.deleteButton,
							new RelativeConstraints(new Binding(Edge.LEFT,
																20,
																Direction.RIGHT,
																Edge.LEFT,
																null), this.bm.below(dueDatePanel)));
		changeTaskPanel.add(this.changeButton,
							new RelativeConstraints(new Binding(Edge.RIGHT,
																20,
																Direction.LEFT,
																Edge.RIGHT,
																null), this.bm.below(dueDatePanel)));

		changeTaskPanel.setBorder(BorderFactory.createTitledBorder("Change Task"));

		return changeTaskPanel;
	}

	/**
	 * Creates and returns the "New Task" panel.
	 * 
	 * @return the "New Task" panel
	 */
	@SuppressWarnings( { "hiding", "synthetic-access" })
	private JPanel createNewTaskPanel() {
		JPanel newTaskPanel = new JPanel(new RelativeLayout());

		JLabel nameLabel = new JLabel("Name:");
		JLabel priorityLabel = new JLabel("Priority:");
		final JTextField newNameField =
			new JTextField("New Task", WOZFrontEndTaskPanel.TEXT_FIELD_WIDTH);
		final JTextField newPriorityField =
			new JTextField("100", WOZFrontEndTaskPanel.TEXT_FIELD_WIDTH);
		final JCheckBox newOnHoldBox = new JCheckBox("On Hold");
		final JCheckBox newCompleteBox = new JCheckBox("Complete");
		final JCheckBox newDueDateBox = new JCheckBox("Due Date:");
		final JDateChooser newDateChooser = new JDateChooser();
		final TimeSpinner newTimeSpinner = new TimeSpinner();

		this.addButton = new JButton("Add Task");
		this.addButton.setPreferredSize(new Dimension(	WOZFrontEndTaskPanel.BUTTON_WIDTH,
														this.addButton.getPreferredSize().height));
		this.addButton.addActionListener(new AddButtonActionListener(	newDateChooser,
																		newPriorityField,
																		newCompleteBox,
																		newTimeSpinner,
																		newNameField,
																		newDueDateBox,
																		newOnHoldBox));
		this.resizeToFit(nameLabel);
		this.resizeToFit(priorityLabel);
		nameLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		priorityLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		newTimeSpinner.setPreferredSize(new Dimension(75, newTimeSpinner.getPreferredSize().height));

		RelativeConstraints[] nameConstraints =
			this.constraintsForLabelPair(nameLabel, newNameField, null, false);
		RelativeConstraints[] priorityConstraints =
			this.constraintsForLabelPair(priorityLabel, newPriorityField, newNameField, false);
		RelativeConstraints[] onHoldConstraints =
			this.constraintsForLabelPair(newOnHoldBox, null, newPriorityField, false);
		RelativeConstraints[] completeConstraints =
			this.constraintsForLabelPair(newCompleteBox, null, newOnHoldBox, false);
		RelativeConstraints[] dueDateConstraints =
			this.constraintsForLabelPair(newDueDateBox, newDateChooser, newCompleteBox, false);

		newTaskPanel.setBorder(BorderFactory.createTitledBorder("Add Task"));

		newTaskPanel.add(nameLabel, nameConstraints[0]);
		newTaskPanel.add(newNameField, nameConstraints[1]);
		newTaskPanel.add(priorityLabel, priorityConstraints[0]);
		newTaskPanel.add(newPriorityField, priorityConstraints[1]);
		newTaskPanel.add(newOnHoldBox, onHoldConstraints[0]);
		newTaskPanel.add(newCompleteBox, completeConstraints[0]);
		newTaskPanel.add(newDueDateBox, dueDateConstraints[0]);
		newTaskPanel.add(newDateChooser, dueDateConstraints[1]);
		newTaskPanel.add(	newTimeSpinner,
							new RelativeConstraints(this.bm.below(newCompleteBox),
													this.bm.rightOf(newDateChooser)));
		newTaskPanel.add(this.addButton, new RelativeConstraints(	new Binding(Edge.RIGHT,
																				20,
																				Direction.LEFT,
																				Edge.RIGHT,
																				null),
																	this.bm.below(newDateChooser)));

		return newTaskPanel;
	}

	/**
	 * Populates the fields and controls in the "Change Task" panel with the information from the
	 * given task.
	 * 
	 * @param task the task to use to populate the panel
	 */
	private void populateChangeTaskPanel(Task task) {
		this.nameChangeField.setText(task.getName());
		this.priorityChangeField.setText(new Double(task.getPriority()).toString());
		this.onHoldChangeBox.setSelected(task.getPendingState().isPending());
		this.completeChangeBox.setSelected(task.isComplete());
		boolean hasDueDate = task.getDueDate() != null;
		this.dueDateChangeBox.setSelected(hasDueDate);
		this.dateChangeChooser.setEnabled(hasDueDate);
		this.timeChangeSpinner.setEnabled(hasDueDate);
		if (hasDueDate) {
			this.dateChangeChooser.setDate(task.getDueDate().getTime());
			this.timeChangeSpinner.setValue(task.getDueDate().getTime());
		}
	}

	/**
	 * Resizes the given label to have a width of LABEL_WIDTH.
	 * 
	 * @param nameLabel the label to resize
	 */
	private void resizeToFit(JLabel nameLabel) {
		nameLabel.setPreferredSize(new Dimension(	WOZFrontEndTaskPanel.LABEL_WIDTH,
													nameLabel.getPreferredSize().height));
		nameLabel.setHorizontalAlignment(SwingConstants.RIGHT);
	}

	/**
	 * Sets up the visual appearance of this panel.
	 */
	@SuppressWarnings("synthetic-access")
	private void setup() {
		this.setLayout(new RelativeLayout());

		this.newTaskPanel = this.createNewTaskPanel();
		this.changeTaskPanel = this.createChangeTaskPanel();
		this.taskList = new JList();

		this.taskViewPanel = new JPanel(new RelativeLayout());

		this.add(this.newTaskPanel, new RelativeConstraints(this.bm.topEdge(), this.bm.leftEdge()));
		this.add(this.changeTaskPanel, new RelativeConstraints(	this.bm.below(this.newTaskPanel),
																this.bm.leftEdge()));
		this.add(	new JScrollPane(this.taskList),
					new RelativeConstraints(this.bm.topEdge(),
											this.bm.rightOf(this.newTaskPanel),
											this.bm.rightEdge(),
											this.bm.above(this.taskViewPanel)));
		this.add(this.taskViewPanel, new RelativeConstraints(	this.bm.bottomEdge(),
																this.bm.rightEdge(),
																this.bm.rightOf(this.newTaskPanel)));
	}

}
