/* File: WizardOfOzFrontEnd.java
 *
 * Contains class WizardOfOzFrontEnd
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.thinkaloud;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Observable;

import javax.swing.AbstractListModel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.messagePassing.Message;
import edu.cmu.hcii.calo.messagePassing.MessageType;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage;
import edu.cmu.hcii.calo.messagePassing.TaskMessage;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage;
import edu.cmu.hcii.calo.messagePassing.NotificationMessage.NotificationMessageType;
import edu.cmu.hcii.calo.messagePassing.TaskMessage.TaskMessageType;
import edu.cmu.hcii.calo.messagePassing.TasklikeObjectMessage.TasklikeObjectMessageType;
import edu.cmu.hcii.calo.messagePassing.WizardOfOzMessage.WizardOfOzMessageType;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Person;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TasklikeObject;
import edu.cmu.hcii.calo.model.Notification.NotificationType;

/**
 * The front-end application for running the Wizard of Oz wizard.
 * 
 * @author Brian Ellis
 * 
 */
public class WizardOfOzFrontEnd extends Observable implements Serializable, WindowListener {

	/**
	 * The model for the notification list.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class NotificationListModel extends AbstractListModel {

		/** Prevent serialization collisions. */
		private static final long serialVersionUID = -7613383682898134619L;

		/**
		 * Gets the element at the given index.
		 * 
		 * @param index the index
		 */
		@SuppressWarnings("synthetic-access")
		public Object getElementAt(int index) {
			return index >= WizardOfOzFrontEnd.this.notifications.size()
					? "(invalid task)"
					: WizardOfOzFrontEnd.this.notifications.get(index).getName();
		}

		/**
		 * Gets the size of the list.
		 */
		@SuppressWarnings("synthetic-access")
		public int getSize() {
			return WizardOfOzFrontEnd.this.notifications == null
					? 0
					: WizardOfOzFrontEnd.this.notifications.size();
		}

		/**
		 * Tells the list that its contents may have changed and it should update itself.
		 */
		public void updateList() {
			this.fireContentsChanged(this, 0, this.getSize());
		}
	}

	/**
	 * Loads the selected notification's information into the appropriate fields.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class NotificationListSelectionListener implements ListSelectionListener {

		/**
		 * Invoked when the list's selection changes.
		 */
		@SuppressWarnings("synthetic-access")
		public void valueChanged(ListSelectionEvent e) {
			final int index =
				WizardOfOzFrontEnd.this.frame.getNotificationList().getSelectedIndex();

			Notification note =
				index >= 0 ? WizardOfOzFrontEnd.this.notifications.get(index) : null;

			WizardOfOzFrontEnd.this.frame.setSelectedNotification(note);
		}
	}

	/**
	 * The model for the task list.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class TaskListModel extends AbstractListModel {

		/** Prevent serialization collision. */
		private static final long serialVersionUID = 6839875586075726699L;

		/**
		 * Gets the element at the given index.
		 * 
		 * @param index the index
		 */
		@SuppressWarnings("synthetic-access")
		public Object getElementAt(int index) {
			return index >= WizardOfOzFrontEnd.this.tasks.size()
					? "(invalid task)"
					: WizardOfOzFrontEnd.this.tasks.get(index).getName();
		}

		/**
		 * Gets the size of the list.
		 */
		@SuppressWarnings("synthetic-access")
		public int getSize() {
			return WizardOfOzFrontEnd.this.tasks == null ? 0 : WizardOfOzFrontEnd.this.tasks.size();
		}

		/**
		 * Tells the list that its contents may have changed and it should update itself.
		 */
		public void updateList() {
			this.fireContentsChanged(this, 0, this.getSize());
		}
	}

	/**
	 * Loads the selected task's information into the appropriate fields.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class TaskListSelectionListener implements ListSelectionListener {

		/**
		 * Invoked when the list's selection changes.
		 */
		@SuppressWarnings("synthetic-access")
		public void valueChanged(ListSelectionEvent e) {
			final int index = WizardOfOzFrontEnd.this.frame.getTaskList().getSelectedIndex();

			Task task = index >= 0 ? WizardOfOzFrontEnd.this.tasks.get(index) : null;

			WizardOfOzFrontEnd.this.frame.setSelectedTask(task);
		}
	}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 461154975424237010L;

	/**
	 * Main function. Run me to launch the Wizard.
	 * 
	 * @param args command-line args, none of which matter
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}
		@SuppressWarnings("unused")
		WizardOfOzFrontEnd woz = new WizardOfOzFrontEnd();
	}

	/**
	 * The thread that looks for input from the engine.
	 */
	Thread frontEndInputLoopThread;

	/**
	 * The client socket.
	 */
	Socket clientSocket = null;

	/**
	 * The wizard frame.
	 */
	private WOZFrontEndFrame frame;

	/**
	 * The task list model.
	 */
	private List<Task> tasks;

	/**
	 * The notification list model.
	 */
	private List<Notification> notifications;

	/**
	 * The stream of serialized objects that this front end writes to to allow the engine to
	 * communicate to it.
	 */
	private ObjectOutputStream output;

	/**
	 * The stream of serialized object that this front end reads to allow the engine to communicate
	 * to it.
	 */
	private ObjectInputStream input;

	/**
	 * Constructs a WizardOfOzFrontEnd.
	 */
	@SuppressWarnings("synthetic-access")
	public WizardOfOzFrontEnd() {
		super();
		this.tasks = new ArrayList<Task>();
		this.performThreadyMagic();

		this.frame = new WOZFrontEndFrame(this);
		this.frame.getTaskList().setModel(new TaskListModel());
		this.frame.getTaskList().addListSelectionListener(new TaskListSelectionListener());

		this.frame.getNotificationList().setModel(new NotificationListModel());
		this.frame.getNotificationList().addListSelectionListener(new NotificationListSelectionListener());

		this.frame.addWindowListener(this);

		this.frame.setSize(800, 650);
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.frame.setVisible(true);
	}

	/**
<<<<<<< .mine
	 * Adds a new notification remotely.
	 * 
	 * @param name the name of the new notification
	 * @param text the text of the email, if this is an email notification
	 * @param person the person who should appear to have sent the email, if this is an email
	 *            notification
	 * @param priority the priority of this notification (over 100 won't coalesce)
	 * @param type the type of this notification
=======
	 * Tells the engine to add a new notification to the notification manager.
	 * 
	 * @param name the name of this notification
	 * @param text this notification's text
	 * @param person the person associated with this notification, if any
	 * @param priority this notification's priority
	 * @param type this notification's type
>>>>>>> .r244
	 */
	public void addNewNotification(String name,
			String text,
			Person person,
			double priority,
			NotificationType type) {
		Notification n = new Notification(type, name, text, person);
		n.setPriority(priority);
		this.addNotification(n);
		// Request an update everything!
		this.requestNotificationList();
	}

	/**
<<<<<<< .mine
	 * Adds a new task remotely.
	 * 
	 * @param name the name of this task
	 * @param priority the priority of this task
	 * @param onHold whether this task should be on hold
	 * @param complete whether this task should be complete
	 * @param hasDueDate whether this task has a due date
	 * @param dueDate if \p hasDueDate is \c true, the due date of this task (otherwise undefined
	 *            and probably null)
=======
	 * Tells the engine to add a new task to the task manager.
	 * 
	 * @param name this task's name
	 * @param priority this task's priority
	 * @param onHold whether this task is on hold
	 * @param complete whether this task is complete
	 * @param hasDueDate whether this task has a set due date
	 * @param dueDate the due date for this task, if any
>>>>>>> .r244
	 */
	public void addNewTask(String name,
			double priority,
			boolean onHold,
			boolean complete,
			boolean hasDueDate,
			Calendar dueDate) {
		Task t = new Task(name, true);
		t.setPriority(priority);
		PendingState pendingState = new PendingState();
		pendingState.setPending(onHold);
		pendingState.setPendingNote("");
		t.setPendingState(pendingState);
		t.setComplete(complete);
		if (hasDueDate) {
			t.setDueDate(dueDate);
		} else {
			t.setDueDate(null);
		}
		this.addTasklikeObject(t);
		// Request an update everything!
		this.requestTaskList();

	}

	/**
<<<<<<< .mine
	 * Changes the notification that is currently selected in the list remotely.
	 * 
	 * @param name the name of the new notification
	 * @param text the text of the email, if this is an email notification
	 * @param person the person who should appear to have sent the email, if this is an email
	 *            notification
	 * @param priority the priority of this notification (over 100 won't coalesce)
=======
	 * Changes the values in a given notification.
	 * 
	 * @param name the notification's new name
	 * @param text the notification's new text
	 * @param person the new person associated with the notification
	 * @param priority the notification's new priority
>>>>>>> .r244
	 */
	public void changeSelectedNotification(String name, String text, Person person, double priority) {
		int index = this.frame.getNotificationList().getSelectedIndex();
		if (index < 0) {
			return;
		}
		Notification n = this.notifications.get(index);
		this.ignoreNotificationModelUpdates(true);
		this.setNotificationPriority(n, priority);
		this.setNotificationText(n, text);
		this.setNotificationName(n, name);
		this.setNotificationPerson(n, person);
		this.ignoreNotificationModelUpdates(false);
		// Request an update everything!
		this.requestNotificationList();
	}

	/**
	 * Tells the engine to make sure the controller ignores updates from the notification model to
	 * ensure atomic updates for animations.
	 * 
	 * @param isIgnoring whether to ignore updates or not
	 */
	private void ignoreNotificationModelUpdates(boolean isIgnoring) {
		HashMap<String, Object> userInfo = new HashMap<String, Object>();
		userInfo.put(WizardOfOzMessage.NEW_IGNORE, isIgnoring);
		userInfo.put(WizardOfOzMessage.OLD_IGNORE, !isIgnoring);
		WizardOfOzMessage msg =
			new WizardOfOzMessage(	this,
									WizardOfOzMessageType.NOTIFICATION_CONTROLLER_IGNORE_CHANGE,
									userInfo);
		this.sendMessage(msg);
	}

	/**
<<<<<<< .mine
	 * Changes the task that is currently selected in the list remotely.
	 * 
	 * @param name the name of this task
	 * @param priority the priority of this task
	 * @param onHold whether this task should be on hold
	 * @param complete whether this task should be complete
	 * @param hasDueDate whether this task has a due date
	 * @param dueDate if \p hasDueDate is \c true, the due date of this task (otherwise undefined
	 *            and probably null)
=======
	 * Changes the values in a given task.
	 * 
	 * @param name the task's new name
	 * @param priority the task's new priority
	 * @param onHold whether this task is now on hold
	 * @param complete whether this task is now complete
	 * @param hasDueDate whether this task now has a due date
	 * @param dueDate the task's new due date if any
>>>>>>> .r244
	 */
	public void changeSelectedTask(String name,
			double priority,
			boolean onHold,
			boolean complete,
			boolean hasDueDate,
			Calendar dueDate) {
		int index = this.frame.getTaskList().getSelectedIndex();
		if (index < 0) {
			return;
		}
		Task t = this.tasks.get(index);
		this.setTaskPriority(t, priority);
		this.setTaskPending(t, onHold);
		this.setTaskCompleted(t, complete);
		// / @todo Set task due date here
		// Request an update everything!
		this.requestTaskList();
	}

	/**
	 * Delete the notification that is currently selected in the list if it still exists.
	 */
	public void deleteSelectedNotification() {
		int index = this.frame.getNotificationList().getSelectedIndex();
		Notification n;
		try {
			n = this.notifications.get(index);
		} catch (java.lang.IndexOutOfBoundsException e) {
			return;
		}
		this.removeNotification(n);
		// Request an update everything!
		this.requestNotificationList();
	}

	/**
	 * Delete the task that is currently selected in the list if it still exists.
	 */
	public void deleteSelectedTask() {
		int index = this.frame.getTaskList().getSelectedIndex();
		Task t;
		try {
			t = this.tasks.get(index);
		} catch (java.lang.IndexOutOfBoundsException e) {
			return;
		}
		this.removeTasklikeObject(t);
		// Request an update everything!
		this.requestTaskList();
	}

	/**
	 * Requests the list of notifications managed on the engine side. Hopefully, this will cause an
	 * update in the near-term future.
	 */
	public void requestNotificationList() {
		WizardOfOzMessage msg =
			new WizardOfOzMessage(this, WizardOfOzMessageType.NOTIFICATION_ITERATOR_REQUEST);
		this.sendMessage(msg);
	}

	/**
	 * Requests the list of tasks managed on the engine side. Hopefully, this will cause an update
	 * in the near-term future.
	 */
	public void requestTaskList() {
		WizardOfOzMessage msg =
			new WizardOfOzMessage(this, WizardOfOzMessageType.TASKLIKE_OBJECT_ITERATOR_REQUEST);
		this.sendMessage(msg);
	}

	/**
	 * Loops to read from the input stream until done is set to true. Puts messages onto a queue for
	 * reading later.
	 */
	public void startInputReadingLoop() {
		Object o = null;

		// Read input from the input stream
		while (true) {
			try {
				o = this.input.readObject();
			} catch (SocketException se) {
				return;
			} catch (IOException e) {
				System.err.println("FrontEnd: Error reading from stream.");
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				System.err.println("FrontEnd: Could not find class for object read from stream.");
			}

			this.onInputReceived(o);
		}
	}

	/**
	 * Does nothing when the window is activated.
	 * 
	 * @param e the window activation event
	 * @see java.util.eventlistener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * Does nothing when the window is closed.
	 * 
	 * @param e the window closed event
	 * @see java.util.eventlistener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * When the window is closing, perfoms socket cleanup and shuts off the engine.
	 * 
	 * @param e the window closing event
	 * @see java.util.eventlistener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		this.stopEngine();
		try {
			this.clientSocket.close();
		} catch (IOException ioe) {
			System.err.println("Client socket failed to close.");
		}
		System.exit(0);
	}

	/**
	 * Does nothing when the window is deactivated.
	 * 
	 * @param e the window deactivation event
	 * @see java.util.eventlistener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * Does nothing when the window is deiconified.
	 * 
	 * @param e the window deiconification event
	 * @see java.util.eventlistener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * Does nothing when the window is iconified.
	 * 
	 * @param e the window iconification event
	 * @see java.util.eventlistener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * Does nothing when the window is opened.
	 * 
	 * @param e the window open event
	 * @see java.util.eventlistener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
	// Intentionally left blank
	}

	/**
	 * Adds a notification to the notification manager via the engine.
	 * 
	 * @param notification the notification who will be added
	 * @throws IOException
	 */
	private void addNotification(Notification notification) {
		NotificationMessage msg =
			new NotificationMessage(NotificationManager.getManager(),
									NotificationMessageType.NOTIFICATION_ADDED,
									notification);
		this.sendMessage(msg);
	}

	/**
	 * Adds a tasklike object to the task manager via the engine.
	 * 
	 * @param task the task who will be added
	 * @throws IOException
	 */
	private void addTasklikeObject(TasklikeObject task) {
		TasklikeObjectMessage msg =
			new TasklikeObjectMessage(	TaskManager.getManager(),
										TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED,
										task);
		this.sendMessage(msg);
	}

	/**
	 * Takes input from off of the input stream and determines what to do with the messages
	 * received. It ignores things that it can't translate.
	 * 
<<<<<<< .mine
	 * @param o the object we received off the stream
	 * 
	 * @todo Break this up better, an if list seems a bit crude.
	 * 
=======
>>>>>>> .r244
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	private void onInputReceived(Object o) {

		// Ignore non-messages
		if (!(o instanceof Message)) {
			return;
		}

		Message msg = (Message)o;
		MessageType curType = msg.getMessageType();

		// Parse the message

		// Wizard generated messages
		if (curType == WizardOfOzMessageType.NOTIFICATION_TEXT_CHANGED_STATUS) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Notification text change failed.");
			}
		} else if (curType == WizardOfOzMessageType.NOTIFICATION_PRIORITY_CHANGED_STATUS) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Notification priority change failed.");
			}
		} else if (curType == WizardOfOzMessageType.NOTIFICATION_ITERATOR_REQUEST) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Notification iterator request failed.");
			}
			this.updateNotifications((List<Notification>)msg.getUserInfo().get(WizardOfOzMessage.NOTIFICATION_ITERATOR));
		} else if (curType == WizardOfOzMessageType.TASK_PRIORITY_CHANGED_STATUS) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Task priority change failed.");
			}
		} else if (curType == WizardOfOzMessageType.TASKLIKE_OBJECT_ITERATOR_REQUEST) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Tasklike object iterator request failed.");
			}
			this.updateTasks((List<Task>)msg.getUserInfo().get(WizardOfOzMessage.TASK_ITERATOR));
		} else if (curType == WizardOfOzMessageType.TASK_COMPLETED_CHANGED_STATUS) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Task complete status change failed.");
			}
			this.requestTaskList();
		} else if (curType == WizardOfOzMessageType.TASK_PENDING_CHANGED_STATUS) {
			if (!(Boolean)msg.getUserInfo().get(WizardOfOzMessage.SUCCESS_STATUS)) {
				System.err.println("Front End: Task pending state change failed.");
			}
			this.requestTaskList();

			// User generated messages
		} else if (curType == TaskMessageType.TASK_PRIORITY_CHANGED) {
			this.frame.getConsole().append("\nUser changed priority for: "
					+ ((TaskMessage)msg).getTask() + " from "
					+ msg.getUserInfo().get(TaskMessage.OLD_PRIORITY));
			this.requestTaskList();
		} else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_ADDED) {
			this.frame.getConsole().append("\nUser added: "
					+ ((TasklikeObjectMessage)msg).getTasklikeObject());
			this.requestTaskList();
		} else if (curType == TaskMessageType.TASK_COMPLETE_CHANGED) {
			this.frame.getConsole().append("\nUser changed completion: "
					+ ((TaskMessage)msg).getTask());
			this.requestTaskList();
		} else if (curType == TaskMessageType.TASK_PENDING_STATE_CHANGED) {
			this.frame.getConsole().append("\nUser changed pending status: "
					+ ((TaskMessage)msg).getTask());
			this.requestTaskList();
		} else if (curType == TaskMessageType.TASK_NAME_CHANGED) {
			this.frame.getConsole().append("\nUser changed name: " + ((TaskMessage)msg).getTask()
					+ " from " + msg.getUserInfo().get(TaskMessage.OLD_NAME));
			this.requestTaskList();
		} else if (curType == TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED) {
			this.frame.getConsole().append("\nUser deleted: "
					+ ((TasklikeObjectMessage)msg).getTasklikeObject());
			this.requestTaskList();
		} else if (curType == NotificationMessageType.NOTIFICATION_REMOVED) {
			this.frame.getConsole().append("\nUser deleted: "
					+ ((NotificationMessage)msg).getNotification());
			this.requestNotificationList();
		}
	}

	/**
	 * Initializes and connect the sockets and threads we need to connect to the sidebar, showing a
	 * dialog to the user to determine what address to connect to.
	 */
	private void performThreadyMagic() {
		final String hostName =
			JOptionPane.showInputDialog("Please enter the IP address or hostname the Wizard should connect to.",
										"127.0.0.1");
		if (hostName == null) { // user canceled
			System.exit(0);
		}

		// Connect to server
		try {
			this.clientSocket = new Socket(hostName, 48192);
		} catch (IOException e) {
			System.err.println("Front End: Couldn't get I/O for " + "the connection to: "
					+ hostName);
			JOptionPane.showMessageDialog(	null,
											"The Wizard couldn't connect to "
													+ hostName
													+ ".  Make sure there are no firewalls in the way, and try to use an internal IP address (one starting with \"10\" or \"192\") if possible.",
											"Connection Failure",
											JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}

		// Create object output stream to write to
		try {
			this.input = new ObjectInputStream(this.clientSocket.getInputStream());
			this.output = new ObjectOutputStream(this.clientSocket.getOutputStream());
			this.output.flush();
		} catch (IOException e) {
			System.err.println("Front End: Could not create ObjectOutputStream.");
			JOptionPane.showMessageDialog(	null,
											"An error occurred while connecting to the remote machine.  Try shutting down CALO on the other machine, then restart it and try again.",
											"Connection Failure",
											JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}

		// Start looking for data passed in by the engine
		Runnable frontEndTask = new Runnable() {

			public void run() {
				WizardOfOzFrontEnd.this.startInputReadingLoop();
			}
		};

		this.frontEndInputLoopThread = new Thread(frontEndTask);
		this.frontEndInputLoopThread.start();

		this.requestTaskList();
	}

	/**
	 * Reads in a serialized front end. Just checks against the serialVersionUID since this is not
	 * used except for error checking.
	 * 
	 * @param in the input stream to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		if (in.readLong() != WizardOfOzFrontEnd.serialVersionUID) {
			throw new ClassNotFoundException();
		}
	}

	/**
	 * Removes a notification from the notification manager via the engine.
	 * 
	 * @param notification the notification who will be removed
	 */
	private void removeNotification(Notification notification) {
		NotificationMessage msg =
			new NotificationMessage(NotificationManager.getManager(),
									NotificationMessageType.NOTIFICATION_REMOVED,
									notification);
		this.sendMessage(msg);
	}

	/**
	 * Removes a tasklike object from the task manager via the engine.
	 * 
	 * @param task the task who will be removed
	 * @throws IOException
	 */
	private void removeTasklikeObject(TasklikeObject task) {
		TasklikeObjectMessage msg =
			new TasklikeObjectMessage(	TaskManager.getManager(),
										TasklikeObjectMessageType.TASKLIKE_OBJECT_REMOVED,
										task);
		this.sendMessage(msg);
	}

	/**
	 * Sends a message to the engine.
	 * 
	 * @param msg the message to send to the engine
	 */
	private synchronized void sendMessage(Message msg) {
		// Write the notification message to the engine
		try {
			this.output.writeObject(msg);
			this.output.flush();
		} catch (SocketException se) {
			System.err.println("Front End: Server socket closed.");
		} catch (IOException e) {
			System.err.println("Front End: Error writing to stream.");
			e.printStackTrace();
		}
	}

	/**
	 * Determines and sets the name of a notification.
	 * 
	 * @param notification the notification whose name will be updated
	 * @param newName the name to change to
	 * @throws IOException
	 */
	private void setNotificationName(Notification notification, String newName) {
		if (!notification.getName().equals(newName)) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(NotificationMessage.NEW_NAME, newName);
			NotificationMessage msg =
				new NotificationMessage(NotificationManager.getManager(),
										NotificationMessageType.NOTIFICATION_NAME_CHANGED,
										notification,
										userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Determines and sets the person for a notification.
	 * 
	 * @param notification the notification whose priority will be updated
	 * @param newPerson the person to change
	 * @throws IOException
	 */
	private void setNotificationPerson(Notification notification, Person newPerson) {
		if (!notification.getPerson().equals(newPerson)) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(NotificationMessage.NEW_PERSON, newPerson);
			NotificationMessage msg =
				new NotificationMessage(NotificationManager.getManager(),
										NotificationMessageType.NOTIFICATION_PERSON_CHANGED,
										notification,
										userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Determines and sets the priority of a notification.
	 * 
<<<<<<< .mine
	 * @param notification the notification whose priority will be updated
	 * @param newPriority the priority to change to
=======
	 * @param notification
	 *            the notification whose priority will be updated
	 * @param newPriority
	 *            the priority to change to
>>>>>>> .r244
	 */
	private void setNotificationPriority(Notification notification, double newPriority) {
		if (notification.getPriority() != newPriority) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(NotificationMessage.NEW_PRIORITY, newPriority);
			NotificationMessage msg =
				new NotificationMessage(NotificationManager.getManager(),
										NotificationMessageType.NOTIFICATION_PRIORITY_CHANGED,
										notification,
										userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Changes a notification's text via the engine.
	 * 
	 * @param notification the notification to change
	 * @param newText the text to change
	 */
	private void setNotificationText(Notification notification, String newText) {
		if (!notification.getText().equals(newText)) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(NotificationMessage.NEW_TEXT, newText);
			NotificationMessage msg =
				new NotificationMessage(NotificationManager.getManager(),
										NotificationMessageType.NOTIFICATION_TEXT_CHANGED,
										notification,
										userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Sets the completed state of a task. Only updates if the completed status changes.
	 * 
	 * @param task the task whose completed status will be updated
	 * @param newCompleted the completed state to update to
	 * @throws IOException
	 */
	private void setTaskCompleted(Task task, boolean newCompleted) {
		if (task.isComplete() != newCompleted) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(TaskMessage.NEW_COMPLETE, newCompleted);
			TaskMessage msg =
				new TaskMessage(TaskManager.getManager(),
								TaskMessageType.TASK_COMPLETE_CHANGED,
								task,
								userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Sets the pending state of a task. Only updated if pending status changes.
	 * 
	 * @param task the task whose pending status will be updated
	 * @param newPending the pending state to update to
	 * @throws IOException
	 */
	private void setTaskPending(Task task, boolean newPending) {
		if (task.getPendingState().isPending() != newPending) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			PendingState nps = new PendingState();
			nps.setPending(newPending);
			userInfo.put(TaskMessage.NEW_PENDING_STATE, nps);
			TaskMessage msg =
				new TaskMessage(TaskManager.getManager(),
								TaskMessageType.TASK_PENDING_STATE_CHANGED,
								task,
								userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Sets the priority of a task. Only updates if the priority changes.
	 * 
	 * @param task the task whose priority will be updated
	 * @param newPriority the priority to update to
	 * @throws IOException
	 */
	private void setTaskPriority(Task task, double newPriority) {
		if (task.getPriority() != newPriority) {
			HashMap<String, Object> userInfo = new HashMap<String, Object>();
			userInfo.put(TaskMessage.NEW_PRIORITY, newPriority);
			TaskMessage msg =
				new TaskMessage(TaskManager.getManager(),
								TaskMessageType.TASK_PRIORITY_CHANGED,
								task,
								userInfo);
			this.sendMessage(msg);
		}
	}

	/**
	 * Stops the engine for cleanup.
	 */
	private void stopEngine() {
		WizardOfOzMessage msg = new WizardOfOzMessage(this, WizardOfOzMessageType.CLOSE);
		this.sendMessage(msg);
	}

	/**
	 * Updates the GUI.
	 */
	private void updateNotifications(List<Notification> newNotes) {
		this.notifications = newNotes;
		if (this.frame != null) {
			((NotificationListModel)(this.frame.getNotificationList().getModel())).updateList();
			this.frame.getNotificationList().repaint();
		}
	}

	/**
	 * Updates the GUI.
	 */
	private void updateTasks(List<Task> newTasks) {
		this.tasks = newTasks;
		if (this.frame != null) {
			((TaskListModel)(this.frame.getTaskList().getModel())).updateList();
			this.frame.getTaskList().repaint();
		}
	}

	/**
	 * Writes this front end to stream. Since I don't use this except for confirmation, I just write
	 * the ID for error checking.
	 * 
	 * @param out the output stream to write to
	 * @throws IOException
	 */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeLong(WizardOfOzFrontEnd.serialVersionUID);
	}

}
