/*
 * File: AnimatedPane.java
 * 
 * Contains class AnimatedPane
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import javax.swing.SwingUtilities;

import edu.cmu.hcii.calo.view.animation.Animation;
import edu.cmu.hcii.calo.view.animation.AnimationListener;

/**
 * An abstract class that supports the animation of a JComponent-based pane.
 * 
 * Classes extending AnimatedPane should do several things:
 * 
 * <ul>
 * <li>implement doUpdate() to change the layout of the pane to match what it should look like once
 * the animation is finished,</li>
 * <li>implement prepareAnimations() and doAnimations() to record the old and new component
 * locations, respectively, and start the Animation objects in doAnimations(),</li>
 * <li>call setTotalAnimations(int) from doAnimations() when the number of needed animations are
 * known, and call incrementNumAnimations() each time you start a new Animation,</li>
 * <li>implement safeUpdateAnimation(Animation) to set the bounds of the appropriate component.</li>
 * </ul>
 * 
 * This may seem like a lot of work, but the alternative is Swing thread hell. AnimatedPane
 * automatically runs all necessary code in the AWT Event Dispatch thread, so you don't have to
 * surround anything with calls to <code>SwingUtilities.invokeLater</code>. This class is
 * guaranteed to work all of the time 99% of the time, and most of the time the other 1% of the
 * time. Have fun.
 * 
 * @author Brian Ellis
 */
public abstract class AnimatedPane extends CALOComponent implements AnimationListener {

	/**
	 * Whether we are ignoring paint requests right now.
	 */
	private boolean ignorePaintRequests = false;

	/**
	 * How many animations are currently in progress.
	 */
	private int numAnimating = 0;

	/**
	 * Image buffer that we paint when we're ignoring paint requests.
	 */
	private BufferedImage imageBuffer;

	/**
	 * The total number of animations we expect to complete before we're done.
	 */
	private int totalAnimations = 0;

	/**
	 * Whether we should prevent our height from growing smaller before all animations are complete.
	 */
	private boolean preventCollapse = true;

	/**
	 * Constructor for AnimatedPane.
	 */
	public AnimatedPane() {
		super();
	}

	/**
	 * Called by the AnimationManager when an animation is updated. Subclasses should NOT override
	 * this method to handle animations, and should instead implement
	 * safeUpdateAnimation(Animation).
	 * 
	 * @param a the animation being updated
	 */
	public void animationDidUpdate(final Animation a) {

		assert SwingUtilities.isEventDispatchThread() : "we are not on the event dispatch thread!";

		if (getNumAnimating() >= getTotalAnimations() && getIgnorePaintRequests()) {
			setIgnorePaintRequests(false);
			repaint();
		}

		safeUpdateAnimation(a);

		if (a.isComplete()) {
			decrementNumAnimating();
			if (getNumAnimating() == 0) {
				finalizeAnimation();
			}
		}

	}

	/**
	 * Calls super.doLayout() unless we have at least one animation in progress, in which case no
	 * layout is performed. This prevents ancestor views from messing up the animation by forcing
	 * the container to recompute its layout in the middle.
	 * 
	 * @see java.awt.Container#doLayout()
	 */
	@Override
	public void doLayout() {
		if (getNumAnimating() == 0) {
			super.doLayout();
		}
		repaint();
	}

	/**
	 * Calls super.paint() unless ignorePaintRequests is true, in which case we draw our stored
	 * buffer instead. This ameliorates the effect of the views snapping to their eventual
	 * destination positions when validate is called just before the animation begins, though it
	 * does not completely eliminate the effect in all instances.
	 * 
	 * @param g the Graphics instance to paint into
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	@Override
	public void paint(Graphics g) {
		if (!this.getIgnorePaintRequests()) {
			super.paint(g);
		} else {
			g.drawImage(this.imageBuffer, 0, 0, null);
		}
	}

	/**
	 * Returns whether this pane should avoid becoming smaller than its initial state during an
	 * animation. If \c true, the pane will instead resize at the end of the animation.
	 * 
	 * @return \c true if this pane prevents collapse, \c false otherwise
	 */
	public boolean preventsCollapse() {
		return this.preventCollapse;
	}

	/**
	 * Sets whether this pane should avoid becoming smaller than its initial state during an
	 * animation. If \c true, the pane will instead resize at the end of the animation.
	 * 
	 * @param preventCollapse \c true if this pane should prevent collapse, \c false otherwise
	 */
	public void setPreventsCollapse(boolean preventCollapse) {
		this.preventCollapse = preventCollapse;
	}

	/**
	 * Causes this view to synchronize itself with the TaskManager. Specifically, the view first
	 * calls prepareAnimations() to allow the old positions of its components to be recorded. Next,
	 * it sets ignorePaintRequests to true if the view is visible onscreen, and calls doUpdate().
	 * Then, doLayout() is called to recompute the positions of all the components in the view, and
	 * the view is resized to its preferred size. Lastly, doAnimations() is called to allow the new
	 * positions of the components to be recorded and the animations to be started.
	 */
	public void update() {
		assert SwingUtilities.isEventDispatchThread() : "we are not on the event dispatch thread!";

		prepareAnimations();
		if (getWidth() > 0 && getHeight() > 0) {
			setIgnorePaintRequests(true);
		}
		Dimension currentSize = getSize();
		doUpdate();
		doLayout();
		if (preventsCollapse()) {
			Dimension preferredSize = getPreferredSize();
			Dimension maxExtents =
				new Dimension(	Math.max(preferredSize.width, currentSize.width),
								Math.max(preferredSize.height, currentSize.height));
			this.setPreferredSize(maxExtents);
		}
		if (!doAnimations()) {
			finalizeAnimation();
		}
	}

	/**
	 * Subtracts one from the number of running animations. Exactly equivalent to
	 * <code>setNumAnimating(getNumAnimating() - 1)</code>. You usually do not need to call this
	 * method; the number of animations will automatically be decremented by AnimatedPane when each
	 * animation finishes.
	 * 
	 */
	protected void decrementNumAnimating() {
		this.setNumAnimating(this.getNumAnimating() - 1);
	}

	/**
	 * Should be implemented to record the positions of all the components in the view so they can
	 * be animated from their old positions, and then create and start the necessary Animation
	 * objects to perform these animations.
	 * 
	 */
	protected abstract boolean doAnimations();

	/**
	 * Should be implemented to synchronize the view with the model on which it depends.
	 * AnimatedPane provides a public update() method to the client code which calls doUpdate
	 * automatically.
	 * 
	 */
	protected abstract void doUpdate();

	/**
	 * Invoked when all animations have finished. Performs cleanup to ensure that the pane is in the
	 * correct state after all animations are done.
	 */
	protected void finalizeAnimation() {
		setPreferredSize(null);
		setIgnorePaintRequests(false);
		revalidate();
	}

	/**
	 * Gets the number of animations that have successfully started running. This number will change
	 * when you call #setNumAnimating(int) or #incrementNumAnimating(), and will also decrement
	 * automatically when animations finish.
	 * 
	 * @return the number of running animations
	 */
	public int getNumAnimating() {
		return this.numAnimating;
	}

	/**
	 * Gets the total number of animations that must complete before the entire set of animations
	 * should be declared finished.
	 * 
	 * @return the total number of animations to be run
	 */
	protected int getTotalAnimations() {
		return this.totalAnimations;
	}

	/**
	 * Adds one to the number of running animations. Exactly equivalent to
	 * <code>setNumAnimating(getNumAnimating() + 1)</code>.
	 * 
	 */
	protected void incrementNumAnimating() {
		this.setNumAnimating(this.getNumAnimating() + 1);
	}

	/**
	 * Should be implemented to record the positions of all the components in the view so they can
	 * be animated to their new positions later. prepareAnimations() is called from update() before
	 * doUpdate() is invoked, so the component positions are unchanged from what they were before
	 * the update.
	 */
	protected abstract void prepareAnimations();

	/**
	 * Should be implemented to process an updated animation, just like
	 * AnimationListener#animationDidUpdate(Animation). This method exists because AnimatedPane
	 * performs some automatic setup and teardown in animationDidUpdate(Animation), and calls
	 * safeUpdateAnimation(Animation) in between.
	 * 
	 * @param a the animation that is updating
	 */
	protected abstract void safeUpdateAnimation(Animation a);

	/**
	 * Determines whether or not the view should ignore requests to paint itself. If set to \c true,
	 * AnimatedPane will make a cached buffer of the window contents as they appear right now, and
	 * any subsequent requests to paint will be fulfilled by using this buffer. Thus, there will be
	 * no flickering or "giant eraser" effect, but any changes to the layout will be hidden until
	 * you are ready to show them.
	 * 
	 * @note This differs from JComponent#setIgnoreRepaint(boolean) in that setIgnorePaintRequests
	 *       affects ALL paint requests, including paint events generated in software by the AWT.
	 *       While ignorePaintRequests is set, the screen is guaranteed never to update.
	 * 
	 * @param ignorePaintRequests \c true to ignore paint requests, \c false to service them as
	 *            usual.
	 */
	protected void setIgnorePaintRequests(boolean ignorePaintRequests) {
		if (ignorePaintRequests) {
			// Make a cached image of the current state
			this.imageBuffer =
				new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2 = this.imageBuffer.createGraphics();
			this.paint(g2);
		}
		this.ignorePaintRequests = ignorePaintRequests;
	}

	/**
	 * Sets the number of animations that have successfully started running. It is often more
	 * convenient to use incrementNumAnimating() instead.
	 * 
	 * @param numAnimating the new number of running animations
	 * @see #incrementNumAnimating()
	 */
	protected void setNumAnimating(int numAnimating) {
		this.numAnimating = numAnimating;
	}

	/**
	 * Sets the total number of animations that must complete before the entire set of animations
	 * should be declared finished. This should be called in doAnimations() so that AnimatedPane can
	 * determine when to begin servicing paint events again.
	 * 
	 * @param totalAnimations the total number of animations to be run
	 */
	protected void setTotalAnimations(int totalAnimations) {
		this.totalAnimations = totalAnimations;
	}

	/**
	 * Returns whether the view should ignore requests to paint itself.
	 * 
	 * @return the value of ignorePaintRequests
	 * @see #setIgnorePaintRequests(boolean)
	 */
	protected boolean getIgnorePaintRequests() {
		return this.ignorePaintRequests;
	}

}
