/*
 * File: CALOComponent.java
 * 
 * Contains class CALOComponent
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.Scrollable;

/**
 * The base class for a CALO view. The only differences between this and a TranslucentView is that
 * the JComponent#firePropertyChange(String, Object, Object) method is promoted from protected to
 * public to allow PropertyChangeForwarder to work, and a PropertyChangeForwarder is automatically
 * installed in any components added to this one with this CALOComponent as the delegator. This
 * means that to the extent that your view hierarchy consists solely of CALOComponent-derived panes,
 * property change events will be propagated up the hierarchy automatically.
 * 
 * @author Brian Ellis
 * 
 */
public class CALOComponent extends TranslucentView implements Scrollable {

	/**
	 * Adds the given component to this container with the specified constraints at the specified
	 * index. Overridden to also attach a PropertyChangeForwarder to the component being added.
	 * 
	 * @param comp the component to add
	 * @param constraints the constraints to add it with
	 * @param index the index to add it at
	 */
	@Override
	protected void addImpl(Component comp, Object constraints, int index) {
		super.addImpl(comp, constraints, index);
		comp.removePropertyChangeListener(this.forwarder);
		comp.addPropertyChangeListener(this.forwarder);
	}

	/** Prevent serialization collisions */
	private static final long serialVersionUID = 893219346685374644L;

	/**
	 * The PropertyChangeForwarder that will be attached to all children of this component.
	 */
	protected PropertyChangeForwarder forwarder = new PropertyChangeForwarder(this);

	/**
	 * Calls <code>super.firePropertyChange(propertyName, oldValue, newValue)</code>. Overridden
	 * to increase the visibility to \c public so the PropertyChangeForwarder can fire property
	 * change events with object parameters on behalf of this view.
	 * 
	 * @param propertyName the name of the property that is changing
	 * @param oldValue the old value of the property
	 * @param newValue the new value of the property
	 * 
	 * @see java.awt.Component#firePropertyChange(java.lang.String, java.lang.Object,
	 *      java.lang.Object)
	 */
	@Override
	public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		super.firePropertyChange(propertyName, oldValue, newValue);
	}

	/**
	 * Returns the preferred size of this component's viewport when it's in a scroll pane.
	 * 
	 * @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return this.getPreferredSize();
	}

	/**
	 * Returns the scrollable block increment for this control when it's in a scroll pane.
	 * 
	 * @param theVisibleRect the currently visible rectangle within the scroll pane
	 * @param theOrientation the orientation of this component
	 * @param theDirection the direction the pane is being scrolled
	 * 
	 * @see javax.swing.Scrollable#getScrollableBlockIncrement(java.awt.Rectangle, int, int)
	 */
	public int getScrollableBlockIncrement(Rectangle theVisibleRect,
			int theOrientation,
			int theDirection) {
		return 50;
	}

	/**
	 * Returns whether this component should resize itself to match the height of the viewport of
	 * its containing scroll pane. Always returns \c false (i.e., there will be a vertical scrollbar
	 * when necessary).
	 * 
	 * @see javax.swing.Scrollable#getScrollableTracksViewportHeight()
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	/**
	 * Returns whether this component should resize itself to match the width of the viewport of its
	 * containing scroll pane. Always returns \c true (i.e., there will never be a horizontal
	 * scrollbar).
	 * 
	 * @see javax.swing.Scrollable#getScrollableTracksViewportWidth()
	 */
	public boolean getScrollableTracksViewportWidth() {
		return true;
	}

	/**
	 * Returns the scrollable unit increment for this control when it's in a scroll pane.
	 * 
	 * @param theVisibleRect the currently visible rectangle within the scroll pane
	 * @param theOrientation the orientation of this component
	 * @param theDirection the direction the pane is being scrolled
	 * 
	 * @see javax.swing.Scrollable#getScrollableUnitIncrement(java.awt.Rectangle, int, int)
	 */
	public int getScrollableUnitIncrement(Rectangle theVisibleRect,
			int theOrientation,
			int theDirection) {
		return 10;
	}
}
