/* File: CALOScrollPane.java
 *
 * Contains class CALOScrollPane
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;

/**
 * A scroll pane that is also a CALOComponent, and will therefore forward property changes from its
 * children to its parent.
 * 
 * @note Due to the lack of multiple inheritance in Java, CALOScrollPane does not actually subclass
 *       JScrollPane. As a result, many of the methods of JScrollPane are not available.
 * 
 * @author Brian Ellis
 * 
 */
public class CALOScrollPane extends CALOComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 9110620568939812305L;

	/**
	 * The actual JScrollPane contained within this pane.
	 */
	private JScrollPane scrollPane;

	/**
	 * Constructs a CALOScrollPane containing the given component.
	 * 
	 * @param c the component to add
	 */
	public CALOScrollPane(Component c) {
		super();
		this.scrollPane = new JScrollPane(c);
		this.scrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
		this.setup();
		// this.scrollPane.setBorder(new LineBorder(Color.MAGENTA));
		// this.scrollPane.setViewportBorder(new LineBorder(Color.BLACK));
		c.addPropertyChangeListener(this.forwarder);
	}

	/**
	 * Recalculates the layout for this container. Overridden to also send doLayout() to the actual
	 * JScrollPane contained inside this one.
	 */
	@Override
	public void doLayout() {
		this.scrollPane.doLayout();
		super.doLayout();
	}

	/**
	 * Returns the preferred size of this component. Overridden to always return the preferred size
	 * of the actual scroll pane contained inside this one unless a preferred size has been
	 * explicitly set, in which case that is returned instead.
	 */
	@Override
	public Dimension getPreferredSize() {
		return this.scrollPane.getPreferredSize();
	}

	/**
	 * Sets the preferred size of this component to the given dimension.
	 * 
	 * @param preferredSize the new preferred size
	 * 
	 * @see javax.swing.JComponent#setPreferredSize(java.awt.Dimension)
	 */
	@Override
	public void setPreferredSize(Dimension preferredSize) {
		this.scrollPane.setPreferredSize(preferredSize);
	}

	/**
	 * Sets the size of this component. Overridden to also set the size of the actual scroll pane
	 * contained inside this one.
	 * 
	 * @param d the size to set
	 */
	@Override
	public void setSize(Dimension d) {
		super.setSize(d);
		this.scrollPane.setSize(d);
		this.scrollPane.validate();
	}

	/**
	 * Sets up the visual appearance of this scroll pane.
	 */
	private void setup() {
		this.setLayout(new BorderLayout());
		this.add(this.scrollPane, BorderLayout.CENTER);
	}

	/**
	 * Sets the background of this scroll pane. Overridden to set the background of the scroll
	 * pane's viewport instead.
	 * 
	 * @param bg the background to set
	 */
	@Override
	public void setBackground(Color bg) {
		this.scrollPane.getViewport().setBackground(bg);
	}

}
