/* File: DisclosureButton.java
 *
 * Contains class DisclosureButton
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;

/**
 * A toggle button that displays a disclosure triangle.
 * 
 * @todo The transition between states should ideally be animated.
 * 
 * @author Brian Ellis
 * 
 */
public class DisclosureButton extends JToggleButton {

	/** Prevent serialization collisions */
	private static final long serialVersionUID = -8742538298469444232L;

	/**
	 * The length of a single frame of the animation between states (the one that currently doesn't
	 * happen).
	 */
	@SuppressWarnings("unused")
	private static final double ANIMATION_FRAME_LENGTH = 0.10;

	/**
	 * The icon to use when the triangle is representing a collapsed state.
	 */
	private static final Icon COLLAPSED = new ImageIcon("Resources/disclosure-collapsed.png");

	/**
	 * The icon to use when the triangle is representing a collapsed state and has been clicked.
	 */
	private static final Icon PRESSED_COLLAPSED =
		new ImageIcon("Resources/disclosure-pressed-collapsed.png");

	/**
	 * The icon to use when the triangle is representing an expanded state.
	 */
	private static final Icon EXPANDED = new ImageIcon("Resources/disclosure-expanded.png");

	/**
	 * The icon to use when the triangle is representing an expanded state and has been clicked.
	 */
	private static final Icon PRESSED_EXPANDED =
		new ImageIcon("Resources/disclosure-pressed-expanded.png");

	/**
	 * The icon to use when the triangle is animating. Currently unused due to the lack of
	 * animation.
	 */
	@SuppressWarnings("unused")
	private static final Icon ANIMATING = new ImageIcon("Resources/disclosure-animating.png");

	/**
	 * The text to use as the tooltip for this triangle when it's expanded.
	 */
	private String selectedToolTipText;

	/**
	 * The text to use as the tooltip for this triangle when it's collapsed.
	 */
	private String unselectedToolTipText;

	/**
	 * Constructs a disclosure triangle in the collapsed state.
	 */
	public DisclosureButton() {
		super();
		this.setup();
	}

	/**
	 * Constructs a disclosure triangle in the collapsed state with the given action.
	 * 
	 * @param a the action to attach
	 */
	public DisclosureButton(Action a) {
		super(a);
		this.setup();
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param icon icon
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(Icon icon) {
		super(icon);
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param icon icon
	 * @param selected selected
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(Icon icon, boolean selected) {
		super(icon, selected);
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param title title
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(String title) {
		super(title);
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param title title
	 * @param selected selected
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(String title, boolean selected) {
		super(title, selected);
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param title title
	 * @param icon icon
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(String title, Icon icon) {
		super(title, icon);
	}

	/**
	 * Superclass constructor. Do not use.
	 * 
	 * @param title title
	 * @param icon icon
	 * @param selected selected
	 */
	@SuppressWarnings("unused")
	private DisclosureButton(String title, Icon icon, boolean selected) {
		super(title, icon, selected);
	}

	/**
	 * Returns the preferred size of this button. It is always the size of the icon, unless its
	 * preferred size has been explicitly set, in which case that value is returned instead.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		} else {
			return new Dimension(this.getIcon().getIconWidth(), this.getIcon().getIconHeight());
		}
	}

	/**
	 * Sets the text to use on the tooltip for this button when it is expanded.
	 * 
	 * @param selectedToolTipText the tooltip text
	 */
	public void setSelectedToolTipText(String selectedToolTipText) {
		this.selectedToolTipText = selectedToolTipText;
	}

	/**
	 * Sets the text to use on the tooltip for this button when it is collapsed.
	 * 
	 * @param unselectedToolTipText the tooltip text
	 */
	public void setUnselectedToolTipText(String unselectedToolTipText) {
		this.unselectedToolTipText = unselectedToolTipText;
	}

	/**
	 * Fires the state changed event for this button. Overridden to set the tooltip text to the
	 * appropriate string as well.
	 * 
	 * @see javax.swing.AbstractButton#fireStateChanged()
	 */
	@Override
	protected void fireStateChanged() {
		super.fireStateChanged();
		if (this.isSelected()) {
			this.setToolTipText(this.getSelectedToolTipText());
		} else {
			this.setToolTipText(this.getUnselectedToolTipText());
		}
	}

	/**
	 * Returns the text that will be displayed in the tooltip when the disclosure button is
	 * expanded.
	 * 
	 * @return the tooltip text
	 */
	private String getSelectedToolTipText() {
		return this.selectedToolTipText;
	}

	/**
	 * Returns the text that will be displayed in the tooltip when the disclosure button is
	 * collapsed.
	 * 
	 * @return the tooltip text
	 */
	private String getUnselectedToolTipText() {
		return this.unselectedToolTipText;
	}

	/**
	 * Sets the appearance of the disclosure button to what it should be when it's collapsed.
	 */
	private void setCollapsedAppearance() {
		this.setPressedIcon(DisclosureButton.PRESSED_COLLAPSED);
	}

	/**
	 * Sets the appearance of the disclosure button to what it should be when it's expanded.
	 */
	private void setExpandedAppearance() {
		this.setPressedIcon(DisclosureButton.PRESSED_EXPANDED);
	}

	/**
	 * Sets up the visual appearance of this button.
	 */
	private void setup() {
		this.setIcon(DisclosureButton.COLLAPSED);
		this.setSelectedIcon(DisclosureButton.EXPANDED);

		this.setCollapsedAppearance();

		this.setBorder(null);
		this.setBorderPainted(false);
		this.setContentAreaFilled(false);
		this.setFocusPainted(false);

		this.setToolTipText(this.getUnselectedToolTipText());

		this.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				if (DisclosureButton.this.isSelected()) {
					DisclosureButton.this.setExpandedAppearance();
				} else {
					DisclosureButton.this.setCollapsedAppearance();
				}
			}

		});
	}

}
