/* File: EditableLabelListener.java
 *
 * Contains class EditableLabelListener
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

/**
 * A listener class that synchronizes a clickable label "viewer" with a text field "editor". This
 * allows the creation of a control that appears as a simple label until clicked, at which point it
 * becomes editable. This class also provides a mouseover and cursor affordance when the user hovers
 * over the label.
 * 
 * To use this class, first ensure that the "viewer" (label) is visible and that the "editor" (text
 * field) is not. Then create an EditableLabelListener for the viewer and editor. It is not
 * necessary to attach this class as a listener to either the viewer or the editor, as this is done
 * automatically in the constructor.
 * 
 * @warning To restate the above: it is NOT necessary to explicitly attach instances of this class
 *          as a listener to any control. Where by "it is not necessary", we mean "it will make
 *          strange things happen, so don't do it". This class only derives from listener classes
 *          and interfaces because it needs to attach itself, and Java doesn't believe in private
 *          inheritance.
 * 
 * @author Brian Ellis
 * 
 */
public class EditableLabelListener extends MouseAdapter implements ActionListener, FocusListener {

	/**
	 * @name Borders
	 * 
	 * Border styles for the label.
	 */

	// @{
	/**
	 * The normal label border, which is just empty with the same insets as the text field.
	 */
	public static final EmptyBorder NORMAL_LABEL_BORDER = new EmptyBorder(3, 3, 3, 3);
	/**
	 * The mouseover label border, which draws a translucent black line around the edge of the
	 * label.
	 */
	public static final CompoundBorder MOUSEOVER_LABEL_BORDER =
		new CompoundBorder(new LineBorder(new Color(0f, 0f, 0f, 0.5f)), new EmptyBorder(2, 2, 2, 2));

	// @}

	/**
	 * The component that contains the viewer and editor.
	 */
	private Component parent;

	/**
	 * The viewer component.
	 */
	private JLabel viewer;

	/**
	 * The editor component.
	 */
	private JTextField editor;

	/**
	 * Constructs an EditableLabelListener with the given parent, viewer, and editor components. The
	 * viewer must be a JLabel, and the editor must be a JTextField. The parent is only necessary to
	 * determine whether the interaction should be enabled.
	 * 
	 * @param parent the parent component of the editor and viewer
	 * @param viewer the viewer component
	 * @param editor the editor component
	 */
	public EditableLabelListener(Component parent, JLabel viewer, JTextField editor) {
		this.parent = parent;
		this.viewer = viewer;
		this.editor = editor;

		viewer.addMouseListener(this);
		editor.addActionListener(this);
		editor.addFocusListener(this);
	}

	/**
	 * Called when the user hits the return key inside the text field.
	 * 
	 * @param e the action event
	 */
	@SuppressWarnings("synthetic-access")
	public void actionPerformed(ActionEvent e) {
		if (!this.parent.isEnabled()) {
			return;
		}
		this.viewer.setText(this.fieldTransform(this.editor.getText()));
		this.editor.setVisible(false);
		this.viewer.setVisible(true);
	}

	/**
	 * Returns the string that should be transferred to the viewer component when the editor
	 * component is done being edited. This is implemented simply to return the contents of the
	 * editor, but subclasses can override it to define custom behavior (like changing the empty
	 * string into a placeholder for the label).
	 * 
	 * @param fieldText the text that was placed into the editor
	 * @return the text that should be shown in the viewer
	 */
	protected String fieldTransform(String fieldText) {
		return fieldText;
	}

	/**
	 * Does nothing when the focus is gained.
	 * 
	 * @param e the focus event
	 * 
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent e) {
	// Intentionally left blank
	}

	/**
	 * Replaces the editor with the viewer when the focus of the editor is lost.
	 * 
	 * @param e the focus event
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	public void focusLost(FocusEvent e) {
		if (!this.parent.isEnabled()) {
			return;
		}
		this.editor.postActionEvent();
		this.editor.setVisible(false);
		this.viewer.setVisible(true);
	}

	/**
	 * Returns the string that should be transferred from the viewer component to the editor
	 * component when the editor component is shown. This is implemented simply to return the
	 * contents of the viewer, but subclasses can override it to define custom behavior (like
	 * changing some placeholder into the empty string for the purposes of editing).
	 * 
	 * @param labelText the text that is being shown in the viewer
	 * @return the text that should be placed into the editor
	 */
	protected String labelTransform(String labelText) {
		return labelText;
	}

	/**
	 * Replaces the viewer with the editor when the mouse is clicked in the viewer.
	 * 
	 * @param e the mouse event
	 */
	@SuppressWarnings("synthetic-access")
	@Override
	public void mouseClicked(MouseEvent e) {
		if (!this.parent.isEnabled()) {
			return;
		}
		this.editor.setText(this.labelTransform(this.viewer.getText()));
		this.editor.setVisible(true);
		this.viewer.setVisible(false);
		this.editor.requestFocusInWindow();
	}

	/**
	 * Displays a mouseover border on the viewer when the mouse is moved into it.
	 * 
	 * @param e the mouse event
	 */
	@SuppressWarnings("synthetic-access")
	@Override
	public void mouseEntered(MouseEvent e) {
		if (!this.parent.isEnabled()) {
			return;
		}
		// if (!TaskView.this.isPressed) {
		this.viewer.setBorder(EditableLabelListener.MOUSEOVER_LABEL_BORDER);
		// }
	}

	/**
	 * Removes the mouseover border from the viewer when the mouse is moved off of it.
	 * 
	 * @param e the mouse event
	 */
	@SuppressWarnings("synthetic-access")
	@Override
	public void mouseExited(MouseEvent e) {
		if (!this.parent.isEnabled()) {
			return;
		}
		// if (!TaskView.this.isPressed) {
		this.viewer.setBorder(EditableLabelListener.NORMAL_LABEL_BORDER);
		// }
	}
}
