/*
 * File: FlatButton.java
 * 
 * Contains class FlatButton
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

/**
 * A transparent button that appears flat with no border until it is moused over, when a border
 * appears around it. The border becomes darker when the button is clicked. Flat buttons also subtly
 * lighten their backgrounds when hovered over or clicked.
 * 
 * @author Brian Ellis
 * 
 */
public class FlatButton extends JButton {

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * @name Borders
	 * 
	 * Border styles for the FlatButton.
	 */

	// @{
	/**
	 * The pressed border style.
	 */
	protected Border pressedBorder;

	/**
	 * The mouseover border style.
	 */
	protected Border mouseoverBorder;

	/**
	 * The normal border style.
	 */
	protected Border normalBorder;

	// @}

	/**
	 * Whether the mouse is currently being held down after having been pressed inside this button.
	 */
	private boolean isMouseDown;

	/**
	 * Whether the mouse is currently hovering over this button.
	 */
	private boolean isMouseOver;

	/**
	 * @name Colors
	 * 
	 * Colors used when drawing the button.
	 */

	// @{
	/**
	 * The color that is lain over the effective background color.
	 */
	private Color backgroundOverlay;

	/**
	 * The base color of this button. By default the button is transparent.
	 */
	private Color effectiveBackground;

	// @}

	/**
	 * Constructs a FlatButton.
	 */
	public FlatButton() {
		super();
		this.setup();
	}

	/**
	 * Constructs a FlatButton with the given action.
	 * 
	 * @param a the action
	 */
	public FlatButton(Action a) {
		super(a);
		this.setup();
	}

	/**
	 * Constructs a FlatButton with the given icon.
	 * 
	 * @param icon the icon
	 */
	public FlatButton(Icon icon) {
		super(icon);
		this.setup();
	}

	/**
	 * Constructs a FlatButton with the given label.
	 * 
	 * @param text the label
	 */
	public FlatButton(String text) {
		super(text);
		this.setup();
	}

	/**
	 * Constructs a FlatButton with the given label and icon.
	 * 
	 * @param text the label
	 * @param icon the icon
	 */
	public FlatButton(String text, Icon icon) {
		super(text, icon);
		this.setup();
	}

	/**
	 * Sets the "padding" of the button, which is the space that will be left between the edge of
	 * the label (and icon, if applicable) and the edge of the button. The same padding is always
	 * used for left and right, and for top and bottom, so the width of the button, for example,
	 * will be the width of the label and icon plus twice the horizontal padding.
	 * 
	 * @param horizontalPadding the horizontal padding
	 * @param verticalPadding the vertical padding
	 */
	public void setPadding(int horizontalPadding, int verticalPadding) {
		EmptyBorder padding =
			new EmptyBorder(new Insets(	verticalPadding,
										horizontalPadding,
										verticalPadding,
										horizontalPadding));
		this.pressedBorder =
			new CompoundBorder(new LineBorder(new Color(0f, 0f, 0f, 0.5f)), padding);
		this.mouseoverBorder =
			new CompoundBorder(new LineBorder(new Color(0f, 0f, 0f, 0.25f)), padding);
		this.normalBorder =
			new CompoundBorder(new LineBorder(new Color(0f, 0f, 0f, 0.0f)), padding);
		this.updateAppearance();
	}

	/**
	 * Sets up the visual appearance of this button.
	 */
	private void setup() {
		this.setOpaque(false);
		this.setFocusPainted(false);
		this.setContentAreaFilled(false);
		this.setBackground(new Color(0f, 0f, 0f, 0f));

		this.addMouseListener(new MouseAdapter() {

			@Override
			@SuppressWarnings("synthetic-access")
			public void mouseEntered(MouseEvent e) {
				FlatButton.this.isMouseOver = true;
				FlatButton.this.updateAppearance();
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public void mouseExited(MouseEvent e) {
				FlatButton.this.isMouseOver = false;
				FlatButton.this.updateAppearance();
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public void mousePressed(MouseEvent e) {
				FlatButton.this.isMouseDown = true;
				FlatButton.this.updateAppearance();
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public void mouseReleased(MouseEvent e) {
				FlatButton.this.isMouseDown = false;
				FlatButton.this.updateAppearance();
			}

		});

		this.setPadding(4, 2);

	}

	/**
	 * Causes this button to update its appearance. This does not repaint the button, it merely sets
	 * the state correctly for the next time the button is painted.
	 */
	private void updateAppearance() {
		if (this.isMouseOver && this.isEnabled()) {
			if (this.isMouseDown) {
				this.setBorder(this.pressedBorder);
				this.backgroundOverlay = new Color(1f, 1f, 1f, 0.5f);
			} else {
				this.setBorder(this.mouseoverBorder);
				this.backgroundOverlay = new Color(1f, 1f, 1f, 0.25f);
			}
		} else {
			this.setBorder(this.normalBorder);
			this.backgroundOverlay = new Color(1f, 1f, 1f, 0f);
		}
	}

	/**
	 * Returns the base (effective) background color of this button. This will be transparent unless
	 * setBackground() has been called with a different color at some point in the past.
	 * 
	 * @see java.awt.Component#getBackground()
	 */
	@Override
	public Color getBackground() {
		return this.effectiveBackground;
	}

	/**
	 * Sets the base (effective) background color of this button to the given color, which may use
	 * an alpha channel for transparency.
	 * 
	 * @param bg the new background color
	 * 
	 * @see javax.swing.JComponent#setBackground(java.awt.Color)
	 */
	@Override
	public void setBackground(Color bg) {
		this.effectiveBackground = bg;
	}

	/**
	 * Paints this button.
	 * 
	 * @param g the graphics context to paint into
	 */
	@Override
	protected void paintComponent(Graphics g) {
		g.setColor(this.effectiveBackground);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		g.setColor(this.backgroundOverlay);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		// this.setBackground(this.backgroundOverlay);
		super.paintComponent(g);
		// this.setBackground(bg);
	}
}
