/*
 * File: IconButton.java
 * 
 * Contains class IconButton
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Dimension;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;

import edu.cmu.hcii.calo.view.IconRenderer.IconState;

/**
 * A simple transparency-aware push button that uses icons for its normal, mouseover, and pressed
 * states.
 * 
 * @author Brian Ellis
 * 
 */
@SuppressWarnings("serial")
public class IconButton extends JButton {

	/**
	 * The icon to use for this button in its normal state.
	 */
	private ImageIcon normalIcon;

	/**
	 * The icon to use for this button in its pressed state.
	 */
	private Icon pressedIcon;

	/**
	 * The icon to use for this button in its mouseover state.
	 */
	private Icon mouseoverIcon;

	/**
	 * Constructs an IconButton with the given normal icon, which must be an ImageIcon. Mouseover
	 * and pressed states are generated automatically.
	 * 
	 * @param normalIcon the normal icon
	 */
	public IconButton(ImageIcon normalIcon) {
		this.normalIcon = normalIcon;
		this.mouseoverIcon = IconRenderer.getInstance().renderIcon(normalIcon, IconState.MOUSEOVER);
		this.pressedIcon = IconRenderer.getInstance().renderIcon(normalIcon, IconState.PRESSED);

		this.setup();
	}

	/**
	 * Constructs an IconButton with the given normal icon, which must be an ImageIcon, and pressed
	 * icon. The mouseover state is generated automatically.
	 * 
	 * @param normalIcon the normal icon
	 * @param pressedIcon the pressed icon
	 * @param pressedIcon
	 */
	public IconButton(ImageIcon normalIcon, Icon pressedIcon) {
		this.normalIcon = normalIcon;
		this.mouseoverIcon = IconRenderer.getInstance().renderIcon(normalIcon, IconState.MOUSEOVER);
		this.pressedIcon = pressedIcon;

		this.setup();
	}

	/**
	 * Constructs a IconButton with the given normal, mouseover, and pressed icons.
	 * 
	 * @param normalIcon the normal icon
	 * @param mouseoverIcon the mouseover icon
	 * @param pressedIcon the pressed icon
	 */
	public IconButton(ImageIcon normalIcon, Icon mouseoverIcon, Icon pressedIcon) {
		super();

		this.normalIcon = normalIcon;
		this.pressedIcon = pressedIcon;
		this.mouseoverIcon = mouseoverIcon;

		this.setup();
	}

	/**
	 * Constructs a IconButton with the given normal, mouseover, and pressed icons and the given
	 * Action.
	 * 
	 * @param normalIcon the normal icon
	 * @param mouseoverIcon the mouseover icon
	 * @param pressedIcon the pressed icon
	 * @param a the Action to set
	 */
	public IconButton(ImageIcon normalIcon, Icon mouseoverIcon, Icon pressedIcon, Action a) {
		super(a);

		this.normalIcon = normalIcon;
		this.pressedIcon = pressedIcon;
		this.mouseoverIcon = mouseoverIcon;

		this.setup();
	}

	/**
	 * Returns the preferred size of this button. It is always the size of the icon, unless its
	 * preferred size has been explicitly set, in which case that value is returned instead.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		} else {
			return new Dimension(this.getIcon().getIconWidth(), this.getIcon().getIconHeight());
		}
	}

	/**
	 * Sets up the visual appearance of the button.
	 */
	private void setup() {

		this.setIcon(this.normalIcon);
		this.setPressedIcon(this.pressedIcon);
		this.setRolloverEnabled(true);
		this.setRolloverIcon(this.mouseoverIcon);

		this.setDisabledIcon(IconRenderer.getInstance().renderIcon(	this.normalIcon,
																	IconState.DISABLED));

		this.setBorder(null);
		this.setBorderPainted(false);
		this.setContentAreaFilled(false);
		this.setFocusPainted(false);
	}

}
