/* File: IconRenderer.java
 *
 * Contains class IconRenderer
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.font.GlyphMetrics;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.UIManager;

import sun.awt.image.ToolkitImage;

/**
 * A utility class that can generate alternate states for icons (including mouseover, pressed,
 * selected, and disabled states) based on the pixel data of the normal state, as well as badge them
 * and draw text on them.
 * 
 * @note Although you can freely create instances of IconRenderer, in most cases it is more
 *       efficient to call the static getInstance() method to return the shared IconRenderer.
 * 
 * @author Brian Ellis
 * 
 */
public class IconRenderer {

	/**
	 * The possibilities for badge positioning.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum BadgePosition {
		/**
		 * Place the badge in the top left corner.
		 */
		TOP_LEFT,
		/**
		 * Place the badge in the bottom left corner.
		 */
		BOTTOM_LEFT,
		/**
		 * Place the badge in the top right corner.
		 */
		TOP_RIGHT,
		/**
		 * Place the badge in the bottom right corner.
		 */
		BOTTOM_RIGHT,
		/**
		 * Place the badge in the center of the icon.
		 */
		CENTER
	}

	/**
	 * The states in which an icon may be rendered by the IconRenderer.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum IconState {
		/**
		 * The normal state. No transformation is applied to the icon.
		 */
		NORMAL,

		/**
		 * The disabled state. The icon is partially composited onto transparency.
		 */
		DISABLED,

		/**
		 * The mouseover icon. A super secret recipe is applied to the icon's pixel data to make it
		 * look even more clickable than usual. Try it and see.
		 */
		MOUSEOVER,

		/**
		 * The pressed icon. The icon is darkened.
		 */
		PRESSED,

		/**
		 * The selected icon. The icon is overlain with the system selection color, a la Windows
		 * desktop icons.
		 */
		SELECTED
	}

	/**
	 * The shared IconRenderer instance.
	 */
	private static IconRenderer sharedInstance = null;

	/**
	 * Returns the shared IconRenderer instance, creating it if necessary.
	 * 
	 * @return
	 */
	public static IconRenderer getInstance() {
		if (IconRenderer.sharedInstance == null) {
			IconRenderer.sharedInstance = new IconRenderer();
		}
		return IconRenderer.sharedInstance;
	}

	/**
	 * Badges the given base icon with the given badge icon, positioning the badge over the icon
	 * based on the position constant specified. Note that the badge icon should really be smaller
	 * than the base icon, or it won't look very good. In particular, the resulting badged icon will
	 * always be the same size as the base icon.
	 * 
	 * @param icon the base icon
	 * @param badge the badge icon to apply to the base icon
	 * @param position the position of the badge on the icon
	 * @return a badged version of the base icon
	 */
	public ImageIcon badgeIcon(ImageIcon icon, ImageIcon badge, BadgePosition position) {
		if (icon.getIconWidth() <= 0 || icon.getIconHeight() <= 0 || badge.getIconWidth() <= 0
				|| badge.getIconHeight() <= 0) {
			return icon;
		}

		BufferedImage img =
			new BufferedImage(	icon.getIconWidth(),
								icon.getIconHeight(),
								BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D)img.getGraphics();

		if (!g.drawImage(icon.getImage(), 0, 0, null)) {
			System.err.println("IconRenderer: Warning: icon image not entirely loaded!");
		}

		Point badgePosition;

		switch (position) {
		case TOP_LEFT:
			badgePosition = new Point(0, 0);
			break;
		case BOTTOM_LEFT:
			badgePosition = new Point(0, icon.getIconHeight() - badge.getIconHeight());
			break;
		case TOP_RIGHT:
			badgePosition = new Point(icon.getIconWidth() - badge.getIconWidth(), 0);
			break;
		case BOTTOM_RIGHT:
			badgePosition =
				new Point(icon.getIconWidth() - badge.getIconWidth(), icon.getIconHeight()
						- badge.getIconHeight());
			break;
		case CENTER:
			badgePosition =
				new Point(icon.getIconWidth() / 2 - badge.getIconWidth() / 2, icon.getIconHeight()
						/ 2 - badge.getIconHeight() / 2);
			break;
		default:
			assert false : "unknown badge position";
			badgePosition = new Point(0, 0);
			break;
		}

		g.drawImage(badge.getImage(), badgePosition.x, badgePosition.y, null);

		return new ImageIcon(img);
	}

	/**
	 * Draws the given text string in the center of the icon with the given font and color. The size
	 * of the font is ignored, however, and the String is drawn with a small enough size so as to
	 * completely fit within the bounds of the icon.
	 * 
	 * @param icon the icon to draw the text over
	 * @param text the text to draw
	 * @param font the font to draw the text in
	 * @param textColor the color to draw the text with
	 * @return the resulting icon
	 */
	public ImageIcon drawTextOnIcon(ImageIcon icon, String text, Font font, Color textColor) {
		if (icon.getIconWidth() <= 0 || icon.getIconHeight() <= 0) {
			return icon;
		}

		BufferedImage img =
			new BufferedImage(	icon.getIconWidth(),
								icon.getIconHeight(),
								BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D)img.getGraphics();

		if (!g.drawImage(icon.getImage(), 0, 0, null)) {
			System.err.println("IconRenderer: Warning: icon image not entirely loaded!");
		}

		g.setFont(font.deriveFont((float)icon.getIconHeight()));
		g.setColor(textColor);

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setRenderingHint(	RenderingHints.KEY_FRACTIONALMETRICS,
							RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		g.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);

		Rectangle2D fontRect = g.getFontMetrics().getStringBounds(text, g);
		while (fontRect.getWidth() > img.getWidth() - 2
				|| fontRect.getHeight() > img.getHeight() - 2) {
			if (fontRect.getWidth() / 2 > img.getWidth()
					|| fontRect.getHeight() / 2 > img.getHeight()) {
				g.setFont(font.deriveFont(g.getFont().getSize2D() / 2.0f));
			} else {
				g.setFont(font.deriveFont(g.getFont().getSize2D() - 1.0f));
			}
			fontRect = g.getFontMetrics().getStringBounds(text, g);
		}

		double maxHeight = 0.0;

		GlyphVector gv = g.getFont().createGlyphVector(g.getFontRenderContext(), text);
		for (int i = 0; i < gv.getNumGlyphs(); i++ ) {
			GlyphMetrics gm = gv.getGlyphMetrics(i);
			maxHeight = Math.max(maxHeight, gm.getBounds2D().getHeight());
		}

		AffineTransform originalTransform = g.getTransform();
		g.translate(img.getWidth() / 2 - fontRect.getWidth() / 2, img.getHeight() / 2 + maxHeight
				/ 2);
		g.drawString(text, 0.0f, 0.0f);
		g.setTransform(originalTransform);

		return new ImageIcon(img);
	}

	/**
	 * Renders the given ImageIcon transformed into the state specified, and returns the resulting
	 * icon. See the IconState enum for a description of the state constants.
	 * 
	 * @param icon the icon to render
	 * @param state the state to render the icon in.
	 * @return the transformed icon
	 */
	public ImageIcon renderIcon(ImageIcon icon, IconState state) {
		if (icon.getIconWidth() <= 0 || icon.getIconHeight() <= 0) {
			return icon;
		}

		BufferedImage img =
			new BufferedImage(	icon.getIconWidth(),
								icon.getIconHeight(),
								BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D)img.getGraphics();

		switch (state) {
		case NORMAL:
			// Really boring, just return the original icon:
			return icon;
		case DISABLED:
			// Boring, just draw the original icon at 50% opacity:
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			g.drawImage(icon.getImage(), 0, 0, null);
			return new ImageIcon(img);
		case MOUSEOVER:
			// We could do something pedestrian and boring for this, but let's
			// have a little fun! :)
			// We're going to convert every pixel in the normal icon to HSB,
			// then brighten it a bit
			// and bump up the saturation a lot, and then convert it back to RGB
			// and make a new icon
			// out of that. This creates a really neat mousover effect.

			// We need to get at the BufferedImage for this icon so we can frob
			// the pixels:
			BufferedImage buf;
			if (icon.getImage() instanceof ToolkitImage) {
				ToolkitImage iconImage = (ToolkitImage)icon.getImage();
				buf = iconImage.getBufferedImage();
			} else if (icon.getImage() instanceof BufferedImage) {
				buf = (BufferedImage)icon.getImage();
			} else {
				assert false : "could not get a buffered image from this icon";
				return icon;
			}
			int[] imageBytes =
				buf.getRGB(0, 0, img.getWidth(), img.getHeight(), null, 0, img.getWidth());

			// Now, for each pixel, convert the integer to a Color and convert
			// it to HSB:
			for (int x = 0; x < img.getWidth(); ++x) {
				for (int y = 0; y < img.getHeight(); ++y) {
					int pixel = imageBytes[y * img.getWidth() + x];
					Color pixelColor = new Color(pixel, true);
					float hsb[] =
						Color.RGBtoHSB(	pixelColor.getRed(),
										pixelColor.getGreen(),
										pixelColor.getBlue(),
										null);

					// Now we'll apply our transformations to the brightness
					// (hsb[2]) and saturation
					// (hsb[1]). We multiply rather than adding so that we don't
					// magnify random hue
					// variations in nearly-completely-gray pixels and create
					// the dreaded "angry
					// fruit salad" look.
					hsb[2] = Math.min(1.0f, hsb[2] * 1.1f);
					hsb[1] = Math.min(1.0f, hsb[1] * 2.0f);

					// Lastly, we'll convert everything back to RGB, salvage the
					// alpha value from
					// the original pixel (since we lose that in the
					// conversion), and apply it to
					// the pixel data of our output image (img).
					pixel = Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]);
					Color newColor = new Color(pixel);
					newColor =
						new Color(	newColor.getRed(),
									newColor.getGreen(),
									newColor.getBlue(),
									pixelColor.getAlpha());
					int newPixel = newColor.getRGB();
					img.setRGB(x, y, newPixel);
				}

			}

			// Now we return it. :)
			return new ImageIcon(img);
		case PRESSED:
			// Boring, just darken the icon a bit:
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
			g.drawImage(icon.getImage(), 0, 0, null);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, 0.5f));
			g.setColor(Color.BLACK);
			g.fillRect(0, 0, img.getWidth(), img.getHeight());
			return new ImageIcon(img);
		case SELECTED:
			// Shade it with the selection color (a la Windows Explorer):
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
			g.drawImage(icon.getImage(), 0, 0, null);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, 0.5f));
			g.setColor(UIManager.getColor("TextArea.selectionBackground"));
			g.fillRect(0, 0, img.getWidth(), img.getHeight());
			return new ImageIcon(img);

		default:
			assert false : "unrecognized icon state";
			return icon;
		}
	}
}
