/*
 * File: IconToggleButton.java
 * 
 * Contains class IconToggleButton
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Dimension;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;

import edu.cmu.hcii.calo.view.IconRenderer.IconState;

/**
 * A simple transparency-aware toggle button that uses icons for its normal, mouseover, and pressed
 * states in both selected and unselected modes.
 * 
 * @author Brian Ellis
 * 
 */
@SuppressWarnings("serial")
public class IconToggleButton extends JToggleButton {

	/**
	 * The text that should be shown in the tooltip for this button when it is unselected.
	 */
	private String unselectedToolTipText;

	/**
	 * The text that should be shown in the tooltip for this button when it is selected.
	 */
	private String selectedToolTipText;

	/**
	 * @name Icons
	 * 
	 * The following icons are stored because JToggleButton is only capable of keeping track of a
	 * single pressed icon, whereas we need two: one for when the button is selected, and one for
	 * when it is not.
	 */

	/**
	 * The icon that should be used for this button when it is pressed, but not selected.
	 */
	private Icon pressedIcon;

	/**
	 * The icon that should be used for this button when it is both pressed and selected.
	 */
	private Icon pressedSelectedIcon;

	/**
	 * Constructs an icon toggle button, initially unselected, with no action. The button has a
	 * normal icon for its unselected state, and a normal icon for its selected state. The button
	 * will generate icons for its rollover and pressed icons using the unselected icon when
	 * unselected, and using its selected icon when selected.
	 * 
	 * @param icon the icon to use when the button is not selected
	 * @param selectedIcon the icon to use when the button is selected
	 */
	public IconToggleButton(ImageIcon icon, ImageIcon selectedIcon) {
		setup(icon, null, null, selectedIcon, null, null);
	}

	/**
	 * Constructs an icon toggle button, initially unselected, with no action. The button has a
	 * normal icon for its unselected state, a normal icon for its selected state, and a pressed
	 * icon that is used in either state. The button will generate mouseover states using its
	 * unselected icon when unselected, and its selected icon when selected.
	 * 
	 * @param icon the icon to use when the button is not selected
	 * @param selectedIcon the icon to use when the button is selected
	 * @param pressedIcon the icon to use when the button is pressed and either selected or
	 *            unselected
	 */
	public IconToggleButton(ImageIcon icon, ImageIcon pressedIcon, ImageIcon selectedIcon) {
		setup(icon, null, pressedIcon, selectedIcon, null, null);
	}

	/**
	 * Constructs an icon toggle button, initially unselected, with no action. The button has a
	 * normal, rollover, and pressed icon for its unselected state, and a normal icon for its
	 * selected state. The button will use the unselected state rollover and pressed icons even when
	 * in the selected state.
	 * 
	 * @param icon the icon to use when the button is unselected
	 * @param rolloverIcon the icon to use when the button is in its mouseover state, whether
	 *            selected or not
	 * @param pressedIcon the icon to use when the button is pressed, whether selected or not
	 * @param selectedIcon the icon to use when the button is selected
	 */
	public IconToggleButton(ImageIcon icon,
			ImageIcon rolloverIcon,
			ImageIcon pressedIcon,
			ImageIcon selectedIcon) {
		this(icon, rolloverIcon, pressedIcon, selectedIcon, null, null);
	}

	/**
	 * Constructs an icon toggle button, initially unselected, with no action. The button has a
	 * normal, rollover, and pressed icon for both of its states (selected and unselected).
	 * 
	 * @note You can pass \c null for any of these arguments except \p icon. The button will use the
	 *       normal icon to intelligently generate states for any whose icons are null.
	 * 
	 * @param icon the normal icon
	 * @param rolloverIcon the mouseover icon, or \c null to automatically generate one
	 * @param pressedIcon the pressed icon, or \c null to automatically generate one
	 * @param selectedIcon the selected icon, or \c null to automatically generate one
	 * @param selectedRolloverIcon the selected mouseover icon, or \c null to automatically generate
	 *            one
	 * @param selectedPressedIcon the selected pressed icon, or \c null to automatically generate
	 *            one
	 */
	public IconToggleButton(ImageIcon icon,
			ImageIcon rolloverIcon,
			ImageIcon pressedIcon,
			ImageIcon selectedIcon,
			ImageIcon selectedRolloverIcon,
			ImageIcon selectedPressedIcon) {
		super();
		setup(	icon,
				rolloverIcon,
				pressedIcon,
				selectedIcon,
				selectedRolloverIcon,
				selectedPressedIcon);
	}

	/**
	 * Constructs an icon toggle button, initially unselected, with the given action. The button has
	 * a normal, rollover, and pressed icon for both of its states (selected and unselected).
	 * 
	 * @note You can pass \c null for any of the icon arguments except \p icon. The button will use
	 *       the normal icon to intelligently generate states for any whose icons are null.
	 * 
	 * @param icon the normal icon
	 * @param rolloverIcon the mouseover icon, or \c null to automatically generate one
	 * @param pressedIcon the pressed icon, or \c null to automatically generate one
	 * @param selectedIcon the selected icon, or \c null to automatically generate one
	 * @param selectedRolloverIcon the selected mouseover icon, or \c null to automatically generate
	 *            one
	 * @param selectedPressedIcon the selected pressed icon, or \c null to automatically generate
	 *            one
	 * @param a the Action to set
	 */
	public IconToggleButton(ImageIcon icon,
			ImageIcon rolloverIcon,
			ImageIcon pressedIcon,
			ImageIcon selectedIcon,
			ImageIcon selectedRolloverIcon,
			ImageIcon selectedPressedIcon,
			Action a) {
		super(a);
		setup(	icon,
				rolloverIcon,
				pressedIcon,
				selectedIcon,
				selectedRolloverIcon,
				selectedPressedIcon);
	}

	/**
	 * Constructs an icon toggle button, initially unselected, with no action. The button has a
	 * normal icon for its unselected state. The button will generate icons for all other states
	 * using this icon.
	 * 
	 * @param icon the icon to use when the button is not selected
	 */
	public IconToggleButton(ImageIcon icon) {
		this(icon, IconRenderer.getInstance().renderIcon(icon, IconState.SELECTED));
	}

	/**
	 * Returns the preferred size of this button. It is always the size of the icon, unless its
	 * preferred size has been explicitly set, in which case that value is returned instead.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		} else {
			return new Dimension(this.getIcon().getIconWidth(), this.getIcon().getIconHeight());
		}
	}

	/**
	 * Returns the pressed selected icon for the button.
	 * 
	 * @return the pressed selected icon
	 */
	public Icon getPressedSelectedIcon() {
		return this.pressedSelectedIcon;
	}

	/**
	 * Returns the pressed unselected icon for the button. Use this instead of
	 * AbstractButton#getPressedIcon() to ensure that you always get the unselected state
	 * (getPressedIcon() will return whichever icon corresponds to the current selection state of
	 * the button).
	 * 
	 * @return the pressed selected icon
	 */
	public Icon getPressedUnselectedIcon() {
		return this.pressedIcon;
	}

	/**
	 * Sets the pressed selected icon for the button.
	 * 
	 * @param pressedSelectedIcon the pressed selected icon
	 */
	public void setPressedSelectedIcon(Icon pressedSelectedIcon) {
		this.pressedSelectedIcon = pressedSelectedIcon;
	}

	/**
	 * Sets the pressed unselected icon for the button. Use this instead of
	 * AbstractButton#setPressedIcon() to avoid having your icon overwritten when the selected state
	 * changes.
	 * 
	 * @param pressedIcon the pressed icon
	 */
	public void setPressedUnselectedIcon(Icon pressedIcon) {
		this.pressedIcon = pressedIcon;
	}

	/**
	 * Sets the text that should be shown on the tooltip for this button when it is selected.
	 * 
	 * @param selectedToolTipText the selected tooltip text
	 */
	public void setSelectedToolTipText(String selectedToolTipText) {
		this.selectedToolTipText = selectedToolTipText;
	}

	/**
	 * Sets the text that should be shown on the tooltip for this button when it is selected.
	 * 
	 * @param unselectedToolTipText the unselected tooltip text
	 */
	public void setUnselectedToolTipText(String unselectedToolTipText) {
		this.unselectedToolTipText = unselectedToolTipText;
	}

	/**
	 * Fires the state changed event for this button. Overridden to also set the pressed icon and
	 * tooltip text correctly.
	 * 
	 * @see javax.swing.AbstractButton#fireStateChanged()
	 */
	@Override
	protected void fireStateChanged() {
		super.fireStateChanged();
		if (this.isSelected()) {
			this.setToolTipText(this.getSelectedToolTipText());
			this.setPressedIcon(this.getPressedSelectedIcon());
		} else {
			this.setToolTipText(this.getUnselectedToolTipText());
			this.setPressedIcon(this.getPressedUnselectedIcon());
		}
	}

	/**
	 * Returns the text that will be shown on the tooltip for this button when it is selected.
	 * 
	 * @return the selected tooltip text
	 */
	public String getSelectedToolTipText() {
		return this.selectedToolTipText;
	}

	/**
	 * Returns the text that will be shown on the tooltip for this button when it is not selected.
	 * Use this instead of JComponent#getTooltipText() to ensure that you always get the unselected
	 * tooltip text (getTooltipText() will return the text of whatever tooltip would be shown if the
	 * mouse were to hover over the control right now).
	 * 
	 * @return the unselected tooltip text
	 */
	public String getUnselectedToolTipText() {
		return this.unselectedToolTipText;
	}

	/**
	 * Sets up the visual appearance of this view.
	 * 
	 * @param icon the normal icon
	 * @param rolloverIcon the mouseover icon, or \c null to automatically generate one
	 * @param pressedIcon the pressed icon, or \c null to automatically generate one
	 * @param selectedIcon the selected icon, or \c null to automatically generate one
	 * @param rolloverSelectedIcon the selected mouseover icon, or \c null to automatically generate
	 *            one
	 * @param pressedSelectedIcon the selected pressed icon, or \c null to automatically generate
	 *            one
	 */
	private void setup(ImageIcon icon,
			ImageIcon rolloverIcon,
			ImageIcon pressedIcon,
			ImageIcon selectedIcon,
			ImageIcon rolloverSelectedIcon,
			ImageIcon pressedSelectedIcon) {

		if (icon == null) {
			throw new IllegalArgumentException("The unselected normal icon cannot be null.");
		}

		ImageIcon theRolloverIcon;
		if (rolloverIcon == null) {
			theRolloverIcon = IconRenderer.getInstance().renderIcon(icon, IconState.MOUSEOVER);
		} else {
			theRolloverIcon = rolloverIcon;
		}

		ImageIcon thePressedIcon;
		if (pressedIcon == null) {
			thePressedIcon = IconRenderer.getInstance().renderIcon(icon, IconState.PRESSED);
		} else {
			thePressedIcon = pressedIcon;
		}

		ImageIcon theSelectedIcon;
		if (selectedIcon == null) {
			theSelectedIcon = IconRenderer.getInstance().renderIcon(icon, IconState.SELECTED);
		} else {
			theSelectedIcon = selectedIcon;
		}

		ImageIcon theRolloverSelectedIcon;
		if (rolloverSelectedIcon == null) {
			theRolloverSelectedIcon =
				IconRenderer.getInstance().renderIcon(theSelectedIcon, IconState.MOUSEOVER);
		} else {
			theRolloverSelectedIcon = rolloverSelectedIcon;
		}

		ImageIcon thePressedSelectedIcon;
		if (pressedSelectedIcon == null) {
			thePressedSelectedIcon =
				IconRenderer.getInstance().renderIcon(theSelectedIcon, IconState.PRESSED);
		} else {
			thePressedSelectedIcon = pressedSelectedIcon;
		}

		this.pressedIcon = thePressedIcon;
		this.pressedSelectedIcon = thePressedSelectedIcon;

		this.setIcon(icon);
		this.setRolloverIcon(theRolloverIcon);
		this.setPressedIcon(thePressedIcon);
		this.setSelectedIcon(theSelectedIcon);
		this.setRolloverSelectedIcon(theRolloverSelectedIcon);
		this.setPressedSelectedIcon(thePressedSelectedIcon);

		this.setDisabledIcon(IconRenderer.getInstance().renderIcon(icon, IconState.DISABLED));

		this.setBorder(null);
		this.setBorderPainted(false);
		this.setContentAreaFilled(false);
		this.setFocusPainted(false);

		this.setToolTipText(this.getUnselectedToolTipText());
	}
}
