/* File: ImageStretchView.java
 *
 * Contains class ImageStretchView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.UIManager;

import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view class that takes up to nine images and lays them out in a grid consisting of four corners,
 * four stretch regions along the edge, and one stretch region in the middle. This is useful for
 * mocking up arbitrarily-sized views from image-based prototypes, though it is currently unused in
 * this application. It is also only somewhat tested.
 * 
 * @author Brian Ellis
 * 
 */
public class ImageStretchView extends CALOComponent {

	/**
	 * The rules that can be used to determine which corner images should overlap which others when
	 * the view is too small to show stretch regions.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum ImageOverlapPolicy {
		TOP_OVER_BOTTOM,
		BOTTOM_OVER_TOP,
		LEFT_OVER_RIGHT,
		RIGHT_OVER_LEFT
	}

	/**
	 * The rules that can be used to determine whether stretch regions should stretch or tile. This
	 * is currently ignored, and images are always stretched.
	 * 
	 * @author Administrator
	 * 
	 */
	public enum ImageStretchPolicy {
		STRETCH,
		TILE
	}

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = -4412068157568869562L;

	/**
	 * Tests ImageStretchView to make sure it works correctly.
	 * 
	 * @param args command-line arguments; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		JFrame testFrame = new JFrame("StretchView test");
		testFrame.setLayout(new RelativeLayout());

		ImageStretchView schedulePane = new ImageStretchView();

		schedulePane.setTopLeftImage(Toolkit.getDefaultToolkit().createImage("Resources/scheduleA.png"));
		schedulePane.setTopRightImage(Toolkit.getDefaultToolkit().createImage("Resources/scheduleB.png"));
		schedulePane.setTopStretchImage(Toolkit.getDefaultToolkit().createImage("Resources/scheduleAB.png"));
		Image whiteDot = Toolkit.getDefaultToolkit().createImage("Resources/whitedot.png");
		schedulePane.setBottomLeftImage(whiteDot);
		schedulePane.setBottomRightImage(whiteDot);
		schedulePane.setBottomStretchImage(whiteDot);
		schedulePane.setLeftStretchImage(whiteDot);
		schedulePane.setRightStretchImage(whiteDot);

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 4, 4);

		RelativeConstraints constraints =
			new RelativeConstraints(bm.topEdge(), bm.bottomEdge(), bm.leftEdge(), bm.rightEdge());

		testFrame.add(schedulePane, constraints);

		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setSize(500, 500);
		testFrame.setVisible(true);
	}

	/**
	 * A MediaTracker to load all our images when the view is constructed.
	 */
	private MediaTracker tracker = new MediaTracker(this);

	/**
	 * @name Images
	 * 
	 * The images that are shown in the view.
	 */

	// @{
	private Image topLeftImage;
	private Image topRightImage;
	private Image bottomLeftImage;
	private Image bottomRightImage;
	private Image topStretchImage;
	private Image leftStretchImage;
	private Image bottomStretchImage;
	private Image rightStretchImage;
	private Image centerStretchImage;
	// @}

	/**
	 * @name Policies
	 * 
	 * The overlap and stretch policies applied to the horizontal and vertical dimensions of the
	 * view.
	 */

	// @{
	private ImageOverlapPolicy horizontalOverlapPolicy = ImageOverlapPolicy.LEFT_OVER_RIGHT;
	private ImageOverlapPolicy verticalOverlapPolicy = ImageOverlapPolicy.TOP_OVER_BOTTOM;
	private ImageStretchPolicy horizontalStretchPolicy = ImageStretchPolicy.STRETCH;
	private ImageStretchPolicy verticalStretchPolicy = ImageStretchPolicy.STRETCH;
	private ImageStretchPolicy centerStretchPolicy = ImageStretchPolicy.STRETCH;

	// @}

	/**
	 * @name Image Accessors
	 * 
	 * Accessors for the various images.
	 */

	// @{
	public Image getBottomLeftImage() {
		return this.bottomLeftImage;
	}

	public Image getBottomRightImage() {
		return this.bottomRightImage;
	}

	public Image getBottomStretchImage() {
		return this.bottomStretchImage;
	}

	public Image getCenterStretchImage() {
		return this.centerStretchImage;
	}

	public Image getLeftStretchImage() {
		return this.leftStretchImage;
	}

	public Image getRightStretchImage() {
		return this.rightStretchImage;
	}

	public Image getTopLeftImage() {
		return this.topLeftImage;
	}

	public Image getTopRightImage() {
		return this.topRightImage;
	}

	public Image getTopStretchImage() {
		return this.topStretchImage;
	}

	// @}

	/**
	 * @name Policy Accessors
	 * 
	 * Accessors for the various overlap and stretch policies.
	 */

	// @{
	public ImageStretchPolicy getCenterStretchPolicy() {
		return this.centerStretchPolicy;
	}

	public ImageOverlapPolicy getHorizontalOverlapPolicy() {
		return this.horizontalOverlapPolicy;
	}

	public ImageStretchPolicy getHorizontalStretchPolicy() {
		return this.horizontalStretchPolicy;
	}

	public ImageOverlapPolicy getVerticalOverlapPolicy() {
		return this.verticalOverlapPolicy;
	}

	public ImageStretchPolicy getVerticalStretchPolicy() {
		return this.verticalStretchPolicy;
	}

	// @}

	/**
	 * Returns the preferred size of this component. Overridden to always return the width and
	 * height required to display all four corners with no stretch regions unless this component's
	 * preferred size has been explicitly set, in which case that value is returned instead.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		}
		int width =
			Math.max(	this.getTopLeftImage().getWidth(null)
								+ this.getTopRightImage().getWidth(null),
						this.getBottomLeftImage().getWidth(null)
								+ this.getBottomRightImage().getWidth(null));
		int height =
			Math.max(	this.getTopLeftImage().getHeight(null)
								+ this.getBottomLeftImage().getHeight(null),
						this.getTopRightImage().getHeight(null)
								+ this.getBottomRightImage().getHeight(null));

		return new Dimension(width, height);
	}

	/**
	 * @name Image Setters
	 * 
	 * Setters for the various images. None of these call repaint(), so you should do this yourself
	 * if you need the new image to display immediately.
	 */

	// @{
	public void setBottomLeftImage(Image bottomLeftImage) {
		this.bottomLeftImage = this.loadImage(bottomLeftImage);
	}

	public void setBottomRightImage(Image bottomRightImage) {
		this.bottomRightImage = this.loadImage(bottomRightImage);
	}

	public void setBottomStretchImage(Image bottomStretchImage) {
		this.bottomStretchImage = this.loadImage(bottomStretchImage);
	}

	public void setCenterStretchImage(Image centerStretchImage) {
		this.centerStretchImage = this.loadImage(centerStretchImage);
	}

	public void setLeftStretchImage(Image leftStretchImage) {
		this.leftStretchImage = this.loadImage(leftStretchImage);
	}

	public void setRightStretchImage(Image rightStretchImage) {
		this.rightStretchImage = this.loadImage(rightStretchImage);
	}

	public void setTopLeftImage(Image topLeftImage) {
		this.topLeftImage = this.loadImage(topLeftImage);

	}

	public void setTopRightImage(Image topRightImage) {
		this.topRightImage = this.loadImage(topRightImage);
	}

	public void setTopStretchImage(Image topStretchImage) {
		this.topStretchImage = this.loadImage(topStretchImage);
	}

	// @}

	/**
	 * @name Policy Setters
	 * 
	 * Setters for the various overlap and stretch policies. None of these call repaint(), so you
	 * should do this yourself if you need the new policy to take effect immediately.
	 */

	// @{
	public void setCenterStretchPolicy(ImageStretchPolicy centerStretchPolicy) {
		this.centerStretchPolicy = centerStretchPolicy;
	}

	public void setHorizontalOverlapPolicy(ImageOverlapPolicy horizontalOverlapPolicy) {
		this.horizontalOverlapPolicy = horizontalOverlapPolicy;
	}

	public void setHorizontalStretchPolicy(ImageStretchPolicy horizontalStretchPolicy) {
		this.horizontalStretchPolicy = horizontalStretchPolicy;
	}

	public void setVerticalOverlapPolicy(ImageOverlapPolicy verticalOverlapPolicy) {
		this.verticalOverlapPolicy = verticalOverlapPolicy;
	}

	public void setVerticalStretchPolicy(ImageStretchPolicy verticalStretchPolicy) {
		this.verticalStretchPolicy = verticalStretchPolicy;
	}

	// @}

	/**
	 * Paints this view.
	 * 
	 * @param g the graphics context to paint into
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		int topMinX = this.topLeftImage == null ? 0 : this.topLeftImage.getWidth(null);
		int topMaxX =
			this.topRightImage == null ? 0 : this.getWidth() - this.topRightImage.getWidth(null);
		int bottomMinX = this.bottomLeftImage == null ? 0 : this.bottomLeftImage.getWidth(null);
		int bottomMaxX =
			this.bottomRightImage == null ? 0 : this.getWidth()
					- this.bottomRightImage.getWidth(null);
		int leftMinY = this.topLeftImage == null ? 0 : this.topLeftImage.getHeight(null);
		int leftMaxY =
			this.bottomLeftImage == null ? 0 : this.getHeight()
					- this.bottomLeftImage.getHeight(null);
		int rightMinY = this.topRightImage == null ? 0 : this.topRightImage.getHeight(null);
		int rightMaxY =
			this.bottomRightImage == null ? 0 : this.getHeight()
					- this.bottomRightImage.getHeight(null);

		if (this.topStretchImage != null) {
			if (this.getHorizontalStretchPolicy() == ImageStretchPolicy.STRETCH) {
				g.drawImage(this.topStretchImage,
							topMinX,
							0,
							topMaxX,
							leftMinY,
							0,
							0,
							this.topStretchImage.getWidth(null),
							this.topStretchImage.getHeight(null),
							null);
			}
		}

		if (this.bottomStretchImage != null) {
			if (this.getHorizontalStretchPolicy() == ImageStretchPolicy.STRETCH) {
				g.drawImage(this.bottomStretchImage,
							bottomMinX,
							leftMaxY,
							bottomMaxX,
							this.getHeight(),
							0,
							0,
							this.bottomStretchImage.getWidth(null),
							this.bottomStretchImage.getHeight(null),
							null);
			}
		}

		if (this.leftStretchImage != null) {
			if (this.getVerticalStretchPolicy() == ImageStretchPolicy.STRETCH) {
				g.drawImage(this.leftStretchImage,
							0,
							leftMinY,
							topMaxX,
							leftMaxY,
							0,
							0,
							this.leftStretchImage.getWidth(null),
							this.leftStretchImage.getHeight(null),
							null);
			}
		}

		if (this.rightStretchImage != null) {
			if (this.getVerticalStretchPolicy() == ImageStretchPolicy.STRETCH) {
				g.drawImage(this.rightStretchImage,
							topMaxX,
							rightMinY,
							this.getWidth(),
							rightMaxY,
							0,
							0,
							this.rightStretchImage.getWidth(null),
							this.rightStretchImage.getHeight(null),
							null);
			}
		}

		if (this.centerStretchImage != null) {
			if (this.getCenterStretchPolicy() == ImageStretchPolicy.STRETCH) {
				g.drawImage(this.centerStretchImage,
							topMinX,
							leftMinY,
							topMaxX,
							leftMaxY,
							0,
							0,
							this.centerStretchImage.getWidth(null),
							this.centerStretchImage.getHeight(null),
							null);
			}
		}

		// Figure out what order to draw the non-stretchy images in:
		List<Image> allImages = this.getOrderedStaticImages();

		// Draw the static images in the appropriate locations:
		for (Image image : allImages) {
			if (image == null) {
				continue;
			}
			if (image == this.topLeftImage) {
				g.drawImage(image, 0, 0, null);
			} else if (image == this.topRightImage) {
				g.drawImage(image, this.getWidth() - image.getWidth(null), 0, null);
			} else if (image == this.bottomLeftImage) {
				g.drawImage(image, this.getHeight() - image.getHeight(null), 0, null);
			} else if (image == this.bottomRightImage) {
				g.drawImage(image, this.getWidth() - image.getWidth(null), this.getHeight()
						- image.getHeight(null), null);
			} else {
				assert false : "tried to draw a non-static image in the static drawing phase";
			}
		}

	}

	/**
	 * Returns all the static (corner) images in the order they should be drawn given the current
	 * overlap policies.
	 * 
	 * @return an ordered list of the static images
	 */
	private List<Image> getOrderedStaticImages() {
		List<Image> topImages = new ArrayList<Image>(2);
		topImages.add(this.topLeftImage);
		if (this.getHorizontalOverlapPolicy() == ImageOverlapPolicy.RIGHT_OVER_LEFT) {
			topImages.add(this.topRightImage);
		} else {
			topImages.add(0, this.topRightImage);
		}
		List<Image> bottomImages = new ArrayList<Image>(2);
		bottomImages.add(this.bottomLeftImage);
		if (this.getHorizontalOverlapPolicy() == ImageOverlapPolicy.RIGHT_OVER_LEFT) {
			bottomImages.add(this.bottomRightImage);
		} else {
			bottomImages.add(0, this.bottomRightImage);
		}

		List<Image> allImages = new ArrayList<Image>(4);
		if (this.getVerticalOverlapPolicy() == ImageOverlapPolicy.BOTTOM_OVER_TOP) {
			allImages.addAll(topImages);
			allImages.addAll(bottomImages);
		} else {
			allImages.addAll(bottomImages);
			allImages.addAll(topImages);
		}
		return allImages;
	}

	/**
	 * Causes the view to load the given image and wait for it to be fully rendered before
	 * continuing.
	 * 
	 * @param img the image to load
	 */
	private Image loadImage(Image img) {
		this.tracker.addImage(img, 0);
		try {
			this.tracker.waitForID(0);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return img;
	}
}
