/* File: PlaceholderTextField.java
 *
 * Contains class PlaceholderTextField
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.text.Document;

/**
 * A text field that displays placeholder text when it is empty and not focused.
 * 
 * @author Brian Ellis
 * 
 */
public class PlaceholderTextField extends JTextField {

	/**
	 * The color of the placeholder text.
	 */
	protected static final Color PLACEHOLDER_TEXT_COLOR = Color.GRAY;
	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -798004097970200065L;

	/**
	 * The placeholder text.
	 */
	private String placeholderText = "";

	/**
	 * The actual foreground color of the text in the field when it's focused.
	 */
	private Color actualForeground;

	/**
	 * Constructs a PlaceholderTextField with a blank placeholder text and no initial text.
	 */
	public PlaceholderTextField() {
		super();
		this.setup();
	}

	/**
	 * Constructs a PlaceholderTextField with the given document, text, and width expressed in
	 * columns.
	 * 
	 * @param doc the document
	 * @param text the text
	 * @param columns the width
	 */
	public PlaceholderTextField(Document doc, String text, int columns) {
		super(doc, text, columns);
		this.setup();
	}

	/**
	 * Constructs a PlaceholderTextField with a blank placeholder text, no initial text, and the
	 * given width expressed in columns.
	 * 
	 * @param columns the width
	 */
	public PlaceholderTextField(int columns) {
		super(columns);
		this.setup();
	}

	/**
	 * Constructs a PlaceholderTextField with a blank placeholder text and the given initial text.
	 * 
	 * @param text the text
	 */
	public PlaceholderTextField(String text) {
		super(text);
		this.setup();
	}

	/**
	 * Constructs a PlaceholderTextField with a blank placeholder text, the given initial text, and
	 * the given width expressed in columns.
	 * 
	 * @param text the text
	 * @param columns the width
	 */
	public PlaceholderTextField(String text, int columns) {
		super(text, columns);
		this.setup();
	}

	/**
	 * Gets the placeholder text for this field.
	 * 
	 * @return the placeholder text
	 */
	public String getPlaceholderText() {
		return this.placeholderText;
	}

	/**
	 * Sets the placeholder text for this field to the given string.
	 * 
	 * @param placeholderText the placeholder text to set
	 */
	public void setPlaceholderText(String placeholderText) {
		this.placeholderText = placeholderText;
		this.displayPlaceholderText();
	}

	/**
	 * Displays the placeholder text in the text field, setting the foreground color correctly, if
	 * and only if the field is empty.
	 */
	private void displayPlaceholderText() {
		if (this.getText().equals("")) {
			this.setForeground(PlaceholderTextField.PLACEHOLDER_TEXT_COLOR);
			this.setText(PlaceholderTextField.this.placeholderText);
		}
	}

	/**
	 * Sets up the visual appearance of this field.
	 */
	private void setup() {

		this.actualForeground = this.getForeground();
		this.displayPlaceholderText();

		this.addFocusListener(new FocusListener() {

			/**
			 * Clears out the placeholder text and sets the foreground color correctly when the
			 * field is focused.
			 * 
			 * @param e the focus event
			 */
			@SuppressWarnings("synthetic-access")
			public void focusGained(FocusEvent e) {
				PlaceholderTextField.this.setForeground(PlaceholderTextField.this.actualForeground);
				if (PlaceholderTextField.this.getText().equals(PlaceholderTextField.this.placeholderText)) {
					PlaceholderTextField.this.setText("");
				}
			}

			@SuppressWarnings("synthetic-access")
			public void focusLost(FocusEvent e) {
				PlaceholderTextField.this.displayPlaceholderText();
			}

		});
	}

}
