/*
 * File: PropertyChangeForwarder.java
 * 
 * Contains class PropertyChangeForwarder
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * A delegate class that will take all property change events it receives and rebroadcast them on
 * behalf of its delegator. More precisely, when any PropertyChangeEvent is received by this class,
 * the CALOComponent#firePropertyChange(String, Object, Object) method will be called on the
 * delegator instance with the same property name and values as the event that was received.
 * 
 * @author Brian Ellis
 * 
 */
public class PropertyChangeForwarder implements PropertyChangeListener {

	/**
	 * The delegator for this forwarder.
	 */
	private CALOComponent delegator;

	/**
	 * Constructs a PropertyChangeForwarder that will forward any property change events it receives
	 * such that they appear to have emanated from the given delegator component. The delegator must
	 * be a CALOComponent because otherwise the firePropertyChange(String, Object, Object) method is
	 * protected rather than public, preventing us from forwarding property change events that have
	 * Object values.
	 * 
	 * @param delegator the delegator
	 */
	public PropertyChangeForwarder(CALOComponent delegator) {
		super();
		this.delegator = delegator;
	}

	/**
	 * Invoked whenever a property change event occurs on a component to which this listener is
	 * attached. Forwards the event on to the delegator.
	 * 
	 * @param e the property change event
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent e) {
		this.delegator.firePropertyChange(e.getPropertyName(), e.getOldValue(), e.getNewValue());
	}

}
