/*
 * File: SearchField.java
 * 
 * Contains class SearchField
 * 
 * Author: Elliott Hughes (modified by Brian Ellis)
 */

package edu.cmu.hcii.calo.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.MouseInputListener;

/**
 * 
 * A text field for search/filter interfaces. The extra functionality includes a placeholder string
 * (when the user hasn't yet typed anything), and a button to clear the currently-entered text.
 * 
 * Largely undocumented due to the fact that I (Brian) did not write it.
 * 
 * @author Elliott Hughes
 * @author Brian Ellis (modifications)
 * 
 */

//
// TODO: add a menu of recent searches.
// TODO: make recent searches persistent.
// TODO: use rounded corners, at least on Mac OS X.
//
public class SearchField extends JTextField {

	/**
	 * 
	 * Draws the cancel button as a gray circle with a white cross inside.
	 * 
	 */
	static class CancelBorder extends EmptyBorder {

		/** Prevent serialization collisions. */
		private static final long serialVersionUID = -9024139253534811547L;

		/**
		 * A gray color for drawing the pressed cancel button.
		 */
		private static final Color GRAY = new Color(0.7f, 0.7f, 0.7f);

		/**
		 * Constructs a CancelBorder.
		 */
		CancelBorder() {
			super(0, 0, 0, 15);
		}

		/**
		 * Paints the border.
		 */
		@SuppressWarnings("synthetic-access")
		@Override
		public void paintBorder(Component c,
				Graphics oldGraphics,
				int x,
				int y,
				int width,
				int height) {

			SearchField field = (SearchField)c;

			if (field.showingPlaceholderText || field.getText().length() == 0) {

				return;

			}

			Graphics2D g = (Graphics2D)oldGraphics;

			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

			final int circleL = 14;

			final int circleX = x + width - circleL;

			final int circleY = y + (height - 1 - circleL) / 2;

			g.setColor(field.armed ? Color.GRAY : CancelBorder.GRAY);

			g.fillOval(circleX, circleY, circleL, circleL);

			final int lineL = circleL / 2 - 2;

			final int lineX = circleX + (circleL / 4) + 1;

			final int lineY = circleY + (circleL / 4) + 1;

			g.setColor(Color.WHITE);
			g.setStroke(new BasicStroke(1.5f));

			g.drawLine(lineX, lineY, lineX + lineL, lineY + lineL);

			g.drawLine(lineX, lineY + lineL, lineX + lineL, lineY);

		}

	}

	/**
	 * 
	 * Handles a click on the cancel button by clearing the text and notifying any ActionListeners.
	 * 
	 */
	class CancelListener extends MouseInputAdapter {

		/**
		 * Invoked when the mouse is dragged.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseDragged(MouseEvent e) {
			if (this.isOverButton(e)) {
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			} else {
				setCursor(new Cursor(Cursor.TEXT_CURSOR));
			}
			arm(e);

		}

		/**
		 * Invoked when the mouse enters the field.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseEntered(MouseEvent e) {

			arm(e);

		}

		/**
		 * Invoked when the mouse exits the field.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseExited(MouseEvent e) {

			disarm();

		}

		/**
		 * Invoked when the mouse is moved inside the field.
		 * 
		 * @param e the mouse event
		 * 
		 * @see java.awt.event.MouseAdapter#mouseMoved(java.awt.event.MouseEvent)
		 */
		@Override
		public void mouseMoved(MouseEvent e) {
			if (this.isOverButton(e)) {
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			} else {
				setCursor(new Cursor(Cursor.TEXT_CURSOR));
			}
		}

		/**
		 * Invoked when the mouse is pressed inside the field.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mousePressed(MouseEvent e) {

			arm(e);

		}

		/**
		 * Invoked when the mouse is released inside the field.
		 * 
		 * @param e the mouse event
		 */
		@SuppressWarnings("synthetic-access")
		@Override
		public void mouseReleased(MouseEvent e) {

			if (SearchField.this.armed) {

				cancel();

			}

			disarm();

		}

		/**
		 * "Arms" the cancel button if the mouse event was over it. Armed means that when the mouse
		 * is released, the field will be cleared.
		 * 
		 * @param e the mouse event
		 */
		@SuppressWarnings("synthetic-access")
		private void arm(MouseEvent e) {

			SearchField.this.armed = (isOverButton(e) && SwingUtilities.isLeftMouseButton(e));

			repaint();

		}

		/**
		 * "Disarms" the cancel button. See above for definition of "armed".
		 */
		@SuppressWarnings("synthetic-access")
		private void disarm() {

			SearchField.this.armed = false;

			repaint();

		}

		/**
		 * Returns whether the mouse event was over the cancel button.
		 * 
		 * @param e the mouse event
		 * @return \c true if the mouse event was over the cancel button, \c false otherwise
		 */
		private boolean isOverButton(MouseEvent e) {

			// If the button is down, we might be outside the component

			// without having had mouseExited invoked.

			if (contains(e.getPoint()) == false) {

				return false;

			}

			// In lieu of proper hit-testing for the circle, check that

			// the mouse is somewhere in the border.

			Rectangle innerArea = SwingUtilities.calculateInnerArea(SearchField.this, null);

			if (innerArea.contains(e.getPoint())) {
				return false;
			}

			if (new Rectangle(innerArea.width, 0, 25, innerArea.height + 10).contains(e.getPoint())) {
				return true;
			}

			return false;

		}

	}

	/**
	 * 
	 * Replaces the entered text with a gray placeholder string when the search field doesn't have
	 * the focus. The entered text returns when we get the focus back.
	 * 
	 */
	class PlaceholderText implements FocusListener {

		/**
		 * The placeholder text.
		 */
		private String placeholderText;

		/**
		 * The previous text that was in the field.
		 */
		private String previousText = "";

		/**
		 * The previous color of the field's text.
		 */
		private Color previousColor;

		/**
		 * Constructs a PlaceholderText with the given, er, placeholder text.
		 * 
		 * @param placeholderText the placeholder text
		 */
		PlaceholderText(String placeholderText) {

			this.placeholderText = placeholderText;

			focusLost(null);

		}

		/**
		 * Restores the field to a focused appearance.
		 */
		@SuppressWarnings("synthetic-access")
		private void doFocusGained() {
			setForeground(this.previousColor);

			setText(this.previousText);

			SearchField.this.showingPlaceholderText = false;

		}

		/**
		 * Invoked when the field is focused.
		 * 
		 * @param e the focus event
		 */
		@SuppressWarnings("synthetic-access")
		public void focusGained(FocusEvent e) {
			this.doFocusGained();
		}

		/**
		 * Invoked when the field is unfocused.
		 * 
		 * @param e the focus event
		 */
		@SuppressWarnings("synthetic-access")
		public void focusLost(FocusEvent e) {

			this.previousText = getText();

			this.previousColor = getForeground();

			if (this.previousText.length() == 0) {

				SearchField.this.showingPlaceholderText = true;

				setForeground(Color.GRAY);

				setText(this.placeholderText);

			}

		}

	}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 8584952661844977286L;

	/**
	 * The border of this field.
	 */
	private static final Border CANCEL_BORDER = new CancelBorder();

	/**
	 * Whether the field posts action events every time the text changes (as opposed to just when
	 * the user unfocuses the field or hits return).
	 */
	private boolean sendsNotificationForEachKeystroke = false;

	/**
	 * Whether the field is showing placeholder text.
	 */
	private boolean showingPlaceholderText = false;

	/**
	 * Whether the cancel button is armed.
	 */
	private boolean armed = false;

	/**
	 * The placeholder text listener.
	 */
	private PlaceholderText placeholderTextListener;

	/**
	 * Constructs a search field with placeholder text of "Search".
	 */
	public SearchField() {

		this("Search");

	}

	/**
	 * Constructs a search field with the given placeholder text.
	 * 
	 * @param placeholderText the placeholder text to use
	 */
	public SearchField(String placeholderText) {

		super(15);

		this.placeholderTextListener = new PlaceholderText(placeholderText);
		addFocusListener(this.placeholderTextListener);

		initBorder();

		initKeyListener();

	}

	/**
	 * Forces the field to gain focus.
	 */
	@SuppressWarnings("synthetic-access")
	public void forceGrabFocus() {
		this.placeholderTextListener.doFocusGained();
	}

	/**
	 * Sets whether the field posts an action event after each keystroke, as opposed to just when
	 * the field loses focus or the user hits the return key.
	 * 
	 * @param eachKeystroke \c true if the field should post an action event after each keystroke,
	 *            \c false otherwise
	 */
	public void setSendsNotificationForEachKeystroke(boolean eachKeystroke) {

		this.sendsNotificationForEachKeystroke = eachKeystroke;

	}

	/**
	 * Causes the search field to clear out its text.
	 */
	private void cancel() {

		setText("");

		postActionEvent();

	}

	/**
	 * Initializes the field's border.
	 */
	private void initBorder() {

		setBorder(new CompoundBorder(getBorder(), SearchField.CANCEL_BORDER));

		MouseInputListener mouseInputListener = new CancelListener();

		addMouseListener(mouseInputListener);

		addMouseMotionListener(mouseInputListener);

	}

	/**
	 * Initializes the key listener for this field.
	 */
	private void initKeyListener() {

		addKeyListener(new KeyAdapter() {

			@SuppressWarnings("synthetic-access")
			@Override
			public void keyReleased(KeyEvent e) {

				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {

					cancel();

				} else if (SearchField.this.sendsNotificationForEachKeystroke) {

					maybeNotify();

				}

			}

		});

	}

	/**
	 * Posts an action event if the field is not showing placeholder text.
	 */
	private void maybeNotify() {

		if (this.showingPlaceholderText) {

			return;

		}

		postActionEvent();

	}

}
