/*
 * File: TranslucentView.java
 * 
 * Contains class TranslucentView
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

/**
 * A simple JComponent subclass that supports composited (i.e., translucent) drawing using an image
 * buffer.
 * 
 * @author Brian Ellis
 * 
 */
public class TranslucentView extends JComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -6453869157181923665L;

	/**
	 * The current opacity of this view.
	 */
	private double opacity = 1.0;

	/**
	 * Constructs a fully opaque TranslucentView.
	 */
	public TranslucentView() {
		super();
	}

	/**
	 * Returns the opacity of this view. 0.0 is completely transparent, and 1.0 is completely
	 * opaque.
	 * 
	 * @return the opacity
	 */
	public double getOpacity() {
		return this.opacity;
	}

	/**
	 * Paints this view. If the view's opacity is at least 1.0 (fully opaque), the view is painted
	 * normally. Otherwise, it is painted to an offscreen image buffer and then composited into
	 * place with the given transparency, and will therefore appear translucent. Note that this is
	 * somewhat more expensive than painting normally.
	 * 
	 * @param g the graphics context to paint into
	 */
	@Override
	public void paint(Graphics g) {
		if (this.opacity >= 1.0) {
			super.paint(g);
		} else {
			Image img =
				new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_4BYTE_ABGR);
			super.paint(img.getGraphics());
			Graphics2D g2 = (Graphics2D)g.create();
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)this.opacity));
			g2.drawImage(img, 0, 0, null);
			g2.dispose();
		}
	}

	/**
	 * Paints this component. Overridden to ensure that this component's background is fully filled
	 * in with the background color if it is marked as "opaque" (which is different from having an
	 * opacity of 1.0).
	 * 
	 * @param g the graphics context to paint into
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (this.isOpaque()) {
			g.setColor(this.getBackground());
			g.fillRect(0, 0, this.getWidth(), this.getHeight());
		}
	}

	/**
	 * Sets the opacity of this TranslucentView. 0.0 is completely transparent, and 1.0 is
	 * completely opaque.
	 * 
	 * @param opacity the opacity to set
	 */
	public void setOpacity(double opacity) {
		this.opacity = opacity;
		this.repaint();
	}

}
