/* File: Animation.java
 *
 * Contains class Animation
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.animation;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Calendar;

/**
 * Represents an animation, a smooth change in some set of numeric values. Animations are created by
 * the AnimationManager. Animations can be used to animate individual scalar values, arbitrary
 * arrays of these values, or the special cases of Points and Rectangles (modeled using Point2D and
 * Rectangle2D). All of these are modeled to double precision. You can therefore use Animations to
 * animate changes in size, position, bounds, color, or anything else you can represent with
 * numbers. When you call start() on an Animation, you must pass an AnimationListener that will be
 * notified when that Animation changes by calls to the
 * AnimationListener#animationDidUpdate(Animation) method. This method will be called as frequently
 * as possible throughout the duration of the Animation, and will be called at least once when the
 * animation is complete. You can implement this method to do whatever is required with the current
 * value of the Animation's modeled state, such as setting the size of a component to the current
 * Rectangle or setting the color from an array of three scalar values representing red, green, and
 * blue.
 * 
 * @author Brian Ellis
 */
public class Animation {

	/**
	 * Represents the mathematical equation used to determine how the animation progresses.
	 * 
	 * @author Brian Ellis
	 */
	public enum AnimationCurve {
		/**
		 * The animation is linear, progressing smoothly from the beginning to the end.
		 */
		LINEAR,
		/**
		 * The animation starts out slow and speeds up at a constant rate until the end.
		 */
		EASE_IN,
		/**
		 * The animation starts out fast and slows down at a constant rate until the end.
		 */
		EASE_OUT,
		/**
		 * The animation starts and ends slowly, but speeds up in the middle. It will be fastest at
		 * the halfway point, following a sinusoidal curve.
		 */
		EASE_IN_OUT,
		/**
		 * The animation starts at 0 and pulses 5 times over the course of the animation, ending at
		 * 1.
		 */
		PULSE,
		/**
		 * The animation starts at 0, fades up to 1, stays for a bit, then fades back to 0
		 */
		GROWL
	}

	/**
	 * Represents the type of Animation, defined by the sort of thing that is being animated.
	 * 
	 * @author Brian Ellis
	 */
	public enum AnimationType {
		/**
		 * We are animating a single double value.
		 */
		SCALAR,
		/**
		 * We are animating a Point.
		 */
		POINT,
		/**
		 * We are animating a Rectangle.
		 */
		RECTANGLE,
		/**
		 * We are animating an array of scalars (double values).
		 */
		ARRAY
	}

	/**
	 * The point in time at which this animation was started.
	 */
	private Calendar startTime;

	/**
	 * How long (in seconds) this animation should take.
	 */
	private double duration = 1.0;

	/**
	 * The type of this animation.
	 */
	private AnimationType animationType;

	/**
	 * The curve for this animation.
	 */
	private AnimationCurve animationCurve;

	/**
	 * The initial state of this animation, which is always represented as an array of doubles.
	 */
	private double[] initialState;

	/**
	 * The final state of this animation, which is always represented as an array of doubles.
	 */
	private double[] finalState;

	/**
	 * The timer assigned to this animation, if any. This field exists solely to ensure that the
	 * timer doesn't get garbage collected before the animation it controls is finished.
	 */
	@SuppressWarnings("unused")
	private AnimationTimer timer;

	/**
	 * The listener for this animation. For simplicity, there can only be one AnimationListener at a
	 * time for any given Animation.
	 */
	private AnimationListener listener;

	/**
	 * Whether this animation is complete.
	 */
	private boolean isComplete = false;

	/**
	 * The arbitrary object data associated with this Animation, if any.
	 */
	private Object objectData;

	/**
	 * Constructs an Animation designed to animate a single scalar value.
	 * 
	 * @note External classes should use the AnimationManager to create Animations, which is why
	 *       this constructor is package protected.
	 * @param initialValue the initial value of this scalar
	 * @param finalValue the final value of this scalar
	 * @param curve the AnimationCurve to use for this Animation
	 * @param duration the intended duration of this Animation
	 */
	Animation(double initialValue, double finalValue, AnimationCurve curve, double duration) {
		this.initialState = new double[] { initialValue };
		this.finalState = new double[] { finalValue };
		this.animationType = AnimationType.SCALAR;
		this.animationCurve = curve;
		this.duration = duration;
	}

	/**
	 * Constructs an Animation designed to animate an arbitrary list of scalar values.
	 * 
	 * @note External classes should use the AnimationManager to create Animations, which is why
	 *       this constructor is package protected.
	 * @param initialValues the initial values of the array
	 * @param finalValues the final values of the array
	 * @param curve the AnimationCurve to use for this Animation
	 * @param duration the intended duration of this Animation
	 */
	Animation(double[] initialValues, double[] finalValues, AnimationCurve curve, double duration) {
		this.initialState = initialValues;
		this.finalState = finalValues;
		this.animationType = AnimationType.ARRAY;
		this.animationCurve = curve;
		this.duration = duration;
	}

	/**
	 * Constructs an Animation designed to animate a point (or a size, or any other two-element pair
	 * for that matter).
	 * 
	 * @note External classes should use the AnimationManager to create Animations, which is why
	 *       this constructor is package protected.
	 * @param initialPoint the initial Point
	 * @param finalPoint the final Point
	 * @param curve the AnimationCurve to use for this Animation
	 * @param duration the intended duration of this Animation
	 */
	Animation(Point2D initialPoint, Point2D finalPoint, AnimationCurve curve, double duration) {
		this.initialState = new double[] { initialPoint.getX(), initialPoint.getY() };
		this.finalState = new double[] { finalPoint.getX(), finalPoint.getY() };
		this.animationType = AnimationType.POINT;
		this.animationCurve = curve;
		this.duration = duration;
	}

	/**
	 * Constructs an Animation designed to animate a rectangle (or any other four-element tuple).
	 * 
	 * @note External classes should use the AnimationManager to create Animations, which is why
	 *       this constructor is package protected.
	 * @param initialRect the initial Rectangle
	 * @param finalRect the final Rectangle
	 * @param curve the AnimationCurve to use for this Animation
	 * @param duration the intended duration of this Animation
	 */
	Animation(Rectangle2D initialRect, Rectangle2D finalRect, AnimationCurve curve, double duration) {
		this.initialState =
			new double[] { initialRect.getMinX(), initialRect.getMinY(), initialRect.getMaxX(),
					initialRect.getMaxY() };
		this.finalState =
			new double[] { finalRect.getMinX(), finalRect.getMinY(), finalRect.getMaxX(),
					finalRect.getMaxY() };
		this.animationType = AnimationType.RECTANGLE;
		this.animationCurve = curve;
		this.duration = duration;
	}

	/**
	 * Gets the animation curve for this animation.
	 * 
	 * @return the animation curve
	 */
	public AnimationCurve getAnimationCurve() {
		return this.animationCurve;
	}

	/**
	 * Gets the type of this animation.
	 * 
	 * @return the animation type
	 */
	public AnimationType getAnimationType() {
		return this.animationType;
	}

	/**
	 * Returns the current value of the Point that is being animated.
	 * 
	 * @return the current value of the Point
	 * @throws IllegalStateException if this Animation is not currently animating a Point.
	 */
	public Point2D getCurrentPoint() {
		if (this.animationType != AnimationType.POINT) {
			throw new IllegalStateException("Tried to get a Point value from an Animation with a type of "
					+ this.animationType);
		}
		double x =
			this.getInterpolatedValue(this.initialState[0], this.finalState[0], this.getProgress());
		double y =
			this.getInterpolatedValue(this.initialState[1], this.finalState[1], this.getProgress());
		return new Point2D.Double(x, y);
	}

	/**
	 * Returns the current value of the Rectangle that is being animated.
	 * 
	 * @return the current value of the Rectangle
	 * @throws IllegalStateException if this Animation is not currently animating a Rectangle.
	 */
	public Rectangle2D getCurrentRectangle() {
		if (this.animationType != AnimationType.RECTANGLE) {
			throw new IllegalStateException("Tried to get a Rectangle value from an Animation with a type of "
					+ this.animationType);
		}
		double left =
			this.getInterpolatedValue(this.initialState[0], this.finalState[0], this.getProgress());
		double top =
			this.getInterpolatedValue(this.initialState[1], this.finalState[1], this.getProgress());
		double right =
			this.getInterpolatedValue(this.initialState[2], this.finalState[2], this.getProgress());
		double bottom =
			this.getInterpolatedValue(this.initialState[3], this.finalState[3], this.getProgress());
		return new Rectangle2D.Double(left, top, right - left, bottom - top);
	}

	/**
	 * Returns the current value of the single scalar that is being animated.
	 * 
	 * @return the current value of the scalar
	 * @throws IllegalStateException if this Animation is not currently animating a single scalar
	 *             value.
	 */
	public double getCurrentValue() throws IllegalStateException {
		if (this.animationType != AnimationType.SCALAR) {
			throw new IllegalStateException("Tried to get a scalar value from an Animation with a type of "
					+ this.animationType);
		}
		return this.getInterpolatedValue(	this.initialState[0],
											this.finalState[0],
											this.getProgress());
	}

	/**
	 * Returns the current value of the array of scalars that is being animated.
	 * 
	 * @return the current value of the scalar array
	 * @throws IllegalStateException if this Animation is not currently animating an array of scalar
	 *             values.
	 */
	public double[] getCurrentValues() {
		if (this.animationType != AnimationType.ARRAY) {
			throw new IllegalStateException("Tried to get an array of values from an Animation with a type of "
					+ this.animationType);
		}
		double[] currentValues = new double[this.initialState.length];
		for (int i = 0; i < this.initialState.length; i++ ) {
			currentValues[i] =
				this.getInterpolatedValue(	this.initialState[i],
											this.finalState[i],
											this.getProgress());
		}
		return currentValues;
	}

	/**
	 * Gets the duration of this Animation. Note that this means the full duration even if the
	 * Animation has been started, and not just the time from now until the end of the animation.
	 * 
	 * @note The AnimationManager will attempt to honor this duration on a best-effort basis. As
	 *       with much computer timing, the actual animation may take a slightly longer time than
	 *       this value to fully complete. The Animation will, however, never take less time than
	 *       this to go from start to finish.
	 * @return the duration of this Animation
	 */
	public double getDuration() {
		return this.duration;
	}

	/**
	 * Returns an interpolated value for the given initial and final values, based on the current
	 * Animation's curve and the progress (which is passed in as \c weight so that you can ask about
	 * progress values other than the current one). If animating more than one value (e.g., a
	 * rectangle or an array of scalars), call this method once for each value you want to animate.
	 * 
	 * @param initialValue the initial value of this particular scalar
	 * @param finalValue the final value of this particular scalar
	 * @param weight the point along the line between these two values that should be interpolated,
	 *            expressed as a number between 0 and 1 (usually the same as the progress value)
	 * @return
	 */
	private double getInterpolatedValue(double initialValue, double finalValue, double weight) {

		double range = finalValue - initialValue;
		double y;
		switch (this.animationCurve) {
		case LINEAR: {
			y = weight;
			break;
		}
		case EASE_IN: {
			y = (weight * weight);
			break;
		}
		case EASE_OUT: {
			y = Math.sqrt(weight);
			break;
		}
		case EASE_IN_OUT: {
			y = Math.sin(weight * Math.PI - Math.PI / 2) / 2 + 0.5;
			break;
		}
		case PULSE: {
			y = 0.5 * (Math.cos((weight + (1.0 / 9.0)) * 9.0 * Math.PI)) + 0.5;
			break;
		}
		case GROWL: {
			y = -64 * (Math.pow((weight - 0.5), 6)) + 1;
			break;
		}
		default: {
			throw new UnsupportedOperationException("Unknown animation curve "
					+ this.animationCurve);
		}
		}
		return initialValue + (y * range);
	}

	/**
	 * Gets the AnimationListener for this Animation.
	 * 
	 * @return the AnimationListener for this Animation
	 */
	public AnimationListener getListener() {
		return this.listener;
	}

	/**
	 * Gets the arbitrary object associated with this Animation, if one has been set.
	 * 
	 * @return this Animation's object data, or \c null if none has been set
	 */
	public Object getObjectData() {
		return this.objectData;
	}

	/**
	 * Gets the progress of this animation as a floating-point number in (0, 1]. 0 indicates that
	 * the animation has not yet started, and 1 indicates that it is complete. The animation's
	 * progress will be greater than zero when the AnimationListener's animationDidUpdate() method
	 * is called for the first time. The AnimationListener's animationDidUpdate() method is
	 * guaranteed to be called at least once when this Animation's progress is exactly 1.0. Also,
	 * this Animation's isComplete() method will return \c true if and only if this method returns
	 * 1.0.
	 * 
	 * @note The progress of the animation is not the same as the value of the scalar that is being
	 *       animated, so be careful not to confuse these two cases. The progress is always a single
	 *       number between 0 and 1.
	 * @return the progress of this Animation
	 */
	public double getProgress() {
		if (this.isComplete) {
			return 1.0;
		}
		// Set the new progress based on the duration and the startTime:
		double millisPassed =
			Calendar.getInstance().getTimeInMillis() - this.getStartTime().getTimeInMillis();

		double secondsPassed = millisPassed / 1000; // convert to seconds
		double progress = secondsPassed / this.getDuration();
		return Math.min(progress, 1.0);
	}

	/**
	 * Gets the time at which this animation was begun. Returns null if the animation has not yet
	 * been started.
	 * 
	 * @return the start time of this Animation
	 */
	public Calendar getStartTime() {
		return this.startTime;
	}

	/**
	 * Returns \c true if this Animation is complete, and \c false otherwise. If this method returns
	 * \c true, the getProgress() method is guaranteed to return 1.0.
	 * 
	 * @return \c true if the Animation is complete, \c false otherwise.
	 */
	public boolean isComplete() {
		return this.isComplete;
	}

	/**
	 * Marks this Animation as complete. This method does not, however, stop the timer associated
	 * with the event. When an event is marked as complete, calls to getProgress() will always
	 * return 1.0. If a formerly complete event is marked incomplete, the getProgress() method will
	 * return whatever it last returned before the Animation was marked as complete.
	 * 
	 * @param complete Whether or not this Animation is complete.
	 */
	void setComplete(boolean complete) {
		this.isComplete = complete;

	}

	/**
	 * Associates an arbitrary object with this Animation which can later be retrieved using
	 * getObjectData(). This allows you to avoid conditional logic in your AnimationListener's
	 * update method if you just want to map an animation to, for example, the object it is
	 * animating. This object is not used in any way by the Animation class; it is provided totally
	 * for your convenience.
	 * 
	 * @param objectData the arbitrary object to associate with this Animation
	 */
	public void setObjectData(Object objectData) {
		this.objectData = objectData;
	}

	/**
	 * Sets the timer associated with this Animation. This method only exists to ensure that the
	 * timer will not be garbage collected before this Animation has completed, and does not
	 * actually start the timer or associate it with the Animation in any other way.
	 * 
	 * @param t the timer associated with this Animation
	 */
	void setTimer(AnimationTimer t) {
		this.timer = t;
	}

	/**
	 * Starts this animation immediately.
	 * 
	 * @param l the AnimationListener that will be notified with a call to
	 *            AnimationListener#animationDidUpdate(Animation) when this Animation updates.
	 */
	public void start(AnimationListener l) {
		this.startTime = Calendar.getInstance();
		this.listener = l;
		AnimationManager.getManager().startAnimationTimer(this);
	}
}
