/* File: AnimationManager.java
 *
 * Contains class AnimationManager
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.animation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Set;

import javax.swing.Timer;

import edu.cmu.hcii.calo.view.animation.Animation.AnimationCurve;

/**
 * A Singleton class that manages all Animations currently in progress. You can create a new
 * Animation by calling one of the animate... methods, AnimationManager#animateValue(double, double,
 * AnimationCurve, double), animateValues(double[], double[], AnimationCurve, double),
 * animatePoint(Point2D, Point2D, AnimationCurve, double), and animateRect(Rectangle2D, Rectangle2D,
 * AnimationCurve, double).
 * 
 * @author Brian Ellis
 * 
 */
public class AnimationManager implements ActionListener {

	/**
	 * The singleton instance of this manager.
	 */
	private static AnimationManager uniqueInstance = null;

	/**
	 * Returns the singleton instance of this manager.
	 * 
	 * @return the singleton instance
	 */
	public static AnimationManager getManager() {
		if (AnimationManager.uniqueInstance == null) {
			AnimationManager.uniqueInstance = new AnimationManager();
		}
		return AnimationManager.uniqueInstance;
	}

	/**
	 * The current set of Animations being managed.
	 */
	@SuppressWarnings("unused")
	private Set<Animation> animations;

	/**
	 * Private constructor. Does nothing special.
	 * 
	 */
	private AnimationManager() {
	// Intentionally left blank
	}

	/**
	 * Called when an Animation's timer fires.
	 * 
	 * @param e the action event
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		AnimationTimer t = (AnimationTimer)e.getSource();
		Animation a = t.getAnimation();

		// If we're done, we'll send an update with a progress of exactly 1.0
		// to ensure that the animation completes correctly, and also stop the
		// timer so we won't get any more messages.
		if (a.getProgress() >= 1.0) {
			t.stop();
			a.setComplete(true);
		}

		a.getListener().animationDidUpdate(a);
	}

	/**
	 * Returns a new Animation object that will animate a given Point from an initial to a final
	 * value over a given duration using a particular animation curve.
	 * 
	 * @note Animation objects returned by this method must be started by calling
	 *       Animation#start(AnimationListener) before they will actually perform the animation.
	 * @note Although the AnimationManager will do its best to ensure that the animation takes
	 *       exactly as long as it's supposed to, the nature of computer timers is such that it may
	 *       take slightly longer than this to fully complete the animation. The animation will not,
	 *       however, take less time than the duration indicates it should.
	 * 
	 * @param initialPoint the initial Point
	 * @param finalPoint the final Point
	 * @param curve the AnimationCurve to use for this animation
	 * @param duration the intended duration of the animation.
	 * @return a new Animation specified by the above parameters
	 */
	public Animation animatePoint(Point2D initialPoint,
			Point2D finalPoint,
			AnimationCurve curve,
			double duration) {
		Animation a = new Animation(initialPoint, finalPoint, curve, duration);
		return a;
	}

	/**
	 * Returns a new Animation object that will animate a given Rectangle from an initial to a final
	 * value over a given duration using a particular animation curve.
	 * 
	 * @note Animation objects returned by this method must be started by calling
	 *       Animation#start(AnimationListener) before they will actually perform the animation.
	 * @note Although the AnimationManager will do its best to ensure that the animation takes
	 *       exactly as long as it's supposed to, the nature of computer timers is such that it may
	 *       take slightly longer than this to fully complete the animation. The animation will not,
	 *       however, take less time than the duration indicates it should.
	 * 
	 * @param initialRect the initial Rectangle
	 * @param finalRect the final Rectangle
	 * @param curve the AnimationCurve to use for this animation
	 * @param duration the intended duration of the animation.
	 * @return a new Animation specified by the above parameters
	 */
	public Animation animateRect(Rectangle2D initialRect,
			Rectangle2D finalRect,
			AnimationCurve curve,
			double duration) {
		Animation a = new Animation(initialRect, finalRect, curve, duration);
		return a;
	}

	/**
	 * Returns a new Animation object that will animate a given scalar value from an initial to a
	 * final value over a given duration using a particular animation curve.
	 * 
	 * @note Animation objects returned by this method must be started by calling
	 *       Animation#start(AnimationListener) before they will actually perform the animation.
	 * @note Although the AnimationManager will do its best to ensure that the animation takes
	 *       exactly as long as it's supposed to, the nature of computer timers is such that it may
	 *       take slightly longer than this to fully complete the animation. The animation will not,
	 *       however, take less time than the duration indicates it should.
	 * 
	 * @param initialValue the initial value of the scalar
	 * @param finalValue the final value of the scalar
	 * @param curve the AnimationCurve to use for this animation
	 * @param duration the intended duration of the animation.
	 * @return a new Animation specified by the above parameters
	 */
	public Animation animateValue(double initialValue,
			double finalValue,
			AnimationCurve curve,
			double duration) {
		Animation a = new Animation(initialValue, finalValue, curve, duration);
		return a;
	}

	/**
	 * Returns a new Animation object that will animate each scalar in a given array of scalar
	 * values from an initial to a final value over a given duration using a particular animation
	 * curve.
	 * 
	 * @note Animation objects returned by this method must be started by calling
	 *       Animation#start(AnimationListener) before they will actually perform the animation.
	 * @note Although the AnimationManager will do its best to ensure that the animation takes
	 *       exactly as long as it's supposed to, the nature of computer timers is such that it may
	 *       take slightly longer than this to fully complete the animation. The animation will not,
	 *       however, take less time than the duration indicates it should.
	 * 
	 * @param initialValues the initial array
	 * @param finalValues the final array
	 * @param curve the AnimationCurve to use for this animation
	 * @param duration the intended duration of the animation.
	 * @return a new Animation specified by the above parameters
	 */
	public Animation animateValues(double[] initialValues,
			double[] finalValues,
			AnimationCurve curve,
			double duration) {
		Animation a = new Animation(initialValues, finalValues, curve, duration);
		return a;
	}

	/**
	 * Starts the timer associated with this animation, and ensures that the timer will not be
	 * garbage collected before the Animation is finished.
	 * 
	 * @note This is not the method to use to start an Animation. To do that, call
	 *       Animation#start(AnimationListener) instead.
	 * 
	 * @param animation the Animation to start a timer for
	 */
	void startAnimationTimer(Animation animation) {
		AnimationTimer t = new AnimationTimer(1, this);
		animation.setTimer(t);
		t.setAnimation(animation);
		t.start();
	}
}

/**
 * A subclass of Timer that keeps track of what Animation it is timing.
 * 
 * @author Brian Ellis
 * 
 */
@SuppressWarnings("serial")
class AnimationTimer extends Timer {

	/**
	 * The Animation this Timer is timing.
	 */
	private Animation animation;

	/**
	 * Constructs an AnimationTimer.
	 * 
	 * @param delay how often the timer fires
	 * @param listener what object should be notified when the timer fires
	 * @see javax.swing.Timer#Timer(int, ActionListener)
	 */
	public AnimationTimer(int delay, ActionListener listener) {
		super(delay, listener);
	}

	/**
	 * Gets the Animation associated with this timer.
	 * 
	 * @return the Animation associated with this timer
	 */
	public Animation getAnimation() {
		return this.animation;
	}

	/**
	 * Sets the Animation associated with this timer.
	 * 
	 * @param a the Animation associated with this timer
	 */
	public void setAnimation(Animation a) {
		this.animation = a;
	}

}
