/* File: IconWellView.java
 *
 * Contains class IconWellView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.iconwell;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.MatteBorder;

import edu.cmu.hcii.calo.controller.TaskViewerController;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.task.viewer.TaskViewerPanel;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view that shows a matrix of icons, one of which shows the task viewer window and all the others
 * of which do nothing because they're not implemented.
 * 
 * The view also intelligently rearranges its icons when its width changes.
 * 
 * @author Brian Ellis
 * 
 */
public class IconWellView extends CALOComponent {

	/**
	 * An action listener subclass that displays a message saying what the clicked button would do
	 * if it were implemented.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class ThisDontWorkYetListener implements ActionListener {

		/**
		 * The label of the button that was clicked.
		 */
		private JLabel label;

		/**
		 * Constructs a ThisDontWorkYetListener with the given label, which will be used in the
		 * message.
		 * 
		 * @param label the label
		 */
		public ThisDontWorkYetListener(JLabel label) {
			this.label = label;
		}

		/**
		 * Called when the button is clicked.
		 * 
		 * @param e the action event
		 */
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(	IconWellView.this,
											"In the finished application, clicking this button would show the "
													+ this.label.getText()
													+ " window.\n"
													+ "As it stands, this isn't implemented yet, but it should be possible to complete this task without using it.");
		}
	}

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = -2349200075305110326L;

	/**
	 * The task viewer panel.
	 */
	protected TaskViewerPanel taskViewer;

	/**
	 * A binding maker for layout purposes.
	 */
	private BindingMaker bm;

	/**
	 * The panel on which the buttons will be placed.
	 */
	private JPanel buttonPanel;

	/**
	 * The list of all buttons we intend to put on the panel.
	 */
	private List<IconButton> buttons;

	/**
	 * The mapping between each button and its label.
	 */
	private Map<IconButton, JLabel> labels;

	/**
	 * The controller for the task viewer.
	 */
	@SuppressWarnings("unused")
	// This is used in the thread, but not locally
	private TaskViewerController taskViewerController;

	/**
	 * Constructs an IconWellView.
	 */
	public IconWellView() {

		this.taskViewer = new TaskViewerPanel();
		this.taskViewerController = new TaskViewerController(this.taskViewer.getTaskList());

		this.buttons = new ArrayList<IconButton>();
		this.labels = new HashMap<IconButton, JLabel>();

		ImageIcon taskViewerIcon = new ImageIcon("Resources/TaskViewer.png");
		ImageIcon calendarIcon = new ImageIcon("Resources/Calendar.png");
		ImageIcon peopleIcon = new ImageIcon("Resources/People.png");
		ImageIcon emailIcon = new ImageIcon("Resources/Email.png");
		ImageIcon automationIcon = new ImageIcon("Resources/Automation.png");
		ImageIcon documentationIcon = new ImageIcon("Resources/Documentation.png");
		ImageIcon preferencesIcon = new ImageIcon("Resources/Preferences.png");
		ImageIcon helpIcon = new ImageIcon("Resources/Help.png");

		this.addIconAndLabel(taskViewerIcon, "Task Viewer", "Task Viewer");
		this.addIconAndLabel(calendarIcon, "Calendar", "Calendar");
		this.addIconAndLabel(peopleIcon, "People", "People");
		this.addIconAndLabel(emailIcon, "Email", "Email");
		this.addIconAndLabel(automationIcon, "Automation", "Automation");
		this.addIconAndLabel(documentationIcon, "Learning Log", "Learning Log");
		this.addIconAndLabel(preferencesIcon, "Preferences", "Preferences");
		this.addIconAndLabel(helpIcon, "Help", "Help");

		this.buttonPanel = new JPanel(new RelativeLayout());

		this.setup();
		this.addComponentListener(new ComponentAdapter() {

			@SuppressWarnings("synthetic-access")
			@Override
			public void componentResized(ComponentEvent e) {
				IconWellView.this.recomputeLayout();
			}

		});
	}

	/**
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Color shadow = new Color(0f, 0f, 0f, 0.33f);
		Color blackTint = new Color(0f, 0f, 0f, 0.05f);
		GradientPaint innerShadow = new GradientPaint(0f, 0f, shadow, 0f, 8f, blackTint);
		((Graphics2D)g).setPaint(innerShadow);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
	}

	/**
	 * Adds the given icon and label to the button list, with the given tooltip text.
	 * 
	 * @param icon the icon
	 * @param labelText the text of the label
	 * @param tooltipText the text that should go on the tool tip
	 */
	private void addIconAndLabel(ImageIcon icon, String labelText, String tooltipText) {
		IconButton taskViewerButton = new IconButton(icon);
		taskViewerButton.setToolTipText(tooltipText);
		JLabel taskViewerLabel = new JLabel(labelText);
		this.buttons.add(taskViewerButton);
		this.labels.put(taskViewerButton, taskViewerLabel);
	}

	/**
	 * Does what the method name implies. The panel must be using a RelativeLayout for this to work.
	 * 
	 * @param panel the panel to lay out the buttons on
	 * @param numColumns the number of columns to use
	 * @param numColumns
	 */
	private void layoutButtonsInColumns(JPanel panel, int numColumns) {

		assert panel.getLayout() instanceof RelativeLayout : "whoa there, buddy, this isn't going to work at all well if your panel doesn't have a RelativeLayout on it!";

		int thisColumn = 0;
		JLabel previousRowLabel = null;
		IconButton previousRowButton = null;
		IconButton previousColumnButton = null;
		for (IconButton button : this.buttons) {
			// Add the button:
			RelativeConstraints buttonConstraints = new RelativeConstraints();
			if (previousRowLabel == null) {
				buttonConstraints.addBinding(this.bm.topEdge());
			} else {
				if (previousRowLabel.getPreferredSize().height > 0) {
					buttonConstraints.addBinding(this.bm.below(previousRowLabel));
				} else {
					buttonConstraints.addBinding(this.bm.below(previousRowButton));
				}
			}
			if (thisColumn == 0) {
				buttonConstraints.addBinding(this.bm.leftEdge());
			} else {
				buttonConstraints.addBinding(this.bm.rightOf(previousColumnButton));
			}

			// Add the label:
			JLabel label = this.labels.get(button);
			RelativeConstraints labelConstraints = new RelativeConstraints();
			labelConstraints.addBinding(this.bm.below(button));
			labelConstraints.addBinding(this.bm.horizontallyCenterAlign(button));

			panel.add(button, buttonConstraints);
			panel.add(label, labelConstraints);

			previousColumnButton = button;
			thisColumn = (thisColumn + 1) % numColumns;
			if (thisColumn == 0) {
				previousRowLabel = label;
				previousRowButton = button;
			}
		}
	}

	/**
	 * Causes the button panel to lay itself out again, possibly changing the number of columns in
	 * which it displays buttons and also possibly showing or hiding text labels depending on the
	 * available space.
	 */
	private void recomputeLayout() {
		boolean wasShowingLabels = (this.labels.get(this.buttons.get(0)).getWidth() > 0);
		boolean isShowingLabels = true;

		if (!wasShowingLabels) {
			this.bm.setLeftMargin(12);
			this.bm.setHorizontalSpacing(28);
			for (IconButton button : this.buttons) {
				JLabel label = this.labels.get(button);
				label.setPreferredSize(null);
			}
			this.layoutButtonsInColumns(this.buttonPanel, 4);
			this.buttonPanel.setBorder(new EmptyBorder(0, 0, 4, 0));
		}
		int numColumns = 4;
		while (this.buttonPanel.getPreferredSize().width > this.getWidth() && numColumns > 0) {
			// Is the sidebar too small for us to show labels?
			int preferredWidth = this.buttonPanel.getPreferredSize().width;
			if (preferredWidth + 8 > this.getWidth()) {
				// If so, hide them:
				for (IconButton button : this.buttons) {
					JLabel label = this.labels.get(button);
					label.setPreferredSize(new Dimension(0, 0));
				}

				this.bm.setLeftMargin(2);
				this.bm.setHorizontalSpacing(8);
				this.layoutButtonsInColumns(this.buttonPanel, numColumns);
				this.buttonPanel.setBorder(null);
				isShowingLabels = false;
			}
			--numColumns;
		}

		if (wasShowingLabels != isShowingLabels) {
			this.repaint();
		}
	}

	/**
	 * Sets up the visual appearance of this view.
	 */
	private void setup() {
		this.setOpaque(true);
		this.buttonPanel.setOpaque(false);

		this.setLayout(new RelativeLayout());

		this.bm = new BindingMaker(12, 4, 4, 4, 8, 8);

		for (IconButton button : this.buttons) {
			JLabel label = this.labels.get(button);
			label.setFont(label.getFont().deriveFont(9.0f));
			if (label.getText().equals("Task Viewer")) {
				button.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						JFrame taskViewerWindow = new JFrame("Stardust Task Viewer");
						taskViewerWindow.add(IconWellView.this.taskViewer);

						taskViewerWindow.setSize(640, 400);
						taskViewerWindow.setLocation(50, 50);
						taskViewerWindow.setVisible(true);
					}

				});
			} else {
				button.addActionListener(new ThisDontWorkYetListener(label));
			}
		}

		this.recomputeLayout();

		this.add(this.buttonPanel, new RelativeConstraints(	this.bm.directBottomEdge(),
															this.bm.horizontallyCenterAlign(this)));

		this.setBorder(new MatteBorder(1, 0, 0, 0, SystemColor.controlDkShadow));

	}
}
