/*
 * File: NotificationCenterView.java
 * 
 * Contains class NotificationCenterView
 * 
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JDialog;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import edu.cmu.hcii.calo.manager.NotificationManager;
import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.Notification.NotificationType;
import edu.cmu.hcii.calo.view.AnimatedPane;
import edu.cmu.hcii.calo.view.PropertyChangeForwarder;
import edu.cmu.hcii.calo.view.animation.Animation;
import edu.cmu.hcii.calo.view.animation.AnimationManager;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationCurve;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationType;
import edu.cmu.hcii.calo.view.notification.support.NotificationViewPropertyNames;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The visualization of the notification center pane.
 * 
 * @todo Set the priority cutoff through the controller and engine.
 * 
 * @author Will Haines
 */
public class NotificationCenterView extends AnimatedPane {

	/**
	 * The minimum size of the notification center in pixels. Right now there is space for three
	 * notifications.
	 */
	private static final int DEFAULT_MINIMUM_SIZE = 24 * 3;

	/** The point at which notifications get their own group. */
	public static final double DEFAULT_PRIORITY_CUTOFF = 100.0;

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -4027022402433141088L;

	/** How long animations should take to complete. */
	private static final double ANIMATION_DURATION = .75;

	/** How long an animation pulse should take to complete. */
	private static final double PULSE_DURATION = 2.0;

	/** How long a growl stays on screen. */
	private static final double GROWL_DURATION = 5.0;

	/** The cutoff for notification coalescing. */
	private double priorityCutoff;

	/** The old view for purposes of storing animation start locations. */
	private Map<Notification, NotificationGroupView> oldViewState;

	/** The old order of notifications as viewed here. */
	private ArrayList<Notification> oldNotificationOrder;

	/** The views to remove from upon update. */
	private ArrayList<NotificationGroupView> toRemove;

	/** The new views to be added sorted. */
	private List<NotificationGroupView> sortedLayout;

	/** This view is minimized */
	private boolean isMini;

	/** The view to show off the sidebar. */
	private JDialog outsideView;

	/** The notification group to show off the sidebar. */
	private NotificationGroupView outsideGroup;

	/** The mini group view referenced by the outside sidebar group. */
	private NotificationGroupView outsideReference;

	/** The next mini group to reference by outside the sidebar group. */
	private NotificationGroupView newReference;

	/** Supress animations. */
	private boolean dontAnimate;

	/**
	 * Create a new NotificationCenterView.
	 */
	public NotificationCenterView(boolean isMini) {
		this.isMini = isMini;
		this.setLayout(new RelativeLayout());
		this.priorityCutoff = NotificationCenterView.DEFAULT_PRIORITY_CUTOFF;
		this.setOpaque(true);
		this.addComponentListener(new ComponentAdapter() {

			@SuppressWarnings("synthetic-access")
			@Override
			public void componentResized(ComponentEvent e) {
				// Only update if the bar is being change to or from a mini-bar
				if (e.getComponent().getWidth() != NotificationCenterView.this.getWidth()) {
					NotificationCenterView.this.update();
					NotificationCenterView.this.repaint();
				}
			}

		});
	}

	/**
	 * Close the outside window.
	 */
	public void closeNotificationGroupOutsideSidebar() {
		// Destroy the old view
		if (this.outsideView != null) {
			this.outsideView.dispose();
			this.outsideView = null;
			this.newReference = null;
			this.outsideGroup = null;
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doAnimations()
	 * 
	 * @todo Make breaking out of a notification group to high priority work
	 * @todo Make intra-group animations work
	 */
	@Override
	protected boolean doAnimations() {

		// Don't animate during mini bar transitions
		if (this.dontAnimate) {
			return false;
		}

		ArrayList<Animation> anims = new ArrayList<Animation>();

		// Figure out what this view now looks like at the notification level
		Map<Notification, NotificationGroupView> newViewState =
			new HashMap<Notification, NotificationGroupView>();
		ArrayList<Notification> newNotificationOrder = new ArrayList<Notification>();
		for (NotificationGroupView ngv : this.sortedLayout) {

			// Store where this component lives in the view
			for (Notification n : ngv.getNotifications()) {
				newViewState.put(n, ngv);
				newNotificationOrder.add(n);
			}
		}

		// Move the outside notification if it exists
		if (this.isMini && this.outsideGroup != null && this.newReference != null) {
			Rectangle oldBounds = (Rectangle)this.outsideView.getBounds().clone();
			this.showNotificationGroupOutsideSidebar(this.newReference);

			Animation a1 =
				AnimationManager.getManager().animateRect(	oldBounds,
															(Rectangle)this.outsideView.getBounds().clone(),
															AnimationCurve.EASE_IN_OUT,
															NotificationCenterView.ANIMATION_DURATION);

			a1.setObjectData(this.outsideView);
			anims.add(a1);
		}

		// Figure out what needs animated
		int oldCursor = 0;
		int newCursor = 0;
		boolean outOfNew = false;
		boolean outOfOld = false;
		Notification oldNote = null;
		Notification newNote = null;
		while (true) {

			try {
				oldNote = this.oldNotificationOrder.get(oldCursor);
			} catch (IndexOutOfBoundsException e) {
				outOfOld = true;
			}
			try {
				newNote = newNotificationOrder.get(newCursor);
			} catch (IndexOutOfBoundsException e) {
				outOfNew = true;
			}

			// We're done!
			if (outOfOld && outOfNew) {
				break;
			}

			NotificationGroupView oldView = this.oldViewState.get(oldNote);
			NotificationGroupView newViewOldLocation = this.oldViewState.get(newNote);
			NotificationGroupView newView = newViewState.get(newNote);
			NotificationGroupView oldViewNewLocation = newViewState.get(oldNote);

			// Find the end of the groups
			int oldGroupEnd = oldCursor;
			int oldGroupSize = 0;
			int newGroupEnd = newCursor;
			int newGroupSize = 0;
			try {
				while (oldView == this.oldViewState.get(this.oldNotificationOrder.get(oldGroupEnd))) {
					oldGroupEnd++ ;
					oldGroupSize++ ;
				}
			} catch (IndexOutOfBoundsException e) {
				oldGroupEnd = this.oldNotificationOrder.size();
			}
			try {
				while (newView == newViewState.get(newNotificationOrder.get(newGroupEnd))) {
					newGroupEnd++ ;
					newGroupSize++ ;
				}
			} catch (IndexOutOfBoundsException e) {
				newGroupEnd = newNotificationOrder.size();
			}

			// Find the old location of this group if it existed
			if (newGroupSize > 1) {
				for (int i = newCursor; i < newGroupEnd; i++ ) {
					newViewOldLocation = this.oldViewState.get(newNotificationOrder.get(i));
					if (newViewOldLocation != null) {
						break;
					}
				}
			}

			// Keep uncollapsed for shrinks
			boolean dontRemove = false;

			// New group view should be added
			if (!outOfNew && newViewOldLocation == null) {

				// It's important, flash it
				if (newView.getPriority() >= this.priorityCutoff) {
					Animation a1 =
						AnimationManager.getManager().animateValues(new double[] { 0.93, 0.93, 0.93 },
																	new double[] { 0.9375, 0.8789,
																			0.1562 },
																	AnimationCurve.PULSE,
																	NotificationCenterView.PULSE_DURATION);
					a1.setObjectData(newView);
					anims.add(a1);

					// Show the notification outside of the mini bar
					if (this.isMini && this.getTopLevelAncestor() != null) {
						this.showNotificationGroupOutsideSidebar(newView);

						Animation a2 =
							AnimationManager.getManager().animateValue(	0,
																		1,
																		AnimationCurve.GROWL,
																		NotificationCenterView.GROWL_DURATION);
						a2.setObjectData(this.outsideView);
						anims.add(a2);

						Point convert = new Point(0, 0);
						SwingUtilities.convertPointToScreen(convert, this.outsideReference);

						Rectangle startBounds =
							new Rectangle(	convert.x,
											convert.y,
											this.outsideView.getPreferredSize().width,
											this.outsideView.getPreferredSize().height);

						Animation a3 =
							AnimationManager.getManager().animateRect(	startBounds,
																		(Rectangle)this.outsideView.getBounds().clone(),
																		AnimationCurve.GROWL,
																		NotificationCenterView.GROWL_DURATION);

						a3.setObjectData(this.outsideView);
						this.outsideView.setAlwaysOnTop(false);

						this.firePropertyChange(NotificationViewPropertyNames.NOTIFICATION_OUTSIDE_CHANGE,
												"OutsideNotificationGroupChanged",
												null);

						anims.add(a3);
					}
				}

				// It's normal, just fade it in
				else {
					Animation a1 =
						AnimationManager.getManager().animateValue(	0,
																	1,
																	AnimationCurve.EASE_IN,
																	NotificationCenterView.ANIMATION_DURATION);
					a1.setObjectData(newView);
					anims.add(a1);
				}
			}

			// New view needs updated, probably moved
			else if (!outOfNew) {

				if (newViewOldLocation != null) {
					// In place shrink
					if (newViewOldLocation.getBounds().height > newView.getBounds().height
							&& newView.getY() == newViewOldLocation.getY()) {
						// Normal shrink
						Animation a1 =
							AnimationManager.getManager().animateRect(	(Rectangle)newViewOldLocation.getBounds().clone(),
																		(Rectangle)newView.getBounds().clone(),
																		AnimationCurve.EASE_IN_OUT,
																		NotificationCenterView.ANIMATION_DURATION);
						a1.setObjectData(oldView);
						anims.add(a1);

						dontRemove = true;

						// Shrink to size 1 group
						if (newGroupSize <= 1) {
							Animation a2 =
								AnimationManager.getManager().animateValue(	1,
																			0,
																			AnimationCurve.EASE_OUT,
																			NotificationCenterView.ANIMATION_DURATION);
							a2.setObjectData(oldView);
							anims.add(a2);
						}
					}

					// Normal move
					else {
						Animation a1 =
							AnimationManager.getManager().animateRect(	(Rectangle)newViewOldLocation.getBounds().clone(),
																		(Rectangle)newView.getBounds().clone(),
																		AnimationCurve.EASE_IN_OUT,
																		NotificationCenterView.ANIMATION_DURATION);
						a1.setObjectData(newView);
						anims.add(a1);
					}
				}
			}

			// Move to the next new group
			newCursor = newGroupEnd;

			// Find the new location of the old group if it exists
			if (oldGroupSize > 1) {
				for (int i = oldCursor; i < oldGroupEnd; i++ ) {
					oldViewNewLocation = newViewState.get(this.oldNotificationOrder.get(i));
					if (oldViewNewLocation != null) {
						break;
					}
				}
			}

			// Old view needs to go away
			if (!outOfOld && !dontRemove) {

				// It's a real remove; fade out
				if (oldViewNewLocation == null) {
					Animation a1 =
						AnimationManager.getManager().animateValue(	1,
																	0,
																	AnimationCurve.EASE_OUT,
																	NotificationCenterView.ANIMATION_DURATION);
					a1.setObjectData(oldView);
					anims.add(a1);
				}

				// It is being moved, just cut it
				else {
					oldView.setVisible(false);
				}
			}

			// Move to the next old group
			oldCursor = oldGroupEnd;
		}

		if (anims.size() == 0) {
			return false;
		}

		this.setTotalAnimations(anims.size());

		for (Animation animation : anims) {
			animation.start(this);
			this.incrementNumAnimating();
		}

		return true;
	}

	/**
	 * Updates the look of this view by dividing up the notification list from the manager into
	 * important single-item notification groups, then coalescing the rest into type-based groups.
	 * 
	 * @todo Store the expanded state correctly.
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doUpdate()
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected void doUpdate() {

		this.dontAnimate = false;

		// Stores the state of the notification groups for update purposes.
		Set<NotificationGroupView> toLayout = new HashSet<NotificationGroupView>();
		Map<NotificationType, NotificationGroupView> groups =
			new HashMap<NotificationType, NotificationGroupView>();
		this.toRemove = new ArrayList<NotificationGroupView>();

		// Handle outside view resizing
		if (this.outsideView != null) {
			this.outsideView.setPreferredSize(new Dimension(200,
															this.outsideGroup.getPreferredSize().height));
			this.outsideView.pack();
		}
		this.newReference = null;

		// Remove all notification group views from this view
		for (int i = this.getComponents().length - 1; i >= 0; i-- ) {
			Component component = this.getComponent(i);
			if (!(component instanceof NotificationGroupView)) {
				continue;
			}
			if (((NotificationGroupView)component).getTitle().isMini() != this.isMini) {
				this.remove(component);
				this.dontAnimate = true;
			} else {
				this.toRemove.add((NotificationGroupView)component);
			}
		}

		// Add them back in the correct order, creating new views for any new
		// notifications.
		Component lastView = null;

		// Grab every notification in the manager and divide them up
		Notification[] notificationList = NotificationManager.getManager().getNotifications();
		for (int i = 0; i < notificationList.length; i++ ) {

			// Put this notification into the correct notification group
			Notification thisNotification = notificationList[i];
			NotificationGroupView view;

			// Is it above the priority cutoff? If so, make it its own group
			if (thisNotification.getPriority() >= this.priorityCutoff
					|| thisNotification.getType() == NotificationType.REMINDER) {

				view =
					new NotificationGroupView(thisNotification.getType(), this.isMini, true, this);

				// Set up the add time to be correct
				NotificationGroupView oldView = this.oldViewState.get(thisNotification);
				if (oldView != null) {
					view.setSize(oldView.getSize());
					view.setAddTime(oldView.getAddTime());
					view.setImportantColorToggle(oldView.isImportantColorSet());

					// Where should the outside view go now?
					if (oldView == this.outsideReference) {
						this.newReference = view;
					}
				}

				view.addNotification(thisNotification);
			}

			// Otherwise, add it to a coalesced group
			else {

				// Does the group exist yet?
				NotificationType type = thisNotification.getType();
				// Set up the expanded state to be correct based on the old
				// expanded state
				NotificationGroupView oldView = this.oldViewState.get(thisNotification);

				// If this doesn't belong in some already created group
				if (!groups.containsKey(type)) {
					groups.put(type, new NotificationGroupView(type, this.isMini, false, this));
				}
				view = groups.get(type);

				// Set up state to be correct
				long addTime = view.getAddTime();
				boolean isExpanded = view.isExpanded();
				if (oldView != null) {
					view.setSize(oldView.getSize());
					addTime = oldView.getAddTime();
					isExpanded = oldView.isExpanded() || isExpanded;

					// Where should the outside view go now?
					if (oldView == this.outsideReference) {
						this.newReference = view;
					}
				}

				view.setAddTime(addTime);
				view.setExpanded(isExpanded);

				view.addNotification(thisNotification);
			}

			// This is a set, so it doesn't add duplicates
			toLayout.add(view);
		}

		this.sortedLayout = new ArrayList<NotificationGroupView>(toLayout);

		Collections.sort(this.sortedLayout, new Comparator<NotificationGroupView>() {

			public int compare(NotificationGroupView o1, NotificationGroupView o2) {

				if (o1.getPriority() > o2.getPriority()) {
					return -1;
				} else if (o2.getPriority() > o1.getPriority()) {
					return 1;
				} else {
					// Silly break ties by time added
					return (int)(o2.getAddTime() - o1.getAddTime());
				}
			}

		});

		// Now layout all the views
		for (NotificationGroupView view : this.sortedLayout) {

			BindingMaker bm = new BindingMaker(0, 0, 0, 0, 0, 0);

			// Now generate the constraints for this view:
			RelativeConstraints constraints =
				new RelativeConstraints(bm.leftEdge(), bm.rightEdge());
			if (lastView != null) {
				if (lastView instanceof NotificationGroupView
						&& ((NotificationGroupView)lastView).isExpanded() && !this.isMini) {
					constraints.addBinding(new Binding(	Edge.TOP,
														1,
														Direction.BELOW,
														Edge.BOTTOM,
														lastView));
				} else {
					constraints.addBinding(new Binding(	Edge.TOP,
														-1,
														Direction.BELOW,
														Edge.BOTTOM,
														lastView));
				}
			} else {
				constraints.addBinding(new Binding(Edge.TOP, 0, Direction.BELOW, Edge.TOP, this));
			}

			// Make sure that groups that are small collapse
			if (view.getNotifications().size() <= 1) {
				view.setExpanded(false);
			}

			view.update();

			// Add the view:
			this.add(view, constraints);

			lastView = view;
		}
	}

	/**
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#finalizeAnimation()
	 */
	@Override
	protected void finalizeAnimation() {
		// Remove the old components
		for (NotificationGroupView n : this.toRemove) {
			this.remove(n);
		}

		super.finalizeAnimation();
	}

	/**
	 * Returns the preferred size of the notification center, which is its normal size unless it
	 * reaches a minimum size, at which point it won't shrink any more.
	 * 
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		Dimension size = super.getPreferredSize();

		// Too short?
		if (size.height < NotificationCenterView.DEFAULT_MINIMUM_SIZE) {
			size = new Dimension(size.width, NotificationCenterView.DEFAULT_MINIMUM_SIZE);
		}

		return size;
	}

	/**
	 * Hide the outside window
	 */
	public void hideNotificationGroupOutsideSidebar() {
		// Destroy the old view
		if (this.outsideView != null) {
			this.outsideView.dispose();
		}
	}

	/**
	 * A getter for isMini.
	 * 
	 * @return isMini
	 */
	public boolean isMini() {
		return this.isMini;
	}

	/**
	 * @see java.awt.Component#isValid()
	 */
	@Override
	public boolean isValid() {
		if (this.getNumAnimating() > 0) {
			return true;
		} else {
			return super.isValid();
		}
	}

	/**
	 * Make the notification center pretty with a gradient and roundedness.
	 * 
	 * @param arg0 the graphics context to paint with
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics arg0) {
		super.paintComponent(arg0);

		Graphics2D g = (Graphics2D)arg0;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		Color solid = UIManager.getColor("InternalFrame.activeTitleBackground");
		float[] hsb = Color.RGBtoHSB(solid.getRed(), solid.getGreen(), solid.getBlue(), null);
		hsb[1] = Math.max(0.0f, hsb[1] / 2.0f);
		solid = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
		Color transparent = new Color(0.741f, 0.827f, 0.937f, 0.0f);

		Paint gradient = new GradientPaint(0, 0, solid, 0, this.getHeight(), transparent);

		g.setPaint(gradient);

		Rectangle2D roundrect = new Rectangle2D.Float(0, 0, this.getWidth(), this.getHeight());
		Area a = new Area(roundrect);

		a.add(new Area(new Rectangle2D.Float(	0,
												this.getHeight() / 2,
												this.getWidth(),
												this.getHeight() / 2)));

		g.fill(a);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#prepareAnimations()
	 */
	@Override
	protected void prepareAnimations() {
		this.oldViewState = new HashMap<Notification, NotificationGroupView>();
		this.oldNotificationOrder = new ArrayList<Notification>();

		for (int i = this.getComponents().length - 1; i >= 0; i-- ) {
			Component component = this.getComponent(i);
			if (!(component instanceof NotificationGroupView)) {
				continue;
			}

			// Store where this component used to live in the view
			for (Notification n : ((NotificationGroupView)component).getNotifications()) {
				this.oldViewState.put(n, (NotificationGroupView)component);
				this.oldNotificationOrder.add(n);
			}
			Collections.sort(this.oldNotificationOrder, new Comparator<Notification>() {

				@SuppressWarnings("synthetic-access")
				public int compare(Notification n1, Notification n2) {

					int y1 = NotificationCenterView.this.oldViewState.get(n1).getY();
					int y2 = NotificationCenterView.this.oldViewState.get(n2).getY();

					return y1 - y2;
				}
			});
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#safeUpdateAnimation
	 *      (edu.cmu.hcii.calo.view.animation.Animation)
	 */
	@Override
	protected void safeUpdateAnimation(Animation a) {
		Component currentView = (Component)a.getObjectData();
		if (a.getAnimationType() == AnimationType.RECTANGLE) {
			Rectangle2D currentRect = a.getCurrentRectangle();
			Rectangle currentBounds =
				new Rectangle(	Math.round((int)currentRect.getMinX()),
								(int)Math.round(currentRect.getMinY()),
								(int)Math.round(currentRect.getWidth()),
								(int)Math.round(currentRect.getHeight()));
			currentView.setBounds(currentBounds);
			currentView.repaint();
		} else if (a.getAnimationType() == AnimationType.SCALAR) {
			if (currentView instanceof NotificationGroupView) {
				((NotificationGroupView)currentView).setOpacity(a.getCurrentValue());
			}
		} else if (a.getAnimationType() == AnimationType.ARRAY) {
			if (currentView instanceof NotificationGroupView) {
				((NotificationGroupView)currentView).setColor(a.getCurrentValues());
				this.repaint();
			}
		} else {
			assert false : "the type of this animation is not one we know how to deal with";
		}
	}

	/**
	 * Set the notification center to be mini or not.
	 * 
	 * @param isMini whether or not this notification is a mini one
	 */
	public void setMini(boolean isMini) {
		this.isMini = isMini;
		this.update();
	}

	/**
	 * Shows this notification group expanded and outside of the sidebar for the mini bar view.
	 * 
	 * @param ngv the notification group view to show.
	 */
	public void showNotificationGroupOutsideSidebar(NotificationGroupView ngv) {

		// Save old expand state
		boolean wasExpanded = false;
		if (this.outsideGroup != null) {
			wasExpanded = this.outsideGroup.isExpanded();
		}

		// Build a new one
		this.outsideReference = ngv;
		this.outsideGroup = new NotificationGroupView(ngv, this);
		this.outsideGroup.setExpanded(wasExpanded);
		this.outsideGroup.update();
		PropertyChangeForwarder outsideForwarder = new PropertyChangeForwarder(this);
		this.outsideGroup.addPropertyChangeListener(outsideForwarder);

		JDialog tempView = new JDialog();
		tempView.setUndecorated(true);
		tempView.setLayout(new RelativeLayout());
		tempView.setPreferredSize(new Dimension(200, this.outsideGroup.getPreferredSize().height));
		BindingMaker bm = new BindingMaker(0, 0, 0, 0, 0, 0);
		tempView.add(this.outsideGroup, new RelativeConstraints(bm.leftAlign(tempView),
																bm.topAlign(tempView),
																bm.bottomAlign(tempView),
																bm.rightAlign(tempView)));
		tempView.pack();

		// Make sure the sidebar doesn't cover the frame!
		Point convert = ngv.getLocation();
		SwingUtilities.convertPointToScreen(convert, ngv.getParent());
		tempView.setLocation(convert.x - tempView.getWidth() - 3, convert.y);

		tempView.setResizable(false);
		tempView.setAlwaysOnTop(true);
		tempView.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

		// Destroy the old view
		if (this.outsideView != null) {
			this.outsideView.setVisible(false);
		}

		this.outsideView = tempView;
		this.outsideView.setVisible(true);

		this.firePropertyChange(NotificationViewPropertyNames.NOTIFICATION_OUTSIDE_CHANGE,
								"OutsideNotificationGroupChanged",
								this.outsideReference);
		this.outsideView.repaint();
	}
}
