/* File: NotificationGroupPropertyActionListener.java
 *
 * Contains class NotificationGroupPropertyActionListener
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.notification.support.NotificationGroupState;
import edu.cmu.hcii.calo.view.notification.support.NotificationViewPropertyNames;
import edu.cmu.hcii.calo.view.notification.support.NotificationGroupState.NotificationGroupStateProperty;

/**
 * An ActionListener subclass that will construct NotificationGroupStates for some property of the
 * old and new states of a notification and fire a NotificationGroupGroupState property change on
 * behalf of its view.
 * 
 * When you construct a NotificationGroupPropertyActionListener, you pass the view, notifications,
 * and property that it will use to fire property change events. When the listener detects an action
 * from the component to which it is listening, it will call getOldValue() and getNewValue() to
 * determine what values to put in the NotificationGroupStates. Then it will call
 * CALOComponent#firePropertyChange(String, Object, Object) in the view to automatically fire the
 * property change on its behalf.
 * 
 * @author Will Haines
 * 
 */
abstract class NotificationGroupPropertyActionListener implements ActionListener {

	/** The notification associated with this change. */
	private List<Notification> notifications;

	/** The view from which this change originated. */
	private CALOComponent view;

	/** The property that changed. */
	private NotificationGroupStateProperty property;

	/**
	 * Create a new NotificationGroupPropertyActionListener.
	 */
	public NotificationGroupPropertyActionListener(CALOComponent view,
			List<Notification> nl,
			NotificationGroupState.NotificationGroupStateProperty property) {
		this.view = view;
		this.notifications = nl;
		this.property = property;

	}

	/**
	 * Notify others about the action being performed.
	 * 
	 * @param e the event to forward
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@SuppressWarnings("synthetic-access")
	public void actionPerformed(ActionEvent e) {
		NotificationGroupState oldState =
			new NotificationGroupState(this.notifications, this.property, this.getOldValue());
		NotificationGroupState newState =
			new NotificationGroupState(this.notifications, this.property, this.getNewValue());

		this.view.firePropertyChange(	NotificationViewPropertyNames.NOTIFICATION_GROUP_STATE,
										oldState,
										newState);
	}

	/**
	 * A getter for newValue.
	 * 
	 * @return newValue
	 */
	public abstract Object getNewValue();

	/**
	 * A getter for oldValue
	 * 
	 * @return oldValue
	 */
	public abstract Object getOldValue();
}
