/* File: NotificationGroupTitleView.java
 *
 * Contains class NotificationGroupTitleView
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;

import edu.cmu.hcii.calo.model.Notification.NotificationType;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.DisclosureButton;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.notification.support.NotificationViewPropertyNames;
import edu.cmu.hcii.calo.view.training.TrainingMenuListener;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view class that displays a single notification group title.
 * 
 * @author Will Haines
 */
public class NotificationGroupTitleView extends CALOComponent implements MouseListener,
		MouseMotionListener {

	/** Prevent serialization collisions */
	private static final long serialVersionUID = -7164304356867859223L;

	/**
	 * @name Colors
	 * 
	 * These constants represent colors that are used in displaying the NotificationView.
	 */

	// @{
	private static final Color NOTIFICATION_COLOR = new Color(0.93f, 0.93f, 0.93f);

	private static final Color IMPORTANT_NOTIFICATION_COLOR = new Color(0.9375f, 0.87f, 0.15f);

	private static final Color IMPORTANT_MOUSEOVER_NOTIFICATION_COLOR =
		new Color(0.93f, 0.96f, 0.20f);

	private static final Color MOUSEOVER_NOTIFICATION_COLOR = new Color(0.93f, 0.97f, 0.97f);

	private static final Color EXPANDED_NOTIFICATION_COLOR = new Color(1.0f, 1.0f, 0.87f);

	private static final Color MOUSEOVER_EXPANDED_NOTIFICATION_COLOR = new Color(1.0f, 1.0f, 0.93f);

	private static final Color NOTIFICATION_STROKE_COLOR = SystemColor.controlShadow;

	// @}

	/**
	 * @name Borders
	 * 
	 * These constants are the various border styles that are applied to the NotificationView under
	 * various circumstances. Note that the rather badly-named Border actually determines the
	 * background color of the NotificationView as well.
	 */

	// @{
	private final NotificationViewBorder MOUSEOVER_EXPANDED_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.MOUSEOVER_EXPANDED_NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder EXPANDED_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.EXPANDED_NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder MOUSEOVER_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.MOUSEOVER_NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder NORMAL_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder INITIAL_IMPORTANT_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.IMPORTANT_NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder INITIAL_MOUSEOVER_IMPORTANT_BORDER =
		new NotificationViewBorder(	NotificationGroupTitleView.IMPORTANT_MOUSEOVER_NOTIFICATION_COLOR,
									NotificationGroupTitleView.NOTIFICATION_STROKE_COLOR);

	// @}

	/**
	 * Whether or not the NotificationGroupView is currently pressed. This may be true even if the
	 * mouse is not within the bounds of the view.
	 */
	private boolean isPressed;

	/** Whether or not the mouse is currently over the view. */
	private boolean isMouseOver;

	/**
	 * Whether or not the view is currently expanded to show multiple notifications.
	 */
	private boolean isExpanded;

	/** The label showing the text of this notification group. */
	private JLabel notificationGroupTextLabel;

	/** The icon associated with this notification group. */
	private IconButton notificationGroupIcon;

	/** The button to delete this notification group. */
	protected IconButton deleteButton;

	/** The button to expand this notification group. */
	private DisclosureButton expandButton;

	/** The text associated with this notification group. */
	private String notificationGroupText;

	/** The image for this notification icon. */
	private ImageIcon notificationGroupIconImage;

	/** The number of notifications in the enclosing group. */
	private int numNotifications;

	/** This is a mini bar style title. */
	protected boolean isMini;

	/** The parent of this title. */
	private NotificationGroupView parent;

	/** The notification center to which this view refers. */
	protected NotificationCenterView notificationCenter;

	/**
	 * Track if this field was clicked to set the set the initial color to the correct value;
	 */
	private boolean importantColorSet;

	/**
	 * Constructs a NotificationGroupTitleView.
	 */
	public NotificationGroupTitleView(boolean isExpanded,
			boolean isMini,
			boolean isImportantColorSet,
			NotificationGroupView parent,
			NotificationCenterView notificationCenter) {
		super();

		this.notificationCenter = notificationCenter;
		this.notificationGroupIconImage = new ImageIcon();
		this.notificationGroupIcon =
			new IconButton(	this.notificationGroupIconImage,
							this.notificationGroupIconImage,
							this.notificationGroupIconImage);
		this.notificationGroupIcon.addMouseListener(this);
		this.notificationGroupTextLabel = new JLabel();
		this.notificationGroupText = "";
		this.numNotifications = -1;
		this.isMini = isMini;
		this.parent = parent;
		this.importantColorSet = isImportantColorSet;

		this.expandButton = new DisclosureButton();
		this.expandButton.addMouseListener(this);

		// Make sure the button doesn't update itself; I only want one update
		// per click
		for (ActionListener listener : this.expandButton.getListeners(ActionListener.class)) {
			this.expandButton.removeActionListener(listener);
		}

		ImageIcon deleteIcon = new ImageIcon("Resources/close.png");
		ImageIcon deleteOverIcon = new ImageIcon("Resources/close-over.png");
		ImageIcon deletePressedIcon = new ImageIcon("Resources/close-pressed.png");
		this.deleteButton = new IconButton(deleteIcon, deleteOverIcon, deletePressedIcon);
		this.deleteButton.addMouseListener(this);

		this.isMouseOver = false;
		this.isPressed = false;
		this.isExpanded = isExpanded;

		this.setup();
	}

	/**
	 * A getter for notificationGroupIconImage.
	 * 
	 * @return the notificationGroupIconImage
	 */
	protected ImageIcon getNotificationGroupIconImage() {
		return this.notificationGroupIconImage;
	}

	/**
	 * A getter for this groups title's text.
	 * 
	 * @return the notificationGroupText
	 */
	protected String getNotificationGroupText() {
		return this.notificationGroupText;
	}

	/**
	 * A getter for isExpanded.
	 * 
	 * @return the isExpanded
	 */
	protected boolean isExpanded() {
		return this.isExpanded;
	}

	/**
	 * A getter for isImportantColorSet.
	 * 
	 * @return isImportantColor set's set status
	 */
	public boolean isImportantColorSet() {
		return this.importantColorSet;
	}

	/**
	 * A getter for isMini.
	 * 
	 * @return isMini
	 */
	public boolean isMini() {
		return this.isMini;
	}

	/**
	 * Defines the behavior of a mouse click on a notification group; that is, it expands to show
	 * its component notifications.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.isMetaDown()) {
			return;
		}

		// Set the notification group as attended to
		this.importantColorSet = false;

		// If normal, expand the pane
		if (!this.isMini) {
			this.toggleExpanded();
		}
		// Show the full version in mini
		else {
			this.firePropertyChange(NotificationViewPropertyNames.NOTIFICATION_OUTSIDE_MOVE,
									"NotificationOutside",
									this.parent);
			this.updateAppearance();
		}
	}

	/**
	 * Defines the behavior of a mouse drag on this notification group title view; that is, nothing
	 * happens.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
	// This method intentionally left blank
	}

	/**
	 * Updates the mouse over state of this notification group title view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
		if (!this.isPressed) {
			this.isMouseOver = true;
			this.updateAppearance();
		}
	}

	/**
	 * Updates the mouse over state of this notification group title view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		if (!this.isPressed) {
			this.isMouseOver = false;
			this.updateAppearance();
		}
	}

	/**
	 * Defines the behavior of a mouse motion in the notification group title view; that is, do
	 * nothing.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
	// This method intentionally left blank
	}

	/**
	 * Updates the pressed states of this notification group title view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		this.isPressed = true;
		this.updateAppearance();
	}

	/**
	 * Updates the pressed state of this notification group view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		this.isPressed = false;
		this.updateAppearance();
	}

	/**
	 * A setter for isExpanded.
	 * 
	 * @param isExpanded the value to update to
	 */
	protected void setExpanded(boolean isExpanded) {
		this.isExpanded = isExpanded;
		this.updateAppearance();
	}

	/**
	 * A setter for isImportantColorSet.
	 * 
	 * @param isImportantColorSet
	 */
	public void setImportantColorToggle(boolean isImportantColorSet) {
		this.importantColorSet = isImportantColorSet;
	}

	/**
	 * A setter for notificationGroupIconImage.
	 * 
	 * @param notificationGroupIconImage the notificationGroupIconImage to set
	 */
	protected void setNotificationGroupIconImage(ImageIcon notificationGroupIconImage) {
		this.notificationGroupIconImage = notificationGroupIconImage;
	}

	/**
	 * A setter for notificationGroupText.
	 * 
	 * @param notificationGroupText the notificationGroupText to set
	 */
	public void setNotificationGroupText(String notificationGroupText) {
		this.notificationGroupText = notificationGroupText;
	}

	/**
	 * A setter for numNotifications.
	 * 
	 * @param numNotifications the numNotifications to set
	 */
	protected void setNumNotifications(int numNotifications) {
		this.numNotifications = numNotifications;
	}

	/**
	 * Sets up the appearance of the NotificationGroupTitleView.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(1, 2, 1, 5, 4, 4);

		RelativeConstraints notificationIconConstraints = new RelativeConstraints(bm.topEdge());

		if (this.isMini) {
			notificationIconConstraints.addBinding(bm.horizontallyCenterAlign(null));
		} else {
			notificationIconConstraints.addBinding(bm.leftEdge());
		}

		RelativeConstraints expandButtonConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(this.notificationGroupIcon),
									bm.directlyRightOf(this.notificationGroupIcon));

		RelativeConstraints notificationTextLabelConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(this.notificationGroupIcon),
									bm.rightOf(this.expandButton),
									bm.leftOf(this.deleteButton));

		RelativeConstraints deleteButtonConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(this.notificationGroupIcon),
									bm.rightEdge());

		this.add(this.notificationGroupIcon, notificationIconConstraints);
		this.add(this.expandButton, expandButtonConstraints);
		this.add(this.notificationGroupTextLabel, notificationTextLabelConstraints);
		this.add(this.deleteButton, deleteButtonConstraints);

		this.addMouseListener(new TrainingMenuListener() {

			@SuppressWarnings("synthetic-access")
			@Override
			public JMenuItem[] getItems() {
				if (NotificationGroupTitleView.this.parent.getNotificationGroupType() == NotificationType.EMAIL) {
					return new JMenuItem[] { new JMenuItem("Close"), new JMenuItem("Open email") };
				} else if (NotificationGroupTitleView.this.parent.getNotificationGroupType() == NotificationType.CALO_ACTION) {
					return new JMenuItem[] { new JMenuItem("Close"), new JMenuItem("Undo action") };
				} else {
					return new JMenuItem[] { new JMenuItem("Close") };
				}
			}

		});

		this.expandButton.addActionListener(new ActionListener() {

			/**
			 * Toggle the expanded state of this notification group.
			 */
			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
			// NotificationGroupTitleView.this.toggleExpanded();
			}

		});

		this.setFocusable(true);

		this.updateAppearance();

		this.addMouseListener(this);
		this.addMouseMotionListener(this);

		this.update();
	}

	/**
	 * Toggles whether or not the view is expanded.
	 */
	private void toggleExpanded() {
		if (this.notificationCenter.getNumAnimating() == 0) {
			this.isExpanded = !this.isExpanded;
			this.firePropertyChange(NotificationViewPropertyNames.NOTIFICATION_EXPANDED,
									!this.isExpanded,
									this.isExpanded);
		}

		this.updateAppearance();
	}

	/**
	 * Returns a string representation of this NotificationGroupTitleView. Useful only for
	 * debugging.
	 */
	@Override
	public String toString() {
		return "Title: " + this.hashCode() + " Mini:" + this.isMini;
	}

	/**
	 * Causes the view to synchronize itself with the notification center view.
	 */
	public void update() {

		this.notificationGroupTextLabel.setText(this.notificationGroupText);
		this.setToolTipText(this.notificationGroupText);
		this.notificationGroupIcon.setIcon(this.notificationGroupIconImage);
		this.notificationGroupIcon.setRolloverIcon(this.notificationGroupIconImage);
		this.notificationGroupIcon.setPressedIcon(this.notificationGroupIconImage);

		// Set up visibility for expand and delete
		if (this.numNotifications <= 1) {
			this.expandButton.setVisible(false);
			this.expandButton.setPreferredSize(new Dimension(	0,
																this.expandButton.getPreferredSize().height));
		} else {
			this.expandButton.setVisible(true);
			this.expandButton.setPreferredSize(null);
		}

		if (this.isMouseOver) {
			this.deleteButton.setVisible(true);
			this.deleteButton.setPreferredSize(null);
		} else {
			this.deleteButton.setVisible(false);
			this.deleteButton.setPreferredSize(new Dimension(	0,
																this.deleteButton.getPreferredSize().height));
		}

		// The minimized version has just an icon
		if (this.isMini) {
			this.notificationGroupTextLabel.setVisible(false);
			this.expandButton.setVisible(false);
			this.deleteButton.setVisible(false);
		}

		this.doLayout();
	}

	/**
	 * Updates the appearance of the view when something changes.
	 */
	private void updateAppearance() {

		this.expandButton.setSelected(this.isExpanded);

		// Set up the border:
		if (this.isMouseOver) {
			if (this.isExpanded) {
				this.setBorder(this.MOUSEOVER_EXPANDED_BORDER);
			} else if (this.importantColorSet) {
				this.setBorder(this.INITIAL_MOUSEOVER_IMPORTANT_BORDER);
			} else {
				this.setBorder(this.MOUSEOVER_BORDER);
			}
		} else {
			if (this.isExpanded) {
				this.setBorder(this.EXPANDED_BORDER);
			} else if (this.importantColorSet) {
				this.setBorder(this.INITIAL_IMPORTANT_BORDER);
			} else {
				this.setBorder(this.NORMAL_BORDER);
			}
		}
		this.update();
	}

	/**
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		Dimension size = super.getPreferredSize();
		size = new Dimension(size.width, size.height + 3);
		return size;
	}
}
