/*
 * File: NotificationGroupView.java
 * 
 * Contains class NotificationGroupView
 * 
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.SystemColor;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.Notification.NotificationType;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconRenderer;
import edu.cmu.hcii.calo.view.IconRenderer.BadgePosition;
import edu.cmu.hcii.calo.view.notification.support.NotificationGroupState.NotificationGroupStateProperty;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view class that displays a single notification.
 * 
 * @author Will Haines
 */
public class NotificationGroupView extends CALOComponent {

	/** Prevent serialization collisions */
	private static final long serialVersionUID = -8336327906817748848L;

	/**
	 * Runs a small test program to verify that NotificationGroupView works correctly.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		SwingUtilities.invokeLater(new Runnable() {

			@SuppressWarnings("synthetic-access")
			public void run() {
				final JFrame testFrame = new JFrame("NotificationGroupView Test");
				testFrame.setLayout(new RelativeLayout());

				final NotificationGroupView notificationView =
					new NotificationGroupView(NotificationType.CALO_ACTION, false, false, null);
				testFrame.add(	notificationView,
								new RelativeConstraints(new Binding(Edge.TOP,
																	4,
																	Direction.BELOW,
																	Edge.TOP,
																	null),
														new Binding(Edge.LEFT,
																	4,
																	Direction.RIGHT,
																	Edge.LEFT,
																	null),
														new Binding(Edge.RIGHT,
																	4,
																	Direction.LEFT,
																	Edge.RIGHT,
																	null)));

				testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				testFrame.setSize(500, 100);
				testFrame.setVisible(true);
			}
		});
	}

	/** The notifications being represented by this group view. */
	private List<Notification> notifications;

	/** The notification types in this group view. */
	private NotificationType notificationGroupType;

	/** The title view of this notification group. */
	private NotificationGroupTitleView title;

	/**
	 * This notification group's priority. This is defined as the maximum of all its children's
	 * priorities.
	 */
	private double priority;

	/** This notification group's id to break ties consistently. */
	private long addTime;

	/**
	 * Copy constructor; sets the default to be not mini.
	 * 
	 * @param ngv the old notification group view
	 * @param notificationCenter the notification center
	 */
	public NotificationGroupView(NotificationGroupView ngv,
			NotificationCenterView notificationCenter) {
		super();

		this.addTime = Calendar.getInstance().getTimeInMillis();
		this.notificationGroupType = ngv.getNotificationGroupType();
		this.notifications = ngv.getNotifications();
		this.priority = ngv.getPriority();

		this.title =
			new NotificationGroupTitleView(	false,
											false,
											ngv.title.isImportantColorSet(),
											this,
											notificationCenter);
		this.title.deleteButton.addActionListener(new NotificationGroupPropertyActionListener(this,
				this.notifications,
				NotificationGroupStateProperty.NOTIFICATION_GROUP_DELETED) {

			@Override
			public Object getNewValue() {
				if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(	NotificationGroupView.this,
																			"Are you sure you want to delete all of these notifications?  This action cannot be undone.",
																			"Confirm Notification Deletion",
																			JOptionPane.OK_CANCEL_OPTION)) {
					return true;
				} else {
					return false;
				}
			}

			@Override
			public Object getOldValue() {
				return false;
			}

		});

		this.setup();
	}

	/**
	 * Constructs a NotificationGroupView.
	 */
	public NotificationGroupView(NotificationType type,
			boolean isMini,
			boolean isImportant,
			NotificationCenterView notificationCenter) {
		super();

		this.addTime = Calendar.getInstance().getTimeInMillis();
		this.notificationGroupType = type;
		this.notifications = new ArrayList<Notification>();
		this.priority = Double.NEGATIVE_INFINITY;

		this.title =
			new NotificationGroupTitleView(false, isMini, isImportant, this, notificationCenter);
		this.title.deleteButton.addActionListener(new NotificationGroupPropertyActionListener(this,
				this.notifications,
				NotificationGroupStateProperty.NOTIFICATION_GROUP_DELETED) {

			@Override
			public Object getNewValue() {
				if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(	NotificationGroupView.this,
																			"Are you sure you want to delete all of these notifications?  This action cannot be undone.",
																			"Confirm Notification Deletion",
																			JOptionPane.OK_CANCEL_OPTION)) {
					return true;
				} else {
					return false;
				}
			}

			@Override
			public Object getOldValue() {
				return false;
			}

		});

		this.setup();
	}

	/**
	 * Add a notification to this notification group. Update it's text to reflect what is going on.
	 * 
	 * @param notification
	 */
	public void addNotification(Notification notification) {
		this.notifications.add(notification);

		// Update the priority as necessary
		if (this.priority < notification.getPriority()) {
			this.priority = notification.getPriority();
		}
	}

	/**
	 * Tests to see if this notification group contains a given notification.
	 * 
	 * @param notification the notification to search for
	 * @return true if the group contains this notification, false otherwise
	 */
	public boolean containsNotification(Notification notification) {
		return this.notifications.contains(notification);
	}

	/**
	 * A getter for the add time of this notification.
	 * 
	 * @return addTime
	 */
	public long getAddTime() {
		return this.addTime;
	}

	/**
	 * A getter for notificationGroupType.
	 * 
	 * @return the notificationGroupType
	 */
	protected NotificationType getNotificationGroupType() {
		return this.notificationGroupType;
	}

	/**
	 * Returns the notifications associated with this view.
	 */
	public List<Notification> getNotifications() {
		return this.notifications;
	}

	/**
	 * A getter for priority.
	 * 
	 * @return the priority
	 */
	protected double getPriority() {
		return this.priority;
	}

	/**
	 * A getter for title.
	 * 
	 * @return the title
	 */
	public NotificationGroupTitleView getTitle() {
		return this.title;
	}

	/**
	 * A getter for the title's state of expansion.
	 * 
	 * @return true if expanded, false otherwise
	 */
	protected boolean isExpanded() {
		return this.title.isExpanded();
	}

	/**
	 * A getter for isImportantColorSet.
	 * 
	 * @return if the important color is set
	 */
	public boolean isImportantColorSet() {
		return this.title.isImportantColorSet();
	}

	/**
	 * Remove a notification from this group.
	 * 
	 * @param notification the notification to remove
	 */
	public void removeNotification(Notification notification) {
		this.notifications.remove(notification);
	}

	/**
	 * A setter for add time.
	 * 
	 * @param addTime the add time to change to
	 */
	public void setAddTime(long addTime) {
		this.addTime = addTime;
	}

	/**
	 * Sets the color of this notification group to make it flash.
	 * 
	 * @param currentValues the RGB values for this color
	 */
	public void setColor(double[] currentValues) {
		this.title.setBorder(new NotificationViewBorder(new Color(	(float)currentValues[0],
																	(float)currentValues[1],
																	(float)currentValues[2]),
														SystemColor.controlShadow));
	}

	/**
	 * A setter for isExpanded.
	 * 
	 * @param isExpanded the value to update to
	 */
	protected void setExpanded(boolean isExpanded) {
		this.title.setExpanded(isExpanded);
	}

	/**
	 * A setter for isImportantColorSet.
	 * 
	 * @param isImportantColorSet the value to set
	 */
	public void setImportantColorToggle(boolean isImportantColorSet) {
		this.title.setImportantColorToggle(isImportantColorSet);
	}

	/**
	 * @see java.awt.Component#setSize(java.awt.Dimension)
	 */
	@Override
	public void setSize(Dimension d) {
		this.title.setSize(d.width, this.getPreferredSize().height);
		super.setSize(d);
	}

	/**
	 * Sets up the appearance of the NotificationGroupView.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 4, 4);

		RelativeConstraints titleConstraints =
			new RelativeConstraints(bm.directLeftEdge(), bm.directTopEdge(), bm.directRightEdge());

		this.add(this.title, titleConstraints);

		this.setFocusable(true);

		this.update();
	}

	/**
	 * @see java.awt.Component#toString()
	 */
	@Override
	public String toString() {
		return "Group: " + this.hashCode() + " " + this.title.getNotificationGroupText();
	}

	/**
	 * Causes this group to update its state based on the NotificationManager's model state.
	 */
	@SuppressWarnings("synthetic-access")
	protected void update() {

		// Remove all NotificationViews from the NotificationGroupView
		for (int i = this.getComponents().length - 1; i >= 0; i-- ) {
			Component component = this.getComponent(i);
			if (!(component instanceof NotificationView)) {
				continue;
			}
			this.remove(component);
		}

		if (!this.title.isMini && this.title.isExpanded() && this.notifications.size() > 1) {

			Component lastView = this.title;
			for (Notification n : this.notifications) {

				// Now generate the constraints for this view:
				RelativeConstraints constraints =
					new RelativeConstraints(new Binding(Edge.LEFT,
														4,
														Direction.RIGHT,
														Edge.LEFT,
														this),
											new Binding(Edge.RIGHT,
														4,
														Direction.LEFT,
														Edge.RIGHT,
														this),
											new Binding(Edge.TOP,
														-1,
														Direction.BELOW,
														Edge.BOTTOM,
														lastView));

				final NotificationView view;
				view = new NotificationView(n);

				view.update();

				// Add the view:
				this.add(view, constraints);

				lastView = view;
			}

			this.setComponentZOrder(this.title, this.getComponentCount() - 1);
		}

		// Update the title
		int size = this.notifications.size();
		String text = "";
		ImageIcon icon = new ImageIcon();
		if (size > 0) {
			text = this.notifications.get(0).getName();
			switch (this.notificationGroupType) {
			case CALO_ACTION:
				icon = new ImageIcon("Resources/notification_CALO.png");
				if (size > 1) {
					text = size + " new CALO actions";
				}
				break;
			case EMAIL:
				text =
					this.notifications.get(0).getPerson().getName() + ": "
							+ this.notifications.get(0).getName() + ": "
							+ this.notifications.get(0).getText();
				icon = new ImageIcon("Resources/notification_email.png");
				if (size > 1) {
					text = size + " new emails";
				}
				break;
			case REMINDER:
				Calendar c = new GregorianCalendar();
				c.setTimeInMillis(this.addTime);
				text =
					DateFormat.getTimeInstance(DateFormat.SHORT).format(c.getTime()) + ": " + text;
				icon = new ImageIcon("Resources/notification_reminder.png");
				break;
			}
		}

		if (this.priority >= NotificationCenterView.DEFAULT_PRIORITY_CUTOFF) {
			ImageIcon urgentBadge = new ImageIcon("Resources/badge_urgent.png");
			icon =
				IconRenderer.getInstance().badgeIcon(icon, urgentBadge, BadgePosition.BOTTOM_RIGHT);
		}
		if (this.notifications.size() > 1) {
			ImageIcon countBadge = new ImageIcon("Resources/badge_green_circle.png");
			ImageIcon numberedBadge =
				IconRenderer.getInstance().drawTextOnIcon(	countBadge,
															Integer.toString(this.notifications.size()),
															Font.decode(Font.SANS_SERIF),
															Color.BLACK);
			icon =
				IconRenderer.getInstance().badgeIcon(	icon,
														numberedBadge,
														BadgePosition.BOTTOM_RIGHT);
		}

		this.title.setNumNotifications(this.notifications.size());
		this.title.setNotificationGroupText(text);
		this.title.setNotificationGroupIconImage(icon);

		this.title.update();
	}
}
