/*
 * File: NotificationPropertyActionListener.java
 * 
 * Contains class NotificationPropertyActionListener
 * 
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.notification.support.NotificationState;
import edu.cmu.hcii.calo.view.notification.support.NotificationViewPropertyNames;
import edu.cmu.hcii.calo.view.notification.support.NotificationState.NotificationStateProperty;

/**
 * An ActionListener subclass that will construct NotificationGroupStates for some property of the
 * old and new states of a notification and fire a NotificationState property change on behalf of
 * its view.
 * 
 * When you construct a NotificationPropertyActionListener, you pass the view, notification, and
 * property that it will use to fire property change events. When the listener detects an action
 * from the component to which it is listening, it will call getOldValue() and getNewValue() to
 * determine what values to put in the NotificationStates. Then it will call
 * CALOComponent#firePropertyChange(String, Object, Object) in the view to automatically fire the
 * property change on its behalf.
 * 
 * @author Will Haines
 * 
 */
abstract class NotificationPropertyActionListener implements ActionListener {

	/** The notification associated with this change. */
	private Notification notification;

	/** The view from which this change originated. */
	private CALOComponent view;

	/** The property that changed. */
	private NotificationStateProperty property;

	/**
	 * Create a new NotificationPropertyActionListener.
	 * 
	 * @param view the view from which the change originated
	 * @param n the notification associated with the change
	 * @param property the property that changed
	 */
	public NotificationPropertyActionListener(CALOComponent view,
			Notification n,
			NotificationState.NotificationStateProperty property) {
		this.view = view;
		this.notification = n;
		this.property = property;

	}

	/**
	 * Notify others about the action being performed.
	 * 
	 * @param e the event to forward
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@SuppressWarnings("synthetic-access")
	public void actionPerformed(ActionEvent e) {
		NotificationState oldState =
			new NotificationState(this.notification, this.property, this.getOldValue());
		NotificationState newState =
			new NotificationState(this.notification, this.property, this.getNewValue());

		this.view.firePropertyChange(	NotificationViewPropertyNames.NOTIFICATION_STATE,
										oldState,
										newState);
	}

	/**
	 * A getter for newValue.
	 * 
	 * @return newValue
	 */
	public abstract Object getNewValue();

	/**
	 * A getter for oldValue
	 * 
	 * @return oldValue
	 */
	public abstract Object getOldValue();
}
