/* File: NotificationView.java
 *
 * Contains class NotificationView
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import edu.cmu.hcii.calo.model.Notification;
import edu.cmu.hcii.calo.model.Notification.NotificationType;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.notification.support.NotificationState.NotificationStateProperty;
import edu.cmu.hcii.calo.view.training.TrainingMenuListener;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view class that displays a single notification.
 * 
 * @author Will Haines
 */
public class NotificationView extends CALOComponent implements MouseListener, MouseMotionListener {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 2199089034199693989L;

	/**
	 * @name Colors
	 * 
	 * These constants represent colors that are used in displaying the NotificationView.
	 */

	// @{
	private static final Color NOTIFICATION_COLOR = new Color(0.93f, 0.93f, 0.93f);

	private static final Color MOUSEOVER_NOTIFICATION_COLOR = new Color(0.97f, 0.97f, 0.97f);

	private static final Color NOTIFICATION_STROKE_COLOR = SystemColor.controlShadow;

	// @}

	/**
	 * Runs a small test program to verify that NotificationView works correctly.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		final JFrame testFrame = new JFrame("NotificationView Test");
		testFrame.setLayout(new RelativeLayout());
		final NotificationView notificationView =
			new NotificationView(new Notification(	NotificationType.CALO_ACTION,
													"My Notification",
													"This is a sample notification."));
		testFrame.add(notificationView, new RelativeConstraints(new Binding(Edge.TOP,
																			4,
																			Direction.BELOW,
																			Edge.TOP,
																			null),
																new Binding(Edge.LEFT,
																			4,
																			Direction.RIGHT,
																			Edge.LEFT,
																			null),
																new Binding(Edge.RIGHT,
																			4,
																			Direction.LEFT,
																			Edge.RIGHT,
																			null)));

		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setSize(500, 100);
		testFrame.setVisible(true);
	}

	/** The notification being represented by this view. */
	protected Notification notification;

	/**
	 * @name Borders
	 * 
	 * These constants are the various border styles that are applied to the NotificationView under
	 * various circumstances. Note that the rather badly-named Border actually determines the
	 * background color of the NotificationView as well.
	 */

	// @{
	private final NotificationViewBorder MOUSEOVER_BORDER =
		new NotificationViewBorder(	NotificationView.MOUSEOVER_NOTIFICATION_COLOR,
									NotificationView.NOTIFICATION_STROKE_COLOR);

	private final NotificationViewBorder NORMAL_BORDER =
		new NotificationViewBorder(	NotificationView.NOTIFICATION_COLOR,
									NotificationView.NOTIFICATION_STROKE_COLOR);

	// @}

	/**
	 * Whether or not the NotificationView is currently pressed. This may be true even if the mouse
	 * is not within the bounds of the view.
	 */
	private boolean isPressed;

	/** Whether or not the mouse is currently over the view. */
	private boolean isMouseOver;

	/** The text of this notification. */
	private JLabel notificationTextLabel;

	/** The button to delete this notification. */
	private IconButton deleteButton;

	/**
	 * Constructs a NotificationView.
	 */
	public NotificationView(Notification n) {
		super();

		this.notification = n;
		this.notificationTextLabel = new JLabel();
		ImageIcon deleteIcon = new ImageIcon("Resources/close.png");
		ImageIcon deleteOverIcon = new ImageIcon("Resources/close-over.png");
		ImageIcon deletePressedIcon = new ImageIcon("Resources/close-pressed.png");
		this.deleteButton = new IconButton(deleteIcon, deleteOverIcon, deletePressedIcon);
		this.deleteButton.addActionListener(new NotificationPropertyActionListener(this,
				this.notification,
				NotificationStateProperty.NOTIFICATION_DELETED) {

			@Override
			public Object getNewValue() {
				if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(	NotificationView.this,
																			"Are you sure you want to delete this notification?  This action cannot be undone.",
																			"Confirm Notification Deletion",
																			JOptionPane.OK_CANCEL_OPTION)) {
					return true;
				} else {
					return false;
				}
			}

			@Override
			public Object getOldValue() {
				return false;
			}

		});
		this.deleteButton.addMouseListener(this);
		this.isMouseOver = false;
		this.isPressed = false;

		this.addMouseListener(new TrainingMenuListener() {

			@Override
			public JMenuItem[] getItems() {
				if (NotificationView.this.getNotification().getType() == NotificationType.EMAIL) {
					return new JMenuItem[] { new JMenuItem("Close"), new JMenuItem("Open email") };
				} else if (NotificationView.this.getNotification().getType() == NotificationType.CALO_ACTION) {
					return new JMenuItem[] { new JMenuItem("Close"), new JMenuItem("Undo action") };
				} else {
					return new JMenuItem[] { new JMenuItem("Close") };
				}
			}

		});

		this.setup();
	}

	/**
	 * Returns the Notification associated with this view.
	 */
	public Notification getNotification() {
		return this.notification;
	}

	/**
	 * Defines the behavior of a mouse click on a notification; that is, nothing happens.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
	// This method intentionally left blank
	}

	/**
	 * Defines the behavior of a mouse drag on this notification view; that is, nothing happens.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
	// This method intentionally left blank
	}

	/**
	 * Updates the mouse over state of this notification view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
		if (!this.isPressed) {
			this.isMouseOver = true;
			this.updateAppearance();
		}
	}

	/**
	 * Updates the mouse over state of this notification view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		if (!this.isPressed) {
			this.isMouseOver = false;
			this.updateAppearance();
		}
	}

	/**
	 * Defines the behavior of a mouse motion in the notification view; that is, nothing happens.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
	// Intentionally left blank
	}

	/**
	 * Updates the pressed state of this notification view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		this.isPressed = true;
		this.updateAppearance();
	}

	/**
	 * Updates the pressed state of this notification view.
	 * 
	 * @param e the mouse event
	 * 
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		this.isPressed = false;
		this.updateAppearance();
	}

	/**
	 * Sets up the appearance of the NotificationView.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 4, 4);

		RelativeConstraints notificationTextLabelConstraints =
			new RelativeConstraints(bm.leftEdge(),
									new Binding(Edge.TOP, 4, Direction.BELOW, Edge.TOP, this),
									bm.directlyLeftOf(this.deleteButton));

		RelativeConstraints deleteButtonConstraints =
			new RelativeConstraints(new Binding(Edge.TOP, 4, Direction.BELOW, Edge.TOP, this),
									bm.rightEdge());

		this.add(this.notificationTextLabel, notificationTextLabelConstraints);
		this.add(this.deleteButton, deleteButtonConstraints);

		this.setFocusable(true);
		this.updateAppearance();

		this.addMouseListener(this);
		this.addMouseMotionListener(this);

		this.update();
	}

	/**
	 * Returns a string representation of this NotificationView. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return "NotificationView[" + this.getNotification().toString() + "]";
	}

	/**
	 * Causes the view to synchronize itself with the Notification model.
	 */
	public void update() {
		String text;

		// Email text is special
		if (this.getNotification().getType() == NotificationType.EMAIL) {
			text =
				this.notification.getPerson().getName() + ": " + this.notification.getName() + ": "
						+ this.notification.getText();
		}

		// Everything else is normal
		else {
			text = this.notification.getText();
		}

		this.notificationTextLabel.setText(text);
		this.setToolTipText(text);

		if (this.isMouseOver) {
			this.deleteButton.setVisible(true);
			this.deleteButton.setPreferredSize(null);
		} else {
			this.deleteButton.setVisible(false);
			this.deleteButton.setPreferredSize(new Dimension(	0,
																this.deleteButton.getPreferredSize().height));
		}
	}

	/**
	 * Updates the appearance of the view when something changes.
	 */
	private void updateAppearance() {
		// Set up the border:
		if (this.isMouseOver) {
			this.deleteButton.setVisible(true);
			this.setBorder(this.MOUSEOVER_BORDER);
		} else {
			this.deleteButton.setVisible(false);
			this.setBorder(this.NORMAL_BORDER);
		}
		this.update();
	}

	/**
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		Dimension superSize = super.getPreferredSize();
		return new Dimension(superSize.width, superSize.height + 4);
	}
}
