/* File: NotificationViewBorder.java
 *
 * Contains class NotificationViewBorder
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.border.AbstractBorder;

/**
 * A custom border (actually comprising both a border and a background color) for use with
 * NotificationViews. Also defines lots of useful constants.
 * 
 * @author Will Haines
 * 
 */
public class NotificationViewBorder extends AbstractBorder {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -6905593250955099862L;

	/**
	 * The color we will use to fill the background.
	 */
	private Color fillColor;

	/**
	 * The color we will use for the border's stroke.
	 */
	private Color strokeColor;

	/**
	 * The opacity of the border, on a scale from 0 to 1.
	 */
	private double opacity = 1.0;

	/**
	 * Constructs a NotificationViewBorder with the given fill and stroke colors.
	 * 
	 * @param fillColor the fill color to use
	 * @param strokeColor the stroke color to use
	 */
	public NotificationViewBorder(Color fillColor, Color strokeColor) {
		this(fillColor, strokeColor, false);
	}

	/**
	 * Constructs a new NotificationViewBorder with the given state.
	 * 
	 * @param fillColor the fill color for this border
	 * @param strokeColor the stroke color for this border
	 * @param isDropTarget \c true if this should be drawn as a drop target, \c false otherwise
	 */
	public NotificationViewBorder(Color fillColor, Color strokeColor, boolean isDropTarget) {
		super();
		this.fillColor = fillColor;
		this.strokeColor = strokeColor;
	}

	/**
	 * Private superclass constructor. Do not use.
	 */
	@SuppressWarnings("unused")
	private NotificationViewBorder() {
		super();
	}

	/**
	 * @see javax.swing.border.AbstractBorder#paintBorder(java.awt.Component, java.awt.Graphics,
	 *      int, int, int, int)
	 */
	@Override
	public void paintBorder(Component c, Graphics graphics, int x, int y, int width, int height) {
		super.paintBorder(c, graphics, x, y, width, height);

		Color effectiveFillColor =
			new Color(	this.fillColor.getRed(),
						this.fillColor.getGreen(),
						this.fillColor.getBlue(),
						((int)(this.opacity * 255.0)));
		Color effectiveStrokeColor =
			new Color(	this.strokeColor.getRed(),
						this.strokeColor.getGreen(),
						this.strokeColor.getBlue(),
						((int)(this.opacity * 255.0)));

		if (!(graphics instanceof Graphics2D)) {
			throw new UnsupportedOperationException("The Java graphics subsystem is not compatible with Java2D, so this border cannot be drawn.");
		}
		Graphics2D g = (Graphics2D)graphics;
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setColor(effectiveFillColor);
		g.fillRect(x, y, width, height);
		g.setColor(effectiveStrokeColor);
		float strokeWidth = ((BasicStroke)g.getStroke()).getLineWidth();
		g.drawRect(x, y, (int)(width - strokeWidth), (int)(height - strokeWidth));
	}
}
