/* File: NotificationGroupState.java
 *
 * Contains class NotificationGroupState
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.notification.support;

import java.util.List;

import edu.cmu.hcii.calo.model.Notification;

/**
 * Represents a particular property within the state of a NotificationGroup. When
 * NotificationPaneView generates PropertyChangedEvents with a name of
 * NotificationViewPropertyNames#NOTIFICATION_STATE_PROPERTY_NAME, the old and new values passed in
 * the event will be NotificationGroupState objects representing what has changed in what
 * notification.
 * 
 * @author Will Haines
 * 
 */
public class NotificationGroupState {

	/**
	 * Represents a particular property of a notification that can change.
	 * 
	 * @author Will Haines
	 * 
	 */
	public enum NotificationGroupStateProperty {
		/** A notification group was deleted. */
		NOTIFICATION_GROUP_DELETED, 
		/** The outside sidebar notification group changed. */
		OUTSIDE_NOTIFICATION_GROUP_CHANGED
	}

	/**
	 * The notifications to which this state pertains.
	 */
	private List<Notification> notificationGroup;

	/**
	 * The property that we are concerned with.
	 */
	private NotificationGroupStateProperty property;

	/**
	 * The value of that property.
	 */
	private Object value;

	/**
	 * Constructs a NotificationGroupState for the given notification group, noting that \p property
	 * has a value of \p value.
	 * 
	 * @param notifications the notifications to which this state pertains
	 * @param property the property that we are concerned with
	 * @param value the value of that property
	 */
	public NotificationGroupState(List<Notification> notifications,
			NotificationGroupStateProperty property,
			Object value) {
		this.notificationGroup = notifications;
		this.property = property;
		this.value = value;
	}

	/**
	 * Gets the Notifications for this NotificationGroupState.
	 * 
	 * @return the Notification
	 */
	public List<Notification> getNotifications() {
		return this.notificationGroup;
	}

	/**
	 * Gets the property for this NotificationGroupState.
	 * 
	 * @return the property
	 */
	public NotificationGroupStateProperty getProperty() {
		return this.property;
	}

	/**
	 * The value for this NotificationGroupState.
	 * 
	 * @return the Notification
	 */
	public Object getValue() {
		return this.value;
	}
}
