/* File: CALOPane.java
 *
 * Contains class CALOPane
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.panes;

import java.awt.Dimension;
import java.awt.SystemColor;

import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.border.Border;

import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A generic pane in the CALO sidebar.
 * 
 * @author Will Haines
 */
public class CALOPane extends CALOComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 7319381556003412785L;

	/**
	 * Runs a small test program to verify that CALOPane works correctly.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		final JFrame testFrame = new JFrame("CALOPane Test");
		testFrame.setLayout(new RelativeLayout());
		final PaneTitleView paneTitle = new PaneTitleView("Sample Pane", true);
		testFrame.add(paneTitle, new RelativeConstraints(	new Binding(Edge.TOP,
																		4,
																		Direction.BELOW,
																		Edge.TOP,
																		null),
															new Binding(Edge.LEFT,
																		4,
																		Direction.RIGHT,
																		Edge.LEFT,
																		null),
															new Binding(Edge.RIGHT,
																		4,
																		Direction.LEFT,
																		Edge.RIGHT,
																		null)));

		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setSize(400, 200);
		testFrame.setVisible(true);
	}

	/**
	 * The last known size of this pane when it's expanded.
	 */
	private Dimension savedSize;

	/**
	 * The title of this pane.
	 */
	private String title;

	/**
	 * @name Controls
	 * 
	 * Swing controls displayed in the pane.
	 */

	// @{
	private PaneTitleView titleView;
	private CALOComponent component;
	private DraggyBit draggyBit;
	// @}

	/**
	 * Whether this pane is scrollable.
	 */
	private boolean scrollable;

	/**
	 * Whether this pane is resizable.
	 */
	private boolean resizable;

	/**
	 * Whether this pane is collapsible.
	 */
	private boolean collapsible;

	/**
	 * Whether this pane was expanded before the last time update() was called on it.
	 */
	private boolean wasExpandedBeforeLastUpdate;

	/**
	 * Whether this pane was expanded before the title view was removed from it.
	 */
	private boolean wasExpandedBeforeTitleViewRemoved;

	/**
	 * Constructs a CALOPane with the given properties.
	 * 
	 * @param title the title of the pane
	 * @param component the main component that should be displayed inside the pane
	 * @param scrollable if \c true, this pane will expand its component to fill it, which is the
	 *            desired behavior if the component is inside a scroll pane
	 * @param collapsible if \c true, this pane's title region will have a collapse button
	 * @param resizable if \c true, this pane will have a draggy bit at the bottom of it to allow
	 *            the user to resize the pane
	 */
	public CALOPane(String title,
			CALOComponent component,
			boolean scrollable,
			boolean collapsible,
			boolean resizable) {
		this.title = title;

		this.titleView = new PaneTitleView(title, collapsible);
		this.component = component;
		this.draggyBit = new DraggyBit();

		this.scrollable = scrollable;
		this.resizable = resizable;
		this.collapsible = collapsible;

		this.wasExpandedBeforeLastUpdate = false;

		this.setLayout(new RelativeLayout());

		this.component.setOpaque(true);
		this.component.setBackground(SystemColor.control);
		// this.component.setBorder(new LineBorder(Color.RED));

		this.update();
	}

	/**
	 * Returns the component contained in this pane.
	 * 
	 * @return the component
	 */
	public CALOComponent getCALOComponent() {
		return this.component;
	}

	/**
	 * Returns the title of this pane.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Returns whether the pane is collapsible.
	 * 
	 * @return whether the pane is collapsible
	 */
	public boolean isCollapsible() {
		return this.collapsible;
	}

	/**
	 * Returns whether the pane is resizable.
	 * 
	 * @return whether the pane is resizable
	 */
	public boolean isResizable() {
		return this.resizable;
	}

	/**
	 * Returns whether the pane is scrollable.
	 * 
	 * @return whether the pane is scrollable
	 */
	public boolean isScrollable() {
		return this.scrollable;
	}

	/**
	 * Removes the title view, component, and draggy bit from this pane and re-adds them with the
	 * appropriate constraints. This allows the pane's appearance to be updated if its
	 * collapsibility or resizability are changed after it is created.
	 */
	private void reAddComponents() {
		this.remove(this.titleView);
		this.remove(this.component);
		this.remove(this.draggyBit);

		BindingMaker bm =
			new BindingMaker(0, 0, this.getInsets().bottom * 2, this.getInsets().right * 2, 0, 0);
		RelativeConstraints titleViewConstraints =
			new RelativeConstraints(bm.topEdge(), bm.leftEdge(), bm.rightEdge());
		RelativeConstraints paneConstraints =
			new RelativeConstraints(bm.below(this.titleView), bm.leftEdge(), bm.rightEdge());
		RelativeConstraints draggyBitConstraints =
			new RelativeConstraints(bm.bottomEdge(), bm.leftEdge(), bm.rightEdge());

		this.add(this.titleView, titleViewConstraints);

		if (this.scrollable) {
			if (this.resizable) {
				paneConstraints.addBinding(bm.above(this.draggyBit));
				this.add(this.draggyBit, draggyBitConstraints);
			} else {
				paneConstraints.addBinding(bm.bottomEdge());
			}
		}
		this.add(this.component, paneConstraints);
	}

	/**
	 * Sets the border of this pane. This is overridden because reAddComponents() has to take into
	 * account the border size for some strange reason.
	 * 
	 * @param border the border to set
	 */
	@Override
	public void setBorder(Border border) {
		super.setBorder(border);
		reAddComponents();
	}

	/**
	 * Sets whether this pane is collapsible.
	 * 
	 * @param collapsible whether this pane is collapsible
	 */
	public void setCollapsible(boolean collapsible) {
		if (collapsible == this.collapsible) {
			return;
		}

		this.collapsible = collapsible;

		if (!this.collapsible) {
			this.wasExpandedBeforeTitleViewRemoved = this.titleView.isExpanded();
		}

		// We have to remove this here because otherwise it won't be the same reference when we go
		// to remove it in reAddComponents().
		this.remove(this.titleView);

		this.titleView = new PaneTitleView(this.title, this.collapsible);

		if (this.collapsible) {
			this.titleView.setExpanded(this.wasExpandedBeforeTitleViewRemoved);
		}

		this.reAddComponents();
	}

	/**
	 * Sets whether this pane is resizable
	 * 
	 * @param resizable whether this pane is resizable
	 */
	public void setResizable(boolean resizable) {
		this.resizable = resizable;
		this.reAddComponents();
	}

	/**
	 * Sets whether this pane is scrollable
	 * 
	 * @param scrollable whether this pane is scrollable
	 */
	public void setScrollable(boolean scrollable) {
		this.scrollable = scrollable;
		this.reAddComponents();
	}

	/**
	 * Updates the visualization of this pane. Sets up the expansion.
	 */
	public void update() {

		if (this.wasExpandedBeforeLastUpdate == this.titleView.isExpanded()) {
			return;
		}

		if (this.titleView.isExpanded()) {
			this.reAddComponents();
			this.setPreferredSize(this.savedSize);
		} else {
			this.savedSize = this.getPreferredSize();
			this.setPreferredSize(null);
			this.remove(this.component);
			this.remove(this.draggyBit);
		}

		this.wasExpandedBeforeLastUpdate = this.titleView.isExpanded();
		this.revalidate();
		this.repaint();
	}
}
