/* File: PaneTitleView.java
 *
 * Contains class PaneTitleView
 *
 * Author: Will Haines
 */

package edu.cmu.hcii.calo.view.panes;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.UIManager;

import edu.cmu.hcii.calo.support.SidebarPropertyNames;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconToggleButton;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The title bars for the various CALO panes in the sidebar.
 * 
 * @author Will Haines
 * 
 */
public class PaneTitleView extends CALOComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 3459552455765647160L;

	/**
	 * Runs a small test program to verify that PaneTitle works correctly.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		final JFrame testFrame = new JFrame("PaneTitle Test");
		testFrame.setLayout(new RelativeLayout());
		final PaneTitleView paneTitle = new PaneTitleView("Sample Pane Title", true);
		testFrame.add(paneTitle, new RelativeConstraints(	new Binding(Edge.TOP,
																		4,
																		Direction.BELOW,
																		Edge.TOP,
																		null),
															new Binding(Edge.LEFT,
																		4,
																		Direction.RIGHT,
																		Edge.LEFT,
																		null),
															new Binding(Edge.RIGHT,
																		4,
																		Direction.LEFT,
																		Edge.RIGHT,
																		null)));

		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setSize(400, 200);
		testFrame.setVisible(true);
	}

	/** This pane's title. */
	private String title;

	/** The label for this pane's title. */
	private JLabel titleLabel;

	/** The minimize button */
	private IconToggleButton expandButton;

	/** Is this pane expanded? */
	private boolean isExpanded;

	/**
	 * The label for this pane's title's shadow.
	 */
	private JLabel titleShadowLabel;

	/**
	 * Create a new PaneTitle.
	 * 
	 * @param title this pane's title
	 * @param collapsible whether or not this pane is scrollable
	 */
	public PaneTitleView(String title, boolean collapsible) {
		super();

		this.title = title;
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(2, 6, 2, 6, 2, 2);

		if (collapsible) {
			ImageIcon contractIcon = new ImageIcon("Resources/chevron_up.png");
			ImageIcon contractOverIcon = new ImageIcon("Resources/chevron_up_hovered.png");
			ImageIcon contractPressedIcon = new ImageIcon("Resources/chevron_up_pressed.png");
			ImageIcon expandIcon = new ImageIcon("Resources/chevron_down.png");
			ImageIcon expandOverIcon = new ImageIcon("Resources/chevron_down_hovered.png");
			ImageIcon expandPressedIcon = new ImageIcon("Resources/chevron_down_pressed.png");

			this.expandButton =
				new IconToggleButton(	contractIcon,
										contractOverIcon,
										contractPressedIcon,
										expandIcon,
										expandOverIcon,
										expandPressedIcon);

			this.expandButton.addActionListener(new ActionListener() {

				/**
				 * Toggle the expanded state of this notification group.
				 */
				@SuppressWarnings("synthetic-access")
				public void actionPerformed(ActionEvent e) {
					PaneTitleView.this.toggleExpanded();
				}
			});

			RelativeConstraints minimizeButtonConstraints =
				new RelativeConstraints(bm.rightEdge(), bm.verticallyCenterAlign(null));
			this.add(this.expandButton, minimizeButtonConstraints);
		}

		this.titleLabel = new JLabel(title);
		this.titleLabel.setFont(this.titleLabel.getFont().deriveFont(Font.BOLD));
		this.titleShadowLabel = new JLabel(title);
		this.titleShadowLabel.setFont(this.titleLabel.getFont());

		RelativeConstraints titleConstraints =
			new RelativeConstraints(bm.leftEdge(), bm.verticallyCenterAlign(null));
		if (collapsible) {
			titleConstraints.addBinding(bm.directlyLeftOf(this.expandButton));
		} else {
			titleConstraints.addBinding(bm.rightEdge());
		}

		RelativeConstraints titleShadowConstraints =
			new RelativeConstraints(new Binding(Edge.TOP,
												1,
												Direction.BELOW,
												Edge.TOP,
												this.titleLabel),
									bm.leftAlign(this.titleLabel),
									bm.rightAlign(this.titleLabel));

		this.isExpanded = true;
		this.add(this.titleLabel, titleConstraints);
		this.add(this.titleShadowLabel, titleShadowConstraints);

	}

	/**
	 * Returns the preferred size of the title view, which is always 22 pixels tall.
	 */
	@Override
	public Dimension getPreferredSize() {
		return new Dimension(400, 22);
	}

	/**
	 * A getter for this pane's title.
	 * 
	 * @return the title
	 */
	protected String getTitle() {
		return this.title;
	}

	/**
	 * A getter for the expanded state of this pane.
	 * 
	 * @return the isExpanded
	 */
	protected boolean isExpanded() {
		return this.isExpanded;
	}

	/**
	 * Make the pane title pretty with a gradient and roundedness.
	 * 
	 * @param arg0 the graphics to paint with
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics arg0) {
		super.paintComponent(arg0);

		Graphics2D g = (Graphics2D)arg0;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		Color whiteGloss = new Color(1.0f, 1.0f, 1.0f, 0.75f);
		Color whiteTransparent = new Color(1.0f, 1.0f, 1.0f, 0.0f);
		// Color dark = new Color(0.49f, 0.78f, 0.94f, 1.0f);
		Color dark = UIManager.getColor("InternalFrame.activeTitleBackground");
		// Color lightGlow = new Color(0.6f, 1.0f, 1.0f, 1.0f);
		float[] hsb = Color.RGBtoHSB(dark.getRed(), dark.getGreen(), dark.getBlue(), null);

		// Try to bump up the saturation to make the color brighter. If the
		// color is already very
		// saturated, though, this won't work well and there'll be very little
		// noticeable change.
		// Interestingly, the solution in this case is to DEsaturate the color
		// to bring it closer to
		// white (as the brightness will always be full-on).
		float newSaturation = Math.min(1.0f, hsb[1] * 1.2f);
		if (newSaturation - hsb[1] < 0.1f) {
			hsb[1] = Math.max(0.0f, hsb[1] * 0.67f);
		} else {
			hsb[1] = newSaturation;
		}
		hsb[2] = 1.0f;
		Color lightGlow = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
		Color glowTransparent =
			new Color(	lightGlow.getRed() / 255f,
						lightGlow.getGreen() / 255f,
						lightGlow.getBlue() / 255f,
						0.0f);
		hsb = Color.RGBtoHSB(dark.getRed(), dark.getGreen(), dark.getBlue(), null);
		hsb[2] = Math.max(0.0f, hsb[2] - 0.25f);
		// Color reallyDark = new Color(0.23f, 0.41f, 0.62f, 1.0f);
		Color reallyDark = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));

		Image edgeGlow =
			new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_4BYTE_ABGR);
		Image edgeGlowMask =
			new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_4BYTE_ABGR);

		Graphics2D edgeGlowGraphics = (Graphics2D)edgeGlow.getGraphics();
		Graphics2D edgeGlowMaskGraphics = (Graphics2D)edgeGlowMask.getGraphics();

		edgeGlowGraphics.setColor(reallyDark);
		edgeGlowGraphics.fillRect(0, 0, this.getWidth(), this.getHeight());

		for (int i = 5; i > 0; i-- ) {
			float f = i;
			edgeGlowMaskGraphics.setStroke(new BasicStroke(	f,
															BasicStroke.CAP_BUTT,
															BasicStroke.JOIN_ROUND));
			edgeGlowMaskGraphics.setColor(new Color(0f, 0f, 0f, 0.75f / f));
			edgeGlowMaskGraphics.drawRect(-1, 0, this.getWidth() + 1, this.getHeight());
		}

		edgeGlowGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_ATOP));
		edgeGlowGraphics.drawImage(edgeGlowMask, 0, 0, null);

		Paint glossGradient =
			new GradientPaint(0, 0, whiteGloss, 0, this.getHeight() / 2, whiteTransparent);
		Paint glowGradient =
			new GradientPaint(	0,
								this.getHeight() / 2,
								glowTransparent,
								0,
								this.getHeight(),
								lightGlow);

		g.setColor(dark);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		g.setPaint(glowGradient);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		g.setPaint(glossGradient);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		g.drawImage(edgeGlow, 0, 0, null);

		this.titleLabel.setForeground(UIManager.getColor("InternalFrame.activeTitleForeground"));
		this.titleShadowLabel.setForeground(UIManager.getColor("InternalFrame.activeTitleBackground"));
	}

	/**
	 * A setter for the expanded state of this pane.
	 * 
	 * @param isExpanded the isExpanded to set
	 */
	protected void setExpanded(boolean isExpanded) {
		this.isExpanded = isExpanded;
		this.expandButton.setSelected(!this.isExpanded);
	}

	/**
	 * Toggles whether or not this title view claims the pane is expanded. This doesn't actually
	 * toggle whether the pane is expanded, but it does fire the appropriate property change event
	 * (with a name of PANE_EXPANDED) so the pane can set its state accordingly. It also changes the
	 * appearance of the expand/contract button to match the new state.
	 */
	private void toggleExpanded() {
		this.isExpanded = !this.isExpanded;

		this.firePropertyChange(SidebarPropertyNames.PANE_EXPANDED,
								!this.isExpanded,
								this.isExpanded);
	}
}
