/*
 * File: BlockScheduleView.java
 * 
 * Contains class BlockScheduleView
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.schedule;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.SwingUtilities;
import javax.swing.Timer;

import edu.cmu.hcii.calo.manager.ScheduleManager;
import edu.cmu.hcii.calo.model.ScheduleItem;
import edu.cmu.hcii.calo.view.AnimatedPane;
import edu.cmu.hcii.calo.view.animation.Animation;
import edu.cmu.hcii.calo.view.animation.AnimationManager;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationCurve;

/**
 * A view that displays schedule events in block (as opposed to agenda) view.
 * 
 * @author Brian Ellis
 * 
 */
public class BlockScheduleView extends AnimatedPane {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -4970443534285020287L;

	/** The default zoom level, expressed in pixels per hour */
	private static final float DEFAULT_ZOOM_PPH = 35.0f;

	/**
	 * The zoom increment multiplier. When the user zooms in, multiply the PPH by this amount. When
	 * the user zooms out, divide by this amount.
	 */
	private static final float ZOOM_INCREMENT = 1.5f;

	/**
	 * Returns \c true if the schedule item is within the range in which it should be visible on the
	 * schedule, \c false otherwise.
	 * 
	 * @param item the item to check
	 * @return \c true if the item should be visible, \c false otherwise
	 */
	private static boolean scheduleItemIsVisible(ScheduleItem item) {
		Calendar now = Calendar.getInstance();
		Calendar tomorrow = Calendar.getInstance();
		tomorrow.add(Calendar.DATE, 1);

		return (item.getEndTime().after(now) && item.getStartTime().before(tomorrow));
	}

	/**
	 * The current zoom level in pixels per hour.
	 */
	private float currentZoom = BlockScheduleView.DEFAULT_ZOOM_PPH;

	/**
	 * The mapping between the schedule items and the views that show them.
	 */
	private Map<ScheduleItem, ScheduleItemView> views =
		new HashMap<ScheduleItem, ScheduleItemView>();

	/**
	 * The mapping between the schedule item views and their bounds rectangles.
	 */
	private Map<ScheduleItemView, Rectangle> bounds = new HashMap<ScheduleItemView, Rectangle>();

	/**
	 * Whether or not we should skip animations on the next update.
	 */
	private boolean skipAnimations;

	/**
	 * Constructs a BlockScheduleView.
	 */
	public BlockScheduleView() {
		this.setLayout(null);
		this.setBackground(Color.WHITE);
		this.update();
		Timer t = new Timer(5000, new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (BlockScheduleView.this.getNumAnimating() == 0) {
					BlockScheduleView.this.update();
				}
			}

		});
		t.start();
	}

	/**
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doAnimations()
	 */
	@Override
	protected boolean doAnimations() {
		this.setTotalAnimations(0);
		for (ScheduleItemView view : this.views.values()) {
			Rectangle newBounds = this.getScheduleItemBounds(view);

			if (this.bounds.containsKey(view)) {
				Rectangle oldBounds = this.bounds.get(view);
				if (oldBounds.equals(newBounds)) {
					continue;
				}
				if (this.skipAnimations) {
					view.setBounds(newBounds);
					continue;
				}
				Animation a =
					AnimationManager.getManager().animateRect(	oldBounds,
																newBounds,
																AnimationCurve.EASE_OUT,
																0.25);
				a.setObjectData(view);
				a.start(this);
				this.incrementNumAnimating();
				this.setTotalAnimations(this.getNumAnimating());

			}
		}

		if (this.getTotalAnimations() > 0 && !this.skipAnimations) {
			return true;
		}
		this.skipAnimations = false;
		return false;
	}

	/**
	 * Tells the pane not to perform animations the next time update() is called.
	 */
	public void doNotAnimateNextUpdate() {
		this.skipAnimations = true;
	}

	/**
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doUpdate()
	 */
	@Override
	protected void doUpdate() {
		assert SwingUtilities.isEventDispatchThread() : "not on event dispatch thread!";
		for (ScheduleItem item : this.views.keySet()) {
			// Remove any items that have been removed from the manager:
			if (!ScheduleManager.getManager().hasScheduleItem(item)) {
				this.remove(this.views.get(item));
				this.views.remove(item);
			}

			// Also remove any items that are no longer between now and 24 hours
			// from now:
			if (!BlockScheduleView.scheduleItemIsVisible(item)) {
				this.remove(this.views.get(item));
				this.views.remove(item);
			}
		}

		// Add views for any new items that are in range:
		Iterator<ScheduleItem> iter = ScheduleManager.getManager().scheduleIterator();
		while (iter.hasNext()) {
			ScheduleItem item = iter.next();
			if (!this.views.containsKey(item)) {
				if (BlockScheduleView.scheduleItemIsVisible(item)) {
					ScheduleItemView view = new ScheduleItemView(item);
					this.views.put(item, view);
					this.add(view);
				}
			}
		}
	}

	/**
	 * Gets the desired bounds of the given schedule item view based on the current time.
	 * 
	 * @param view the view to ask about bounds for
	 * @return the desired bounds for the view
	 */
	private Rectangle getScheduleItemBounds(ScheduleItemView view) {
		Calendar startTime = view.getScheduleItem().getStartTime();
		Calendar endTime = view.getScheduleItem().getEndTime();
		Calendar now = Calendar.getInstance();

		float startHoursAway =
			(startTime.getTimeInMillis() - now.getTimeInMillis()) / 1000f / 60f / 60f;
		float endHoursAway =
			(endTime.getTimeInMillis() - now.getTimeInMillis()) / 1000f / 60f / 60f;

		Rectangle newBounds =
			new Rectangle(	0,
							(int)(startHoursAway * this.currentZoom),
							this.getWidth(),
							(int)((endHoursAway - startHoursAway) * this.currentZoom));

		if (newBounds.height < 16) {
			newBounds.height = 16;
		}
		return newBounds;
	}

	/**
	 * @see edu.cmu.hcii.calo.view.TranslucentView#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics graphics) {
		Graphics2D g = (Graphics2D)graphics.create();

		g.setColor(this.getBackground());
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		g.setColor(new Color(0f, 0f, 0f, 0.25f));

		Calendar now = Calendar.getInstance();
		now.add(Calendar.MINUTE, -30);

		// Compute initial line distance:
		float lineDistance = (60.0f - now.get(Calendar.MINUTE)) / 60.0f * this.currentZoom;
		while (lineDistance < this.getHeight()) {
			g.drawLine(0, (int)lineDistance, this.getWidth(), (int)lineDistance);
			now.set(Calendar.MINUTE, 0);
			now.add(Calendar.HOUR_OF_DAY, 1);
			g.drawString(	DateFormat.getTimeInstance(DateFormat.SHORT).format(now.getTime()),
							4,
							lineDistance - 2);

			// Increment by an hour:
			lineDistance += this.currentZoom;
		}

		g.dispose();
	}

	@Override
	protected void prepareAnimations() {
		for (ScheduleItemView view : this.views.values()) {
			this.bounds.put(view, view.getBounds());
		}
	}

	@Override
	protected void safeUpdateAnimation(Animation a) {
		ScheduleItemView view = (ScheduleItemView)a.getObjectData();
		view.setBounds(a.getCurrentRectangle().getBounds());
		view.doLayout();
	}

	/**
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	@Override
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		for (ScheduleItemView view : this.views.values()) {
			view.setBounds(0, view.getY(), width, view.getHeight());
		}
	}

	/**
	 * @see java.awt.Component#setBounds(java.awt.Rectangle)
	 */
	@Override
	public void setBounds(Rectangle r) {
		super.setBounds(r);
		for (ScheduleItemView view : this.views.values()) {
			view.setBounds(0, view.getY(), r.width, view.getHeight());
		}

	}

	/**
	 * Zooms the view in one unit.
	 */
	public void zoomIn() {
		this.currentZoom *= BlockScheduleView.ZOOM_INCREMENT;
		this.update();
	}

	/**
	 * Zooms the view out one unit.
	 */
	public void zoomOut() {
		this.currentZoom /= BlockScheduleView.ZOOM_INCREMENT;
		this.update();
	}
}
