/* File: ScheduleItemView.java
 *
 * Contains class ScheduleItemView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.schedule;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.text.DateFormat;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

import edu.cmu.hcii.calo.model.ScheduleItem;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view that displays a schedule item in block view.
 * 
 * @author Brian Ellis
 * 
 */
public class ScheduleItemView extends CALOComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -2534503572581677568L;

	/**
	 * The schedule item displayed by this view.
	 */
	private ScheduleItem scheduleItem;

	/**
	 * Constructs a ScheduleItemView with the given schedule item.
	 * 
	 * @param scheduleItem the schedule item
	 */
	public ScheduleItemView(ScheduleItem scheduleItem) {
		super();
		this.scheduleItem = scheduleItem;
		this.setup();
	}

	/**
	 * Returns the schedule item displayed by this view.
	 * 
	 * @return the schedule item
	 */
	public ScheduleItem getScheduleItem() {
		return this.scheduleItem;
	}

	/**
	 * @see edu.cmu.hcii.calo.view.TranslucentView#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics graphics) {
		Graphics2D g = (Graphics2D)graphics.create();

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setRenderingHint(	RenderingHints.KEY_FRACTIONALMETRICS,
							RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		g.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);

		g.setColor(new Color(0.73f, 0.80f, 0.88f));
		g.fillRoundRect(0, 0, this.getWidth(), this.getHeight(), 8, 8);

		g.setColor(new Color(0.35f, 0.53f, 0.71f));
		Area a =
			new Area(new RoundRectangle2D.Float(0f, 0f, this.getWidth(), this.getHeight(), 8f, 8f));
		a.subtract(new Area(new Rectangle2D.Float(0f, 16f, this.getWidth(), this.getHeight() - 16f)));
		g.fill(a);
		g.drawRoundRect(0, 0, this.getWidth() - 1, this.getHeight() - 1, 8, 8);

		g.setFont(Font.decode(Font.DIALOG));
		g.setColor(Color.WHITE);
		int textY = 16 - g.getFontMetrics().getDescent();
		String timeString =
			DateFormat.getTimeInstance(DateFormat.SHORT).format(this.scheduleItem.getStartTime().getTime());
		if (this.scheduleItem.getEndTime().after(this.scheduleItem.getStartTime())) {
			timeString +=
				" - "
						+ DateFormat.getTimeInstance(DateFormat.SHORT).format(this.scheduleItem.getEndTime().getTime());
		}
		g.drawString(timeString, 8, textY);

		g.dispose();
	}

	/**
	 * Sets the schedule item displayed by this view.
	 * 
	 * @param scheduleItem the schedule item
	 */
	public void setScheduleItem(ScheduleItem scheduleItem) {
		this.scheduleItem = scheduleItem;
		this.repaint();
	}

	/**
	 * Sets up the visual appearance of this view.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());
		BindingMaker bm = new BindingMaker(18, 8, 4, 8, 4, 4);
		JLabel label = new JLabel("<html><p>" + this.scheduleItem.getName() + "</p></html>");
		label.setVerticalAlignment(SwingConstants.TOP);
		this.add(label, new RelativeConstraints(bm.topEdge(),
												bm.leftEdge(),
												bm.rightEdge(),
												bm.bottomEdge()));

		IconButton packButton = new IconButton(new ImageIcon("Resources/pack.png"));
		packButton.setToolTipText("Click to see a pack for this event.");
		packButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				JOptionPane msg =
					new JOptionPane("<html><p>In the fully working system, this icon would bring up a window "
							+ "showing you resources such as files and emails that are relevant to the event \""
							+ ScheduleItemView.this.scheduleItem.getName()
							+ "\".  However, this feature is not yet implemented.  "
							+ "You should be able to complete this task without using it.</p></html>");
				msg.setMessageType(JOptionPane.INFORMATION_MESSAGE);

				JDialog dialog = msg.createDialog(ScheduleItemView.this, "Sorry!");
				dialog.setSize(new Dimension(600, 130));
				dialog.setLocationRelativeTo(ScheduleItemView.this);
				dialog.setVisible(true);
			}

		});
		this.add(packButton, new RelativeConstraints(bm.rightEdge(), bm.bottomEdge()));
	}
}
