/* File: SchedulePaneView.java
 *
 * Contains class SchedulePaneView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.schedule;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.border.EmptyBorder;

import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.FlatButton;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The schedule pane in the sidebar, which can display the schedule in either block or agenda view.
 * Actually it just displays in block view because agenda view isn't implemented, but you get the
 * idea.
 * 
 * @author Brian Ellis
 * 
 */
public class SchedulePaneView extends CALOComponent {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -2088775096824020566L;

	/**
	 * The view used to display the schedule in block view.
	 */
	private BlockScheduleView blockScheduleView;

	/**
	 * Constructs a SchedulePaneView.
	 */
	public SchedulePaneView() {
		super();
		this.blockScheduleView = new BlockScheduleView();
		this.setup();
	}

	/**
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	@Override
	public void setBounds(int x, int y, int width, int height) {
		Rectangle oldBounds = this.getBounds();
		super.setBounds(x, y, width, height);
		if (!oldBounds.equals(this.getBounds())) {
			this.blockScheduleView.doNotAnimateNextUpdate();
		}
	}

	/**
	 * @see java.awt.Component#setBounds(java.awt.Rectangle)
	 */
	@Override
	public void setBounds(Rectangle r) {
		Rectangle oldBounds = this.getBounds();
		super.setBounds(r);
		if (!oldBounds.equals(this.getBounds())) {
			this.blockScheduleView.doNotAnimateNextUpdate();
		}
	}

	/**
	 * Sets up the visual appearance of this view.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		CALOComponent controlBar = new CALOComponent();
		controlBar.setLayout(new RelativeLayout());

		Icon blockViewIcon = new ImageIcon("Resources/block_schedule_button.png");
		Icon agendaViewIcon = new ImageIcon("Resources/agenda_schedule_button.png");

		FlatButton scheduleViewButton = new FlatButton(blockViewIcon);
		FlatButton agendaViewButton = new FlatButton(agendaViewIcon);

		ImageIcon zoomOutIcon = new ImageIcon("Resources/zoomout.png");
		IconButton zoomOutButton = new IconButton(zoomOutIcon);
		zoomOutButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				SchedulePaneView.this.blockScheduleView.zoomOut();
			}

		});

		ImageIcon zoomInIcon = new ImageIcon("Resources/zoomin.png");
		IconButton zoomInButton = new IconButton(zoomInIcon);
		zoomInButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				SchedulePaneView.this.blockScheduleView.zoomIn();
			}

		});

		IconButton addButton = new IconButton(new ImageIcon("Resources/add_event.png"));
		addButton.setToolTipText("Click to add an event to your schedule.");
		addButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(	SchedulePaneView.this,
												"In the full implementation, this button would allow you to add a new event to your schedule.\nUnfortunately, this is not yet implemented.");
			}

		});

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 2, 2);

		RelativeConstraints scheduleViewButtonConstraints =
			new RelativeConstraints(bm.leftEdge(), bm.topEdge());
		RelativeConstraints agendaViewButtonConstraints =
			new RelativeConstraints(bm.rightOf(scheduleViewButton),
									bm.verticallyCenterAlign(scheduleViewButton));
		RelativeConstraints zoomInButtonConstraints =
			new RelativeConstraints(bm.rightOf(zoomOutButton),
									bm.verticallyCenterAlign(scheduleViewButton));
		RelativeConstraints zoomOutButtonConstraints =
			new RelativeConstraints(bm.rightOf(agendaViewButton),
									bm.verticallyCenterAlign(scheduleViewButton));
		RelativeConstraints addButtonConstraints =
			new RelativeConstraints(bm.rightEdge(), bm.verticallyCenterAlign(scheduleViewButton));

		controlBar.add(scheduleViewButton, scheduleViewButtonConstraints);
		controlBar.add(agendaViewButton, agendaViewButtonConstraints);
		controlBar.add(zoomInButton, zoomInButtonConstraints);
		controlBar.add(zoomOutButton, zoomOutButtonConstraints);
		controlBar.add(addButton, addButtonConstraints);

		controlBar.setBorder(new EmptyBorder(0, 0, 2, 0));

		RelativeConstraints scheduleViewConstraints =
			new RelativeConstraints(bm.directTopEdge(),
									bm.directLeftEdge(),
									bm.directRightEdge(),
									bm.directlyAbove(controlBar));

		RelativeConstraints controlBarConstraints =
			new RelativeConstraints(bm.directBottomEdge(), bm.leftEdge(), bm.rightEdge());

		this.add(this.blockScheduleView, scheduleViewConstraints);
		this.add(controlBar, controlBarConstraints);

		this.update();
	}

	/**
	 * Updates the schedule based on the state of the schedule manager.
	 */
	public void update() {
		this.blockScheduleView.update();
	}
}
