/*
 * File: ChooseDueDatePanel.java
 * 
 * Contains class ChooseDueDatePanel
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.toedter.calendar.JCalendar;

import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.TimeSpinner;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A panel that allows the user to choose a due date and allows other windows to query what date the
 * user chose. This panel is designed to be used in a JDialog in a modal session; show the panel,
 * then call getReturnCode() to see if the user clicked OK, and if so call getDate() to get the date
 * the user chose.
 * 
 * @author Brian Ellis
 * 
 */
public class ChooseDueDatePanel extends JPanel implements PropertyChangeListener, ChangeListener {

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = 7157144022358657916L;

	/**
	 * The ID returned by getReturnCode() when the user clicks the OK button.
	 */
	public static final int OK = 0;

	/**
	 * The ID returned by getReturnCode() when the user clicks the Cancel button or closes the
	 * window without clicking OK.
	 */
	public static final int USER_CANCELED = 1;

	/**
	 * The return code that we should return from getReturnCode().
	 */
	private int returnCode = ChooseDueDatePanel.USER_CANCELED;

	/**
	 * @name Controls
	 * 
	 * The various controls in the panel.
	 */

	// @{
	private JCalendar calendar;
	private TimeSpinner timeSpinner;
	private JRadioButton noDueDateRadio;
	private JRadioButton dueDateRadio;
	private JButton okButton;

	// @}

	/**
	 * Constructs a new DueDatePanel with its initially selected date set to the current date and
	 * time.
	 */
	public ChooseDueDatePanel() {
		this(null);
	}

	/**
	 * Constructs a new DueDatePanel with its initially selected date set to the due date of the
	 * given task, or the current date and time if the task has no due date.
	 * 
	 * @param t the task whose due date we should use to initialize the panel
	 */
	public ChooseDueDatePanel(Task t) {
		super();
		this.setup(t);
		this.calendar.addPropertyChangeListener(this);
		this.timeSpinner.addChangeListener(this);
	}

	/**
	 * Returns the date the user chose from the panel, or \c null if they selected the "no due date"
	 * radio button. Note that this will NOT return \c null if the user clicked Cancel; the return
	 * value in that case is what they would have chosen had they clicked OK. Thus, it is important
	 * to call getReturnCode() before assuming the return value of this method is valid.
	 * 
	 * @return the date chosen by the user, or \c null if the user chose "no due date" as the value
	 */
	public Calendar getDate() {
		if (this.noDueDateRadio.isSelected()) {
			return null;
		}
		Calendar date = this.calendar.getCalendar();
		Calendar time = new GregorianCalendar();
		time.setTime((Date)this.timeSpinner.getValue());
		date.set(Calendar.HOUR_OF_DAY, time.get(Calendar.HOUR_OF_DAY));
		date.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
		date.set(Calendar.SECOND, time.get(Calendar.SECOND));

		return date;
	}

	/**
	 * Returns the OK button. This is necessary so the OK button can be set as the default button in
	 * whatever JFrame or JDialog it is placed into.
	 * 
	 * @return
	 */
	public JButton getOKButton() {
		return this.okButton;
	}

	/**
	 * Gets the return code, which will have the value OK if the user clicked the OK button and
	 * USER_CANCELLED if the user clicked the Cancel button or closed the window without clicking
	 * OK. If getReturnCode() is called while the panel is still visible, USER_CANCELLED is also
	 * returned, so don't call this until you're sure the interaction has finished.
	 * 
	 * @return the return code
	 */
	public int getReturnCode() {
		return this.returnCode;
	}

	/**
	 * Set the due date radio button to selected if the user interacts with the calendar to set the
	 * date.
	 */
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == "calendar") {
			this.dueDateRadio.setSelected(true);
		}
	}

	/**
	 * Sets up the visual appearance of this panel.
	 * 
	 * @param t the task to use to initialize the panel
	 */
	private void setup(Task t) {
		Calendar c = t.getDueDate();
		JLabel titleLabel = new JLabel("Set due date for " + t.getName() + ":");
		titleLabel.setFont(titleLabel.getFont().deriveFont(Font.BOLD));

		this.noDueDateRadio = new JRadioButton("No due date");
		this.dueDateRadio = new JRadioButton("Due at:");
		ButtonGroup dueDateRadios = new ButtonGroup();
		dueDateRadios.add(this.noDueDateRadio);
		dueDateRadios.add(this.dueDateRadio);

		this.noDueDateRadio.setSelected(c == null);
		this.dueDateRadio.setSelected(c != null);
		JButton cancelButton = new JButton("Cancel");
		this.okButton = new JButton("OK");
		Calendar initialDate;
		if (c == null) {
			initialDate = Calendar.getInstance();
		} else {
			initialDate = c;
		}
		this.calendar = new JCalendar(initialDate);
		this.timeSpinner = new TimeSpinner();
		this.timeSpinner.setValue(initialDate.getTime());

		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(8, 8, 8, 8, 4, 8);

		this.add(titleLabel, new RelativeConstraints(bm.topEdge(), bm.leftEdge(), bm.rightEdge()));

		this.add(this.noDueDateRadio, new RelativeConstraints(bm.below(titleLabel), bm.leftEdge()));
		int oldVerticalSpacing = bm.getVerticalSpacing();
		bm.setVerticalSpacing(4);
		this.add(this.dueDateRadio, new RelativeConstraints(bm.below(this.noDueDateRadio),
															bm.leftEdge()));
		bm.setVerticalSpacing(oldVerticalSpacing);
		int oldLeftMargin = bm.getLeftMargin();

		bm.setLeftMargin(28);
		this.add(	this.timeSpinner,
					new RelativeConstraints(bm.rightOf(this.dueDateRadio),
											bm.verticallyCenterAlign(this.dueDateRadio)));
		this.add(this.calendar, new RelativeConstraints(bm.below(this.timeSpinner), bm.leftEdge()));
		bm.setLeftMargin(oldLeftMargin);

		this.add(this.okButton, new RelativeConstraints(bm.below(this.calendar),
														bm.leftOf(cancelButton)));
		this.add(cancelButton, new RelativeConstraints(bm.below(this.calendar), bm.rightEdge()));

		this.okButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				ChooseDueDatePanel.this.returnCode = ChooseDueDatePanel.OK;
				Container topLevelAncestor = ChooseDueDatePanel.this.getTopLevelAncestor();
				topLevelAncestor.setVisible(false);
			}

		});

		cancelButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				ChooseDueDatePanel.this.returnCode = ChooseDueDatePanel.USER_CANCELED;
				Container topLevelAncestor = ChooseDueDatePanel.this.getTopLevelAncestor();
				topLevelAncestor.setVisible(false);
			}

		});

		this.calendar.setBorder(BorderFactory.createLoweredBevelBorder());
		this.setBorder(new EmptyBorder(0, 0, 8, 0));
		this.doLayout();
		this.timeSpinner.setPreferredSize(new Dimension(100,
														this.timeSpinner.getPreferredSize().height));
		this.setPreferredSize(new Dimension(this.calendar.getBounds().x
				+ this.calendar.getBounds().width + 8, this.getPreferredSize().height));

		this.setBorder(new LineBorder(SystemColor.controlShadow));
	}

	/**
	 * Invoked when the value of the time spinner changes.
	 * 
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {
		this.dueDateRadio.setSelected(true);
	}
}
