/*
 * File: ExpandedTaskInfoPanel.java
 * 
 * Contains class ExpandedTaskInfoPanel
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import edu.cmu.hcii.calo.model.Resource;
import edu.cmu.hcii.calo.model.ResourceList;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.util.CommandLauncher;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.EditableLabelListener;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.resources.ResourceListCellRenderer;
import edu.cmu.hcii.calo.view.resources.ResourceListModel;
import edu.cmu.hcii.calo.view.task.support.TaskState;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.hcii.calo.view.task.support.TaskState.TaskStateProperty;
import edu.cmu.hcii.calo.view.training.TrainingMenuListener;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * The panel that contains controls visible only when a TaskView is in its expanded state.
 * 
 * @author Brian Ellis
 * 
 */
public class ExpandedTaskInfoPanel extends CALOComponent {

	/**
	 * The string to display when a task has no defined tags.
	 */
	private static final String NO_TAGS_STRING = "(no tags)";

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Returns a string representation of the given tags suitable for display in the tags field.
	 * 
	 * @param tags the set of tags to stringify
	 * @return the string representation
	 */
	private static String getTagsAsString(Set<String> tags) {
		StringBuffer buf = new StringBuffer();
		Iterator<String> iter = tags.iterator();
		while (iter.hasNext()) {
			String tag = iter.next();
			buf.append(tag);
			if (iter.hasNext()) {
				buf.append(", ");
			}
		}
		return buf.toString();
	}

	/**
	 * The task associated with this panel's parent TaskView.
	 */
	private Task task;

	/**
	 * @name Controls
	 * 
	 * Various controls in this panel.
	 */

	// @{
	private JList resourceList;
	private JLabel tagsLabelLabel;
	private JLabel tagsLabel;
	private JTextField tagsField;
	private IconButton addResourceButton;

	// @}

	/**
	 * Constructs an ExpandedTaskInfoPanel for the given task.
	 * 
	 * @param t the task to represent with this panel
	 */
	public ExpandedTaskInfoPanel(Task t) {
		super();
		this.task = t;

		this.addResourceButton = new IconButton(new ImageIcon("Resources/add_document.png"));

		this.resourceList = new JList(new ResourceListModel(this.task));
		this.resourceList.setCellRenderer(new ResourceListCellRenderer());

		this.resourceList.addMouseListener(new MouseAdapter() {

			@SuppressWarnings("synthetic-access")
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					int index =
						ExpandedTaskInfoPanel.this.resourceList.locationToIndex(e.getPoint());
					assert ExpandedTaskInfoPanel.this.resourceList.getModel().getElementAt(index) instanceof Resource : "resource list model contained something that wasn't a resource";
					Resource r =
						(Resource)ExpandedTaskInfoPanel.this.resourceList.getModel().getElementAt(index);
					CommandLauncher.exec(r.getAssociatedFile().getAbsolutePath());
				}
			}

		});

		this.tagsLabelLabel = new JLabel("Tags:");
		this.tagsLabel = new JLabel();
		this.tagsField = new JTextField();

		@SuppressWarnings("unused")
		EditableLabelListener tagsFieldListener =
			new EditableLabelListener(this, this.tagsLabel, this.tagsField) {

				@Override
				protected String fieldTransform(String fieldText) {
					if (fieldText.equals("")) {
						return NO_TAGS_STRING;
					} else {
						return fieldText;
					}
				}

				@Override
				protected String labelTransform(String labelText) {
					if (labelText.equals(NO_TAGS_STRING)) {
						return "";
					} else {
						return labelText;
					}
				}

			};

		this.tagsField.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.TAGS) {

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getNewValue() {
				return getTagsAsSet(ExpandedTaskInfoPanel.this.tagsField.getText());
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return ExpandedTaskInfoPanel.this.task.getTags();
			}

		});

		this.tagsField.setVisible(false);
		this.tagsLabel.setVisible(true);
		this.tagsLabel.setBorder(EditableLabelListener.NORMAL_LABEL_BORDER);
		this.tagsLabel.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		this.tagsField.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));

		this.setup();
	}

	/**
	 * Returns the Set of tags represented by the given string, which should be in the same format
	 * as the string representation returned by getTagsAsString(). Specifically, the string should
	 * be a comma-delimited list of tags with arbitrary whitespace before or after the commas.
	 * 
	 * @param tags the string to make into a Set
	 * @return the Set of tags
	 */
	private static Set<String> getTagsAsSet(String tags) {
		Set<String> tagSet = new HashSet<String>();
		StringTokenizer tok = new StringTokenizer(tags, ",");
		while (tok.hasMoreTokens()) {
			tagSet.add(tok.nextToken().trim());
		}
		return tagSet;
	}

	/**
	 * Adds the given mouse listener to every component in this panel.
	 * 
	 * @param l the listener to add
	 */
	public void addMouseListenerToComponents(MouseListener l) {
		for (Component c : this.getComponents()) {
			c.addMouseListener(l);
		}

	}

	/**
	 * Tells this panel to update its appearance based on the task state.
	 * 
	 */
	public void update() {
		((ResourceListModel)this.resourceList.getModel()).updateList();
		if (this.task.getTags().size() == 0) {
			this.tagsLabel.setText(NO_TAGS_STRING);
		} else {
			this.tagsLabel.setText(ExpandedTaskInfoPanel.getTagsAsString(this.task.getTags()));
		}
		this.tagsLabel.setToolTipText("<html><p><strong>" + this.tagsLabel.getText()
				+ "</strong></p><p>Click to edit the tags associated with this task.</p></html>");

	}

	/**
	 * Sets up the visual appearance of this panel.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		this.addResourceButton.setToolTipText("Click to associate a file or other resource with this task.");
		this.addResourceButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setMultiSelectionEnabled(true);
				fileChooser.setDialogTitle("Associate Resources");
				int result = fileChooser.showOpenDialog(ExpandedTaskInfoPanel.this);
				if (result != JFileChooser.APPROVE_OPTION) {
					return;
				}

				ResourceList newResources = new ResourceList();
				Iterator<Resource> iter = ExpandedTaskInfoPanel.this.task.getResources().iterator();
				while (iter.hasNext()) {
					Resource rsrc = iter.next();
					newResources.add(rsrc);
				}
				for (File f : fileChooser.getSelectedFiles()) {
					newResources.add(new Resource(f));
				}

				TaskState oldState =
					new TaskState(	ExpandedTaskInfoPanel.this.task,
									TaskStateProperty.RESOURCES,
									ExpandedTaskInfoPanel.this.task.getResources());
				TaskState newState =
					new TaskState(	ExpandedTaskInfoPanel.this.task,
									TaskStateProperty.RESOURCES,
									newResources);

				ExpandedTaskInfoPanel.this.firePropertyChange(	TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME,
																oldState,
																newState);

			}

		});

		Font tagsFont = this.tagsLabel.getFont().deriveFont(9.0f).deriveFont(Font.ITALIC);
		this.tagsLabelLabel.setFont(this.tagsLabelLabel.getFont().deriveFont(9.0f));
		this.tagsLabelLabel.setForeground(Color.DARK_GRAY);
		this.tagsLabel.setFont(tagsFont);
		this.tagsField.setFont(tagsFont);
		this.tagsLabel.setForeground(Color.DARK_GRAY);

		BindingMaker bm = new BindingMaker(0, 0, 0, 0, 2, 2);

		RelativeConstraints tagsLabelLabelConstraints =
			new RelativeConstraints(bm.leftEdge(), bm.verticallyCenterAlign(this.tagsField));

		RelativeConstraints tagsLabelConstraints;
		RelativeConstraints tagsFieldConstraints;
		RelativeConstraints addResourceButtonConstraints;

		if (this.task.getResources().size() > 0) {
			tagsLabelConstraints =
				new RelativeConstraints(bm.rightOf(this.tagsLabelLabel),
										bm.rightEdge(),
										bm.verticallyCenterAlign(this.tagsField));

			tagsFieldConstraints =
				new RelativeConstraints(bm.rightOf(this.tagsLabelLabel),
										bm.rightEdge(),
										bm.topEdge());

			addResourceButtonConstraints =
				new RelativeConstraints(bm.bottomAlign(this.resourceList), bm.rightEdge());
		} else {
			tagsLabelConstraints =
				new RelativeConstraints(bm.rightOf(this.tagsLabelLabel),
										bm.leftOf(this.addResourceButton),
										bm.verticallyCenterAlign(this.tagsField));

			tagsFieldConstraints =
				new RelativeConstraints(bm.rightOf(this.tagsLabelLabel),
										bm.leftOf(this.addResourceButton),
										bm.topEdge());

			addResourceButtonConstraints =
				new RelativeConstraints(bm.bottomAlign(this.tagsField), bm.rightEdge());
		}

		this.add(this.tagsLabelLabel, tagsLabelLabelConstraints);
		this.add(this.tagsLabel, tagsLabelConstraints);
		this.add(this.tagsField, tagsFieldConstraints);

		RelativeConstraints resourceListConstraints =
			new RelativeConstraints(bm.below(this.tagsField),
									bm.leftEdge(),
									bm.leftOf(this.addResourceButton));

		this.resourceList.setBorder(new LineBorder(new Color(0f, 0f, 0f, 0.25f)));

		this.add(this.addResourceButton, addResourceButtonConstraints);
		this.add(this.resourceList, resourceListConstraints);

		if (this.task.getResources().size() > 0) {
			this.setBorder(new EmptyBorder(0, 0, 4, 0));
		} else {
			this.setBorder(new EmptyBorder(0, 0, 1, 0));
		}

		TrainingMenuListener trainingMenuListener = new TrainingMenuListener() {

			@SuppressWarnings("synthetic-access")
			@Override
			public JMenuItem[] getItems() {
				return new JMenuItem[] { new JMenuItem("Remove resource"),
						new JMenuItem("Find similar files...") };
			}

		};
		this.resourceList.addMouseListener(trainingMenuListener);
		trainingMenuListener.getMenu().setCorrectTerminology("relevant", "irrelevant");

		this.update();
	}
}
