/* File: PutOnHoldDialog.java
 *
 * Contains class PutOnHoldDialog
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import edu.cmu.hcii.calo.view.PlaceholderTextField;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A dialog containing fields allowing the user to enter a reason for putting a task on hold, as
 * well as the person on whom they are waiting.
 * 
 * @author Brian Ellis
 * 
 */
public class PutOnHoldDialog extends JDialog {

	/**
	 * Prevent serialization collisions.
	 */
	private static final long serialVersionUID = 3593748306106567966L;

	/**
	 * Tests that PutOnHoldDialog works correctly.
	 * 
	 * @param args command-line arguments, don't bother passing any
	 */
	public static void main(String[] args) {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		PutOnHoldDialog dlg = new PutOnHoldDialog();
		if (dlg.presentDialog()) {
			System.out.println("Pressed OK, reason is " + dlg.getReason() + ", person is "
					+ dlg.getPerson());
		} else {
			System.out.println("Pressed cancel or closed window.");
		}
	}

	/**
	 * @name Controls
	 * 
	 * Various controls in this dialog.
	 */

	// @{
	private PlaceholderTextField whoField;
	private JTextArea reasonField;
	// @}

	/**
	 * True if the OK button has been pressed, false otherwise.
	 */
	private boolean okPressed = false;

	/**
	 * Constructs a PutOnHoldDialog.
	 */
	public PutOnHoldDialog() {
		super();
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog.
	 * 
	 * @param d the dialog to use
	 */
	public PutOnHoldDialog(Dialog d) {
		super(d);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog and modal state.
	 * 
	 * @param d the dialog to use
	 * @param flag whether this should be modal
	 */
	public PutOnHoldDialog(Dialog d, boolean flag) {
		super(d, flag);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog and title.
	 * 
	 * @param d the dialog to use
	 * @param title the title to use
	 */
	public PutOnHoldDialog(Dialog d, String title) {
		super(d, title);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog, title, and modal state.
	 * 
	 * @param d the dialog to use
	 * @param title the title to use
	 * @param flag whether this should be modal
	 */
	public PutOnHoldDialog(Dialog d, String title, boolean flag) {
		super(d, title, flag);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog, title, modal state, and graphics
	 * configuration.
	 * 
	 * @param d the dialog to use
	 * @param title the title to use
	 * @param flag whether this should be modal
	 * @param g the graphics configuration to use
	 */
	public PutOnHoldDialog(Dialog d, String title, boolean flag, GraphicsConfiguration g) {
		super(d, title, flag, g);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog from the given frame.
	 * 
	 * @param f the frame to use
	 */
	public PutOnHoldDialog(Frame f) {
		super(f);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given dialog and modal state.
	 * 
	 * @param f the frame to use
	 * @param flag whether this should be modal
	 */
	public PutOnHoldDialog(Frame f, boolean flag) {
		super(f, flag);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given frame and title.
	 * 
	 * @param f the frame to use
	 * @param title the title to use
	 */
	public PutOnHoldDialog(Frame f, String title) {
		super(f, title);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given frame, title, and modal state.
	 * 
	 * @param f the frame to use
	 * @param title the title to use
	 * @param flag whether this should be modal
	 */
	public PutOnHoldDialog(Frame f, String title, boolean flag) {
		super(f, title, flag);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given frame, title, modal state, and graphics
	 * configuration.
	 * 
	 * @param f the frame to use
	 * @param title the title to use
	 * @param flag whether this should be modal
	 * @param g the graphics configuration to use
	 */
	public PutOnHoldDialog(Frame f, String title, boolean flag, GraphicsConfiguration g) {
		super(f, title, flag, g);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog from the given window
	 * 
	 * @param w the window to use
	 */
	public PutOnHoldDialog(Window w) {
		super(w);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given window and modality type.
	 * 
	 * @param w the window to use
	 * @param type the modality type to use
	 */
	public PutOnHoldDialog(Window w, ModalityType type) {
		super(w, type);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given window and title.
	 * 
	 * @param w the window to use
	 * @param title the title to use
	 */
	public PutOnHoldDialog(Window w, String title) {
		super(w, title);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given window, title, and modality type.
	 * 
	 * @param w the window to use
	 * @param title the title to use
	 * @param type the modality type to use
	 */
	public PutOnHoldDialog(Window w, String title, ModalityType type) {
		super(w, title, type);
		this.setup();
	}

	/**
	 * Constructs a PutOnHoldDialog with the given window, title, and modality type.
	 * 
	 * @param w the window to use
	 * @param title the title to use
	 * @param type the modality type to use
	 * @param g the graphics configuration to use
	 */
	public PutOnHoldDialog(Window w, String title, ModalityType type, GraphicsConfiguration g) {
		super(w, title, type, g);
		this.setup();
	}

	/**
	 * Gets the string value entered by the user as the person on whom this task is waiting.
	 * 
	 * @return the person
	 */
	public String getPerson() {
		return this.whoField.getText();
	}

	/**
	 * Gets the string value entered by the user as the reason this task is on hold.
	 * 
	 * @return the reason
	 */
	public String getReason() {
		return this.reasonField.getText();
	}

	/**
	 * Sets the size of this dialog to a reasonable one, sets it to be modal, shows it, and then
	 * returns \c true if the user clicked OK and \c false otherwise. You should use this instead of
	 * invoking setVisible() directly.
	 * 
	 * @return \c true if the user clicked OK, \c false otherwise
	 */
	public boolean presentDialog() {
		this.pack();
		this.setSize(450, this.getPreferredSize().height);
		this.setModal(true);
		this.setVisible(true);
		// modal loop causes this line to be delayed until the window is hidden.
		return this.okPressed;

	}

	/**
	 * Sets up the visual appearance of this dialog.
	 */
	private void setup() {
		if (this.getTitle() == null) {
			this.setTitle("Put Task On Hold");
		}
		this.setLayout(new RelativeLayout());

		JLabel whoLabel = new JLabel("Whose action is required to take this task off hold?");
		this.whoField = new PlaceholderTextField();
		this.whoField.setPlaceholderText("(optional)");

		JLabel noteLabel = new JLabel("Please enter the reason this task is on hold:");
		this.reasonField = new JTextArea(3, 50);
		// this.reasonField.setFont(Font.getFont(Font.DIALOG_INPUT));
		this.reasonField.setLineWrap(true);
		this.reasonField.setWrapStyleWord(true);

		JButton okButton = new JButton("OK");
		okButton.setPreferredSize(new Dimension(80, okButton.getPreferredSize().height));
		okButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				PutOnHoldDialog.this.okPressed = true;
				PutOnHoldDialog.this.setVisible(false);
			}

		});

		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				PutOnHoldDialog.this.setVisible(false);
			}

		});
		cancelButton.setPreferredSize(new Dimension(80, cancelButton.getPreferredSize().height));

		BindingMaker bm = new BindingMaker();
		this.add(noteLabel, new RelativeConstraints(bm.topEdge(), bm.leftEdge(), bm.rightEdge()));
		JScrollPane scrollPane = new JScrollPane(this.reasonField);
		this.add(scrollPane, new RelativeConstraints(	bm.below(noteLabel),
														bm.leftEdge(),
														bm.rightEdge()));
		int oldVerticalSpacing = bm.getVerticalSpacing();
		bm.setVerticalSpacing(12);
		this.add(whoLabel, new RelativeConstraints(	bm.below(scrollPane),
													bm.leftEdge(),
													bm.rightEdge()));
		bm.setVerticalSpacing(oldVerticalSpacing);
		this.add(this.whoField, new RelativeConstraints(bm.below(whoLabel),
														bm.leftEdge(),
														bm.rightEdge()));
		JPanel buttonPanel = new JPanel(new FlowLayout());
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		this.add(buttonPanel, new RelativeConstraints(	bm.below(this.whoField),
														bm.horizontallyCenterAlign(null)));
		this.whoField.setPreferredSize(new Dimension(	this.reasonField.getPreferredSize().width + 2,
														this.whoField.getPreferredSize().height));
		this.getRootPane().setDefaultButton(okButton);

		this.getRootPane().setBorder(new EmptyBorder(	0,
														0,
														bm.getRightMargin(),
														bm.getBottomMargin()));
	}

}
