/* File: TaskPaneView.java
 *
 * Contains class TaskPaneView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.MatteBorder;

import net.iharder.dnd.FileDrop;
import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Resource;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TasklikeObject;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.CALOScrollPane;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.PlaceholderTextField;
import edu.cmu.hcii.calo.view.SearchField;
import edu.cmu.hcii.calo.view.task.support.TaskFilter;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view that shows a scrollable task pane with three subpanes (to-do, on hold, and completed).
 * 
 * @author Brian Ellis
 */
public class TaskPaneView extends CALOComponent {

	/**
	 * Test instance.
	 */
	static TaskPaneView paneView;

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 7083012944101041073L;

	/**
	 * The default filter for the to-do subpane.
	 */
	private static final TaskFilter TODO_FILTER = new TaskFilter() {

		@Override
		public boolean shouldShowTask(Task t) {
			if (t.isComplete()) {
				return false;
			}
			if (t.getPendingState().isPending()) {
				return false;
			}
			return true;
		}

	};

	/**
	 * The default filter for the pending tasks subpane.
	 */
	private static final TaskFilter PENDING_FILTER = new TaskFilter() {

		@Override
		public boolean shouldShowTask(Task t) {
			if (t.getPendingState().isPending() && !t.isComplete()) {
				return true;
			}
			return false;
		}

	};

	/**
	 * The default filter for the completed tasks subpane.
	 */
	private static final TaskFilter COMPLETED_FILTER = new TaskFilter() {

		@Override
		public boolean shouldShowTask(Task t) {
			if (t.isComplete()) {
				return true;
			}
			return false;
		}

	};

	/**
	 * A test program that creates and shows a TaskPaneView.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		for (int i = 0; i < 5; i++ ) {
			Task t = new Task("Sample Task " + (i + 1), false);
			t.setPriority(100 - i);
			TaskManager.getManager().addTasklikeObject(t);
		}

		for (int i = 0; i < 1; i++ ) {
			Task t = new Task("Pending Task " + (i + 1), false);
			t.setPriority(100 - i);
			PendingState pendingState = new PendingState();
			pendingState.setPending(true);
			t.setPendingState(pendingState);
			TaskManager.getManager().addTasklikeObject(t);
		}

		for (int i = 0; i < 1; i++ ) {
			Task t = new Task("Completed Task " + (i + 1), false);
			t.setPriority(100 - i);
			t.setComplete(true);
			TaskManager.getManager().addTasklikeObject(t);
		}

		JFrame testFrame = new JFrame("TaskPaneView test");

		TaskPaneView.paneView = new TaskPaneView();

		JScrollPane scrollPane = new JScrollPane(TaskPaneView.paneView);
		testFrame.setLayout(new BorderLayout());
		testFrame.add(scrollPane, BorderLayout.CENTER);
		testFrame.setSize(250, 650);
		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setVisible(true);

		new Timer().schedule(new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {

						ArrayList<Task> bigOlHack = new ArrayList<Task>();
						Iterator<TasklikeObject> iter = TaskManager.getManager().taskIterator();
						while (iter.hasNext()) {
							bigOlHack.add((Task)iter.next());
						}

						bigOlHack.get(1).setPriority(65);
						bigOlHack.get(3).setPriority(105);

						Task task = new Task("Sample Task 6", false);
						task.setPriority(96);
						TaskManager.getManager().addTasklikeObject(task);

						Task anotherTask = new Task("Sample Task 7", false);
						anotherTask.setPriority(96);
						TaskManager.getManager().addTasklikeObject(anotherTask);
						// TaskManager.getManager().removeTask(bigOlHack.get(2));

						TaskPaneView.paneView.todoList.update();
					}
				});

			}

		}, 2500);

	}

	/**
	 * The to-do list subpane.
	 */
	private TaskSubpaneView todoList;

	/**
	 * The pending tasks subpane.
	 */
	private TaskSubpaneView pendingTasks;

	/**
	 * The completed tasks subpane.
	 */
	private TaskSubpaneView completedTasks;

	/**
	 * @name Controls
	 * 
	 * Various controls in the pane.
	 */

	// @{
	private SearchField filterField;

	private PlaceholderTextField inputField;

	private IconButton addTaskButton;

	private TaskSubpaneContainerView subpaneContainer;

	private CALOScrollPane scrollPane;

	private JLabel searchResultsLabel;

	// @}

	/**
	 * Constructs a TaskPaneView.
	 */
	public TaskPaneView() {
		super();
		this.setup();
	}

	/**
	 * Updates the filters applied to each of the subpanes to match whether the filter field has
	 * text in it, and updates the subpanes so they will re-filter their results. Also shows or
	 * hides the search results label, and updates the number of matching tasks it displays, as
	 * appropriate.
	 */
	protected void doFilter() {
		if (this.filterField.getText().length() == 0) {
			this.searchResultsLabel.setPreferredSize(new Dimension(	this.searchResultsLabel.getPreferredSize().width,
																	0));
			this.todoList.setFilter(null);
			this.pendingTasks.setFilter(null);
			this.completedTasks.setFilter(null);
		} else {
			this.searchResultsLabel.setPreferredSize(null);
			if (this.todoList.getFilter() == null) {
				this.todoList.setFilter(new TaskFilter() {

					@SuppressWarnings("synthetic-access")
					@Override
					public boolean shouldShowTask(Task t) {
						return TaskPaneView.this.taskMatchesFilter(t);
					}

				});
			} else {
				this.todoList.update();
			}
			if (this.pendingTasks.getFilter() == null) {
				this.pendingTasks.setFilter(new TaskFilter() {

					@SuppressWarnings("synthetic-access")
					@Override
					public boolean shouldShowTask(Task t) {
						return TaskPaneView.this.taskMatchesFilter(t);
					}

				});
			} else {
				this.pendingTasks.update();
			}
			if (this.completedTasks.getFilter() == null) {
				this.completedTasks.setFilter(new TaskFilter() {

					@SuppressWarnings("synthetic-access")
					@Override
					public boolean shouldShowTask(Task t) {
						return (t.getName().toUpperCase().contains(TaskPaneView.this.filterField.getText().toUpperCase()));
					}

				});
			} else {
				this.completedTasks.update();
			}
			int shown =
				this.todoList.countShownTasks() + this.pendingTasks.countShownTasks()
						+ this.completedTasks.countShownTasks();
			int total = TaskManager.getManager().countTasks();
			String tasks = shown == 1 ? "task" : "tasks";
			this.searchResultsLabel.setText("Showing " + shown + " matching " + tasks + " out of "
					+ total);
		}
		this.validate();
	}

	/**
	 * Creates a new task using the text the user has entered into the input field. If the input
	 * field is empty, does nothing. Also sets the input field text back to the empty string.
	 */
	protected void makeNewTask() {
		if (this.inputField.getText().isEmpty()) {
			return;
		}

		Task newTask = new Task(this.inputField.getText(), false);
		TaskManager.getManager().addTasklikeObject(newTask);
		this.inputField.setText("");
		this.todoList.update();

	}

	/**
	 * Sets up the visual appearance of the view.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		this.filterField = new SearchField("Search");
		this.filterField.setSendsNotificationForEachKeystroke(true);

		this.inputField = new PlaceholderTextField();
		this.inputField.setPlaceholderText("Type to add a new task");
		this.inputField.addKeyListener(new KeyAdapter() {

			/*
			 * (non-Javadoc)
			 * 
			 * @see java.awt.event.KeyAdapter#keyPressed(java.awt.event.KeyEvent)
			 */
			@SuppressWarnings("synthetic-access")
			@Override
			public void keyReleased(KeyEvent e) {
				if (TaskPaneView.this.inputField.getText().length() == 0) {
					TaskPaneView.this.addTaskButton.setEnabled(false);
				} else {
					TaskPaneView.this.addTaskButton.setEnabled(true);
				}
			}

			/*
			 * (non-Javadoc)
			 * 
			 * @see java.awt.event.KeyAdapter#keyTyped(java.awt.event.KeyEvent)
			 */
			@Override
			public void keyTyped(KeyEvent e) {
				if (e.getKeyChar() == '\n') {
					TaskPaneView.this.makeNewTask();
				}
			}

		});
		this.filterField.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				if (TaskPaneView.this.filterField.getText().length() == 0) {
					TaskPaneView.this.addTaskButton.setEnabled(false);
				}
				TaskPaneView.this.doFilter();
			}

		});

		this.subpaneContainer = new TaskSubpaneContainerView();

		new FileDrop(this, new FileDrop.Listener() {

			@SuppressWarnings("synthetic-access")
			public void filesDropped(final File[] files) {
				if (files.length > 0) {
					Task newTask = new Task(files[0].getName(), false);
					for (File file : files) {
						newTask.getResources().add(new Resource(file));
					}
					TaskManager.getManager().addTasklikeObject(newTask);
					TaskPaneView.this.todoList.update();
				}
			}

		});

		ImageIcon addIcon = new ImageIcon("Resources/add.png");
		ImageIcon addOverIcon = new ImageIcon("Resources/add-over.png");
		ImageIcon addPressedIcon = new ImageIcon("Resources/add-pressed.png");

		this.addTaskButton = new IconButton(addIcon, addOverIcon, addPressedIcon);
		this.addTaskButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				TaskPaneView.this.makeNewTask();
			}

		});
		this.addTaskButton.setEnabled(false);

		this.todoList = new TaskSubpaneView("To Do", 4, TaskPaneView.TODO_FILTER);
		this.pendingTasks = new TaskSubpaneView("On Hold", 2, TaskPaneView.PENDING_FILTER);
		this.completedTasks = new TaskSubpaneView("Complete", 2, TaskPaneView.COMPLETED_FILTER);

		this.searchResultsLabel = new JLabel("Showing all tasks.");
		this.searchResultsLabel.setBorder(new CompoundBorder(	new MatteBorder(new Insets(	1,
																							0,
																							0,
																							0),
																				SystemColor.controlShadow),
																new EmptyBorder(2, 6, 2, 6)));
		this.searchResultsLabel.setFont(this.searchResultsLabel.getFont().deriveFont(9.0f));
		this.searchResultsLabel.setPreferredSize(new Dimension(	this.searchResultsLabel.getPreferredSize().width,
																0));
		BindingMaker bm = new BindingMaker(4, 2, 4, 2, 4, 4);

		RelativeConstraints inputFieldConstraints = new RelativeConstraints();
		inputFieldConstraints.addBindings(bm.topEdge(), bm.leftEdge(), new Binding(	Edge.RIGHT,
																					25,
																					Direction.LEFT,
																					Edge.RIGHT,
																					null));

		RelativeConstraints addTaskButtonConstraints = new RelativeConstraints();
		addTaskButtonConstraints.addBindings(	bm.verticallyCenterAlign(this.inputField),
												new Binding(Edge.RIGHT,
															6,
															Direction.RIGHT,
															Edge.RIGHT,
															null));

		RelativeConstraints filterFieldConstraints = new RelativeConstraints();
		filterFieldConstraints.addBindings(bm.below(this.inputField), bm.leftEdge(), bm.rightEdge());

		RelativeConstraints searchResultsLabelConstraints = new RelativeConstraints();
		searchResultsLabelConstraints.addBindings(	bm.below(this.filterField),
													bm.directLeftEdge(),
													bm.directRightEdge());

		int leftMargin = bm.getLeftMargin();
		int rightMargin = bm.getRightMargin();
		int bottomMargin = bm.getBottomMargin();

		bm.setLeftMargin(0);
		bm.setRightMargin(0);
		bm.setBottomMargin(0);

		RelativeConstraints subpaneContainerConstraints = new RelativeConstraints();
		subpaneContainerConstraints.addBindings(bm.directlyBelow(this.searchResultsLabel),
												bm.directLeftEdge(),
												bm.directRightEdge(),
												bm.directBottomEdge());

		bm.setLeftMargin(leftMargin);
		bm.setRightMargin(rightMargin);
		bm.setBottomMargin(bottomMargin);
		bm.setTopMargin(2);

		RelativeConstraints todoListConstraints = new RelativeConstraints();
		todoListConstraints.addBindings(bm.topEdge(), bm.leftEdge(), bm.rightEdge());

		RelativeConstraints pendingTasksConstraints = new RelativeConstraints();
		pendingTasksConstraints.addBindings(bm.below(this.todoList), bm.leftEdge(), bm.rightEdge());

		RelativeConstraints completedTasksConstraints = new RelativeConstraints();
		completedTasksConstraints.addBindings(	bm.below(this.pendingTasks),
												bm.leftEdge(),
												bm.rightEdge());

		this.add(this.inputField, inputFieldConstraints);
		this.add(this.addTaskButton, addTaskButtonConstraints);

		this.add(this.filterField, filterFieldConstraints);
		this.add(this.searchResultsLabel, searchResultsLabelConstraints);

		this.subpaneContainer.setLayout(new RelativeLayout());

		this.subpaneContainer.add(this.todoList, todoListConstraints);
		this.subpaneContainer.add(this.pendingTasks, pendingTasksConstraints);
		this.subpaneContainer.add(this.completedTasks, completedTasksConstraints);

		this.scrollPane = new CALOScrollPane(this.subpaneContainer);
		this.subpaneContainer.setBorder(new EmptyBorder(0, 0, 2, 0));
		this.add(this.scrollPane, subpaneContainerConstraints);
		this.scrollPane.setBorder(new MatteBorder(new Insets(1, 0, 0, 0), SystemColor.controlShadow));
		this.setBorder(new EmptyBorder(0, 0, 0, 0));
	}

	/**
	 * Returns whether or not the given task matches the text currently in the filter field. This is
	 * called by the filters applied to the subpanes in doFilter(), and is safe (even though it uses
	 * the view state) because doFilter() is called whenever the filter field text changes.
	 * 
	 * @param t the task to match against the filter field text
	 * @return \c true if the task matches the filter, \c false otherwise
	 */
	protected boolean taskMatchesFilter(Task t) {
		if (t.getName().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else if (t.getTags().toString().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else if (t.getDueDateString().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else if (t.getDueDate() != null
				&& DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.SHORT).format(t.getDueDate().getTime()).toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else if (t.getPendingState().getPendingNote().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else if (t.getPendingState().getPerson().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
			return true;
		} else {
			Iterator<Resource> it = t.getResources().iterator();
			while (it.hasNext()) {
				Resource rsrc = it.next();
				if (rsrc.getFileName().toUpperCase().contains(this.filterField.getText().toUpperCase())) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Causes this pane and all its subpanes to update themselves to match the state of the
	 * TaskManager.
	 */
	public void update() {
		assert SwingUtilities.isEventDispatchThread() : "we are not on the event dispatch thread!";
		this.subpaneContainer.update();
		this.validate();
	}

	/**
	 * Causes all the task views within each subpane to update themselves to match the state of the
	 * TaskManager.
	 * 
	 * @note This differs from update() in that it does not update the subpanes themselves (meaning
	 *       the adding, removing, or reordering of tasks will not be correctly reflected), but it
	 *       is much, much less expensive than a full update().
	 */
	public void updateTasks() {
		assert SwingUtilities.isEventDispatchThread() : "we are not on the event dispatch thread!";
		this.subpaneContainer.updateTasks();
	}
}
