/* File: TaskPropertyActionListener.java
 *
 * Contains class TaskPropertyActionListener
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.task.support.TaskState;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.hcii.calo.view.task.support.TaskState.TaskStateProperty;

/**
 * An ActionListener subclass that will construct TaskStates for some property of the old and new
 * states of a task and fire a TaskState property change on behalf of its view.
 * 
 * When you construct a TaskPropertyActionListener, you pass the view, task, and property that it
 * will use to fire property change events. When the listener detects an action from the component
 * to which it is listening, it will call getOldValue() and getNewValue() to determine what values
 * to put in the TaskStates. Then it will call CALOComponent#firePropertyChange(String, Object,
 * Object) in the view to automatically fire the property change on its behalf.
 * 
 * For example, imagine that there is a toggle button that determines whether or not the task
 * represented by the check box's enclosing view is complete. We could attach a
 * TaskPropertyActionListener to the button, passing it the check box's enclosing view as the \p
 * view, the task represented by that view as the \p task, and TaskStateProperty.COMPLETE as the \p
 * property. When the button is clicked, the TaskPropertyActionListener would call getOldValue()
 * (which we would implement to return the current completed state of the task), then getNewValue()
 * (which we would implement to return the opposite of that state), and then would fire a property
 * change event on behalf of the view with a name of TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME
 * whose old and new values are TaskState instances created from our task and the values we returned
 * from getOldValue() and getNewValue() respectively. This would propagate up the hierarchy of
 * CALOComponents to the controller, which would handle the state change.
 * 
 * @author Brian Ellis
 * 
 */
abstract class TaskPropertyActionListener implements ActionListener {

	/**
	 * The view on behalf of which we will send property change events.
	 */
	private CALOComponent view;

	/**
	 * The task represented by the view we are listening to.
	 */
	private Task task;

	/**
	 * The property of the task that should be marked as changed when the action fires.
	 */
	private TaskStateProperty property;

	/**
	 * Constructs a TaskPropertyActionListener with the given view, task, and property.
	 * 
	 * @param view the view on behalf of which we will fire a property change event
	 * @param t the task we will mark as having changed
	 * @param property the property of the task we will mark as having changed
	 */
	public TaskPropertyActionListener(CALOComponent view,
			Task t,
			TaskState.TaskStateProperty property) {
		this.view = view;
		this.task = t;
		this.property = property;

	}

	/**
	 * Fired when the component to which this listener is attached fires an action event. Invokes
	 * getOldValue() and getNewValue() to get the old and new values of our property, then fires a
	 * property change event on behalf of our view with a property name of
	 * TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME. The old and new values of the property change
	 * event are new TaskState instances whose task is our task, whose property is our property, and
	 * whose old and new states are what was returned from getOldValue() and getNewValue().
	 * 
	 * @param e the action event
	 */
	@SuppressWarnings("synthetic-access")
	public void actionPerformed(ActionEvent e) {
		if (!this.view.isEnabled()) {
			return;
		}
		TaskState oldState = new TaskState(this.task, this.property, this.getOldValue());
		TaskState newState = new TaskState(this.task, this.property, this.getNewValue());

		this.view.firePropertyChange(	TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME,
										oldState,
										newState);
	}

	/**
	 * Should be implemented by instances to return the value our property had before the action
	 * fired.
	 * 
	 * @return the old value
	 */
	public abstract Object getNewValue();

	/**
	 * Should be implemented by instances to return the value our property should have now that the
	 * action has fired.
	 * 
	 * @return the new value
	 */
	public abstract Object getOldValue();
}
