/* File: TaskSubpaneContainerView.java
 *
 * Contains class TaskSubpaneContainerView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.cmu.hcii.calo.view.AnimatedPane;
import edu.cmu.hcii.calo.view.animation.Animation;
import edu.cmu.hcii.calo.view.animation.AnimationManager;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationCurve;

/**
 * A container view (as the name implies) that holds all three subpanes of the task pane and
 * animates them when their contents change.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskSubpaneContainerView extends AnimatedPane {

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = -1152108861501116678L;

	/**
	 * The mapping of the subpanes to their bounds rectangles.
	 */
	private Map<Component, Rectangle> bounds = new HashMap<Component, Rectangle>();

	/*
	 * Animates the subpanes.
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doAnimations()
	 */
	@Override
	protected boolean doAnimations() {
		List<Animation> animations = new ArrayList<Animation>();
		for (Component c : getComponents()) {
			if (!c.getBounds().equals(this.bounds.get(c))) {
				Animation a =
					AnimationManager.getManager().animateRect(	this.bounds.get(c),
																c.getBounds(),
																AnimationCurve.EASE_IN_OUT,
																0.75);
				a.setObjectData(c);
				animations.add(a);
			}
		}
		this.setTotalAnimations(animations.size());
		for (Animation animation : animations) {
			animation.start(this);
			this.incrementNumAnimating();
		}

		return (!animations.isEmpty());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#doUpdate()
	 */
	@Override
	protected void doUpdate() {

		for (Component c : getComponents()) {
			assert c instanceof TaskSubpaneView;
			((TaskSubpaneView)c).update();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#prepareAnimations()
	 */
	@Override
	protected void prepareAnimations() {
		this.bounds.clear();
		for (Component c : getComponents()) {
			assert c instanceof TaskSubpaneView;
			this.bounds.put(c, c.getBounds());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.cmu.hcii.calo.view.AnimatedPane#safeUpdateAnimation(edu.cmu.hcii.calo.view.animation.Animation)
	 */
	@Override
	protected void safeUpdateAnimation(Animation a) {
		TaskSubpaneView view = (TaskSubpaneView)a.getObjectData();
		Rectangle2D currentRect = a.getCurrentRectangle();
		Rectangle currentBounds =
			new Rectangle(	Math.round((int)currentRect.getMinX()),
							(int)Math.round(currentRect.getMinY()),
							(int)Math.round(currentRect.getWidth()),
							(int)Math.round(currentRect.getHeight()));
		view.setBounds(currentBounds);
		view.repaint();
	}

	/**
	 * Causes the task views within each subpane to update, without actually updating the subpanes
	 * themselves.
	 * 
	 * @note This differs from update() in that the adding, removing, and reordering of tasks will
	 *       not be correctly reflected, but it is much, much less expensive than a full update().
	 */
	public void updateTasks() {
		for (Component c : getComponents()) {
			assert c instanceof TaskSubpaneView;
			((TaskSubpaneView)c).updateTasks();
		}
	}

}
