/* File: TaskSubpaneTitleView.java
 *
 * Contains class TaskSubpaneTitleView
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.UIManager;

import edu.cmu.hcii.calo.view.IconToggleButton;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view that shows a title bar for a TaskSubpaneView, containing a collapse button. When the
 * button is clicked, the state of the title bar changes, and a propertyChange is fired that can be
 * listened for by the enclosing view. The widget also changes to reflect whether the subpane is now
 * collapsed or expanded.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskSubpaneTitleView extends JComponent {

	/**
	 * A listener that toggles the collapsed state of the view and fires off the appropriate
	 * property change event so the subpane itself can change its state.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public class CollapseButtonListener implements ActionListener {

		/**
		 * When the action event fires, toggles the collapsed state of the view and fires off the
		 * appropriate property change event so the subpane itself can change its state.
		 * 
		 * @param e the action event
		 * 
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			TaskSubpaneTitleView.this.setCollapsed(!TaskSubpaneTitleView.this.isCollapsed());
		}

	}

	/** Prevent serialization collisions. */
	private static final long serialVersionUID = 5016556472007849539L;

	/**
	 * @name Controls
	 * 
	 * The various controls in the view.
	 */

	// @{
	/**
	 * The minimize button.
	 */
	IconToggleButton minimizeButton;

	/**
	 * The label containing the title for this title bar.
	 */
	private JLabel titleLabel;

	/**
	 * The label containing the shadow of the title for this title bar.
	 */
	private JLabel titleShadowLabel;

	// @}

	/**
	 * Whether this title view is currently showing a collapsed state.
	 */
	protected boolean isCollapsed = false;

	/**
	 * The title that should be displayed by this title bar.
	 */
	private String title;

	/**
	 * Constructs a title view with the given title
	 * 
	 * @param title the title to show in this view
	 */
	public TaskSubpaneTitleView(String title) {
		this.title = title;
		this.setup();
	}

	/**
	 * Returns the preferred size of the title view, which will be either 19 or 46 pixels high
	 * depending on whether it is collapsed or not. If setPreferredSize() is called on this view,
	 * the size that has been set will take precedence as usual.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		}
		if (this.isCollapsed()) {
			return new Dimension(500, 19);
		} else {
			return new Dimension(500, 46);
		}
	}

	/**
	 * Returns whether this title view is currently showing a collapsed state.
	 * 
	 * @return \c true if this title view is collapsed, \c false otherwise
	 */
	public boolean isCollapsed() {
		return this.isCollapsed;
	}

	/**
	 * Draws a pretty gradient on the title view using the system theme colors.
	 * 
	 * @param graphics the graphics context to paint into
	 */
	@Override
	protected void paintComponent(Graphics graphics) {
		super.paintComponent(graphics);

		Graphics2D g = (Graphics2D)graphics;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		Color solid = UIManager.getColor("InternalFrame.activeTitleBackground");
		float[] hsb = Color.RGBtoHSB(solid.getRed(), solid.getGreen(), solid.getBlue(), null);
		hsb[1] = Math.max(0.0f, hsb[1] / 2.0f);
		solid = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));

		Color transparent =
			new Color(solid.getRed() / 255f, solid.getGreen() / 255f, solid.getBlue() / 255f, 0.0f);

		Paint gradient = new GradientPaint(0, 0, solid, 0, this.getHeight(), transparent);

		if (this.isCollapsed()) {
			g.setColor(solid);
		} else {
			g.setPaint(gradient);
		}

		RoundRectangle2D roundrect =
			new RoundRectangle2D.Float(0, 0, this.getWidth(), this.getHeight(), 8, 8);
		Area a = new Area(roundrect);
		a.add(new Area(new Rectangle2D.Float(	0,
												this.getHeight() / 2,
												this.getWidth(),
												this.getHeight() / 2)));

		g.fill(a);

		this.titleLabel.setForeground(UIManager.getColor("InternalFrame.activeTitleForeground"));
		this.titleShadowLabel.setForeground(UIManager.getColor("InternalFrame.activeTitleBackground"));
	}

	/**
	 * Sets whether this view is drawing itself in a collapsed state. This is called automatically
	 * when the collapse/expand widget is clicked, and may also be invoked externally. When called,
	 * it updates the visual state of the view and then fires a property change with a name of
	 * TaskViewPropertyNames.SUBPANE_COLLAPSED_PROPERTY_NAME indicating the previous and current
	 * collapsed states (as booleans, true meaning collapsed). This is fired even if the previous
	 * and current states are the same.
	 * 
	 * @param isCollapsed \c true if the view should appear collapsed, \c false otherwise
	 */
	public void setCollapsed(boolean isCollapsed) {

		boolean wasCollapsed = this.isCollapsed;
		this.isCollapsed = isCollapsed;

		this.repaint();

		this.firePropertyChange(TaskViewPropertyNames.SUBPANE_COLLAPSED_PROPERTY_NAME,
								wasCollapsed,
								this.isCollapsed);

	}

	/**
	 * Sets up the visual appearance of this view.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(4, 8, 4, 4, 4, 4);

		this.titleLabel = new JLabel(this.title);
		this.titleLabel.setFont(this.titleLabel.getFont().deriveFont(Font.BOLD));

		this.titleShadowLabel = new JLabel(this.title);
		this.titleShadowLabel.setFont(this.titleLabel.getFont());

		ImageIcon collapseIcon = new ImageIcon("Resources/down.png");
		ImageIcon collapseOverIcon = new ImageIcon("Resources/down-over.png");
		ImageIcon expandIcon = new ImageIcon("Resources/up.png");
		ImageIcon expandOverIcon = new ImageIcon("Resources/up-over.png");

		this.minimizeButton =
			new IconToggleButton(	collapseIcon,
									collapseOverIcon,
									null,
									expandIcon,
									expandOverIcon,
									null);
		this.minimizeButton.addActionListener(new CollapseButtonListener());

		// Define Bindings and RelativePositions we're going to need later:

		// Make our title constraints, but leave them empty:
		RelativeConstraints titleConstraints = new RelativeConstraints();

		RelativeConstraints minimizeButtonConstraints = new RelativeConstraints(bm.topEdge());

		titleConstraints.addBindings(bm.leftEdge(), bm.verticallyCenterAlign(this.minimizeButton));
		titleConstraints.addBinding(bm.leftOf(this.minimizeButton));

		RelativeConstraints titleShadowConstraints =
			new RelativeConstraints(bm.leftAlign(this.titleLabel),
									bm.rightAlign(this.titleLabel),
									new Binding(Edge.TOP,
												1,
												Direction.BELOW,
												Edge.TOP,
												this.titleLabel));

		minimizeButtonConstraints.addBinding(bm.rightEdge());

		this.add(this.minimizeButton, minimizeButtonConstraints);

		this.add(this.titleLabel, titleConstraints);
		this.add(this.titleShadowLabel, titleShadowConstraints);
	}

	/**
	 * Sets the title of this view.
	 * 
	 * @param string the title to set
	 */
	public void setTitle(String string) {
		this.title = string;
		this.titleLabel.setText(string);
		this.titleShadowLabel.setText(string);
		this.validate();
	}

}
