/*
 * File: TaskSubpaneView.java
 * 
 * Contains class TaskSubpaneView
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MouseInfo;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TasklikeObject;
import edu.cmu.hcii.calo.view.AnimatedPane;
import edu.cmu.hcii.calo.view.JLinkButton;
import edu.cmu.hcii.calo.view.animation.Animation;
import edu.cmu.hcii.calo.view.animation.AnimationManager;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationCurve;
import edu.cmu.hcii.calo.view.animation.Animation.AnimationType;
import edu.cmu.hcii.calo.view.task.support.TaskFilter;
import edu.cmu.hcii.calo.view.task.support.TaskOrdering;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;
import edu.cmu.relativelayout.RelativePosition;

/**
 * A view that shows some subset of TaskViews in an list, ordered by priority.
 * 
 * @author Brian Ellis
 * 
 * @todo This needs, someday, to support task groups too.
 */
@SuppressWarnings("serial")
public class TaskSubpaneView extends AnimatedPane {

	/**
	 * A ComponentListener that listens for moving TaskViews when a drag is in progress, and
	 * determines what the drop target would be if the user were to drop the TaskView.
	 * TaskViewDragListener also visually updates the potential drop targets' appearance during the
	 * drag.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class TaskViewDragListener extends ComponentAdapter {

		/**
		 * Called when a TaskView moves.
		 * 
		 * @param e the component event
		 */
		@Override
		public void componentMoved(ComponentEvent e) {
			assert e.getSource() instanceof TaskView : "we attached a TaskViewDragListener to something other than a TaskView";
			TaskView sourceView = (TaskView)e.getSource();
			if (!sourceView.isInDrag()) {
				return;
			}

			boolean isOverOtherView = false;
			TaskView previousSibling = null;
			TaskView nextSibling = null;

			Component draggingView = e.getComponent();
			Point mouseLocation = MouseInfo.getPointerInfo().getLocation();
			SwingUtilities.convertPointFromScreen(mouseLocation, TaskSubpaneView.this);
			boolean isMouseInView =
				(new Rectangle(	0,
								0,
								TaskSubpaneView.this.getWidth(),
								TaskSubpaneView.this.getHeight()).contains(mouseLocation));

			for (Component c : TaskSubpaneView.this.getComponents()) {

				// This is the view being dragged, so ignore it.
				if (c == draggingView) {
					continue;
				}

				// This isn't a TaskView, so ignore it.
				if (!(c instanceof TaskView)) {
					continue;
				}

				// Which TaskView are we on?
				TaskView thisView = (TaskView)c;

				// If the mouse isn't even inside the subpane, just make sure
				// this view isn't set as
				// the current drop target and continue.
				if (!isMouseInView) {
					thisView.setCurrentDropTarget(false);
					continue;
				}

				// So now we know that we're in the subpane somewhere, and we
				// might be dragging near
				// or over this TaskView.

				// Calculate the rectangle the mouse must be over for us to drop
				// onto this TaskView.
				// We "grow" the bounds rect by (0, -2) so that if the mouse is
				// two pixels or less
				// within the boundary of the task, we'll consider it an attempt
				// to drop between
				// rather than on.
				Rectangle dropRegion = thisView.getBounds();
				dropRegion.grow(0, -2);

				// So, is the mouse inside the drop region for this TaskView?
				if (dropRegion.contains(mouseLocation)) {
					// Yes, so we flag this and set the TaskView as the current
					// drop target.
					isOverOtherView = true;
					thisView.setCurrentDropTarget(true);
				} else {
					// No, so first we revoke the drop target status for this
					// view if it has it.
					if (thisView.isCurrentDropTarget()) {
						thisView.setCurrentDropTarget(false);
					}

					// Then we figure out whether this view's drop region is
					// above or below the
					// mouse position.
					int top = dropRegion.y;
					if (mouseLocation.y < top) {
						// It's below, so check whether we already know about a
						// view below the
						// mouse.
						if (nextSibling == null) {
							// If not, this one will do.
							nextSibling = thisView;
						} else {
							// If so, check to see which one's closer and set
							// that as the
							// nextSibling. We can't just assume this one is
							// further away, because
							// we're not getting the components in any
							// particular order.
							int currentTop = nextSibling.getY() + 2;
							if (Math.abs(currentTop - mouseLocation.y) > Math.abs(top
									- mouseLocation.y)) {
								nextSibling = thisView;
							}
						}
					} else {
						// It's above, so check whether we already know about a
						// view above the
						// mouse.
						if (previousSibling == null) {
							// If not, this one will do.
							previousSibling = thisView;
						} else {
							// If so, check to see which one's closer and set
							// that as the
							// previousSibling. We can't just assume this one is
							// closer, because
							// we're not getting the components in any
							// particular order.
							int currentTop = previousSibling.getY() + 2;
							if (Math.abs(currentTop - mouseLocation.y) > Math.abs(top
									- mouseLocation.y)) {
								previousSibling = thisView;
							}
						}
					}
				}
			}

			// If we are indeed dropping between TaskViews, let's draw the
			// affordance for the drop:
			if (!isOverOtherView) {
				TaskSubpaneView.this.setDropBetween(previousSibling, nextSibling);
			} else {
				// Otherwise, we need to clear it:
				TaskSubpaneView.this.clearDrop();
			}
		}
	}

	/**
	 * A PropertyChangeListener that listens for TaskViewPropertyNames.DRAGGING_PROPERTY_NAME events
	 * coming from a task view that indicate that the view is no longer being dragged, and performs
	 * the drop operation on the appropriate component.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class TaskViewDropListener implements PropertyChangeListener {

		/**
		 * Called when a TaskView starts or stops being dragged.
		 * 
		 * @param e the property change event
		 */
		@SuppressWarnings("synthetic-access")
		public void propertyChange(PropertyChangeEvent e) {
			if (!e.getPropertyName().equals(TaskViewPropertyNames.DRAGGING_PROPERTY_NAME)) {
				// We only care about DRAGGING_PROPERTY_NAME property changes.
				return;
			}
			assert e.getNewValue() instanceof Boolean;
			if ((Boolean)e.getNewValue() == true) {
				// We don't care about the start of a drag, only the end.
				return;
			}

			Point mouseLocation = MouseInfo.getPointerInfo().getLocation();
			SwingUtilities.convertPointFromScreen(mouseLocation, TaskSubpaneView.this);
			boolean isMouseInView =
				(new Rectangle(	0,
								0,
								TaskSubpaneView.this.getWidth(),
								TaskSubpaneView.this.getHeight()).contains(mouseLocation));
			if (!isMouseInView) {
				// Don't do anything if the drop happened outside the view (this expresses a
				// "cancel").
				TaskSubpaneView.this.doLayout();
				return;
			}

			TaskView sourceView = ((TaskView)e.getSource());
			if (TaskSubpaneView.this.viewBelowDropTarget == null
					&& TaskSubpaneView.this.viewAboveDropTarget == null) {
				// They must have dropped onto a TaskView, so we need to go through and figure out
				// which one is the current drop target.
				for (Component c : TaskSubpaneView.this.getComponents()) {
					if (!(c instanceof TaskView)) {
						continue;
					}
					TaskView possibleDropTarget = (TaskView)c;
					if (possibleDropTarget.isCurrentDropTarget()) {
						possibleDropTarget.setCurrentDropTarget(false);
						JOptionPane.showMessageDialog(	TaskSubpaneView.this,
														"You have chosen to group the tasks "
																+ possibleDropTarget.getTask().getName()
																+ " and "
																+ sourceView.getTask().getName()
																+ ".  Unfortunately, this feature is not yet implemented.",
														"Create Task Group",
														JOptionPane.WARNING_MESSAGE);
						TaskSubpaneView.this.doLayout();
						return;
					}
				}

				// If we got here, they weren't actually over another view at all, which can happen
				// if there's only one taskview in the subpane. So we'll just snap it back to its
				// original position and return.
				TaskSubpaneView.this.doLayout();
				return;
			} else {
				// They dropped a TaskView between two other TaskViews, so they want to reorder.
				TaskOrdering taskList = new TaskOrdering(sourceView.getTask());
				taskList.setPreviousTask(TaskSubpaneView.this.viewBelowDropTarget == null
						? null
						: TaskSubpaneView.this.viewBelowDropTarget.getTask());

				taskList.setNextTask(TaskSubpaneView.this.viewAboveDropTarget == null
						? null
						: TaskSubpaneView.this.viewAboveDropTarget.getTask());
				TaskSubpaneView.this.firePropertyChange(TaskViewPropertyNames.TASK_ORDER_PROPERTY_NAME,
														null,
														taskList);
				TaskSubpaneView.this.clearDrop();
			}

		}
	}

	/**
	 * A PropertyChangeListener that listens for TaskViewPropertyNames.EXPANDED_PROPERTY_NAME
	 * property change events coming from task views and causes the subpane to redo its layout.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private final class TaskViewExpansionListener implements PropertyChangeListener {

		/**
		 * Called when a TaskView is expanded or collapsed.
		 * 
		 * @param evt the property change event
		 */
		public void propertyChange(PropertyChangeEvent evt) {
			TaskSubpaneView.this.validate();
		}
	}

	/**
	 * A helper class that remembers the previous state of the TaskManager and the mappings between
	 * Tasks and TaskViews. This allows the subpane to determine what tasks have been added,
	 * removed, and reordered since the last update, which allows it to determine what it should
	 * animate and what it should not.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	class TaskWatcher {

		/**
		 * The ordered list of relevant tasks that were in the TaskManager the last time
		 * rememberCurrentTasks() was called.
		 */
		private ArrayList<TasklikeObject> oldTasks = new ArrayList<TasklikeObject>();

		/**
		 * The rectangles of all the views that have been remembered (using rememberRect()) since
		 * the last time forgetRects() was called.
		 */
		private HashMap<Component, Rectangle> oldRects = new HashMap<Component, Rectangle>();

		/**
		 * The views associated with all the tasks that have been remembered (using rememberView())
		 * since the last time forgetViews() was called.
		 */
		private HashMap<Task, TaskView> taskMap = new HashMap<Task, TaskView>();

		/**
		 * The filter that determines which tasks are considered relevant.
		 */
		private TaskFilter taskFilter;

		/**
		 * Constructs a new TaskWatcher.
		 * 
		 * @param filter the filter to use when determining whether we need to know about changes to
		 *            a Task
		 */
		public TaskWatcher(TaskFilter filter) {
			this.taskFilter = filter;
		}

		/**
		 * Causes the TaskWatcher to forget about all the rects it has stored for various TaskViews.
		 * 
		 */
		public void forgetRects() {
			this.oldRects.clear();
		}

		/**
		 * Causes the TaskWatcher to forget about all the views it has stored for various Tasks.
		 * 
		 */
		public void forgetViews() {
			this.taskMap.clear();
		}

		/**
		 * Gets a list of all the Tasks that have been added to the TaskManager since the last time
		 * rememberCurrentTasks() was called.
		 * 
		 * @return the list of added tasks
		 */
		public List<TasklikeObject> getAddedTasks() {
			List<TasklikeObject> tasks = this.getFilteredTasks();
			List<TasklikeObject> newTasks = new ArrayList<TasklikeObject>();
			for (TasklikeObject object : tasks) {
				if (!this.oldTasks.contains(object)) {
					newTasks.add(object);
				}
			}
			return newTasks;
		}

		/**
		 * Gets a list of all relevant tasks in the TaskManager, where relevant is defined by the
		 * filter passed to this TaskWatcher in the constructor.
		 * 
		 * @return the list of relevant tasks
		 */
		private List<TasklikeObject> getFilteredTasks() {
			TasklikeObject[] taskArray = TaskManager.getManager().getTasks();
			List<TasklikeObject> tasks = new ArrayList<TasklikeObject>(taskArray.length);
			for (TasklikeObject object : taskArray) {
				if (this.taskFilter.shouldShowTask((Task)object)) {
					tasks.add(object);
				}
			}
			return tasks;
		}

		/**
		 * Returns the remembered Rectangle for the given TaskView, or null if we do not remember
		 * this view (that is, if rememberRect() has not been called with this view as the argument
		 * since the last time forgetRects() was called).
		 * 
		 * @param view the view whose Rectangle should be remembered
		 * @return the Rectangle, or \c null if we don't remember it
		 */
		public Rectangle getRememberedRect(TaskView view) {
			return this.oldRects.get(view);
		}

		/**
		 * Gets a list of all the Tasks that have been removed from the TaskManager since the last
		 * time rememberCurrentTasks() was called.
		 * 
		 * @return the list of removed tasks
		 */
		public List<TasklikeObject> getRemovedTasks() {
			List<TasklikeObject> tasks = this.getFilteredTasks();
			List<TasklikeObject> removedTasks = new ArrayList<TasklikeObject>(tasks.size());
			for (TasklikeObject object : this.oldTasks) {
				if (!tasks.contains(object)) {
					removedTasks.add(object);
				}
			}
			return removedTasks;
		}

		/**
		 * Gets a list of all the Tasks that have been reordered within the TaskManager since the
		 * last time rememberCurrentTasks() was called.
		 * 
		 * @note Reordering is defined as follows: A task has been reordered if:
		 *       <ol>
		 *       <li>it exists in the TaskManager and also existed the last time
		 *       rememberCurrentTasks() was called, and</li>
		 *       <li>its index in the ordered list of all relevant tasks currently in the manager
		 *       (including those that have since been added) is not equal to its index in the
		 *       ordered list of all relevant tasks in the manager when rememberCurrentTasks() was
		 *       called (including those that have since been removed).</li>
		 *       </ol>
		 * 
		 * @return the list of reordered tasks
		 */
		public List<TasklikeObject> getReorderedTasks() {
			List<TasklikeObject> keptTasks = this.getFilteredTasks();

			List<TasklikeObject> reorderedTasks = new ArrayList<TasklikeObject>(keptTasks.size());

			for (int i = 0; i < keptTasks.size(); i++ ) {
				if (i >= this.oldTasks.size() || !keptTasks.get(i).equals(this.oldTasks.get(i))) {
					if (this.oldTasks.contains(keptTasks.get(i))) {
						reorderedTasks.add(keptTasks.get(i));
					}
				}
			}
			return reorderedTasks;
		}

		/**
		 * Returns the TaskView associated with the given Task, or \c null if we do not remember a
		 * view for this Task (that is, if rememberView() has not been called with this Task as the
		 * parameter since the last time forgetViews() was called).
		 * 
		 * @param task the task to remember a view for
		 * @return the view for this task, or \c null if we don't remember a view for this task
		 */
		public TaskView getViewForTask(Task task) {
			return this.taskMap.get(task);
		}

		/**
		 * Removes from the list of remembered views any views associated with Tasks that are no
		 * longer in the TaskManager.
		 */
		public void pruneViews() {
			HashMap<Task, TaskView> newTaskMap = new HashMap<Task, TaskView>();
			for (Task t : this.taskMap.keySet()) {
				if (TaskManager.getManager().hasTask(t)) {
					newTaskMap.put(t, this.taskMap.get(t));
				}
			}
			this.taskMap = newTaskMap;
		}

		/**
		 * Causes subsequent calls to getAddedTasks(), getRemovedTasks(), and getReorderedTasks() to
		 * return the tasks that have been added, removed, or reordered (respectively) since the
		 * moment this method was called. Remembering the current tasks causes TaskWatcher to forget
		 * about any previous tasks it might already have been remembering.
		 */
		public void rememberCurrentTasks() {
			List<TasklikeObject> tasks = this.getFilteredTasks();

			this.oldTasks.clear();

			this.oldTasks.ensureCapacity(tasks.size());

			for (TasklikeObject task : tasks) {
				this.oldTasks.add(task);
			}

		}

		/**
		 * Remembers the Rectangle defining the bounds of this view at the time this method is
		 * called. Subsequent calls to getRememberedRect() with this view as the parameter will
		 * return this Rectangle.
		 * 
		 * @param view the view whose rectangle we should remember
		 */
		public void rememberRect(TaskView view) {
			this.oldRects.put(view, view.getBounds());
		}

		/**
		 * Remembers the mapping between this view and its associated task. Subsequent calls to
		 * getViewForTask() with this view's task as the parameter will return this view.
		 * 
		 * @param view the view to remember
		 */
		public void rememberView(TaskView view) {
			this.taskMap.put(view.getTask(), view);
		}
	}

	/**
	 * The duration of subpane animations, in seconds.
	 */
	public static final double ANIMATION_DURATION = 0.75;

	/**
	 * A test program that creates and shows a TaskSubpaneView.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		for (int i = 0; i < 5; i++ ) {
			Task t = new Task("Sample Task " + (i + 1), false);
			t.setPriority(100 - i);
			TaskManager.getManager().addTasklikeObject(t);
		}

		JFrame testFrame = new JFrame("TaskSubpaneView test");

		final TaskSubpaneView subpaneView = new TaskSubpaneView("All Tasks", 4, new TaskFilter() {

			@Override
			public boolean shouldShowTask(Task theT) {
				return true;
			}

		});

		JScrollPane scrollPane = new JScrollPane(subpaneView);
		testFrame.add(scrollPane);

		testFrame.setSize(400, 300);
		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setVisible(true);

		final Timer t = new Timer();
		t.schedule(new TimerTask() {

			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {

					public void run() {

						ArrayList<Task> bigOlHack = new ArrayList<Task>();
						Iterator<TasklikeObject> iter = TaskManager.getManager().taskIterator();
						while (iter.hasNext()) {
							bigOlHack.add((Task)iter.next());
						}

						TaskManager.getManager().removeTasklikeObject(bigOlHack.get(2));
						bigOlHack.get(1).setPriority(65);
						bigOlHack.get(3).setPriority(105);

						Task task = new Task("Sample Task 6", false);
						task.setPriority(96);
						// TaskManager.getManager().addTask(task);

						Task anotherTask = new Task("Sample Task 7", false);
						anotherTask.setPriority(96);
						// TaskManager.getManager().addTask(anotherTask);

						subpaneView.update();
					}
				});

			}

		}, 5000);

	}

	/**
	 * The title of this subpane.
	 */
	protected String title;

	/**
	 * Whether or not this subpane is maximized, that is, showing all tasks (rather than just the
	 * first few).
	 */
	private boolean isShowingAll = false;

	/**
	 * The number of items that should be shown in this view when it is in the "normal" state.
	 */
	int normalCount = 0;

	/**
	 * The TaskWatcher that helps us keep track of what's going on in the subpane between updates.
	 */
	private TaskWatcher taskWatcher;

	/**
	 * The title view for this subpane.
	 */
	private TaskSubpaneTitleView titleView;

	/**
	 * The link at the bottom of the subpane that allows the user to see more or fewer items.
	 */
	JButton moreLink;

	/**
	 * @name Filtering
	 * 
	 * Fields used in filtering tasks within the subpane.
	 */

	// @{
	/**
	 * The number of tasks that made it through both the base filter and any additional filter.
	 */
	private int numFilteredTasks;

	/**
	 * The TaskFilter that is used to determine which tasks show up in this subpane before any other
	 * filter is applied.
	 */
	private TaskFilter baseFilter;

	/**
	 * An additional filter to be applied on top of the base filter for this subpane.
	 */
	private TaskFilter filter;

	// @}

	/**
	 * @name Drag and Drop
	 * 
	 * Fields used when dealing with drag-and-drop of task views within the subpane.
	 * 
	 */

	// @{
	/**
	 * The view that is directly above the current drop target, or \c null if the drop target is the
	 * first view.
	 */
	private TaskView viewAboveDropTarget;

	/**
	 * The view that is directly below the current drop target, or \c null if the drop target is the
	 * last view.
	 */
	private TaskView viewBelowDropTarget;

	// @}

	/**
	 * Constructs a TaskSubpaneView with the given normal task count and task taskFilter. It will be
	 * populated with tasks from the TaskManager. Note that an item will be shown in the maximized
	 * state as long as it matches the taskFilter, and in normal mode only if it both matches the
	 * taskFilter and is one of the \p normalCount highest-priority tasks that match the taskFilter.
	 * 
	 * @param title the title of this subpane
	 * @param normalCount the number of tasks to show in the normal state
	 * @param filter the task taskFilter to use for this subpane
	 */
	public TaskSubpaneView(String title, int normalCount, TaskFilter filter) {
		super();
		this.title = title;
		this.normalCount = normalCount;
		this.baseFilter = filter;
		this.taskWatcher = new TaskWatcher(filter);
		this.setup();
	}

	/**
	 * Resets all the subpane state relating to drag and drop and repaints the subpane so that no
	 * drop target is drawn.
	 */
	protected void clearDrop() {
		this.viewAboveDropTarget = null;
		this.viewBelowDropTarget = null;

		this.repaint();
	}

	/**
	 * Returns the number of tasks that can be shown in the subpane. This does NOT take into account
	 * tasks that may not be visible due to being "below the fold" or because the subpane is
	 * collapsed, but does take into account whether the subpane is filtered. This method will
	 * always return the number of tasks that would be shown in the subpane if it were maximized.
	 * 
	 * @return the number of shown tasks
	 */
	public int countShownTasks() {
		return this.numFilteredTasks;
	}

	@Override
	protected boolean doAnimations() {

		ArrayList<Animation> anims = new ArrayList<Animation>();

		for (TasklikeObject obj : this.taskWatcher.getAddedTasks()) {

			Component c = this.taskWatcher.getViewForTask((Task)obj);
			if (c == null) {
				continue;
			}

			Animation a =
				AnimationManager.getManager().animateValue(	0,
															1,
															AnimationCurve.EASE_IN,
															TaskSubpaneView.ANIMATION_DURATION);
			a.setObjectData(c);
			anims.add(a);
		}

		for (TasklikeObject obj : this.taskWatcher.getRemovedTasks()) {

			Component c = this.taskWatcher.getViewForTask((Task)obj);
			if (c == null) {
				continue;
			}
			Animation a =
				AnimationManager.getManager().animateValue(	1,
															0,
															AnimationCurve.EASE_OUT,
															TaskSubpaneView.ANIMATION_DURATION);
			a.setObjectData(c);
			anims.add(a);
		}

		for (TasklikeObject obj : this.taskWatcher.getReorderedTasks()) {
			TaskView c = this.taskWatcher.getViewForTask((Task)obj);
			if (c == null) {
				continue;
			}
			Rectangle oldRect = this.taskWatcher.getRememberedRect(c);

			if (oldRect == null || oldRect.getY() > this.getHeight()) {
				// This rectangle was filtered out before, so let's pretend it
				// was off the bottom
				// edge of the view.
				int y = this.getHeight() + 4;
				oldRect = new Rectangle(6, y, this.getWidth() - 12, c.getPreferredSize().height);
			}

			Animation a =
				AnimationManager.getManager().animateRect(	oldRect,
															(Rectangle)c.getBounds().clone(),
															AnimationCurve.EASE_IN_OUT,
															TaskSubpaneView.ANIMATION_DURATION);
			c.setBounds(oldRect);
			a.setObjectData(c);
			anims.add(a);
		}

		this.taskWatcher.rememberCurrentTasks();

		if (anims.size() == 0) {
			return false;
		}

		this.setTotalAnimations(anims.size());

		for (Animation animation : anims) {
			animation.start(this);
			this.incrementNumAnimating();
		}

		return true;
	}

	/**
	 * Lays out all the TaskManager's tasks in order of descending priority.
	 */
	@SuppressWarnings("synthetic-access")
	@Override
	protected void doUpdate() {

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 4, 2);

		// Define some bindings and relative positions:
		RelativePosition overlap = new RelativePosition(Edge.TOP, 24, Direction.ABOVE, Edge.BOTTOM);

		// Remove all TaskViews from the subpaneView, and store them in the map:
		for (int i = this.getComponents().length - 1; i >= 0; i-- ) {
			Component component = this.getComponent(i);
			if (!(component instanceof TaskView)) {
				continue;
			}
			this.remove(component);
			this.taskWatcher.rememberView((TaskView)component);
		}
		this.remove(this.moreLink);

		int numRelevantTasks = 0;
		int numDisplayedTasks = 0;
		this.numFilteredTasks = 0;

		TasklikeObject objects[] = TaskManager.getManager().getTasks();

		ArrayList<TaskView> orderedTaskViews = new ArrayList<TaskView>();

		for (int i = 0; i < objects.length; i++ ) {

			// If this task shouldn't appear due to the taskFilter on this
			// subpane, go on to the next task:
			Task thisTask = (Task)objects[i];
			if (!this.baseFilter.shouldShowTask(thisTask)) {
				continue;
			}

			numRelevantTasks++ ;

			if (this.filter != null && !this.filter.shouldShowTask(thisTask)) {
				continue;
			}

			if (this.filter != null && this.isCollapsed()) {
				this.setCollapsed(false);
				update();
				return;
			}

			// Otherwise, increment the number of tasks that match the
			// taskFilter.
			this.numFilteredTasks++ ;

			// Now, if the task shouldn't appear due to being "below the
			// fold" of a normal view, go on to the next task:
			if (this.isCollapsed() || !this.isShowingAll() && numDisplayedTasks >= this.normalCount) {
				continue;
			}

			// Otherwise, increment the number of tasks that will be
			// displayed.
			numDisplayedTasks++ ;

			// If the view existed before we removed everything, we'll just
			// add it back. Otherwise, we'll make a new view for this task
			// and add that.
			final TaskView view;
			TaskView viewForTask = this.taskWatcher.getViewForTask(thisTask);
			if (viewForTask != null) {
				view = viewForTask;
				// Reset the drop target status of the task in case it was
				// just dropped onto.
				view.setCurrentDropTarget(false);
			} else {
				view = new TaskView(thisTask);

				// We only add the listeners if this is a new task, since we
				// don't want to attach more than one of each!
				PropertyChangeListener taskViewExpansionListener = new TaskViewExpansionListener();
				view.addPropertyChangeListener(	TaskViewPropertyNames.EXPANDED_PROPERTY_NAME,
												taskViewExpansionListener);

				PropertyChangeListener taskViewDropListener = new TaskViewDropListener();
				view.addPropertyChangeListener(	TaskViewPropertyNames.DRAGGING_PROPERTY_NAME,
												taskViewDropListener);

				ComponentListener taskViewDragListener = new TaskViewDragListener();
				view.addComponentListener(taskViewDragListener);

				this.taskWatcher.rememberView(view);
			}

			orderedTaskViews.add(view);
		}

		this.taskWatcher.pruneViews();

		Collections.sort(orderedTaskViews, new Comparator<TaskView>() {

			public int compare(TaskView o1, TaskView o2) {

				if (o1.getTask().getPriority() > o2.getTask().getPriority()) {
					return -1;
				} else if (o2.getTask().getPriority() > o1.getTask().getPriority()) {
					return 1;
				} else {
					return 0;
				}
			}

		});

		TaskView lastView = null;
		for (TaskView view : orderedTaskViews) {
			// Now generate the constraints for this view:
			RelativeConstraints constraints = new RelativeConstraints();
			constraints.addBindings(bm.leftEdge(), bm.rightEdge());
			if (lastView != null) {
				constraints.addBinding(bm.below(lastView));
			} else {
				constraints.addBinding(new Binding(overlap, this.titleView));
			}

			// Add the view:
			this.add(view, constraints);

			lastView = view;
		}

		this.updateTasks();

		RelativeConstraints moreLinkConstraints = new RelativeConstraints();
		moreLinkConstraints.addBindings(bm.leftEdge(),
										bm.below(this.getComponent(this.getComponentCount() - 1)));

		if (!this.isCollapsed()) {
			if (!this.isShowingAll()) {
				this.moreLink.setText("Show all " + this.numFilteredTasks + "...");
			} else {
				String showText = "Show top " + this.normalCount + "...";
				this.moreLink.setText(showText);
			}
			if (this.numFilteredTasks > this.normalCount) {
				this.moreLink.setSize(this.moreLink.getPreferredSize());
				this.add(this.moreLink, moreLinkConstraints);
			}
		}

		this.setComponentZOrder(this.titleView, this.getComponentCount() - 1);
		String fullTitle = this.title + "  (" + numDisplayedTasks + "/" + numRelevantTasks + ")";
		this.titleView.setTitle(fullTitle);
	}

	/**
	 * Gets the filter currently being applied to this subpane if there is one, or \c null
	 * otherwise.
	 * 
	 * @return the filter
	 */
	public TaskFilter getFilter() {
		return this.filter;
	}

	/**
	 * Returns whether this subpane is currently collapsed.
	 * 
	 * @return \c true if the subpane is collapsed, \c false otherwise
	 */
	public boolean isCollapsed() {
		return this.titleView.isCollapsed();
	}

	/**
	 * Returns whether this subpane is currently showing all tasks (i.e., maximized).
	 * 
	 * @return \c true if the subpane is showing all tasks, \c false otherwise
	 */
	public boolean isShowingAll() {
		return this.isShowingAll;
	}

	/**
	 * Called by Swing to determine whether to revalidate the layout of this component. We override
	 * it to return \c true whenever there are animations in progress, which prevents Swing from
	 * redoing the layout and putting things where they don't belong.
	 */
	@Override
	public boolean isValid() {
		if (this.getNumAnimating() > 0) {
			return true;
		} else {
			return super.isValid();
		}
	}

	/**
	 * Paints the subpane normally, then superimposes a drop target indicator if a drag is currently
	 * in progress that would result in the dragging task being dropped between two other tasks.
	 * 
	 * @param graphics the graphics context to paint into
	 */
	@Override
	public void paint(Graphics graphics) {
		super.paint(graphics);

		Graphics2D g = ((Graphics2D)graphics);
		Stroke originalStroke = g.getStroke();
		Color lineColor = SystemColor.textHighlight;
		if (this.viewBelowDropTarget != null) {
			g.setStroke(new BasicStroke(4.0f));
			g.setColor(lineColor);
			g.drawLine(	0,
						this.viewBelowDropTarget.getY() - 2,
						this.getWidth(),
						this.viewBelowDropTarget.getY() - 2);
			g.setStroke(originalStroke);
		} else if (this.viewAboveDropTarget != null) {
			g.setStroke(new BasicStroke(4.0f));
			g.setColor(lineColor);
			g.drawLine(0, this.viewAboveDropTarget.getY() + this.viewAboveDropTarget.getHeight()
					+ 2, this.getWidth(), this.viewAboveDropTarget.getY()
					+ this.viewAboveDropTarget.getHeight() + 2);
			g.setStroke(originalStroke);
		}
	}

	/**
	 * Paints the subpane's background, including a nice pretty gradient along the bottom to match
	 * the one painted by the title view.
	 * 
	 * @param graphics the graphics context to paint into
	 */
	@Override
	public void paintComponent(Graphics graphics) {
		super.paintComponent(graphics);
		if (!(graphics instanceof Graphics2D)) {
			throw new UnsupportedOperationException("The Java graphics subsystem is not compatible with Java2D, so this component cannot be drawn.");
		}
		Graphics2D g = (Graphics2D)graphics;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		Color solid = UIManager.getColor("InternalFrame.activeTitleBackground");
		float[] hsb = Color.RGBtoHSB(solid.getRed(), solid.getGreen(), solid.getBlue(), null);
		hsb[1] = Math.max(0.0f, hsb[1] / 2.0f);
		solid = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
		Color transparent =
			new Color(solid.getRed() / 255f, solid.getGreen() / 255f, solid.getBlue() / 255f, 0.0f);

		if (!this.isCollapsed()) {
			RoundRectangle2D roundrect =
				new RoundRectangle2D.Float(0, this.getHeight() - 32, this.getWidth(), 32, 8, 8);
			Area fill = new Area(roundrect);
			Area frame = new Area(roundrect);
			frame.subtract(new Area(new RoundRectangle2D.Float(	4,
																this.getHeight() - 40,
																this.getWidth() - 8,
																36,
																8,
																8)));

			Paint gradient =
				new GradientPaint(0, this.getHeight() - 32, transparent, 0, this.getHeight(), solid);
			g.setPaint(gradient);
			g.fill(fill);
		} else {
			g.setColor(solid);
			g.fillRoundRect(0, 0, this.getWidth(), this.getHeight(), 8, 8);
		}
	}

	@Override
	protected void prepareAnimations() {
		this.taskWatcher.forgetRects();
		for (Component c : this.getComponents()) {
			if (!(c instanceof TaskView)) {
				continue;
			}
			this.taskWatcher.rememberRect((TaskView)c);
		}
	}

	@Override
	protected void safeUpdateAnimation(Animation a) {
		Component currentView = (Component)a.getObjectData();
		if (a.getAnimationType() == AnimationType.RECTANGLE) {
			Rectangle2D currentRect = a.getCurrentRectangle();
			Rectangle currentBounds =
				new Rectangle(	Math.round((int)currentRect.getMinX()),
								(int)Math.round(currentRect.getMinY()),
								(int)Math.round(currentRect.getWidth()),
								(int)Math.round(currentRect.getHeight()));
			currentView.setBounds(currentBounds);
			currentView.repaint();
		} else if (a.getAnimationType() == AnimationType.SCALAR) {
			TaskView tview = (TaskView)currentView;
			tview.setOpacity(a.getCurrentValue());
		} else {
			assert false : "the type of this animation is not one we know how to deal with";
		}
	}

	/**
	 * Sets whether this subpane is collapsed.
	 * 
	 * @param isCollapsed whether or not this subpane is collapsed
	 */
	public void setCollapsed(boolean isCollapsed) {
		this.titleView.setCollapsed(isCollapsed);
	}

	/**
	 * Sets the drag-and-drop related state of this subpane based on the views passed, and repaints
	 * the subpane to show the drop target indicator.
	 * 
	 * @param previousSibling the view above the drop target
	 * @param nextSibling the view below the drop target
	 */
	protected void setDropBetween(TaskView previousSibling, TaskView nextSibling) {
		this.viewAboveDropTarget = previousSibling;
		this.viewBelowDropTarget = nextSibling;

		this.repaint();
	}

	/**
	 * Sets the filter used in this subpane. Pass \c null to clear the filter.
	 * 
	 * @param filter the filter to set
	 */
	public void setFilter(TaskFilter filter) {
		this.filter = filter;
		this.update();
	}

	/**
	 * Sets whether this subpane is showing all tasks (i.e., maximized).
	 * 
	 * @param isShowingAll \c true if the subpane should show all tasks, \c false otherwise
	 */
	public void setShowingAll(boolean isShowingAll) {
		boolean wasShowingAll = this.isShowingAll;
		this.isShowingAll = isShowingAll;
		this.firePropertyChange(TaskViewPropertyNames.SUBPANE_SHOWING_ALL_PROPERTY_NAME,
								wasShowingAll,
								this.isShowingAll);
	}

	/**
	 * Sets up the appearance of the view.
	 */
	private void setup() {
		this.setLayout(new RelativeLayout());

		Binding topEdgeNoMargin = new Binding(Edge.TOP, 0, Direction.BELOW, Edge.TOP, null);
		Binding leftEdgeNoMargin = new Binding(Edge.LEFT, 0, Direction.RIGHT, Edge.LEFT, null);
		Binding rightEdgeNoMargin = new Binding(Edge.RIGHT, 0, Direction.LEFT, Edge.RIGHT, null);

		this.titleView = new TaskSubpaneTitleView(this.title);

		this.moreLink = new JLinkButton();
		this.moreLink.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				TaskSubpaneView.this.setShowingAll(!TaskSubpaneView.this.isShowingAll());

			}

		});

		RelativeConstraints titleViewConstraints = new RelativeConstraints();
		titleViewConstraints.addBindings(leftEdgeNoMargin, rightEdgeNoMargin, topEdgeNoMargin);
		this.add(this.titleView, titleViewConstraints);

		this.setBorder(new EmptyBorder(0, 0, 4, 0));
		this.setPreventsCollapse(false);

		this.update();
	}

	/**
	 * Updates the state of all task views in this subpane, without actually updating the subpane
	 * itself.
	 * 
	 * @note This differs from update() in that only the contents of the tasks will change, and so
	 *       the adding, removing, or reordering of tasks within the subpane will not be correctly
	 *       reflected. It is much, much less expensive than a full update(), however.
	 */
	public void updateTasks() {
		for (Component c : this.getComponents()) {
			if (c instanceof TaskView) {
				TaskView view = (TaskView)c;
				view.update();
			}
		}
	}

}
