/*
 * File: TaskView.java
 * 
 * Contains class TaskView
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.border.Border;

import net.iharder.dnd.FileDrop;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Resource;
import edu.cmu.hcii.calo.model.ResourceList;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.DisclosureButton;
import edu.cmu.hcii.calo.view.EditableLabelListener;
import edu.cmu.hcii.calo.view.FlatButton;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.IconToggleButton;
import edu.cmu.hcii.calo.view.task.support.TaskState;
import edu.cmu.hcii.calo.view.task.support.TaskViewPropertyNames;
import edu.cmu.hcii.calo.view.task.support.TaskState.TaskStateProperty;
import edu.cmu.hcii.calo.view.training.TrainingMenuListener;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A view class that displays a single task in the task pane.
 * 
 * @author Brian Ellis
 */
@SuppressWarnings("serial")
public class TaskView extends CALOComponent {

	/**
	 * A MouseListener that handles dragging for this task view.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	class DragListener extends MouseAdapter {

		/**
		 * Invoked when the mouse is dragged after having been pressed in the view.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseDragged(MouseEvent e) {
			TaskView.this.scrollRectToVisible(new Rectangle(e.getPoint()));
			if (!TaskView.this.isEnabled()) {
				return;
			}
			if (!TaskView.this.isDragging) {
				TaskView.this.getParent().setComponentZOrder(TaskView.this, 0);
				TaskView.this.isDragging = true;
				TaskView.this.firePropertyChange(	TaskViewPropertyNames.DRAGGING_PROPERTY_NAME,
													false,
													true);
				TaskView.this.updateAppearance();
			}
			Point newLoc = new Point(e.getLocationOnScreen());
			SwingUtilities.convertPointFromScreen(newLoc, TaskView.this.getParent());
			newLoc.translate(0 - TaskView.this.clickPoint.x, 0 - TaskView.this.clickPoint.y);
			TaskView.this.setLocation(newLoc);
		}

		/**
		 * Invoked when the mouse is pressed in the view.
		 * 
		 * @param e the mouse event
		 * 
		 */
		@Override
		public void mousePressed(MouseEvent e) {
			if (!TaskView.this.isEnabled()) {
				return;
			}
			TaskView.this.isPressed = true;
			TaskView.this.clickPoint = e.getPoint();
			TaskView.this.updateAppearance();
		}

		/**
		 * Invoked when the mouse is released after having been pressed in the view.
		 * 
		 * @param e the mouse event
		 * 
		 */
		@Override
		public void mouseReleased(MouseEvent e) {
			if (!TaskView.this.isEnabled()) {
				return;
			}
			if (TaskView.this.isDragging) {
				TaskView.this.firePropertyChange(	TaskViewPropertyNames.DRAGGING_PROPERTY_NAME,
													true,
													false);
			}
			TaskView.this.isPressed = false;
			TaskView.this.isDragging = false;
			TaskView.this.updateAppearance();
		}

	}

	/**
	 * A MouseListener that handles when the user clicks on the task view to expand or collapse it.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	class ExpansionListener extends MouseAdapter {

		/**
		 * Invoked when the mouse is clicked in the view.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseClicked(MouseEvent e) {
			if (e.isMetaDown()) {
				return;
			}
			TaskView.this.toggleExpanded();
			TaskView.this.updateAppearance();
		}

	}

	/**
	 * A MouseListener that handles displaying the mouseover state of the task view when the user
	 * hovers the mouse over it.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	@SuppressWarnings("synthetic-access")
	class HighlightListener extends MouseAdapter {

		/**
		 * Invoked when the mouse enters the view.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseEntered(MouseEvent e) {
			if (!TaskView.this.isPressed) {
				TaskView.this.isMouseOver = true;
				TaskView.this.updateAppearance();
			}
		}

		/**
		 * Invoked when the mouse exits the view.
		 * 
		 * @param e the mouse event
		 */
		@Override
		public void mouseExited(MouseEvent e) {
			if (!TaskView.this.isPressed) {
				TaskView.this.isMouseOver = false;
				TaskView.this.updateAppearance();
			}
		}

	}

	/**
	 * @name Colors
	 * 
	 * These constants represent colors that are used in displaying the TaskView.
	 */

	// @{
	private static final Color TASK_COLOR = new Color(0.93f, 0.93f, 0.93f);

	private static final Color EXPANDED_TASK_COLOR = new Color(1.0f, 1.0f, 0.87f);

	private static final Color MOUSEOVER_TASK_COLOR = new Color(0.97f, 0.97f, 0.97f);

	private static final Color MOUSEOVER_EXPANDED_TASK_COLOR = new Color(1.0f, 1.0f, 0.93f);

	private static final Color TASK_STROKE_COLOR = SystemColor.controlShadow;

	// @}

	/**
	 * Runs a small test program to verify that TaskView works correctly.
	 * 
	 * @param args command-line args; don't bother passing any
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		final JFrame testFrame = new JFrame("TaskView test");
		testFrame.setLayout(new RelativeLayout());
		final TaskView taskView = new TaskView(new Task("My Task", false));
		testFrame.add(taskView, new RelativeConstraints(new Binding(Edge.TOP,
																	4,
																	Direction.BELOW,
																	Edge.TOP,
																	null),
														new Binding(Edge.LEFT,
																	4,
																	Direction.RIGHT,
																	Edge.LEFT,
																	null),
														new Binding(Edge.RIGHT,
																	4,
																	Direction.LEFT,
																	Edge.RIGHT,
																	null)));

		taskView.addPropertyChangeListener(	TaskViewPropertyNames.EXPANDED_PROPERTY_NAME,
											new PropertyChangeListener() {

												public void propertyChange(PropertyChangeEvent e) {
													taskView.setSize(taskView.getPreferredSize());
													testFrame.validate();
												}

											});

		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		testFrame.setSize(400, 52);
		testFrame.setVisible(true);
	}

	/**
	 * @name Borders
	 * 
	 * These constants are the various border styles that are applied to the TaskView under various
	 * circumstances. Note that the rather badly-named Border actually determines the background
	 * color of the TaskView as well.
	 */

	// @{
	private final TaskViewBorder MOUSEOVER_BORDER =
		new TaskViewBorder(TaskView.MOUSEOVER_TASK_COLOR, TaskView.TASK_STROKE_COLOR);

	private final TaskViewBorder EXPANDED_BORDER =
		new TaskViewBorder(TaskView.EXPANDED_TASK_COLOR, TaskView.TASK_STROKE_COLOR);

	private final TaskViewBorder MOUSEOVER_EXPANDED_BORDER =
		new TaskViewBorder(TaskView.MOUSEOVER_EXPANDED_TASK_COLOR, TaskView.TASK_STROKE_COLOR);

	private final Border DROP_TARGET_BORDER =
		new TaskViewBorder(TaskView.MOUSEOVER_TASK_COLOR, TaskView.TASK_STROKE_COLOR, true);

	private final TaskViewBorder NORMAL_BORDER =
		new TaskViewBorder(TaskView.TASK_COLOR, TaskView.TASK_STROKE_COLOR);
	// @}

	/**
	 * Whether or not the TaskView is currently pressed. This may be \c true even if the mouse is
	 * not within the bounds of the view.
	 */
	private boolean isPressed = false;

	/**
	 * Whether or not the TaskView is currently expanded to show detailed information.
	 */
	private boolean isExpanded = false;

	/**
	 * Whether or not the mouse is currently over the view.
	 */
	private boolean isMouseOver = false;

	/**
	 * The task being represented by this view.
	 */
	private Task task;

	/**
	 * The shared highlight listener attached to all the controls in this view. This ensures that
	 * the mouseover state of the task will be shown even if the mouse is over a child of this view
	 * rather than over the view itself.
	 */
	private HighlightListener highlightListener = new HighlightListener();

	/**
	 * @name Drag and Drop
	 * 
	 * Fields used to implement dragging and dropping task views inside their subpane.
	 */

	// @{
	/**
	 * Whether or not the user is currently dragging this TaskView around.
	 */
	private boolean isDragging;

	/**
	 * The point at which the user clicked at the beginning of a drag operation.
	 */
	private Point clickPoint;

	/**
	 * Whether this view is currently the target of a drop operation involving a different view.
	 */
	private boolean isCurrentDropTarget;

	// @}

	/**
	 * @name Controls
	 * 
	 * Various controls in the view.
	 */

	// @{
	/**
	 * The panel containing the info that's displayed when the TaskView is expanded.
	 */
	private ExpandedTaskInfoPanel expandedInfoPanel;

	/**
	 * The button allowing the user to star or unstar this task.
	 */
	private IconToggleButton starButton;

	/**
	 * The label showing the name of this task.
	 */
	private JLabel taskNameLabel;

	/**
	 * The field allowing you to edit the name of the task.
	 */
	private JTextField taskNameField;

	/**
	 * The check box determining whether this task is complete.
	 */
	private IconToggleButton completeBox;

	/**
	 * The disclosure triangle allowing this view to be expanded or contracted.
	 */
	private DisclosureButton disclosureTriangle;

	/**
	 * The button showing that this task has been added by CALO and allowing the user to verify that
	 * it is accurate by clicking it.
	 */
	private IconToggleButton caloAddedButton;

	/**
	 * The button that displays the task's current due date and allows the user to change it.
	 */
	private FlatButton dueDateButton;

	/**
	 * The button that allows the user to mark the task as on or off of hold.
	 */
	private IconButton markOnHoldButton;

	/**
	 * The button that allows the user to delete this task.
	 */
	private IconButton deleteButton;

	// @}

	/**
	 * Constructs a TaskView representing the given task.
	 * 
	 * @param t the task to represent with this TaskView
	 */
	public TaskView(Task t) {
		super();

		ImageIcon starIcon = new ImageIcon("Resources/star-off.png");
		ImageIcon starPressedIcon = new ImageIcon("Resources/star-pressed.png");
		ImageIcon starSelectedIcon = new ImageIcon("Resources/star-on.png");

		ImageIcon caloIcon = new ImageIcon("Resources/calo.png");
		ImageIcon caloDimmedIcon = new ImageIcon("Resources/calo-dimmed.png");

		ImageIcon deleteIcon = new ImageIcon("Resources/trash.png");
		ImageIcon deleteOverIcon = new ImageIcon("Resources/trash-over.png");
		ImageIcon deletePressedIcon = new ImageIcon("Resources/trash-pressed.png");

		this.starButton = new IconToggleButton(starIcon, starPressedIcon, starSelectedIcon);
		this.starButton.setUnselectedToolTipText("Click to star this task.");
		this.starButton.setSelectedToolTipText("Click to unstar this task.");

		this.completeBox =
			new IconToggleButton(	new ImageIcon("Resources/complete-unchecked.png"),
									new ImageIcon("Resources/complete-unchecked-over.png"),
									new ImageIcon("Resources/complete-unchecked-pressed.png"),
									new ImageIcon("Resources/complete-checked.png"),
									new ImageIcon("Resources/complete-checked-over.png"),
									new ImageIcon("Resources/complete-checked-pressed.png"));
		this.completeBox.setUnselectedToolTipText("Click to mark this task as completed.");
		this.completeBox.setSelectedToolTipText("Click to mark this task as not completed.");
		this.taskNameLabel = new JLabel();
		this.taskNameField = new JTextField();
		this.disclosureTriangle = new DisclosureButton();
		this.disclosureTriangle.setUnselectedToolTipText("Click to see more information about this task.");
		this.disclosureTriangle.setSelectedToolTipText("Click to show less information about this task.");
		this.caloAddedButton =
			new IconToggleButton(	caloIcon,
									caloIcon,
									caloIcon,
									caloDimmedIcon,
									caloDimmedIcon,
									caloDimmedIcon);
		this.dueDateButton = new FlatButton();
		this.dueDateButton.setFont(this.dueDateButton.getFont().deriveFont(9.0f));
		this.dueDateButton.setForeground(Color.DARK_GRAY);
		this.dueDateButton.setHorizontalAlignment(SwingConstants.LEFT);
		this.dueDateButton.setPadding(4, 3);
		this.markOnHoldButton = new IconButton(new ImageIcon("Resources/onhold.png"));
		this.deleteButton = new IconButton(deleteIcon, deleteOverIcon, deletePressedIcon);
		this.deleteButton.setToolTipText("Click to delete this task.");
		this.task = t;
		this.setup();
	}

	/**
	 * Adds the component to this container at the specified index with the specified constraints,
	 * as usual. Overridden to also add a highlight listener to the component so that this view will
	 * show its mouseover state whenever the mouse is within its bounds.
	 * 
	 * @param comp the component to add
	 * @param constraints the constraints to add it with
	 * @param index the index to add it at
	 */
	@Override
	protected void addImpl(Component comp, Object constraints, int index) {
		super.addImpl(comp, constraints, index);
		comp.addMouseListener(this.highlightListener);
		comp.setCursor(Cursor.getDefaultCursor());
	}

	/**
	 * Returns an appropriate HTML string to use as the text of the tooltip for the on hold button
	 * for this task. This is calculated based on the pending state of the task.
	 * 
	 * @return the tooltip string
	 */
	private String getOnHoldToolTipString() {
		String toolTipString = "<html><p>This task is marked as &ldquo;on hold&rdquo;";
		if (this.task.getPendingState().getPendingNote().length() > 0) {
			toolTipString +=
				":</p>" + "<blockquote><em>" + this.task.getPendingState().getPendingNote()
						+ "</em> ";
		}
		if (this.task.getPendingState().getPerson().length() > 0) {
			toolTipString += ", pending action by " + this.task.getPendingState().getPerson();
		}
		if (this.task.getPendingState().getPendingNote().length() > 0) {
			toolTipString += ".</blockquote><p>";
		} else {
			toolTipString += ".</p><p>";
		}
		toolTipString +=
			"Click to take it off hold.</p>"
					+ "<p>A task that is on hold is waiting for someone or something outside your control.</p></html>";
		return toolTipString;
	}

	/**
	 * Returns the Task associated with this view.
	 */
	public Task getTask() {
		return this.task;
	}

	/**
	 * Returns whether this view is the current target of a drop operation involving some other
	 * TaskView.
	 * 
	 * @return \c true if this view is the drop target, \c false otherwise
	 */
	public boolean isCurrentDropTarget() {
		return this.isCurrentDropTarget;
	}

	/**
	 * Returns whether this view is currently expanded.
	 * 
	 * @return \c true if the view is expanded, \c false otherwise
	 */
	public boolean isExpanded() {
		return this.isExpanded;
	}

	/**
	 * Returns whether this view is currently being dragged by the user.
	 * 
	 * @return \c true if the view is being dragged, \c false otherwise
	 */
	public boolean isInDrag() {
		return this.isDragging;
	}

	/**
	 * Sets whether this view is the current drop target of a drop operation involving some other
	 * task view. Calling setCurrentDropTarget() updates the appearance of this view to reflect
	 * whether it's the drop target or not.
	 * 
	 * @param b \c true if this view should be the current drop target, \c false otherwise
	 */
	public void setCurrentDropTarget(boolean b) {
		this.isCurrentDropTarget = b;
		this.updateAppearance();
	}

	/**
	 * Expands or contracts this view.
	 * 
	 * @param isExpanded \c true if this view should be expanded, \c false otherwise
	 */
	public void setExpanded(boolean isExpanded) {
		if (this.isExpanded == isExpanded) {
			return;
		} else {
			this.toggleExpanded();
		}
	}

	/**
	 * Sets up the visual appearance of the TaskView.
	 */
	private void setup() {
		this.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		this.setLayout(new RelativeLayout());

		BindingMaker bm = new BindingMaker(4, 4, 4, 4, 2, 2);
		Binding centerAlign = bm.verticallyCenterAlign(this.completeBox);

		RelativeConstraints disclosureTriangleConstraints = new RelativeConstraints();
		disclosureTriangleConstraints.addBindings(bm.leftEdge(), centerAlign);

		RelativeConstraints completeBoxConstraints = new RelativeConstraints();
		completeBoxConstraints.addBindings(bm.rightOf(this.disclosureTriangle), bm.topEdge());

		RelativeConstraints starButtonConstraints = new RelativeConstraints();
		starButtonConstraints.addBindings(	bm.below(this.completeBox),
											bm.horizontallyCenterAlign(this.completeBox));

		RelativeConstraints caloAddedButtonConstraints = new RelativeConstraints();
		caloAddedButtonConstraints.addBindings(bm.rightEdge(), centerAlign);

		RelativeConstraints taskNameLabelConstraints = new RelativeConstraints();
		taskNameLabelConstraints.addBindings(centerAlign);
		taskNameLabelConstraints.addBinding(bm.rightOf(this.completeBox));

		RelativeConstraints taskNameFieldConstraints = new RelativeConstraints();
		taskNameFieldConstraints.addBindings(centerAlign);
		taskNameFieldConstraints.addBinding(bm.rightOf(this.completeBox));

		if (this.task.isCaloAdded()) {
			taskNameLabelConstraints.addBinding(bm.leftOf(this.caloAddedButton));
			taskNameFieldConstraints.addBinding(bm.leftOf(this.caloAddedButton));
			this.add(this.caloAddedButton, caloAddedButtonConstraints);
		} else {
			taskNameLabelConstraints.addBinding(bm.rightEdge());
			taskNameFieldConstraints.addBinding(bm.rightEdge());
		}

		RelativeConstraints dueDateLabelConstraints = new RelativeConstraints();
		dueDateLabelConstraints.addBindings(bm.leftOf(this.markOnHoldButton),
											bm.rightOf(this.starButton),
											bm.verticallyCenterAlign(this.starButton));

		RelativeConstraints markOnHoldButtonConstraints =
			new RelativeConstraints(bm.leftOf(this.deleteButton),
									bm.verticallyCenterAlign(this.starButton));

		RelativeConstraints deleteButtonConstraints =
			new RelativeConstraints(bm.rightEdge(), bm.verticallyCenterAlign(this.starButton));

		this.taskNameField.setVisible(false);
		this.taskNameLabel.setVisible(true);
		this.taskNameLabel.setBorder(EditableLabelListener.NORMAL_LABEL_BORDER);

		this.add(this.disclosureTriangle, disclosureTriangleConstraints);
		this.add(this.starButton, starButtonConstraints);
		this.add(this.completeBox, completeBoxConstraints);
		this.add(this.taskNameLabel, taskNameLabelConstraints);
		this.add(this.taskNameField, taskNameFieldConstraints);
		this.add(this.dueDateButton, dueDateLabelConstraints);
		this.add(this.markOnHoldButton, markOnHoldButtonConstraints);
		this.add(this.deleteButton, deleteButtonConstraints);

		this.taskNameLabel.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		this.taskNameField.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));

		this.setFocusable(true);

		new FileDrop(this, new FileDrop.Listener() {

			/**
			 * Invoked when files are dropped onto this task view. Adds them to the task's resource
			 * list.
			 * 
			 * @param files the dropped files
			 */
			@SuppressWarnings("synthetic-access")
			public void filesDropped(final File[] files) {
				if (files.length > 0) {
					if (!TaskView.this.isEnabled()) {
						return;
					}

					ResourceList newResources = new ResourceList();
					Iterator<Resource> iter = TaskView.this.task.getResources().iterator();
					while (iter.hasNext()) {
						Resource rsrc = iter.next();
						newResources.add(rsrc);
					}
					for (File file : files) {
						newResources.add(new Resource(file));
					}

					TaskState oldState =
						new TaskState(	TaskView.this.task,
										TaskStateProperty.RESOURCES,
										TaskView.this.task.getResources());
					TaskState newState =
						new TaskState(TaskView.this.task, TaskStateProperty.RESOURCES, newResources);

					TaskView.this.firePropertyChange(	TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME,
														oldState,
														newState);
				}
			}

		});

		this.updateAppearance();

		this.taskNameField.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.NAME) {

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getNewValue() {
				return TaskView.this.taskNameField.getText();
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return TaskView.this.task.getName();
			}

		});

		@SuppressWarnings("unused")
		EditableLabelListener taskNameFieldListener =
			new EditableLabelListener(this, this.taskNameLabel, this.taskNameField);

		this.completeBox.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.COMPLETE) {

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getNewValue() {
				return TaskView.this.completeBox.isSelected();
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return TaskView.this.task.isComplete();
			}

		});

		this.completeBox.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.DATE_COMPLETED) {

			@Override
			public Object getNewValue() {
				return Calendar.getInstance();
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return TaskView.this.task.getDateCompleted();
			}

		});

		this.starButton.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.STARRED) {

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getNewValue() {
				return TaskView.this.starButton.isSelected();
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return TaskView.this.task.isStarred();
			}

		});

		this.disclosureTriangle.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {

				TaskView.this.toggleExpanded();
			}

		});

		this.dueDateButton.addActionListener(new ActionListener() {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent e) {
				JDialog scheduleDialog =
					new JDialog((JFrame)null, "Due Date For " + TaskView.this.task.getName());
				scheduleDialog.setUndecorated(true);
				final ChooseDueDatePanel chooseDueDatePanel =
					new ChooseDueDatePanel(TaskView.this.task);
				scheduleDialog.add(chooseDueDatePanel);
				scheduleDialog.getRootPane().setDefaultButton(chooseDueDatePanel.getOKButton());

				scheduleDialog.pack();
				scheduleDialog.setLocationRelativeTo(TaskView.this);
				// Make sure the sidebar doesn't cover the dialog!
				scheduleDialog.setLocation(	scheduleDialog.getLocation().x
													- TaskView.this.getTopLevelAncestor().getWidth(),
											scheduleDialog.getLocation().y);
				scheduleDialog.setModal(true);
				scheduleDialog.setResizable(false);
				scheduleDialog.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

				// You might be tempted to attach a WindowListener here instead. Rest assured, that
				// would be a terrible, terrible mistake.
				scheduleDialog.addComponentListener(new ComponentAdapter() {

					@Override
					public void componentHidden(ComponentEvent ce) {
						if (chooseDueDatePanel.getReturnCode() == ChooseDueDatePanel.USER_CANCELED) {
							return;
						}
						// getDueDate() might return null, in which case we clear the due date:
						TaskState oldState =
							new TaskState(	TaskView.this.task,
											TaskStateProperty.DUE_DATE,
											TaskView.this.task.getDueDate());
						TaskState newState =
							new TaskState(	TaskView.this.task,
											TaskStateProperty.DUE_DATE,
											chooseDueDatePanel.getDate());

						TaskView.this.firePropertyChange(	TaskViewPropertyNames.TASK_STATE_PROPERTY_NAME,
															oldState,
															newState);
					}

				});

				scheduleDialog.setVisible(true);
			}

		});

		this.markOnHoldButton.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.ON_HOLD) {

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getNewValue() {
				PendingState p = new PendingState();
				// Remember, this is the OLD state of the task!
				if (TaskView.this.task.getPendingState().isPending()) {
					p.setPending(false);
				} else {
					PutOnHoldDialog dlg =
						new PutOnHoldDialog((Window)null, "Put Task \""
								+ TaskView.this.task.getName() + "\" On Hold");
					dlg.setLocationRelativeTo(TaskView.this);
					// It's maaaaaagic, doo doo doo...
					dlg.setLocation(dlg.getLocation().x - 541, dlg.getLocation().y - 82);
					if (!dlg.presentDialog()) {
						TaskView.this.markOnHoldButton.setSelected(false);
						return TaskView.this.task.getPendingState();
					} else {
						p.setPending(true);
						p.setPendingNote(dlg.getReason());
						p.setPerson(dlg.getPerson());
					}
				}

				return p;
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public Object getOldValue() {
				return TaskView.this.task.getPendingState();
			}

		});

		this.deleteButton.addActionListener(new TaskPropertyActionListener(this,
				this.task,
				TaskStateProperty.DELETED) {

			@Override
			public Object getNewValue() {
				if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(	TaskView.this,
																			"Are you sure you want to delete this task?  This action cannot be undone.",
																			"Confirm Task Deletion",
																			JOptionPane.OK_CANCEL_OPTION)) {
					return true;
				} else {
					return false;
				}
			}

			@Override
			public Object getOldValue() {
				return false;
			}

		});

		this.expandedInfoPanel = new ExpandedTaskInfoPanel(this.task);
		this.expandedInfoPanel.setOpaque(false);

		this.addMouseListener(new ExpansionListener());
		DragListener dragListener = new DragListener();
		this.addMouseListener(dragListener);
		this.addMouseMotionListener(dragListener);
		this.addMouseListener(this.highlightListener);

		this.addMouseListener(new TrainingMenuListener() {

			@SuppressWarnings("synthetic-access")
			@Override
			public JMenuItem[] getItems() {
				String completeItem =
					TaskView.this.task.isComplete() ? "Mark as incomplete" : "Mark as complete";
				String pendingItem =
					TaskView.this.task.getPendingState().isPending()
							? "Mark as no longer on hold"
							: "Mark as on hold";
				String starItem = TaskView.this.task.isStarred() ? "Unstar" : "Star";
				return new JMenuItem[] { new JMenuItem(completeItem), new JMenuItem(pendingItem),
						new JMenuItem(starItem) };
			}

		});

		this.update();

	}

	/**
	 * Toggles whether or not the view is expanded.
	 */
	private void toggleExpanded() {
		this.isExpanded = !this.isExpanded;
		this.disclosureTriangle.setSelected(this.isExpanded);
		if (this.isExpanded) {

			BindingMaker bm = new BindingMaker(4, 4, 4, 4, 2, 8);

			RelativeConstraints expandedPanelConstraints = new RelativeConstraints();
			expandedPanelConstraints.addBindings(	bm.below(this.dueDateButton),
													bm.leftAlign(this.completeBox),
													bm.rightEdge());
			this.add(this.expandedInfoPanel, expandedPanelConstraints);
			this.expandedInfoPanel.update();
		} else {
			this.remove(this.expandedInfoPanel);
		}
		this.firePropertyChange(TaskViewPropertyNames.EXPANDED_PROPERTY_NAME,
								this.isExpanded,
								!this.isExpanded);

		this.updateAppearance();
	}

	/**
	 * Returns a string representation of this task view. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return "TaskView[" + this.getTask().toString() + "]";
	}

	/**
	 * Causes the view to synchronize itself with the Task model.
	 */
	public void update() {
		if (!this.task.isCaloAdded()) {
			this.caloAddedButton.setVisible(false);
			this.caloAddedButton.setPreferredSize(new Dimension(0, 0));
		} else {
			this.caloAddedButton.setVisible(true);
			this.caloAddedButton.setPreferredSize(new Dimension(16, 16));
		}
		this.taskNameLabel.setText(this.task.getName());
		this.taskNameLabel.setToolTipText("<html><p><strong>" + this.task.getName()
				+ "</strong></p><p>Click to change the name of this task.</p></html>");
		this.setToolTipText("<html><p>Drag this task on top of another task to group.</p><p>Drag this task next to another task to reorder.</p></html>");
		this.starButton.setSelected(this.task.isStarred());
		this.completeBox.setSelected(this.task.isComplete());
		this.dueDateButton.setEnabled(!this.task.isComplete());
		if (this.task.isComplete()) {
			if (this.task.getDateCompleted() != null) {
				this.dueDateButton.setText("Completed "
						+ DateFormat.getDateInstance(DateFormat.SHORT).format(this.task.getDateCompleted().getTime()));
			} else {
				this.dueDateButton.setText("Completed");
			}
		} else {
			if (this.task.getDueDate() != null) {
				this.dueDateButton.setText(this.task.getDueDateString());
				this.dueDateButton.setToolTipText("<html><p><strong>"
						+ this.dueDateButton.getText()
						+ "</strong></p><p>Click to change the due date of this task.</p></html>");
			} else {
				this.dueDateButton.setText("No due date");
				this.dueDateButton.setToolTipText("<html><p>This task has no due date.</p>"
						+ "<p>Click to set a due date for this task.</p></html>");
			}
		}
		this.markOnHoldButton.setSelected(this.task.getPendingState().isPending());
		this.markOnHoldButton.setEnabled(!this.task.isComplete());
		String toolTipString, selectedToolTipString;
		if (this.markOnHoldButton.isEnabled()) {
			toolTipString =
				"<html><p>Click to mark this task as &ldquo;on hold&rdquo;.</p>"
						+ "<p>A task that is on hold is waiting for someone or something outside your control.</p>"
						+ "<p>When you put a task on hold, you can tell CALO what you are waiting for.</p></html>";
			selectedToolTipString = this.getOnHoldToolTipString();
		} else {
			toolTipString =
				"<html><p>Marks this task as &ldquo;on hold&rdquo;.</p>"
						+ "<p>Disabled because this task is complete.</p></html>";
			selectedToolTipString = toolTipString;
		}

		if (this.task.getPendingState().isPending()) {
			this.markOnHoldButton.setToolTipText(selectedToolTipString);
		} else {
			this.markOnHoldButton.setToolTipText(toolTipString);
		}

		this.expandedInfoPanel.update();
		this.doLayout();
		SwingUtilities.invokeLater(new Runnable() {

			public void run() {
				// This ensures that all deferred changes to controls are
				// visible once this method
				// exits.
				TaskView.this.repaint();
			}

		});
	}

	/**
	 * Updates the appearance of the view when something changes.
	 */
	private void updateAppearance() {

		// Set up the border:
		if (this.isCurrentDropTarget) {
			this.setBorder(this.DROP_TARGET_BORDER);
		} else if (this.isMouseOver) {
			if (this.isExpanded) {
				this.setBorder(this.MOUSEOVER_EXPANDED_BORDER);
			} else {
				this.setBorder(this.MOUSEOVER_BORDER);
			}
		} else if (this.isExpanded) {
			this.setBorder(this.EXPANDED_BORDER);
		} else {
			this.setBorder(this.NORMAL_BORDER);
		}

		// Set the opacity:
		if (this.isDragging) {
			this.setOpacity(0.75);
		} else {
			this.setOpacity(1.0);
		}
	}

}
