/*
 * File: TaskViewBorder.java
 * 
 * Contains class TaskViewBorder
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.SystemColor;

import javax.swing.border.AbstractBorder;

/**
 * A custom border (actually comprising both a border and a background color) for use with
 * TaskViews.
 * 
 * @author Brian Ellis
 * 
 */
@SuppressWarnings("serial")
public class TaskViewBorder extends AbstractBorder {

	/**
	 * The radius of the rounded corners of the border.
	 */
	private static final int CORNER_RADIUS = 8;

	/**
	 * The color we will use to fill the background.
	 */
	private Color fillColor;

	/**
	 * The color we will use for the border's stroke.
	 */
	private Color strokeColor;

	/**
	 * Whether this border should be drawn to indicate that its view is a drop target.
	 */
	private boolean isDropTarget;

	/**
	 * Constructs a TaskViewBorder with the given fill and stroke colors.
	 * 
	 * @param fillColor the fill color to use
	 * @param strokeColor the stroke color to use
	 */
	public TaskViewBorder(Color fillColor, Color strokeColor) {
		this(fillColor, strokeColor, false);
	}

	/**
	 * Private superclass constructor. Do not use.
	 */
	@SuppressWarnings("unused")
	private TaskViewBorder() {
		super();
	}

	/**
	 * Constructs a TaskViewBorder with the given color and state.
	 * 
	 * @param fillColor the fill color of the border
	 * @param strokeColor the stroke color of the border
	 * @param isDropTarget if \c true, the border will be drawn to indicate its associated view is a
	 *            drop target
	 */
	public TaskViewBorder(Color fillColor, Color strokeColor, boolean isDropTarget) {
		super();
		this.fillColor = fillColor;
		this.strokeColor = strokeColor;
		this.isDropTarget = isDropTarget;
	}

	/**
	 * Paints this border.
	 * 
	 * @param c the component this border is attached to
	 * @param graphics the graphics context into which to paint this border
	 * @param x the x-coordinate of this border's top left corner
	 * @param y the y-coordinate of this border's top left corner
	 * @param width the width of this border's paint region
	 * @param height the height of this border's paint region
	 * 
	 * @see javax.swing.border.AbstractBorder#paintBorder(java.awt.Component, java.awt.Graphics,
	 *      int, int, int, int)
	 */
	@Override
	public void paintBorder(Component c, Graphics graphics, int x, int y, int width, int height) {
		super.paintBorder(c, graphics, x, y, width, height);

		if (!(graphics instanceof Graphics2D)) {
			throw new UnsupportedOperationException("The Java graphics subsystem is not compatible with Java2D, so this border cannot be drawn.");
		}
		Graphics2D g = (Graphics2D)graphics;
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setColor(this.fillColor);
		g.fillRoundRect(x,
						y,
						width,
						height,
						TaskViewBorder.CORNER_RADIUS,
						TaskViewBorder.CORNER_RADIUS);
		g.setColor(this.strokeColor);
		float strokeWidth = ((BasicStroke)g.getStroke()).getLineWidth();
		g.drawRoundRect(x,
						y,
						(int)(width - strokeWidth),
						(int)(height - strokeWidth),
						TaskViewBorder.CORNER_RADIUS,
						TaskViewBorder.CORNER_RADIUS);

		if (this.isDropTarget) {
			Stroke originalStroke = g.getStroke();
			g.setColor(SystemColor.textHighlight);
			g.setStroke(new BasicStroke(4.0f));
			g.drawRoundRect(x + 1,
							y + 1,
							width - 3,
							height - 3,
							TaskViewBorder.CORNER_RADIUS + 1,
							TaskViewBorder.CORNER_RADIUS + 1);
			g.setStroke(originalStroke);
		}

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);

		Stroke s =
			new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, new float[] {
					1.0f, 0.0f }, 0.0f);

		Stroke originalStroke = g.getStroke();

		g.setStroke(s);
		g.setColor(new Color(1.0f, 1.0f, 1.0f, 1.0f));
		g.drawLine(4, c.getHeight() - 7, c.getWidth() - 5, c.getHeight() - 7);
		g.setColor(new Color(0.0f, 0.0f, 0.0f, 0.25f));
		g.drawLine(5, c.getHeight() - 6, c.getWidth() - 4, c.getHeight() - 6);
		g.setColor(new Color(1.0f, 1.0f, 1.0f, 1.0f));
		g.drawLine(4, c.getHeight() - 5, c.getWidth() - 5, c.getHeight() - 5);
		g.setColor(new Color(0.0f, 0.0f, 0.0f, 0.25f));
		g.drawLine(5, c.getHeight() - 4, c.getWidth() - 4, c.getHeight() - 4);

		g.setStroke(originalStroke);

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	}

	/**
	 * Returns the insets of this border for the given component.
	 * 
	 * @param c the component to return insets for
	 */
	@Override
	public Insets getBorderInsets(Component c) {
		return new Insets(1, 1, 8, 1);
	}

}
