/* File: TaskOrdering.java
 *
 * Contains class TaskOrdering
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.support;

import edu.cmu.hcii.calo.model.Task;

/**
 * A small structure that stores a triad of adjacent tasks, of which the middle has just been
 * reordered, in the order they should now appear in the sidebar. If the reordered task should be
 * first or last in the sidebar's task view, then either the previous or next task will be \c null.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskOrdering {

	/**
	 * The task that has been reordered between the previous and next tasks.
	 */
	private Task reorderedTask;

	/**
	 * The task that should appear before the reordered one in the sidebar.
	 */
	private Task previousTask;

	/**
	 * The task that should appear after the reordered one in the sidebar.
	 */
	private Task nextTask;

	/**
	 * Constructs a task ordering, setting the reordered task to \p t.
	 * 
	 * @param t the reordered task
	 */
	public TaskOrdering(Task t) {
		this.reorderedTask = t;
	}

	/**
	 * Returns the task that should appear after the reordered task in the sidebar.
	 * 
	 * @return the next task
	 */
	public Task getNextTask() {
		return this.nextTask;
	}

	/**
	 * Sets the task that should appear after the reordered task in the sidebar.
	 * 
	 * @param nextTask the next task
	 */
	public void setNextTask(Task nextTask) {
		this.nextTask = nextTask;
	}

	/**
	 * Returns the task that should appear before the reordered task in the sidebar.
	 * 
	 * @return the previous task
	 */
	public Task getPreviousTask() {
		return this.previousTask;
	}

	/**
	 * Sets the task that should appear before the reordered task in the sidebar.
	 * 
	 * @param previousTask the previous task
	 */
	public void setPreviousTask(Task previousTask) {
		this.previousTask = previousTask;
	}

	/**
	 * Returns the reordered task.
	 * 
	 * @return the reordered task
	 */
	public Task getReorderedTask() {
		return this.reorderedTask;
	}

	/**
	 * Returns \c true if the task that should appear before the reordered task in the sidebar is \c
	 * null, \c false otherwise.
	 * 
	 * @return whether there is a previous task
	 */
	public boolean hasPreviousTask() {
		return (this.previousTask != null);
	}

	/**
	 * Returns \c true if the task that should apprear after the reordered task in the sidebar is \c
	 * null, \c false otherwise.
	 * 
	 * @return whether there is a next task
	 */
	public boolean hasNextTask() {
		return (this.nextTask != null);
	}

	/**
	 * Returns a string representation of this TaskOrdering. Useful only for debugging.
	 */
	@Override
	public String toString() {
		return "[" + this.previousTask + ", " + this.reorderedTask + ", " + this.nextTask + "]";
	}

}
