/* File: ListSortButton.java
 *
 * Contains class ListSortButton
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.viewer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.SystemColor;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;

import javax.swing.AbstractButton;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

/**
 * A button that emulates the sort buttons employed by Windows Vista's Explorer application to sort,
 * filter, and group files. Sort buttons consist of a toggle button (which makes the clicked sort
 * button active and thereafter toggles the sorting direction) and a pop-up menu button (which
 * invokes a menu allowing more advanced filtering, grouping, and stacking operations). In this
 * implementation, the menu exists only as an unimplemented stub.
 * 
 * @author Brian Ellis
 * 
 */
public class ListSortButton extends AbstractButton {

	/**
	 * The possibilities for sort direction.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	public enum Direction {
		/**
		 * Lower values come before higher values (e.g., names in alphabetical order, or small files
		 * before large ones).
		 */
		ASCENDING,

		/**
		 * Higher values come before lower values (e.g., names in reverse alphabetical order, or
		 * large files before small ones).
		 */
		DESCENDING
	}

	/**
	 * The width of the menu button on the right side of the control.
	 */
	private static final int MENU_BUTTON_WIDTH = 24;

	/**
	 * Whether this button is the currently active sort button for its associated list.
	 */
	private boolean isActive = false;

	/**
	 * The sort direction of this button.
	 */
	private Direction sortDirection = Direction.DESCENDING;

	/**
	 * Whether the mouse is currently over this button.
	 */
	private boolean mouseIsOver = false;

	/**
	 * Whether the mouse is currently being held down after having been pressed inside this button.
	 */
	private boolean mouseIsDown = false;

	/**
	 * If \c true, this button is not actually a button at all, but rather a decorative placard that
	 * looks identical but is not clickable and doesn't display its name. This can be used to extend
	 * the button bar to match the width of the list it's attached to.
	 */
	private boolean isDecorative;

	/**
	 * Creates a list sort button with the given text label. If \p isDecorative is \c true, this
	 * label will be ignored and the button will be blank and unclickable, for use as a decorative
	 * placard to round out the width of the button bar.
	 * 
	 * @param label the label for this button
	 * @param isDecorative whether this button is a decorative placard
	 */
	@SuppressWarnings("synthetic-access")
	public ListSortButton(String label, boolean isDecorative) {
		super();

		this.setText(label);
		this.isDecorative = isDecorative;

		if (!isDecorative) {

			this.addMouseListener(new MouseAdapter() {

				/**
				 * Shows a dummy menu when the menu button segment of the button is clicked. The
				 * menu doesn't actually do anything, but it does present a very stylish haiku.
				 * 
				 * @param e the mouse event
				 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
				 */
				@Override
				public void mouseClicked(MouseEvent e) {
					if (e.getPoint().x >= (getWidth() - MENU_BUTTON_WIDTH)) {
						JPopupMenu menu = new JPopupMenu();
						JLabel notImplementedLabel =
							new JLabel("<html><center>" + "<div>This does not work yet.</div>"
									+ "<div>Use your imagination</div>"
									+ "<div>and have a haiku.</div>" + "</center></html>");
						notImplementedLabel.setForeground(SystemColor.controlShadow);
						notImplementedLabel.setHorizontalAlignment(CENTER);
						notImplementedLabel.setBorder(new EmptyBorder(8, 8, 8, 8));
						menu.add(notImplementedLabel);
						Point pt = new Point(getWidth() - MENU_BUTTON_WIDTH, getHeight());
						SwingUtilities.convertPointToScreen(pt, ListSortButton.this);
						menu.setLocation(pt);
						menu.setVisible(true);
					} else {
						if (!ListSortButton.this.isActive()) {
							ListSortButton.this.setActive(true);
						} else {
							ListSortButton.this.setSortDirection(ListSortButton.this.getSortDirection() == Direction.ASCENDING
									? Direction.DESCENDING
									: Direction.ASCENDING);
						}
					}
				}

				/**
				 * Updates the state and repaints the button.
				 * 
				 * @param e the mouse event
				 */
				@Override
				public void mouseReleased(MouseEvent e) {
					ListSortButton.this.mouseIsDown = false;
					repaint();
				}

				/**
				 * Updates the state and repaints the button.
				 * 
				 * @param e the mouse event
				 */
				@Override
				public void mousePressed(MouseEvent e) {
					ListSortButton.this.mouseIsDown = true;
					repaint();
				}

				/**
				 * Updates the state and repaints the button.
				 * 
				 * @param e the mouse event
				 */
				@Override
				public void mouseExited(MouseEvent e) {
					ListSortButton.this.mouseIsOver = false;
					repaint();
				}

				/**
				 * Updates the state and repaints the button.
				 * 
				 * @param e the mouse event
				 */
				@Override
				public void mouseEntered(MouseEvent e) {
					ListSortButton.this.mouseIsOver = true;
					repaint();
				}

			});
		}
	}

	/**
	 * Constructs a list sort button with the given label. Equivalent to ListSortButton(\p label, \c
	 * false).
	 * 
	 * @param label the label for this button
	 * 
	 */
	public ListSortButton(String label) {
		this(label, false);
	}

	/**
	 * Returns whether this button is the active sort button for its associated list. Active buttons
	 * show their sort direction with an arrow. Note that ListSortButton does not itself guarantee
	 * that only one button will be active at a time; for that, consider using SortButtonGroup as
	 * well.
	 * 
	 * @return \c true if this button is active, \c false otherwise
	 */
	public boolean isActive() {
		return this.isActive;
	}

	/**
	 * Sets this button as the active sort button for its associated list. Active buttons show their
	 * sort direction with an arrow. Note that ListSortButton does not itself guarantee that only
	 * one button will be active at a time; for that, consider using SortButtonGroup as well.
	 * 
	 * @param isActive \c true if the button should be active, \c false otherwise
	 */
	public void setActive(boolean isActive) {
		boolean wasActive = this.isActive;
		this.isActive = isActive;
		this.firePropertyChange("active", wasActive, isActive);
		this.repaint();
	}

	/**
	 * Gets this button's current sort direction.
	 * 
	 * @return the sort direction
	 */
	public Direction getSortDirection() {
		return this.sortDirection;
	}

	/**
	 * Sets this button's current sort direction.
	 * 
	 * @param sortDirection the sort direction
	 */
	public void setSortDirection(Direction sortDirection) {
		Direction oldSortDirection = this.sortDirection;
		this.sortDirection = sortDirection;
		this.firePropertyChange("sortDirection", oldSortDirection, sortDirection);
		this.repaint();
	}

	/**
	 * Paints this button.
	 * 
	 * @param graphics the graphics context to paint into
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics graphics) {
		super.paintComponent(graphics);

		Graphics2D g = (Graphics2D)graphics;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		// Define colors based on the mouse state:

		Color white = new Color(1f, 1f, 1f, 0.5f);
		Color base =
			new Color(	SystemColor.control.getRed() / 255f,
						SystemColor.control.getGreen() / 255f,
						SystemColor.control.getBlue() / 255f,
						0.0f);
		Color dark;
		if (this.mouseIsDown && this.mouseIsOver) {
			dark =
				new Color(	SystemColor.controlShadow.getRed() / 255f,
							SystemColor.controlShadow.getGreen() / 255f,
							SystemColor.controlShadow.getBlue() / 255f,
							0.75f);
		} else if (this.mouseIsOver) {
			dark =
				new Color(	SystemColor.controlShadow.getRed() / 255f,
							SystemColor.controlShadow.getGreen() / 255f,
							SystemColor.controlShadow.getBlue() / 255f,
							0.5f);
		} else {
			dark =
				new Color(	SystemColor.controlShadow.getRed() / 255f,
							SystemColor.controlShadow.getGreen() / 255f,
							SystemColor.controlShadow.getBlue() / 255f,
							0.25f);
		}

		GradientPaint gloss =
			new GradientPaint(	0,
								0,
								white,
								0,
								this.getHeight() / 2,
								new Color(	dark.getRed() / 255f,
											dark.getGreen() / 255f,
											dark.getBlue() / 255f,
											dark.getAlpha() / 255f / 2f));
		GradientPaint shine =
			new GradientPaint(	0,
								this.getHeight() / 2,
								dark,
								0,
								this.getHeight(),
								new Color(	dark.getRed() / 255f,
											dark.getGreen() / 255f,
											dark.getBlue() / 255f,
											dark.getAlpha() / 255f / 4f));

		// Draw the base coloration:
		g.setColor(base);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setPaint(gloss);
		g.fillRect(0, 0, getWidth(), getHeight() / 2);
		g.setPaint(shine);
		g.fillRect(0, getHeight() / 2, getWidth(), getHeight() / 2);
		g.setColor(SystemColor.controlShadow);
		g.drawRect(0, 0, getWidth() - 1, getHeight());

		if (this.isDecorative) {
			return;
		}

		// Draw the pop-up menu affordance, if applicable:
		if (this.mouseIsDown || this.mouseIsOver) {
			Color grooveDarkColor =
				new Color(	SystemColor.controlShadow.getRed() / 255f,
							SystemColor.controlShadow.getGreen() / 255f,
							SystemColor.controlShadow.getBlue() / 255f,
							0.5f);
			Color grooveLightColor =
				new Color(	SystemColor.controlHighlight.getRed() / 255f,
							SystemColor.controlHighlight.getGreen() / 255f,
							SystemColor.controlHighlight.getBlue() / 255f,
							0.5f);

			g.setColor(grooveDarkColor);
			g.drawLine(this.getWidth() - MENU_BUTTON_WIDTH - 1, 1, this.getWidth()
					- MENU_BUTTON_WIDTH - 1, this.getHeight());
			g.setColor(grooveLightColor);
			g.drawLine(	this.getWidth() - MENU_BUTTON_WIDTH,
						1,
						this.getWidth() - MENU_BUTTON_WIDTH,
						this.getHeight());

			AffineTransform transform = g.getTransform();
			Shape popupArrow = new Polygon(new int[] { 0, 4, 8 }, new int[] { 0, 4, 0 }, 3);

			g.setColor(SystemColor.textText);

			g.translate(getWidth() - MENU_BUTTON_WIDTH + 8, 10);
			g.fill(popupArrow);
			g.setTransform(transform);

		}

		// Draw the sort arrow, if applicable:
		AffineTransform transform = g.getTransform();
		if (isActive()) {
			Shape sortArrow = null;

			switch (getSortDirection()) {
			case ASCENDING:
				sortArrow = new Polygon(new int[] { 0, 4, 8 }, new int[] { 4, 0, 4 }, 3);
				break;
			case DESCENDING:
				sortArrow = new Polygon(new int[] { 0, 4, 8 }, new int[] { 0, 4, 0 }, 3);
				break;
			}

			g.setColor(SystemColor.controlShadow);

			g.translate(getWidth() / 2 - 4, 2);
			g.draw(sortArrow);
		}
		g.setTransform(transform);

		// Draw the text:
		g.setColor(SystemColor.controlText);
		g.drawString(this.getText(), 9f, getHeight() / 2f + 4f);

	}

	/**
	 * Gets the preferred size of this button, which will always be 24 pixels tall with a width
	 * depending on the length of its label unless setPreferredSize() has been called with a
	 * non-null argument.
	 * 
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.isPreferredSizeSet()) {
			return super.getPreferredSize();
		}
		Graphics2D g = (Graphics2D)this.getGraphics();
		int width = g.getFontMetrics().stringWidth(this.getText());

		return new Dimension(width + MENU_BUTTON_WIDTH + 16, 24);
	}

}
