/* File: SortButtonGroup.java
 *
 * Contains class SortButtonGroup
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.viewer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

/**
 * A simple class that ensures that only one of the ListSortButtons added to it will be active at
 * any one time by deactivating all the others when a new one is made active.
 * 
 * @note Once a button is added to a group, it will remain in the group until it is explicitly
 *       removed, even if you drop the reference to the button group.
 * 
 * @author Brian Ellis
 * 
 */
public class SortButtonGroup implements PropertyChangeListener {

	/**
	 * The buttons that have been added to this group.
	 */
	private List<ListSortButton> sortButtons = new ArrayList<ListSortButton>();

	/**
	 * Invoked when a sort button becomes active. Deactivates all buttons in the group except that
	 * one.
	 * 
	 * @param e the property change event
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName().equals("active")) {
			if ((Boolean)e.getNewValue() == true) {
				for (ListSortButton sortButton : this.sortButtons) {
					if (sortButton == e.getSource()) {
						continue;
					}
					sortButton.setActive(false);
				}
			}
		}
	}

	/**
	 * Adds a button to this group.
	 * 
	 * @param button the button to add
	 */
	public void add(ListSortButton button) {
		this.sortButtons.add(button);
		button.removePropertyChangeListener("active", this);
		button.addPropertyChangeListener("active", this);
	}

	/**
	 * Removes a button from this group.
	 * 
	 * @param button the button to remove
	 */
	public void remove(ListSortButton button) {
		this.sortButtons.remove(button);
		button.removePropertyChangeListener("active", this);
	}

}
