/* File: TaskViewerPanel.java
 *
 * Contains class TaskViewerPanel
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.viewer;

import java.awt.BorderLayout;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Calendar;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.model.PendingState;
import edu.cmu.hcii.calo.model.Resource;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.FlatButton;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.PlaceholderTextField;
import edu.cmu.hcii.calo.view.SearchField;
import edu.cmu.relativelayout.Binding;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.Direction;
import edu.cmu.relativelayout.Edge;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;
import edu.cmu.relativelayout.RelativePosition;

/**
 * The panel containing the task viewer. This should be added to a generic frame when it is shown.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskViewerPanel extends CALOComponent {

	/**
	 * Tests whether TaskViewerPanel works correctly.
	 * 
	 * @param args command-line args, will be cheerfully eaten
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Error setting native LAF: " + e);
		}

		SwingUtilities.invokeLater(new Runnable() {

			public void run() {
				TaskManager.getManager().addTasklikeObject(new Task("Task 1", false));
				Task task = new Task("Task 2", true);
				task.setDueDate(Calendar.getInstance());
				PendingState pendingState = new PendingState();
				pendingState.setPending(true);
				pendingState.setPendingNote("Sample pending note here.");
				pendingState.setPerson("Will Haines");
				task.setPendingState(pendingState);
				task.addTag("hello");
				task.addTag("world");
				task.getResources().add(new Resource(new File("C:/Boston Expenses.xls")));
				TaskManager.getManager().addTasklikeObject(task);

				final JFrame testFrame = new JFrame("Task Viewer");
				testFrame.setLayout(new BorderLayout());
				testFrame.add(new TaskViewerPanel());

				testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

				testFrame.setSize(640, 400);
				testFrame.setVisible(true);
			}

		});

	}

	/**
	 * The task list contained within this panel.
	 */
	private TaskViewerTaskList taskList;

	/**
	 * Constructs a TaskViewerPanel.
	 */
	public TaskViewerPanel() {
		setup();
	}

	/**
	 * Gets the task list for this panel.
	 * 
	 * @return the task list
	 */
	public TaskViewerTaskList getTaskList() {
		return this.taskList;
	}

	/**
	 * Sets up the visual appearance of this panel.
	 */
	public void setup() {
		this.setLayout(new RelativeLayout());

		SortButtonGroup sortButtons = new SortButtonGroup();

		PlaceholderTextField addField = new PlaceholderTextField(18);
		addField.setPlaceholderText("Add a task (not implemented)");
		IconButton addButton =
			new IconButton(	new ImageIcon("Resources/add.png"),
							new ImageIcon("Resources/add-over.png"),
							new ImageIcon("Resources/add-pressed.png"));
		addButton.addActionListener(new ActionListener() {

			/**
			 * Shows a dialog indicating that this isn't implemented.
			 * 
			 * @param e the action event
			 */
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(	TaskViewerPanel.this,
												"Sorry, this isn't implemented at the moment.");
			}

		});
		FlatButton clearFiltersButton = new FlatButton("Clear all filters");
		clearFiltersButton.addActionListener(new ActionListener() {

			/**
			 * Shows a dialog indicating that this isn't implemented.
			 * 
			 * @param e the action event
			 */
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(	TaskViewerPanel.this,
												"Sorry, this isn't implemented at the moment.");
			}

		});
		SearchField searchField = new SearchField("Search (not implemented)");

		BindingMaker bm = new BindingMaker();

		RelativeConstraints addFieldConstraints =
			new RelativeConstraints(bm.topEdge(), bm.leftEdge(), bm.directlyLeftOf(addButton));
		RelativeConstraints addButtonConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(addField),
									bm.leftOf(clearFiltersButton));
		RelativeConstraints clearFiltersButtonConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(addField), bm.leftOf(searchField));
		RelativeConstraints searchFieldConstraints =
			new RelativeConstraints(bm.verticallyCenterAlign(addField), bm.rightEdge());

		this.add(addField, addFieldConstraints);
		this.add(addButton, addButtonConstraints);
		this.add(clearFiltersButton, clearFiltersButtonConstraints);
		this.add(searchField, searchFieldConstraints);

		String[] sortButtonNames =
			{ "Name", "Priority", "Deadline", "Date Added", "Project", "Tags", "Status", "People" };

		RelativePosition flushRightOf =
			new RelativePosition(Edge.LEFT, 1, Direction.LEFT, Edge.RIGHT);

		ListSortButton lastSortButton = null;
		for (String name : sortButtonNames) {
			RelativeConstraints sortButtonConstraints = new RelativeConstraints(bm.below(addField));

			if (lastSortButton == null) {
				sortButtonConstraints.addBinding(bm.leftEdge());
			} else {
				sortButtonConstraints.addBinding(new Binding(flushRightOf, lastSortButton));
			}

			ListSortButton sortButton = new ListSortButton(name);
			this.add(sortButton, sortButtonConstraints);
			sortButtons.add(sortButton);

			lastSortButton = sortButton;
		}

		ListSortButton sortBar = new ListSortButton("Sort Bar", true);
		RelativeConstraints sortBarConstraints =
			new RelativeConstraints(new Binding(flushRightOf, lastSortButton),
									bm.below(addField),
									bm.rightEdge());

		this.add(sortBar, sortBarConstraints);

		this.taskList = new TaskViewerTaskList();
		RelativeConstraints taskListConstraints =
			new RelativeConstraints(bm.leftEdge(),
									bm.rightEdge(),
									bm.bottomEdge(),
									bm.directlyBelow(lastSortButton));

		JScrollPane scrollPane = new JScrollPane(this.taskList);
		scrollPane.getViewport().setBackground(SystemColor.text);
		this.add(scrollPane, taskListConstraints);

	}
}
