/* File: TaskViewerTaskList.java
 *
 * Contains class TaskViewerTaskList
 *
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.viewer;

import java.util.Arrays;
import java.util.List;

import javax.swing.Scrollable;

import edu.cmu.hcii.calo.manager.TaskManager;
import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.model.TasklikeObject;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A JList-like view that shows tasks as they appear in the task viewer. We can't actually use a
 * JList for this because it uses its cells as "rubber stamps" for painting but does not actually
 * keep them resident in the view, which makes them non-interactive.
 * 
 * @author Brian Ellis
 * 
 */
public class TaskViewerTaskList extends CALOComponent implements Scrollable {

	/**
	 * The tasks represented by this list. Automatically refreshed from the TaskManager upon update.
	 */
	private List<TasklikeObject> tasks;

	/**
	 * Constructs a TaskViewerTaskList.
	 */
	public TaskViewerTaskList() {
		this.setLayout(new RelativeLayout());
		this.update();
	}

	/**
	 * Updates the tasks contained in the task list from the TaskManager. Should be called whenever
	 * the task manager's state changes.
	 * 
	 * @note Currently, every update causes the entire list to be re-created from scratch with all
	 *       new cells. This is somewhat expensive, as you might imagine, so avoid spamming updates
	 *       to the list if possible.
	 * 
	 * @todo Currently, the expanded state of the views is not preserved when updates occur, causing
	 *       all views to collapse.
	 */
	public void update() {
		this.removeAll();

		this.tasks = Arrays.asList(TaskManager.getManager().getTasks());

		BindingMaker bm = new BindingMaker(0, 0, 0, 0, 0, 0);

		CALOComponent previousCell = null;
		for (int i = 0; i < this.tasks.size(); i++ ) {
			assert this.tasks.get(i) instanceof Task : "we have a task list containing task groups, but we don't yet support this!";
			Task task = (Task)this.tasks.get(i);

			RelativeConstraints constraints =
				new RelativeConstraints(bm.leftEdge(), bm.rightEdge());
			if (previousCell == null) {
				constraints.addBinding(bm.topEdge());
			} else {
				constraints.addBinding(bm.below(previousCell));
			}
			CALOComponent cell = this.createListCell(i, task);

			this.add(cell, constraints);

			previousCell = cell;
		}

		this.validate();
		this.repaint();

	}

	/**
	 * Creates and returns a new TaskViewerTaskListCell for the given task at the given index. The
	 * index is only used to determine the background color of the list cell: odd-numbered indices
	 * have the default list background, while even-numbered indices use a light blue background.
	 * 
	 * @param index the index of the task in sorted order, needed to determine the cell's background
	 *            color
	 * @param task the task to be represented by this cell
	 * @return the new TaskViewerTaskListCell
	 */
	private CALOComponent createListCell(int index, Task task) {
		TaskViewerTaskListCell cell = new TaskViewerTaskListCell(this, index, task);
		cell.update();
		return cell;
	}
}
