/*
 * File: TaskViewerTaskListCell.java
 * 
 * Contains class TaskViewerTaskListCell
 * 
 * Author: Brian Ellis
 */

package edu.cmu.hcii.calo.view.task.viewer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.MatteBorder;

import edu.cmu.hcii.calo.model.Task;
import edu.cmu.hcii.calo.view.CALOComponent;
import edu.cmu.hcii.calo.view.IconButton;
import edu.cmu.hcii.calo.view.IconToggleButton;
import edu.cmu.hcii.calo.view.resources.ResourceListCellRenderer;
import edu.cmu.hcii.calo.view.resources.ResourceListModel;
import edu.cmu.relativelayout.BindingMaker;
import edu.cmu.relativelayout.RelativeConstraints;
import edu.cmu.relativelayout.RelativeLayout;

/**
 * A cell in the task list. This is the class that displays each task, analogous to TaskView in the
 * task pane.
 * 
 * @author Brian Ellis
 * 
 */
@SuppressWarnings("synthetic-access")
class TaskViewerTaskListCell extends CALOComponent {

	/**
	 * An action listener that, when invoked, pops up a dialog indicating that the Task Viewer is
	 * only a mockup and can't be used to make changes to tasks.
	 * 
	 * @note Since AWT doesn't seem to consistently support the concept of vetoing or consuming
	 *       events, we can't prevent the other event listeners attached to the various controls
	 *       from firing, and even if we could, there's no way to ensure that we'd be the first
	 *       listener to get invoked. Therefore, rather than trying to prevent the control from
	 *       changing state when clicked, we define an abstract method, undoAction(), that sets it
	 *       right back to the way it was to begin with. This is an annoying, but necessary,
	 *       solution.
	 * 
	 * @author Brian Ellis
	 * 
	 */
	private abstract class OnlyAMockupActionListener implements ActionListener {

		/**
		 * Invoked when a control that ought to change something about the task gets clicked on.
		 * Pops up a dialog stating that that won't work, and undoes the change by calling
		 * undoAction().
		 * 
		 * @param e the action event
		 */
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(	TaskViewerTaskListCell.this.list,
											"The Task Viewer is currently only a concept mockup, "
													+ "and cannot be used to change tasks at the moment.  "
													+ "Please use the task pane in the sidebar instead.  "
													+ "Sorry!");
			this.undoAction();
		}

		/**
		 * Should be implemented by each listener instance to undo whatever change clicking the
		 * control would otherwise have performed (e.g., selecting the button).
		 */
		protected abstract void undoAction();
	}

	/**
	 * The list of which this cell is a part.
	 */
	private TaskViewerTaskList list;

	/**
	 * Whether this cell is in its selected (i.e., expanded) state.
	 */
	private boolean isSelected = false;

	/**
	 * The index of this cell in the enclosing list. We use this to determine the background color
	 * we should use for this cell.
	 */
	private int index;

	/**
	 * The task represented by this cell.
	 */
	private Task task;

	/**
	 * The mouse listener that toggles whether this cell is selected.
	 */
	private MouseAdapter selectionListener;

	/**
	 * @name Controls
	 * 
	 * Various Swing controls contained in this cell.
	 */

	// @{
	private JLabel taskNameLabel;
	private JLabel dueDateLabel;
	private IconToggleButton starButton;
	private IconToggleButton completeBox;
	private IconButton deleteButton;
	private IconToggleButton caloAddedButton;
	private JLabel dateAddedLabel;
	private JLabel dateDueLabel;
	private JLabel statusLabel;
	private JList resourceList;
	private JLabel tagsLabelLabel;
	private JLabel tagsLabel;
	private IconButton onHoldButton;

	// @}

	/**
	 * Constructs a TaskViewerTaskListCell with the given list, index, and task.
	 * 
	 * @param taskViewerTaskList the list of which this cell is a part
	 * @param index the index of this task in the list, used to determine background color
	 * @param task the task represented by this cell
	 */
	public TaskViewerTaskListCell(TaskViewerTaskList taskViewerTaskList, int index, Task task) {
		this.list = taskViewerTaskList;
		this.task = task;
		this.index = index;
		setup();
	}

	/**
	 * Returns whether this cell is selected.
	 * 
	 * @return \c true if the cell is currently selected, \c false otherwise
	 */
	public boolean isSelected() {
		return this.isSelected;
	}

	/**
	 * Sets whether this cell is selected.
	 * 
	 * @param isSelected \c true if the cell should be selected, \c false otherwise
	 */
	public void setSelected(boolean isSelected) {
		this.isSelected = isSelected;
	}

	/**
	 * Causes this cell to update its contents based on the state of its represented task.
	 */
	public void update() {
		this.removeAll();

		this.taskNameLabel = new JLabel(this.task.getName());
		this.dueDateLabel =
			new JLabel(this.task.getDueDate() == null
					? "no due date"
					: this.task.getDueDateString());
		this.dueDateLabel.setForeground(SystemColor.controlShadow);

		ImageIcon starIcon = new ImageIcon("Resources/star-off.png");
		ImageIcon starPressedIcon = new ImageIcon("Resources/star-pressed.png");
		ImageIcon starSelectedIcon = new ImageIcon("Resources/star-on.png");

		this.starButton = new IconToggleButton(starIcon, starPressedIcon, starSelectedIcon);
		this.starButton.setUnselectedToolTipText("Click to star this task.");
		this.starButton.setSelectedToolTipText("Click to unstar this task.");
		this.starButton.setSelected(this.task.isStarred());
		this.starButton.addActionListener(new OnlyAMockupActionListener() {

			/**
			 * Undoes this action.
			 */
			@Override
			protected void undoAction() {
				TaskViewerTaskListCell.this.starButton.setSelected(TaskViewerTaskListCell.this.task.isStarred());
			}

		});

		this.completeBox =
			new IconToggleButton(	new ImageIcon("Resources/complete-unchecked.png"),
									new ImageIcon("Resources/complete-unchecked-over.png"),
									new ImageIcon("Resources/complete-unchecked-pressed.png"),
									new ImageIcon("Resources/complete-checked.png"),
									new ImageIcon("Resources/complete-checked-over.png"),
									new ImageIcon("Resources/complete-checked-pressed.png"));
		this.completeBox.setUnselectedToolTipText("Click to mark this task as completed.");
		this.completeBox.setSelectedToolTipText("Click to mark this task as not completed.");
		this.completeBox.setSelected(this.task.isComplete());
		this.completeBox.addActionListener(new OnlyAMockupActionListener() {

			/**
			 * Undoes this action.
			 */
			@Override
			protected void undoAction() {
				TaskViewerTaskListCell.this.completeBox.setSelected(TaskViewerTaskListCell.this.task.isComplete());
			}

		});

		ImageIcon caloIcon = new ImageIcon("Resources/calo.png");
		ImageIcon caloDimmedIcon = new ImageIcon("Resources/calo-dimmed.png");
		this.caloAddedButton =
			new IconToggleButton(	caloIcon,
									caloIcon,
									caloIcon,
									caloDimmedIcon,
									caloDimmedIcon,
									caloDimmedIcon);

		if (this.task.isCaloAdded() == false) {
			this.caloAddedButton.setPreferredSize(new Dimension(-4, 0));
		} else {
			this.caloAddedButton.setPreferredSize(null);
		}
		this.caloAddedButton.addPropertyChangeListener("pressedIcon", new PropertyChangeListener() {

			/**
			 * Shows a dialog indicating that this is only a mockup when the "CALO Added" button is
			 * clicked. This is a property change listener instead of an action listener because the
			 * CALO Action button doesn't fire action events.
			 * 
			 * @param evt the property change event
			 */
			public void propertyChange(PropertyChangeEvent evt) {
				if (TaskViewerTaskListCell.this.caloAddedButton.isSelected()) {
					JOptionPane.showMessageDialog(	TaskViewerTaskListCell.this,
													"The Task Viewer is currently only a concept mockup, and cannot be used to change tasks at the moment.  Please use the task pane in the sidebar instead.  Sorry!");
					TaskViewerTaskListCell.this.caloAddedButton.setSelected(false);
				}
			}

		});

		ImageIcon onHoldIcon = new ImageIcon("Resources/pending_status.png");
		this.onHoldButton = new IconButton(onHoldIcon, onHoldIcon, onHoldIcon);

		if (this.task.getPendingState().isPending() == false) {
			this.onHoldButton.setPreferredSize(new Dimension(-4, 0));
		} else {
			this.onHoldButton.setPreferredSize(null);
		}

		ImageIcon deleteIcon = new ImageIcon("Resources/trash.png");
		ImageIcon deleteOverIcon = new ImageIcon("Resources/trash-over.png");
		ImageIcon deletePressedIcon = new ImageIcon("Resources/trash-pressed.png");

		this.deleteButton = new IconButton(deleteIcon, deleteOverIcon, deletePressedIcon);
		this.deleteButton.setToolTipText("Click to delete this task.");
		this.deleteButton.addActionListener(new OnlyAMockupActionListener() {

			/**
			 * Does nothing, since there's no need to undo this action.
			 */
			@Override
			protected void undoAction() {
			// nothing to undo
			}

		});

		BindingMaker bm = new BindingMaker(8, 8, 8, 8, 4, 4);

		this.add(this.starButton, new RelativeConstraints(bm.topEdge(), bm.leftEdge()));
		this.add(	this.completeBox,
					new RelativeConstraints(bm.verticallyCenterAlign(this.starButton),
											bm.rightOf(this.starButton)));
		this.add(	this.onHoldButton,
					new RelativeConstraints(bm.verticallyCenterAlign(this.completeBox),
											bm.rightOf(this.completeBox)));
		this.add(	this.taskNameLabel,
					new RelativeConstraints(bm.verticallyCenterAlign(this.onHoldButton),
											bm.rightOf(this.onHoldButton)));
		this.add(	this.dueDateLabel,
					new RelativeConstraints(bm.verticallyCenterAlign(this.taskNameLabel),
											bm.leftOf(this.caloAddedButton)));
		this.add(	this.caloAddedButton,
					new RelativeConstraints(bm.verticallyCenterAlign(this.taskNameLabel),
											bm.leftOf(this.deleteButton)));
		this.add(	this.deleteButton,
					new RelativeConstraints(bm.verticallyCenterAlign(this.dueDateLabel),
											bm.rightEdge()));
		if (isSelected()) {
			this.setBorder(new CompoundBorder(	new MatteBorder(new Insets(0, 0, 1, 0),
																SystemColor.controlShadow),
												new EmptyBorder(0,
																0,
																bm.getRightMargin(),
																bm.getBottomMargin() - 1)));
			this.taskNameLabel.setFont(this.taskNameLabel.getFont().deriveFont(Font.BOLD));
			this.setBackground(new Color(1.0f, 1.0f, 0.87f));
		} else {
			this.setBorder(new EmptyBorder(0, 0, bm.getRightMargin(), bm.getBottomMargin()));
			if (this.index % 2 == 0) {
				this.setBackground(Color.WHITE);
			} else {
				this.setBackground(new Color(232, 242, 254));
			}
		}

		if (this.isSelected()) {
			JPanel propertiesPanel = new JPanel(new RelativeLayout());
			propertiesPanel.setOpaque(false);

			BindingMaker pbm = new BindingMaker(0, 0, 0, 0, 0, 4);

			this.dateAddedLabel =
				new JLabel("Added "
						+ DateFormat.getInstance().format(this.task.getDateAdded().getTime()));
			this.dateDueLabel = null;
			if (this.task.getDueDate() != null) {
				this.dateDueLabel =
					new JLabel("Due "
							+ DateFormat.getInstance().format(this.task.getDueDate().getTime()));
			}

			this.statusLabel = new JLabel();
			if (this.task.getPendingState().isPending()) {
				this.statusLabel.setText(String.format(	"Status: on hold %s %s",
														(this.task.getPendingState().getPendingNote() != null
																&& this.task.getPendingState().getPendingNote().length() > 0
																? "("
																		+ this.task.getPendingState().getPendingNote()
																		+ ")"
																: ""),
														(this.task.getPendingState().getPerson() != null
																&& this.task.getPendingState().getPerson().length() > 0
																? "waiting on "
																		+ this.task.getPendingState().getPerson()
																: "")));
			} else if (this.task.isComplete()) {
				this.statusLabel.setText("Status: complete");
			} else {
				this.statusLabel.setText("Status: active");
			}

			this.resourceList = new JList(new ResourceListModel(this.task));
			this.resourceList.setCellRenderer(new ResourceListCellRenderer());

			propertiesPanel.add(this.dateAddedLabel, new RelativeConstraints(	pbm.topEdge(),
																				pbm.leftEdge()));
			if (this.dateDueLabel != null) {
				propertiesPanel.add(this.dateDueLabel,
									new RelativeConstraints(pbm.below(this.dateAddedLabel),
															pbm.leftEdge()));
				propertiesPanel.add(this.statusLabel,
									new RelativeConstraints(pbm.below(this.dateDueLabel),
															pbm.leftEdge()));
			} else {
				propertiesPanel.add(this.statusLabel,
									new RelativeConstraints(pbm.below(this.dateAddedLabel),
															pbm.leftEdge()));
			}

			this.add(propertiesPanel, new RelativeConstraints(	bm.below(this.taskNameLabel),
																bm.leftAlign(this.taskNameLabel)));

			this.tagsLabelLabel = new JLabel("Tags: ");
			this.tagsLabel = new JLabel();
			this.tagsLabel.setFont(this.tagsLabel.getFont().deriveFont(Font.ITALIC));
			if (this.task.getTags().size() == 0) {
				this.tagsLabel.setText("(no tags)");
			} else {
				Iterator<String> it = this.task.getTags().iterator();

				// This string concatenation is quite possibly the most egregious waste of
				// computer
				// cycles ever created. Al Gore is probably going to come after me for the sheer
				// power wasted by allocating all these strings. This is truly horrifying.
				while (it.hasNext()) {
					String tag = it.next();
					this.tagsLabel.setText(this.tagsLabel.getText() + tag);
					if (it.hasNext()) {
						this.tagsLabel.setText(this.tagsLabel.getText() + ", ");
					}
				}
			}
			int oldThingy = bm.getHorizontalSpacing();
			bm.setHorizontalSpacing(18);

			this.add(this.tagsLabelLabel, new RelativeConstraints(	bm.rightOf(propertiesPanel),
																	bm.below(this.taskNameLabel)));
			this.add(	this.tagsLabel,
						new RelativeConstraints(bm.directlyRightOf(this.tagsLabelLabel),
												bm.rightEdge(),
												bm.below(this.taskNameLabel)));

			RelativeConstraints resourceListConstraints =
				new RelativeConstraints(bm.rightOf(propertiesPanel),
										bm.rightEdge(),
										bm.below(this.tagsLabelLabel));
			Border listBorder = UIManager.getBorder("ScrollPane.border");
			this.resourceList.setBorder(listBorder);

			this.add(this.resourceList, resourceListConstraints);

			bm.setHorizontalSpacing(oldThingy);
		}

		if (this.selectionListener == null) {
			this.selectionListener = new MouseAdapter() {

				/**
				 * Toggles the selected state of this cell when it is clicked.
				 * 
				 * @param e the mouse event
				 */
				@Override
				public void mouseClicked(MouseEvent e) {
					setSelected(!isSelected());
					update();
				}

				/**
				 * Sets the colors of the cell background and all the labels within it to their
				 * selected variants.
				 * 
				 * @param e the mouse event
				 * 
				 * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
				 */
				@Override
				public void mousePressed(MouseEvent e) {
					TaskViewerTaskListCell.this.setBackground(SystemColor.textHighlight);

					if (TaskViewerTaskListCell.this.dueDateLabel != null) {
						TaskViewerTaskListCell.this.dueDateLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.caloAddedButton != null) {
						TaskViewerTaskListCell.this.caloAddedButton.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.dateAddedLabel != null) {
						TaskViewerTaskListCell.this.dateAddedLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.dateDueLabel != null) {
						TaskViewerTaskListCell.this.dateDueLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.statusLabel != null) {
						TaskViewerTaskListCell.this.statusLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.tagsLabel != null) {
						TaskViewerTaskListCell.this.tagsLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.tagsLabelLabel != null) {
						TaskViewerTaskListCell.this.tagsLabelLabel.setForeground(SystemColor.textHighlightText);
					}
					if (TaskViewerTaskListCell.this.taskNameLabel != null) {
						TaskViewerTaskListCell.this.taskNameLabel.setForeground(SystemColor.textHighlightText);
					}

				}

				/**
				 * Sets the colors of the cell background and all the labels within it to their
				 * unselected variants. Normally these values are immediately clobbered by the
				 * invocation of setSelected() in the mouseClicked() event handler, but in the
				 * special case where the user drags the mouse outside the cell before releasing the
				 * button that code is never called, so we need to set things back to the way they
				 * were.
				 * 
				 * @param e the mouse event
				 * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
				 */
				@Override
				public void mouseReleased(MouseEvent e) {
					if (isSelected()) {
						TaskViewerTaskListCell.this.setBackground(new Color(1.0f, 1.0f, 0.87f));
					} else {
						if (TaskViewerTaskListCell.this.index % 2 == 0) {
							TaskViewerTaskListCell.this.setBackground(Color.WHITE);
						} else {
							TaskViewerTaskListCell.this.setBackground(new Color(232, 242, 254));
						}
					}

					if (TaskViewerTaskListCell.this.dueDateLabel != null) {
						TaskViewerTaskListCell.this.dueDateLabel.setForeground(SystemColor.controlShadow);
					}
					if (TaskViewerTaskListCell.this.dateAddedLabel != null) {
						TaskViewerTaskListCell.this.dateAddedLabel.setForeground(SystemColor.textText);
					}
					if (TaskViewerTaskListCell.this.dateDueLabel != null) {
						TaskViewerTaskListCell.this.dateDueLabel.setForeground(SystemColor.textText);
					}
					if (TaskViewerTaskListCell.this.statusLabel != null) {
						TaskViewerTaskListCell.this.statusLabel.setForeground(SystemColor.textText);
					}
					if (TaskViewerTaskListCell.this.tagsLabel != null) {
						TaskViewerTaskListCell.this.tagsLabel.setForeground(SystemColor.textText);
					}
					if (TaskViewerTaskListCell.this.tagsLabelLabel != null) {
						TaskViewerTaskListCell.this.tagsLabelLabel.setForeground(SystemColor.textText);
					}
					if (TaskViewerTaskListCell.this.taskNameLabel != null) {
						TaskViewerTaskListCell.this.taskNameLabel.setForeground(SystemColor.textText);
					}
				}

			};
		}
		this.removeMouseListener(this.selectionListener);
		this.addMouseListener(this.selectionListener);

		this.revalidate();
		this.repaint();
	}

	/**
	 * Sets up the visual appearance of this cell.
	 */
	private void setup() {
		this.setOpaque(true);
		this.setLayout(new RelativeLayout());
	}
}
